@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -29,6 +29,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "gdis.h"
 #include "coords.h"
 #include "edit.h"
+#include "error.h"
 #include "interface.h"
 #include "matrix.h"
 #include "measure.h"
@@ -37,6 +38,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "numeric.h"
 #include "morph.h"
 #include "opengl.h"
+#include "render.h"
 #include "select.h"
 #include "zone.h"
 
@@ -55,20 +57,21 @@ struct shel_pak *shell;
 
 for (item=list ; item ; item=g_slist_next(item))
   {
-  core = (struct core_pak *) item->data;
+  core = item->data;
 
   printf("core: %p, shell: %p ", core, core->shell);
   P3VEC(" : ", core->x);
   }
 for (item=list ; item ; item=g_slist_next(item))
   {
-  shell = (struct shel_pak *) item->data;
+  shell = item->data;
 
   printf("shell: %p, core: %p\n", shell, shell->core);
   P3VEC(" : ", shell->x);
   }
 }
-void dump_cores(GSList *list)
+
+void print_core_list(GSList *list)
 {
 gint i=0;
 GSList *item;
@@ -77,22 +80,22 @@ struct shel_pak *shell;
 
 for (item=list ; item ; item=g_slist_next(item))
   {
-  core = (struct core_pak *) item->data;
+  core = item->data;
 
 /*
 if (i==4)
 */
   {
   printf("[%4s] %1d %1d c %10.4f %10.4f %10.4f : %6.2f\n",
-          core->label, core->primary, core->orig,
+          core->atom_label, core->primary, core->orig,
           core->x[0], core->x[1], core->x[2], core->charge);
 
   if (core->shell)
     {
-    shell = (struct shel_pak *) core->shell;
+    shell = core->shell;
 
     printf("[%4s] %1d %1d s %10.4f %10.4f %10.4f : %6.2f\n",
-            shell->label, shell->primary, shell->orig,
+            shell->shell_label, shell->primary, shell->orig,
             shell->x[0], shell->x[1], shell->x[2], shell->charge);
     }
   }
@@ -100,48 +103,48 @@ if (i==4)
   }
 }
 
+void print_core_list_cart(GSList *list)
+{
+    gint i=0;
+    GSList *item;
+    struct core_pak *core;
+    struct shel_pak *shell;
+    
+    for (item=list ; item ; item=g_slist_next(item))
+    {
+        core = item->data;
+        
+        /*
+         if (i==4)
+         */
+        {
+            printf("[%4s] %1d %1d c %10.4f %10.4f %10.4f : %6.2f\n",
+                   core->atom_label, core->primary, core->orig,
+                   core->rx[0], core->rx[1], core->rx[2], core->charge);
+            
+            if (core->shell)
+            {
+                shell = core->shell;
+                
+                printf("[%4s] %1d %1d s %10.4f %10.4f %10.4f : %6.2f\n",
+                       shell->shell_label, shell->primary, shell->orig,
+                       shell->rx[0], shell->rx[1], shell->rx[2], shell->charge);
+            }
+        }
+        i++;
+    }
+}
+
 /****************/
 /* SPECIAL OBJS */
 /****************/
 #define DEBUG_INIT 0
-void init_objs(gint type, struct model_pak *data)
+void coords_init(gint type, struct model_pak *data)
 {
-GSList *list;
-struct core_pak *core;
-struct shel_pak *shel;
-
 g_return_if_fail(data != NULL);
 
 switch(type)
   {
-/* set all atoms back to element colour */
-  case REFRESH_COLOUR:
-    model_colour_scheme(data->colour_scheme, data);
-    for (list=data->shels ; list ; list=g_slist_next(list))
-      {
-      shel = (struct shel_pak *) list->data;
-      ARR3SET(shel->colour, elements[shel->atom_code].colour);
-      }
-    break;
-
-/* scale all atom colours by their sof */
-  case SOF_COLOUR:
-    if (data->has_sof)
-      for (list=data->cores ; list ; list=g_slist_next(list))
-        {
-        core = (struct core_pak *) list->data;
-        VEC3MUL(core->colour, core->sof);
-        }
-    break;
-
-  case REFRESH_BONDS:
-    for (list=data->cores ; list ; list=g_slist_next(list))
-      {
-      core = (struct core_pak *) list->data;
-      init_elem(core, type, data);
-      }
-    break; 
-
 /* atomic coordinates processing */
   case INIT_COORDS:
 /* find unique elements */
@@ -152,13 +155,13 @@ switch(type)
 /* update electrostatic info */
     calc_emp(data);
 /* repetition here, since there is interdependence */
-/* between centroid (cent_coords) and latmat (calc_coords) calc */
-    cent_coords(1, data);
-    calc_coords(INITIAL, data);
+/* between centroid (coords_center) and latmat (coords_compute) calc */
+    coords_center(1, data);
+    coords_compute(data);
   case CENT_COORDS:
-    cent_coords(1, data);
+    coords_center(1, data);
   case REDO_COORDS:
-    calc_coords(REFRESH, data);
+    coords_compute(data);
     break;
 
   default:
@@ -166,6 +169,109 @@ switch(type)
   }
 }
 
+/******************************************/
+/* convert fractional coords to cartesian */
+/******************************************/
+void coords_make_cartesian(struct model_pak *model)
+{
+GSList *list;
+struct core_pak *core;
+struct shel_pak *shel;
+struct vertex_pak *vec;
+
+g_assert(model != NULL);
+
+/* transform cores */
+for (list=model->cores ; list ; list=g_slist_next(list))
+  {
+  core = list->data;
+  vecmat(model->latmat, core->x);
+  }
+/* transform shells */
+for (list=model->shels ; list ; list=g_slist_next(list))
+  {
+  shel = list->data;
+  vecmat(model->latmat, shel->x);
+  }
+/* morphology vertices */
+for (list=model->vertices ; list ; list=g_slist_next(list))
+  {
+  vec = list->data;
+  vecmat(model->latmat, vec->x);
+  }
+matrix_identity(model->latmat);
+matrix_identity(model->ilatmat);
+matrix_identity(model->rlatmat);
+}
+
+/******************************************/
+/* convert cartesian coords to fractional */
+/******************************************/
+void coords_make_fractional(struct model_pak *model)
+{
+GSList *list;
+struct core_pak *core;
+struct shel_pak *shel;
+struct vertex_pak *vec;
+
+g_assert(model != NULL);
+
+/* transform cores */
+for (list=model->cores ; list ; list=g_slist_next(list))
+  {
+  core = list->data;
+  vecmat(model->ilatmat, core->x);
+  }
+/* transform shells */
+for (list=model->shels ; list ; list=g_slist_next(list))
+  {
+  shel = list->data;
+  vecmat(model->ilatmat, shel->x);
+  }
+/* morphology vertices */
+for (list=model->vertices ; list ; list=g_slist_next(list))
+  {
+  vec = list->data;
+  vecmat(model->ilatmat, vec->x);
+  }
+}
+
+/*************************************/
+/* process coordinate scaling factor */
+/*************************************/
+void coords_init_units(struct model_pak *model)
+{
+gdouble scale=1.0;
+GSList *list;
+struct core_pak *core;
+struct shel_pak *shel;
+
+g_assert(model != NULL);
+
+switch (model->coord_units)
+  {
+  case BOHR:
+    scale = AU2ANG;
+    break;
+  default:
+    return;
+  }
+
+/* transform cores */
+for (list=model->cores ; list ; list=g_slist_next(list))
+  {
+  core = list->data;
+  VEC3MUL(core->x, scale);
+  }
+
+/* transform shells */
+for (list=model->shels ; list ; list=g_slist_next(list))
+  {
+  shel = list->data;
+  VEC3MUL(shel->x, scale);
+  }
+}
+
 /********/
 /* AXES */
 /********/
@@ -203,7 +309,6 @@ VEC3SET(data->cell[6].x, 1.0, b, c);
 VEC3SET(data->cell[7].x, 1.0, 0.0, c);
 }
 
-
 /*********************************/
 /* construct core-shell linkages */
 /*********************************/
@@ -211,26 +316,26 @@ VEC3SET(data->cell[7].x, 1.0, 0.0, c);
 #define MAX_SHELL_DIST 0.6
 void shell_make_links(struct model_pak *model)
 {
-gint i, match, total;
-gchar *text;
+gint match;
 gdouble min, sep, vec[3];
-GSList *list1, *list2, *locality;
+gpointer zone;
+GSList *list1, *list2, *locality, *floating;
 struct core_pak *core;
 struct shel_pak *shel;
 
 /* checks */
 g_assert(model != NULL);
-g_assert(model->zones != NULL);
+g_assert(model->zone_array != NULL);
 
 /* enumerate all shells */
-total = 0;
+floating = NULL;
 for (list1=model->shels ; list1 ; list1=g_slist_next(list1))
   {
-  shel = (struct shel_pak *) list1->data;
+  shel = list1->data;
 
 /* get neighbourhood core list for the shell */
-  i = zone_index(shel->x, model);
-  locality = zone_area_cores(1, model->zones[i], model);
+  zone = zone_get(shel->x, model->zone_array);
+  locality = zone_area_cores(1, zone, model->zone_array);
 
 /* upper bound for core-shell link */
   min = MAX_SHELL_DIST*MAX_SHELL_DIST;
@@ -239,7 +344,7 @@ for (list1=model->shels ; list1 ; list1=g_slist_next(list1))
   match = 0;
   for (list2=locality ; list2 ; list2=g_slist_next(list2))
     {
-    core = (struct core_pak *) list2->data;
+    core = list2->data;
 
     if (shel->atom_code == core->atom_code)
       {
@@ -264,106 +369,37 @@ for (list1=model->shels ; list1 ; list1=g_slist_next(list1))
     }
 /* record any shells with no linkages */
   if (!match)
-    total++;
+    {
+    floating = g_slist_prepend(floating, shel);
+    error_table_entry("Error: found shell with no core.\n");
+    }
+
 /* free the constructed locality */
   g_slist_free(locality);
   }
 
 /* found some floating shells? */
-if (total)
+if (floating)
   {
-  text = g_strdup_printf("Warning: found %d shell(s) with no cores.\n", total);
-  show_text(WARNING, text);
+/*
+  text = g_strdup_printf("Warning: found %d shell(s) with no core(s).\n",
+                         g_slist_length(floating));
+  gui_text_show(WARNING, text);
   g_free(text);
-  }
-}
+*/
 
-/**********/
-/* SHELLS */
-/**********/
-void update_shells(struct model_pak *data)
-{
-gint tot_match, match;
-gdouble min, sep, vec[3];
-GSList *core_list, *shel_list;
-struct core_pak *core;
-struct shel_pak *shel;
-
-
-/* CURRENT */
-shell_make_links(data);
-return;
-
-
-
-/* checks */
-g_assert(data != NULL);
-
-/* find core-shell connections */
-tot_match = 0;
-for (core_list=data->cores ; core_list ; core_list=g_slist_next(core_list))
-  {
-  core = (struct core_pak *) core_list->data;
-  core->shell = NULL;
-
-/* the closest separation found */
-  min = MAX_SHELL_DIST*MAX_SHELL_DIST;
-  match = 0;
-  for (shel_list=data->shels ; shel_list ; shel_list=g_slist_next(shel_list))
+/* flag shells (diff colour?) with no cores */
+  for (list1=floating ; list1 ; list1=g_slist_next(list1))
     {
-    shel = (struct shel_pak *) shel_list->data;
+    shel = list1->data;
 
-    if (shel->atom_code == core->atom_code)
-      {
-/* get the absolute fractional separation */
-      ARR3SET(vec, core->x);
-      ARR3SUB(vec, shel->x);
+/*
+printf("Floating shell=%p\n", shel);
+*/
 
-/* NEW - get the minimum fractional separation */
-      fractional_minsq(vec, data->periodic);
-
-/* convert to cartesian & compare with cutoff */
-      vecmat(data->latmat, vec);
-      sep = VEC3MAGSQ(vec);
-      if (sep < min)
-        {
-/* FIXME - what to do with previous matches? eg set (core->shell)->shell = NULL ? */
-/* reference each other */
-        core->shell = shel;
-        shel->core = core;
-        match++;
-/* keep track of the minimum, so only the closest pair is recorded */
-        min = sep;
-        }
-      }
+    VEC3SET(shel->colour, 1.0, 1.0, 1.0);
     }
-  if (match)
-    tot_match++;
-  }
-
-#if DEBUG_SHELLS
-printf("core-shell connections: %d/%d\n", tot_match, g_slist_length(data->shels));
-#endif
-
-/* do some shells float down here? */
-if (tot_match != g_slist_length(data->shels))
-  {
-/* TODO - somehow only print this once for a model */
-  show_text(WARNING, "Warning: missing core-shell connection(s).\n");
-#if DEBUG_SHELLS
-for (shel_list=data->shels ; shel_list ; shel_list=g_slist_next(shel_list))
-  {
-  shel = (struct shel_pak *) shel_list->data;
-  if (!shel->core)
-    {
-/* NEW - also calculate cartesian values */
-    ARR3SET(shel->rx, shel->x);
-    vecmat(data->latmat, shel->rx);
-    ARR3ADD(shel->rx, data->centroid);
-    print_shell(shel); 
-    }
-  }
-#endif
+  g_slist_free(floating);
   }
 }
 
@@ -380,11 +416,10 @@ struct core_pak *core, *best=NULL;
 struct image_pak *image;
 
 /* tolerance (angstroms squared)  */
-tol = data->rmax * 20.0*data->scale/sysenv.size;
+tol = data->rmax * 20.0/sysenv.size;
 
 #if DEBUG_SEEK_COORD3D
 printf(" size = %d\n", sysenv.size);
-printf("scale = %f\n", data->scale);
 printf(" rmax = %f\n", data->rmax);
 printf("  tol = %f\n", tol);
 #endif
@@ -394,7 +429,7 @@ do
   if (ilist)
     {
 /* retrieve periodic image vector */
-    image = (struct image_pak *) ilist->data;
+    image = ilist->data;
     ARR3SET(piv, image->rx);
     ilist = g_slist_next(ilist);
     }
@@ -407,11 +442,10 @@ do
 /* attempt to match core coords */
   for (list=data->cores ; list ; list=g_slist_next(list))
     {
-    core = (struct core_pak *) list->data;
+    core = list->data;
 
     ARR3SET(vec, core->rx);
     ARR3ADD(vec, piv);
-    VEC3MUL(vec, data->scale);
     ARR3SUB(vec, x);
     d2 = vec[0]*vec[0] + vec[1]*vec[1];
     if (d2 < tol)
@@ -448,12 +482,11 @@ printf("search: [%d,%d]\n", x, y);
 /* TODO - periodic images */
 for (list=data->cores ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
 
   ARR3SET(vec, core->rx);
-  VEC3MUL(vec, data->scale);
 
-  gl_get_window_coords(vec, w);
+  gl_unproject(w, vec, canvas_find(data));
 
 #if DEBUG_SEEK_COORD2D
 printf("  (%d, %d)\n", w[0], w[1]);
@@ -519,7 +552,7 @@ while (list)
 /* COORDS */
 /**********/
 #define DEBUG_CENT 0
-gint cent_coords(gint global, struct model_pak *data)
+gint coords_center(gint global, struct model_pak *data)
 {
 gint i, n;
 gdouble min[3], max[3], vec[3], r, r2, rmax;
@@ -544,7 +577,7 @@ if (!global && g_slist_length(data->selection))
 n=0;
 for (list=clist ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
   if (core->status & DELETED)
     continue;
 
@@ -572,20 +605,16 @@ else
     }
   }
 
-/* if centroid is substantially within repeat cell, make pbc/2 the centroid */
-if (data->periodic)
-  {
-  for (i=data->periodic ; i-- ; )
-    if (fabs(data->centroid[i] - 0.5) < 0.45)
-      data->centroid[i] = 0.5; 
-  }
+/* NEW - always make a pbc/2 the centroid */
+for (i=data->periodic ; i-- ; )
+  data->centroid[i] = 0.5; 
 
 /* adjust for periodic images */
 n=1;
 VEC3SET(vec, 0.0, 0.0, 0.0);
 for (list=data->images ; list ; list=g_slist_next(list))
   {
-  image = (struct image_pak *) list->data;
+  image = list->data;
 
   ARR3ADD(vec, image->pic);
   n++;
@@ -596,7 +625,7 @@ ARR3ADD(data->centroid, vec);
 /* get distance of furtherest displayed item from centroid */
 for (list=clist ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
   if (core->status & DELETED)
     continue;
 
@@ -606,7 +635,7 @@ for (list=clist ; list ; list=g_slist_next(list))
     ARR3SET(vec, core->x);
     if (ilist)
       {
-      image = (struct image_pak *) ilist->data;
+      image = ilist->data;
 /* image */
       ARR3ADD(vec, image->pic);
       ilist = g_slist_next(ilist);
@@ -636,7 +665,7 @@ if (data->periodic)
 /* centroid removal */
     ARR3SUB(vec, data->centroid);
 /* transform */
-    vecmat(data->latmat,vec);  
+    vecmat(data->latmat, vec);  
 /* dist sq. */
     r = VEC3MAGSQ(vec);
     if (r > r2)
@@ -644,7 +673,7 @@ if (data->periodic)
     }
   }
 
-/* check vertices */
+/* check morphology vertices */
 for (list=data->vertices ; list ; list=g_slist_next(list))
   {
   v = list->data;
@@ -653,42 +682,50 @@ for (list=data->vertices ; list ; list=g_slist_next(list))
   vecmat(data->latmat, vec);
   r = VEC3MAGSQ(vec);
   if (r > r2)
-    r2 = r; 
+    r2 = r;
   }
 
 /* check spatial objects */
 for (list=data->spatial ; list ; list=g_slist_next(list))
   {
   spatial = list->data;
-
-/* molecular surface contribution */
-  if (spatial->type == SPATIAL_MOLSURF)
+  p1 = g_slist_nth_data(spatial->list, 0);
+  if (p1)
     {
-    p1 = g_slist_nth_data(spatial->data, 0);
-
     ARR3SET(vec, p1->x);
     ARR3SUB(vec, data->centroid);
     vecmat(data->latmat, vec);  
-
     r = VEC3MAGSQ(vec);
     if (r > r2)
       r2 = r; 
     }
   }
-rmax = sqrt(r2);
+
+/* check camera waypoints */
+for (list=data->waypoint_list ; list ; list=g_slist_next(list))
+  {
+  struct camera_pak *camera = list->data;
+
+  r = VEC3MAGSQ(camera->x);
+  if (r > r2)
+    r2 = r;
+  }
 
 /* assign the calculated value, unless 0 */
+rmax = sqrt(r2);
 if (rmax)
   data->rmax = rmax;
 else
-  data->rmax = RMAX_FUDGE;
-data->zoom = data->rmax/data->scale;
+  data->rmax = 5.0;   /* seems to give ~ correct axes lengths */
+
+data->zoom = data->rmax;
 
 #if DEBUG_CENT
 P3VEC("centroid = ", data->centroid);
 printf("rmax = %f\n", data->rmax);
 printf("zoom = %f\n", data->zoom);
-printf("cent_coords() done.\n");
+printf("coords_center() done.\n");
+P3MAT("latmat", data->latmat);
 #endif
 
 return(0);
@@ -740,6 +777,8 @@ for (i=3 ; i-- ; )
 /* NB: even tho' we check the "fractional" part - the values */
 /* will still be cartesian for isolated (or the appropriate */
 /* mixed frac/cart values for 1D/2D) models */
+/* FIXME - not always true - newly generated surfaces have z values */
+/* in fractional coords */
 /* loop to get limits */
 for (item=list ; item ; item=g_slist_next(item))
   {
@@ -755,16 +794,16 @@ for (item=list ; item ; item=g_slist_next(item))
   }
 }
 
-/**********************/
-/* CALC SCREEN COORDS */
-/**********************/
+/**********************************/
+/* compute cartesian world coords */
+/**********************************/
+/* TODO - make more fine grained ie cores/shells or spatials (or ALL) */
 #define DEBUG_UPDATE_COORDS 0
-void calc_coords(gint mode, struct model_pak *data)
+void coords_compute(struct model_pak *data)
 {
 gint n, ghost, model;
-gdouble scale;
-gdouble rot[9], vec[3];
-gdouble tmp4[16], mat4[16], vec4[4];
+gdouble vec[3];
+gdouble mat4[16], vec4[4];
 GSList *list=NULL, *list1, *list2;
 struct core_pak *core;
 struct shel_pak *shell;
@@ -779,7 +818,7 @@ struct image_pak *image;
 GSList *plist, *glist=NULL, *olist;
 
 #if DEBUG_UPDATE_COORDS
-printf("calc_coords() start.\n");
+printf("coords_compute() start.\n");
 #endif
 
 g_return_if_fail(data != NULL);
@@ -789,49 +828,12 @@ ghost=0;
 orig = data;
 while(data)
   {
-/* get current incremental rotation matrix */
-  make_rotmat(rot, orig->da, mode);
-  switch(mode)
-    {
-    case INITIAL:
-      init_rotmat(data->rotmat);
-      break;
-/* set and constrain (0-360) angles */
-    case YAW:
-      data->angle[0] += R2D*orig->da;
-      ca_deg(&data->angle[0]);
-      break;
-    case PITCH:
-      data->angle[1] += R2D*orig->da;
-      ca_deg(&data->angle[1]);
-      break;
-    case ROLL:
-      data->angle[2] += R2D*orig->da;
-      ca_deg(&data->angle[2]);
-      break;
-    }
-
 #if DEBUG_UPDATE_COORDS
 P3MAT("instantaneous rot matrix", rot);
-#endif
-
-/* update cummulative rotation matrix */
-matmat(rot, data->rotmat);
-/* compute inverse, needed for correct selection handling */
-ARR3SET(&data->irotmat[0], &data->rotmat[0]);
-ARR3SET(&data->irotmat[3], &data->rotmat[3]);
-ARR3SET(&data->irotmat[6], &data->rotmat[6]);
-invmat(data->irotmat);
-
-#if DEBUG_UPDATE_COORDS
 P3MAT("cummulative rot matrix", data->rotmat);
 P3MAT("           irot matrix", data->irotmat);
 #endif
 
-/* TODO - eliminate scale? (use camera position) */
-/* precalc scale */
-scale = data->scale * sysenv.subscale * 0.5 / data->rmax;
-
 /* precalc matrix products */
 ARR3SET(&mat4[0], &data->latmat[0]);
 ARR3SET(&mat4[4], &data->latmat[3]);
@@ -843,15 +845,6 @@ mat4[7] = -vec[1];
 mat4[11] = -vec[2];
 VEC4SET(&mat4[12], 0.0, 0.0, 0.0, 1.0);
 
-ARR3SET(&tmp4[0], &data->rotmat[0]);
-ARR3SET(&tmp4[4], &data->rotmat[3]);
-ARR3SET(&tmp4[8], &data->rotmat[6]);
-tmp4[3] = 0.0;
-tmp4[7] = 0.0;
-tmp4[11] = 0.0;
-VEC4SET(&tmp4[12], 0.0, 0.0, 0.0, 1.0);
-mat4mat(tmp4, mat4);
-
 #if DEBUG_UPDATE_COORDS
 P4MAT("mat4:", mat4);
 #endif
@@ -861,55 +854,42 @@ memcpy(data->display_lattice, mat4, 16*sizeof(gdouble));
 /* update image translation vectors */
 for (list=data->images ; list ; list=g_slist_next(list))
   {
-  image = (struct image_pak *) list->data;
-
+  image = list->data;
   ARR3SET(vec4, image->pic);
-/* NB: is a vector */
   vec4[3] = 0.0;
   vec4mat(mat4, vec4);
   ARR3SET(image->rx, vec4);
   }
 
-/* update atoms */
+/* calculate for atoms */
 for (list=data->cores ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
   if (core->status & DELETED)
     continue;
 
-/* calc coords */
   ARR3SET(vec4, core->x);
-
-ARR3ADD(vec4, core->offset);
-
+  ARR3ADD(vec4, core->offset);
   vec4[3] = 1.0;
   vec4mat(mat4, vec4);
   ARR3SET(core->rx, vec4);
   }
 
-/* rotate shells (if any ) */
+/* calculate for shells */
 for (list=data->shels ; list ; list=g_slist_next(list))
   {
-  shell = (struct shel_pak *) list->data;
+  shell = list->data;
   if (shell->status & DELETED)
     continue;
 
-/* get coordinates */
   ARR3SET(vec4, shell->x);
-
-ARR3ADD(vec4, shell->offset);
-
+  ARR3ADD(vec4, shell->offset);
   vec4[3] = 1.0;
   vec4mat(mat4, vec4);
   ARR3SET(shell->rx, vec4);
   }
 
-/* CURRENT */
-measure_update_global(data);
-
-/********/
-/* CELL */
-/********/
+/* cell */
 for (n=8 ; n-- ; )
   {
   ARR3SET(vec4, data->cell[n].x);
@@ -918,35 +898,25 @@ for (n=8 ; n-- ; )
   ARR3SET(data->cell[n].rx, vec4);
   }
 
-/********/
-/* AXES */
-/********/
+/* axes */
 for (n=6 ; n-- ; )
   {
   ARR3SET(vec4, data->axes[n].x);
   vec4[3] = 0.0;
-
-/* cartesian or unit cell frame */
-  if (data->axes_type == CARTESIAN)
-    vecmat(data->rotmat, vec4);
-  else
+  if (data->axes_type != CARTESIAN)
     {
     vec4mat(mat4, vec4);
     normalize(vec4, 3);
     }
-
   ARR3SET(data->axes[n].rx, vec4);
-/* about a 16th of rmax */
   VEC3MUL(data->axes[n].rx, 0.06*data->rmax);
   }
 
-/******************/
 /* ribbon updates */
-/******************/
 olist = data->ribbons;
 while (olist)
   {
-  odata = (struct object_pak *) olist->data;
+  odata = olist->data;
 
   switch (odata->type)
     {
@@ -954,7 +924,7 @@ while (olist)
       plist = (GSList *) odata->data;
       while (plist != NULL)
         {
-        ribbon = (struct ribbon_pak *) plist->data;
+        ribbon = plist->data;
 
 /* end point 1 */
         ARR3SET(vec4, ribbon->x1);
@@ -1002,12 +972,11 @@ while (olist)
 /* spatials */
 for (olist=data->spatial ; olist ; olist=g_slist_next(olist))
   {
-  spatial = (struct spatial_pak *) olist->data;
-
-  plist = (GSList *) spatial->data;
+  spatial = olist->data;
+  plist = spatial->list;
   while (plist)
     {
-    vector = (struct vec_pak *) plist->data;
+    vector = plist->data;
 
 /* rotate each point */
     ARR3SET(vec4, vector->x);
@@ -1026,14 +995,10 @@ for (olist=data->spatial ; olist ; olist=g_slist_next(olist))
     }
   }
 
-/**************/
 /* morphology */
-/**************/
-if (data->id == MORPH)
-  {
   for (list1=data->vertices ; list1 ; list1=g_slist_next(list1))
     {
-    v = (struct vertex_pak *) list1->data;
+    v = list1->data;
 
     ARR3SET(vec4, v->x);
     vec4[3] = 1.0;
@@ -1041,38 +1006,31 @@ if (data->id == MORPH)
     ARR3SET(v->rx, vec4);
     }
 
-/* facet center calc. */
+/* compute facet center (miller label placement) */
   for (list1=data->planes ; list1 ; list1=g_slist_next(list1))
     {
-    plane = (struct plane_pak *) list1->data;
+    plane = list1->data;
     if (plane->present)
       {
-/* set visible flag */
-      if (facet_visible(data, plane))
-        plane->visible = TRUE;
-      else
-        plane->visible = FALSE;
-
       VEC3SET(vec, 0.0, 0.0, 0.0);
       n = 0; 
       for (list2=plane->vertices ; list2 ; list2=g_slist_next(list2))
         {
-        v = (struct vertex_pak *) list2->data;
+        v = list2->data;
         ARR3ADD(vec, v->rx);
         n++;
         }
 /* assign */
 /* NB: n=0 can sometimes occur with a bad (ie non-closed) polyhedron */
       if (n)
+        {
         VEC3MUL(vec, 1.0/(gdouble) n);
+        }
       ARR3SET(plane->rx, vec);
       }
     }
-  }
 
-/*********************/
 /* ghost processsing */
-/*********************/
   if (!ghost)
     {
     glist = data->ghosts;
@@ -1091,27 +1049,10 @@ if (data->id == MORPH)
   }
 
 #if DEBUG_UPDATE_COORDS
-printf("calc_coords() done.\n");
+printf("coords_compute() done.\n");
 #endif
 }
 
-/***************************************/
-/* unified pixel to coordinate routine */
-/***************************************/
-#define DEBUG_PCM 0
-/* deprec. - use gl_get_world)coords() */
-void pixel_coord_map(gint px, gint py, gdouble *r, struct model_pak *data)
-{
-/* checks */
-g_return_if_fail(data != NULL);
-
-/* the model should be displayed for this to make sense */
-VEC3SET(r, 0.0, 0.0, 0.0);
-gl_get_world_coords(px, py, r);
-
-return;
-}
-
 /***********************************/
 /* print atom coord data primitive */
 /***********************************/
@@ -1122,11 +1063,26 @@ gchar *txt;
 g_assert(core != NULL);
 
 txt = g_strdup_printf("%4s core at (%9.4f,%9.4f,%9.4f).\n",
-                      core->label, core->x[0], core->x[1], core->x[2]);
-show_text(STANDARD, txt);
+                      core->atom_label, core->x[0], core->x[1], core->x[2]);
+gui_text_show(STANDARD, txt);
 g_free(txt);
 }
 
+/***********************************/
+/* print atom coord data primitive */
+/***********************************/
+void print_core_cart(struct core_pak *core)
+{
+    gchar *txt;
+    
+    g_assert(core != NULL);
+    
+    txt = g_strdup_printf("%4s core at (%9.4f,%9.4f,%9.4f).\n",
+                          core->atom_label, core->rx[0], core->rx[1], core->rx[2]);
+    gui_text_show(STANDARD, txt);
+    g_free(txt);
+}
+
 /************************************/
 /* print shell coord data primitive */
 /************************************/
@@ -1137,9 +1093,9 @@ gchar *txt;
 g_assert(shell != NULL);
 
 txt = g_strdup_printf("%4s shel at (%9.4f,%9.4f,%9.4f) [%9.4f, %9.4f, %9.4f]\n",
-                      shell->label, shell->x[0], shell->x[1], shell->x[2],
+                      shell->shell_label, shell->x[0], shell->x[1], shell->x[2],
                       shell->rx[0], shell->rx[1], shell->rx[2]);
-show_text(STANDARD, txt);
+gui_text_show(STANDARD, txt);
 g_free(txt);
 }
 
@@ -1150,12 +1106,25 @@ struct core_pak *core;
 
 for (list=data->cores ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
 
   printf("%4s core at (%9.4f,%9.4f,%9.4f).\n",
-         core->label, core->x[0], core->x[1], core->x[2]);
+         core->atom_label, core->x[0], core->x[1], core->x[2]);
   }
 }
+void print_cores_cart(struct model_pak *data)
+{
+    GSList *list;
+    struct core_pak *core;
+    
+    for (list=data->cores ; list ; list=g_slist_next(list))
+    {
+        core = list->data;
+        
+        printf("%4s core at (%9.4f,%9.4f,%9.4f).\n",
+               core->atom_label, core->rx[0], core->rx[1], core->rx[2]);
+    }
+}
 
 void print_shells(struct model_pak *data)
 {
@@ -1164,10 +1133,10 @@ struct shel_pak *shel;
 
 for (list=data->shels ; list ; list=g_slist_next(list))
   {
-  shel = (struct shel_pak *) list->data;
+  shel = list->data;
 
   printf("%4s shel at (%9.4f,%9.4f,%9.4f).\n",
-         shel->label, shel->x[0], shel->x[1], shel->x[2]);
+         shel->shell_label, shel->x[0], shel->x[1], shel->x[2]);
   }
 }
 
@@ -1183,17 +1152,17 @@ struct shel_pak *shel;
 
 for (list=data->cores ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
   if (core->shell)
     {
-    shel = (struct shel_pak *) core->shell;
+    shel = core->shell;
 
 if (n == 7 || n == 15)
   {
     printf("(%p) %4s core at (%9.4f,%9.4f,%9.4f).\n",
-           core, core->label, core->x[0], core->x[1], core->x[2]);
+           core, core->atom_label, core->x[0], core->x[1], core->x[2]);
     printf("(%p) %4s shel at (%9.4f,%9.4f,%9.4f).\n",
-           shel, shel->label, shel->x[0], shel->x[1], shel->x[2]);
+           shel, shel->shell_label, shel->x[0], shel->x[1], shel->x[2]);
   }
 
     n++;
@@ -1211,6 +1180,8 @@ struct core_pak *core = data;
 g_free(core->atom_label);
 g_free(core->atom_type);
 g_free(core->res_name);
+g_free(core->flags);
+g_free(core->chain);
 g_slist_free(core->bonds);
 
 free_slist(core->vibx_list);
@@ -1229,31 +1200,32 @@ GSList *list;
 
 g_assert(data != NULL);
 
-/* free cores */
-for (list=data->cores ; list ; list=g_slist_next(list))
-  core_free(list->data);
-g_slist_free(data->cores);
+/* free all objects that contain core refs first */
 
 /* free shells */
+/* TODO - free shell data */
 free_slist(data->shels);
+data->shels = NULL;
 
 /* free connectivity */
 free_slist(data->bonds);
+data->bonds = NULL;
 free_slist(data->ubonds);
+data->ubonds = NULL;
 free_mol_list(data);
+data->moles = NULL;
 
 /* others */
 g_slist_free(data->selection);
+data->selection = NULL;
 /* NB: don't free data */
 g_slist_free(data->unique_atom_list);
-
-/* enforce empty lists */
-data->cores = NULL;
-data->shels = NULL;
-data->bonds = NULL;
-data->ubonds = NULL;
-data->moles = NULL;
-data->selection = NULL;
 data->unique_atom_list = NULL;
+
+/* free the cores */
+for (list=data->cores ; list ; list=g_slist_next(list))
+  core_free(list->data);
+g_slist_free(data->cores);
+data->cores = NULL;
 }
 
