@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -24,7 +24,6 @@ The GNU GPL can also be found at http://www.gnu.org
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <strings.h>
 
 #ifndef __WIN32
 #include <sys/times.h>
@@ -37,7 +36,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "opengl.h"
 #include "render.h"
 #include "select.h"
-#include "gtkshorts.h"
+#include "shortcuts.h"
 #include "interface.h"
 #include "zone.h"
 
@@ -79,7 +78,6 @@ VEC3SET(copyc->offset, 0.0, 0.0, 0.0);
 
 /* coords, account for transformation matrices */
 ARR3SET(vec, core->rx);
-vecmat(dest->irotmat, vec);
 vecmat(dest->ilatmat, vec);
 ARR3ADD(vec, dest->centroid);
 ARR3SET(copyc->x, vec);
@@ -89,7 +87,7 @@ dest->cores = g_slist_prepend(dest->cores, copyc);
 /* attached shell? */
 if (copyc->shell)
   {
-  copys = (struct shel_pak *) copyc->shell;
+  copys = copyc->shell;
   items++;
 
 /* main info */
@@ -100,7 +98,6 @@ if (copyc->shell)
 
 /* coords, account for transformation matrices */
   ARR3SET(vec, copys->rx);
-  vecmat(dest->irotmat, vec);
   vecmat(dest->ilatmat, vec);
   ARR3ADD(vec, dest->centroid);
   ARR3SET(copys->x, vec);
@@ -125,7 +122,7 @@ core = g_malloc(sizeof(struct core_pak));
 
 memcpy(core, orig, sizeof(struct core_pak));
 
-/* init for safety */
+/* clear pointers */
 core->bonds = NULL;
 core->shell = NULL;
 core->mol = NULL;
@@ -135,6 +132,7 @@ core->primary_core = NULL;
 core->atom_label = g_strdup(orig->atom_label);
 core->atom_type = g_strdup(orig->atom_type);
 core->res_name = g_strdup(orig->res_name);
+core->flags = g_strdup(orig->flags);
 
 core->vibx_list = NULL;
 core->viby_list = NULL;
@@ -164,18 +162,21 @@ shel = g_malloc(sizeof(struct shel_pak));
 
 memcpy(shel, orig, sizeof(struct shel_pak));
 
-/* init for safety */
+/* clear pointers */
 shel->core = NULL;
 shel->primary_shell = NULL;
 VEC3SET(shel->pic, 0, 0, 0);
 
+/* duplicate strings */
+shel->flags = g_strdup(orig->flags);
+
 return(shel);
 }
 
 /****************************/
 /* base atom initialization */
 /****************************/
-void template_core(gchar *elem, struct core_pak *core, struct model_pak *model)
+void core_init(gchar *elem, struct core_pak *core, struct model_pak *model)
 {
 gint code;
 struct elem_pak elem_data;
@@ -186,43 +187,43 @@ if (model->protein)
 else
   code = elem_test(elem);
 
-if (!code)
-  strcpy(core->label, "X");	/* dummy type */
-else
-  strcpy(core->label, elem);
-
-/* NEW */
+/* general initialization */
+core->atom_code = code;
 core->atom_type = NULL;
 core->atom_label = NULL;
-core->res_name = NULL;
+core->res_name = g_strdup("");
 core->res_no = 1;
+core->chain = NULL;
 core->atom_order = 0;
 
-/* init element related data */
+/* element related initialization */
 get_elem_data(code, &elem_data, model);
-
 core->bond_cutoff = elem_data.cova;
 ARR3SET(core->colour, elem_data.colour);
 core->colour[3] = 1.0;
-core->atom_code = code;
 }
 
-/*******************************/
-/* unified atom initialization */
-/*******************************/
-#define DEBUG_NEW_CORE 0
-struct core_pak *new_core(gchar *elem, struct model_pak *data)
+/***************************/
+/* atom creation primitive */
+/***************************/
+gpointer core_new(gchar *elem, gchar *label, struct model_pak *model)
 {
 struct core_pak *core;
 
-g_return_val_if_fail(data != NULL, NULL);
+g_assert(elem != NULL);
+g_assert(model != NULL);
 
 core = g_malloc(sizeof(struct core_pak));
 
-/* init element related data */
-template_core(elem, core, data);
+/* element related initialization */
+core_init(elem, core, model);
+
+/* general initialization */
+if (label)
+  core->atom_label = g_strdup(label);
+else
+  core->atom_label = g_strdup(elem);
 
-/* init other data */
 VEC4SET(core->x, 0.0, 0.0, 0.0, 1.0);
 VEC4SET(core->rx, 0.0, 0.0, 0.0, 1.0);
 VEC3SET(core->v, 0.0, 0.0, 0.0);
@@ -232,35 +233,59 @@ core->primary = TRUE;
 core->orig = TRUE;
 core->ghost = FALSE;
 core->breathe = FALSE;
+core->growth = FALSE;
+core->translate = FALSE;
 core->render_mode = BALL_STICK;
 core->render_wire = FALSE;
 core->region = REGION1A;
 core->radius = 0.0;
+core->has_sof = FALSE;
 core->sof = 1.0;
 core->charge = 0.0;
 core->lookup_charge = TRUE;
+core->flags = NULL;
 core->bonds = NULL;
 core->mol = NULL;
 core->shell = NULL;
 core->primary_core = NULL;
-strcpy(core->tail,"\0");
 core->vibx_list = NULL;
 core->viby_list = NULL;
 core->vibz_list = NULL;
 
+/* NEW: hydrogen bond capabilities */
+if (g_strrstr(elem, "H") != NULL)
+  core->hydrogen_bond = TRUE;
+else 
+  core->hydrogen_bond = FALSE;
+
 return(core);
 }
 
-/**********************************/
-/* replacement shell init routine */
-/**********************************/
-struct shel_pak *new_shell(gchar *elem, struct model_pak *data)
+/*******************************/
+/* unified atom initialization */
+/*******************************/
+/* deprec */
+#define DEBUG_NEW_CORE 0
+struct core_pak *new_core(gchar *elem, struct model_pak *model)
+{
+struct core_pak *core;
+
+core = core_new(elem, NULL, model);
+
+return(core);
+}
+
+/****************************/
+/* shell creation primitive */
+/****************************/
+gpointer shell_new(gchar *elem, gchar *label, struct model_pak *model)
 {
 gint code;
 struct elem_pak elem_data;
 struct shel_pak *shell;
 
-g_return_val_if_fail(data != NULL, NULL);
+g_assert(elem != NULL);
+g_assert(model != NULL);
 
 shell = g_malloc(sizeof(struct shel_pak));
 
@@ -270,11 +295,14 @@ if (!code)
   printf("Warning: element [%s] not found.\n", elem);
 
 /* init modifiable element data */
-get_elem_data(code, &elem_data, data);
+get_elem_data(code, &elem_data, model);
 ARR3SET(shell->colour, elem_data.colour);
 
 shell->atom_code = code;
-strcpy(shell->label, elem);
+if (label)
+  shell->shell_label = g_strdup(label);
+else
+  shell->shell_label = g_strdup(elem);
 VEC4SET(shell->x, 0.0, 0.0, 0.0, 1.0);
 VEC4SET(shell->rx, 0.0, 0.0, 0.0, 1.0);
 VEC3SET(shell->v, 0.0, 0.0, 0.0);
@@ -283,13 +311,29 @@ shell->status = NORMAL;
 shell->primary = TRUE;
 shell->orig = TRUE;
 shell->breathe = FALSE;
+shell->translate = FALSE;
 shell->region = REGION1A;
 shell->core = NULL;
 shell->primary_shell = NULL;
 shell->radius = 0.0;
 shell->charge = 0.0;
+shell->has_sof = FALSE;
+shell->sof = 1.0;
 shell->lookup_charge = TRUE;
-strcpy(shell->tail,"\0");
+shell->flags = NULL;
+
+return(shell);
+}
+
+/**********************************/
+/* replacement shell init routine */
+/**********************************/
+/* deprec */
+struct shel_pak *new_shell(gchar *elem, struct model_pak *model)
+{
+struct shel_pak *shell;
+
+shell = shell_new(elem, NULL, model);
 
 return(shell);
 }
@@ -335,23 +379,20 @@ return(copy);
 /**************************/
 /* remove duplicate cores */
 /**************************/
+/* NB: assumes zone_init() has been done */
 #define DEBUG_REMOVE_DUPLICATES 0
 #define DEBUG_REMOVE_DUPLICATES_MORE 0
-void remove_duplicates(struct model_pak *model)
+void delete_duplicate_cores(struct model_pak *model)
 {
-gint zi;
 gdouble vec[3];
 GSList *list1, *list2;
+gpointer zone;
 struct core_pak *core1, *core2;
 struct shel_pak *s1, *s2;
-gpointer zone;
 
 /* checks */
 g_assert(model != NULL);
-
-/* NB: it's important to refresh the spatial partitioning */
-/* to get the maximum benefit, especially for large models */
-zone_init(model);
+g_assert(model->zone_array != NULL);
 
 #if DEBUG_REMOVE_DUPLICATES
 printf("Initial cores: %d\n", g_slist_length(model->cores));
@@ -361,52 +402,67 @@ printf("Initial shels: %d\n", g_slist_length(model->shels));
 /* enumerate all cores */
 for (list1=model->cores ; list1 ; list1=g_slist_next(list1))
   {
-  core1 = (struct core_pak *) list1->data;
+  core1 = list1->data;
   if (core1->status & DELETED)
     continue;
 
 /* enumerate cores in current locality */
-  zone = zone_ptr(core1, model);
+  zone = zone_get(core1->x, model->zone_array);
+
+
+#if DEBUG_REMOVE_DUPLICATES_MORE
+printf(" + %s [%p] : [%p] :", core1->atom_label, core1, zone);
+P3VEC(" ", core1->x);
+#endif
+/* should use zone_area_cores() as a very small coord difference */
+/* can result in cores being put in different (neighbouring) zones */
+/*
+  for (list2=zone_cores(zone) ; list2 ; list2=g_slist_next(list2))
+*/
+
+
+  for (list2=zone_area_cores(1, zone, model->zone_array) ; list2 ; list2=g_slist_next(list2))
 
-/* NB: use zone_area_cores() as a very small coord difference */
-/* can result in cores being put in dfferent (neighbouring) zones */
-  for (list2=zone_area_cores(1, zone, model) ; list2 ; list2=g_slist_next(list2))
     {
-    core2 = (struct core_pak *) list2->data;
+    core2 = list2->data;
     if (core2->status & DELETED)
       continue;
-    if (core1 == core2)
+
+/* avoid double counting */
+    if (core1 >= core2)
       continue;
+
     if (core1->atom_code != core2->atom_code)
       continue;
 
 #if DEBUG_REMOVE_DUPLICATES_MORE
-printf(" - %s :", core2->label);
+printf(" - %s :", core2->atom_label);
 P3VEC(" ", core2->x);
 #endif
 
 /* compute and test the minimum separation */
     ARR3SET(vec, core1->x);
     ARR3SUB(vec, core2->x);
-    if (model->periodic)
-      fractional_minsq(vec, model->periodic);
+
+    fractional_minsq(vec, model->periodic);
 
     if (VEC3MAGSQ(vec) < FRACTION_TOLERANCE)
       {
 /* delete core2, unless primary AND core1 is non primary */
       if (core2->primary && !core1->primary)
+        {
         core1->status |= DELETED;
+
+/* FIXME - this may be a problem since we start the core2 loop */
+/* requiring that core1 be undeleted */
+#if DEBUG_REMOVE_DUPLICATES_MORE
+printf(" * rm 1\n");
+#endif
+        }
       else
         {
         core2->status |= DELETED;
 
-/*
-if (core1->region != core2->region)
-  {
-  show_text(ERROR, "ERROR: region crossover bug.\n");
-  }
-*/
-
 #if DEBUG_REMOVE_DUPLICATES_MORE
 printf(" * rm 2\n");
 #endif
@@ -421,16 +477,15 @@ delete_commit(model);
 
 for (list1=model->shels ; list1 ; list1=g_slist_next(list1))
   {
-  s1 = (struct shel_pak *) list1->data;
+  s1 = list1->data;
   if (s1->status & DELETED)
     continue;
 
 /* NEW - enumerate shells in the current locality */
-  zi = zone_index(s1->x, model); 
-  zone = model->zones[zi];
+  zone = zone_get(s1->x, model->zone_array); 
   for (list2=zone_shells(zone) ; list2 ; list2=g_slist_next(list2))
     {
-    s2 = (struct shel_pak *) list2->data;
+    s2 = list2->data;
 
     if (s2->status & DELETED)
       continue;
@@ -441,8 +496,7 @@ for (list1=model->shels ; list1 ; list1=g_slist_next(list1))
     ARR3SUB(vec, s2->x);
 
 /* adjust for periodicity */
-    if (model->periodic)
-      fractional_minsq(vec, model->periodic);
+    fractional_minsq(vec, model->periodic);
 
     if (VEC3MAGSQ(vec) < FRACTION_TOLERANCE)
       {
@@ -456,9 +510,6 @@ for (list1=model->shels ; list1 ; list1=g_slist_next(list1))
   }
 delete_commit(model);
 
-/* refresh core-shell linkages */
-shell_make_links(model);
-
 #if DEBUG_REMOVE_DUPLICATES
 printf("Final cores: %d\n", g_slist_length(model->cores));
 printf("Final shels: %d\n", g_slist_length(model->shels));
@@ -495,9 +546,9 @@ P3VEC("af: ", x);
 #endif
 }
 
-/****************************************/
-/* clamp 3D coords to fractional limits */
-/****************************************/
+/*************************************/
+/* clamp coords to fractional limits */
+/*************************************/
 #define DEBUG_CLAMP 0
 void fractional_clamp(gdouble *vec, gint *mov, gint dim)
 {
@@ -508,12 +559,14 @@ gdouble ip;
 P3VEC("inp: ", vec);
 #endif
 
-for (i=3 ; i-- ; )
-  mov[i] = 0;
-
 /* NB: init ALL 3 coords as isolated molecules call this */
 /* routine & it is expected that mov to be set to 0,0,0 */
+mov[0] = mov[1] = mov[2] = 0;
+
+/*
 for (i=0 ; i<dim ; i++)
+*/
+for (i=dim ; i-- ; )
   {
   mov[i] = -vec[i];
   if (vec[i] < 0.0)
@@ -533,167 +586,61 @@ P3VEC("out: ", vec);
 #endif
 }
 
-/***************************************/
-/* apply PBC to the atoms in the model */
-/***************************************/
+/********************************************/
+/* confine a list of atoms to the unit cell */
+/********************************************/
 #define DEBUG_PBC_ATOMS 0
-void pbc_constrain_atoms(struct model_pak *data)
+void coords_confine_cores(GSList *cores, struct model_pak *model)
 {
 gint xlat[3];
 gdouble mov[3];
-GSList *list, *blist;
+GSList *list;
 struct core_pak *core;
 struct shel_pak *shell;
-struct bond_pak *bond;
 
-g_return_if_fail(data != NULL);
-if (!data->periodic)
+g_assert(model != NULL);
+
+if (!model->periodic)
   return;
 
-#if DEBUG_PBC_ATOMS
-printf("[A] num cores: %d\n", g_slist_length(data->cores));
-printf("[A] num shels: %d\n", g_slist_length(data->shels));
-#endif
-
-/*
-printf(" --- pca start\n");
-dump_bonds(data);
-if (data->periodic == 2)
-  return;
-*/
-
 /* translate cores to within the cell */
-for (list=data->cores ; list ; list=g_slist_next(list))
+for (list=cores ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
-
-/* clamp */
-  VEC3SET(xlat, 0, 0, 0);
-  fractional_clamp(core->x, xlat, data->periodic);
-  ARR3SET(mov, xlat);
+  core = list->data;
+  fractional_clamp(core->x, xlat, model->periodic);
 
 /* move shell */
   if (core->shell)
     {
-    shell = (struct shel_pak *) core->shell;
+    shell = core->shell;
+    ARR3SET(mov, xlat);
     ARR3ADD(shell->x, mov);
     }
-
-/* no translation needed - skip to next */
-  if (VEC3MAGSQ(mov) < POSITION_TOLERANCE)
-    continue;
-
-/* search for periodic bonds to update */
-  for (blist=core->bonds ; blist ; blist=g_slist_next(blist))
-    {
-    bond = (struct bond_pak *) blist->data;
-
-    switch (bond->periodic)
-      {
-      case BOND_MERGED:
-      case BOND_SPLIT:
-        if (core == bond->atom1)
-          {
-          ARR3ADD(bond->pic, mov);
-          }
-        else
-          {
-          ARR3SUB(bond->pic, mov);
-          }
-        break;
-
-/* here we've found a bond that wasn't marked as periodic initially */
-/* yet was broken by the pbc constraint. This can happen for cartesian */
-/* models with a molecule sticking out past a cell edge */
-      default:
-        bond->periodic = BOND_SPLIT;
-        ARR3SET(bond->pic, mov);
-        if (core == bond->atom2)
-          {
-          VEC3MUL(bond->pic, -1.0);
-          }
-        break;
-      }
-    }
   }
-
-/* search for ruptured periodic bonds */
-for (blist=data->bonds ; blist ; blist=g_slist_next(blist))
-  {
-  bond = (struct bond_pak *) blist->data;
-
-  switch (bond->periodic)
-    {
-    case BOND_MERGED:
-      if (VEC3MAGSQ(bond->pic) > FRACTION_TOLERANCE)
-        bond->periodic = BOND_SPLIT;
-      break;
-
-/* were any healed? (shouldn't happen?) */
-/* this can happen if unfrag() fails & this function is called */
-/*
-    case BOND_PERIODIC:
-      if (VEC3MAGSQ(bond->pic) < FRACTION_TOLERANCE)
-        g_assert_not_reached();
-*/
-    }
-  }
-
-/* NEW */
-/*
-remove_duplicates(data);
-*/
-
-/*
-printf(" --- pca end\n");
-dump_bonds(data);
-*/
-
-#if DEBUG_PBC_ATOMS
-printf("[B] num cores: %d\n", g_slist_length(data->cores));
-printf("[B] num shels: %d\n", g_slist_length(data->shels));
-#endif
 }
 
-/**************************************/
-/* apply PBC to the mols in the model */
-/**************************************/
-#define DEBUG_CONFINE_MOLS 0
-void pbc_constrain_mols(struct model_pak *data)
+/**********************************************/
+/* confine a molecule's centroid to unit cell */
+/**********************************************/
+void coords_confine_centroid(struct mol_pak *mol, struct model_pak *model)
 {
 gint xlat[3];
 gdouble mov[3];
-GSList *mlist, *clist;
+GSList *list;
 struct core_pak *core;
-struct mol_pak *mol;
-
-g_return_if_fail(data != NULL);
-
-#if DEBUG_CONFINE_MOLS
-printf("pbc_constrain_mols()\n");
-#endif
-
-/* all orig coords are fractional */
-for (mlist=data->moles ; mlist ; mlist=g_slist_next(mlist))
-  {
-  mol = (struct mol_pak *) mlist->data;
 
 /* calc moves required to bring centroid within pbc */
-  VEC3SET(xlat, 0, 0, 0);
-  fractional_clamp(mol->centroid, xlat, data->periodic);
-  ARR3SET(mov, xlat);
+fractional_clamp(mol->centroid, xlat, model->periodic);
+ARR3SET(mov, xlat);
 
 /* apply to all atoms/shells in this molecule */
-  for (clist=mol->cores ; clist ; clist=g_slist_next(clist))
+for (list=mol->cores ; list ; list=g_slist_next(list))
+  {
+  core = list->data;
+  ARR3ADD(core->x, mov);
+  if (core->shell)
     {
-    core = (struct core_pak *) clist->data;
-
-    ARR3ADD(core->x, mov);
-
-    if (core->shell)
-      {
-      ARR3ADD((core->shell)->x, mov);
-      }
+    ARR3ADD((core->shell)->x, mov);
     }
   }
 }
@@ -728,7 +675,7 @@ gint shel_match(const gchar *label, struct shel_pak *shell)
 gint code;
 
 #if DEBUG_SHELL_MATCH
-printf("[%s] : [%s]\n", label, shell->label);
+printf("[%s] : [%s]\n", label, shell->shell_label);
 #endif
 
 code = elem_symbol_test(label);
@@ -739,7 +686,7 @@ if (code)
   {
   if (g_ascii_strcasecmp(label, elements[shell->atom_code].symbol) != 0)
     {
-    if (g_ascii_strcasecmp(shell->label, label) == 0)
+    if (g_ascii_strcasecmp(shell->shell_label, label) == 0)
       return(1);
     }
   else
@@ -764,7 +711,7 @@ gint core_match(const gchar *label, struct core_pak *core)
 gint code;
 
 #if DEBUG_ATOM_MATCH
-printf("[%s] : [%s]\n", label, core->label);
+printf("[%s] : [%s]\n", label, core->atom_label);
 #endif
 
 code = elem_symbol_test(label);
@@ -775,7 +722,7 @@ if (code)
   {
   if (g_ascii_strcasecmp(label, elements[core->atom_code].symbol) != 0)
     {
-    if (g_ascii_strcasecmp(core->label, label) == 0)
+    if (g_ascii_strcasecmp(core->atom_label, label) == 0)
       return(1);
     }
   else
