@@ -1,16 +1,4 @@
 
-/**************/
-/* structures */
-/**************/
-
-struct transform_pak
-{
-gint id;
-gdouble matrix[9];
-gdouble vector[3];
-gdouble scalar;
-};
-
 /**************/
 /* prototypes */
 /**************/
@@ -27,9 +15,16 @@ void info_torsion(gint, gint, struct model_pak *);
 
 void elem_change_colour(GtkWidget *, gpointer *);
 
-gint copy_gulp_data(struct model_pak *, struct model_pak *);
-gint copy_gulp_extra(struct model_pak *, struct model_pak *);
-gint free_gulp_data(struct model_pak *);
+void gulp_init(gpointer);
+void gulp_files_init(struct model_pak *);
+void gulp_data_free(struct model_pak *);
+void gulp_files_free(struct model_pak *);
+void gulp_data_copy(struct model_pak *, struct model_pak *);
+void gulp_extra_copy(struct model_pak *, struct model_pak *);
+
+gint gulp_cosmo_points(gint, gint, gint);
+
+gint free_moldy_data(struct model_pak *);
 
 gint search_basename(const gchar *);
 gint dialog_active(gint);
@@ -38,6 +33,7 @@ void unhide_atoms(void);
 
 void type_model(const gchar *, struct model_pak *);
 
-void connect_page(GtkWidget *);
 void periodicity_page(GtkWidget *);
 
+void atom_numbers_box(GtkWidget *);
+
