@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -35,7 +35,9 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "matrix.h"
 #include "numeric.h"
 #include "surface.h"
+#include "select.h"
 #include "interface.h"
+#include "shortcuts.h"
 #include "opengl.h"
 
 /* main structures */
@@ -60,8 +62,12 @@ GSList *list;
 /* checks */
 g_return_val_if_fail(fp != NULL, 1);
 
+/* FIXME - num_keys() is returning a value that is too small */
+/*
 n = num_keys();
-set = g_malloc(n * sizeof(gint));
+*/
+n = 1000;
+set = g_malloc0(n * sizeof(gint));
 
 while(!fgetline(fp,line))
   {
@@ -90,7 +96,8 @@ while(!fgetline(fp,line))
           if (list)
             {
             key = GPOINTER_TO_INT(list->data);
-            switch(key)
+g_assert(key < n);
+            switch (key)
               {
 /* these shouldn't be modified if not default */
               case SYMBOL:
@@ -128,9 +135,13 @@ while(!fgetline(fp,line))
                 break; 
               case COLOUR:
                 g_assert(num_tokens > 3);
-                elem.colour[0] = (gint) str_to_float(*(buff+1));
-                elem.colour[1] = (gint) str_to_float(*(buff+2));
-                elem.colour[2] = (gint) str_to_float(*(buff+3));
+                elem.colour[0] = str_to_float(*(buff+1));
+                elem.colour[1] = str_to_float(*(buff+2));
+                elem.colour[2] = str_to_float(*(buff+3));
+if (VEC3MAGSQ(elem.colour) > 3.0)
+  {
+  VEC3MUL(elem.colour, 1.0/65535.0);
+  }
                 set[key]++;
                 break; 
               case GDIS_END:
@@ -241,8 +252,8 @@ g_return_val_if_fail(fp != NULL, 1);
 list = sysenv.elements;
 while(list != NULL)
   {
-/* CURRENT - only write those values that are different to default library */
-  elem = (struct elem_pak *) list->data;
+/* only write those values that are different to default library */
+  elem = list->data;
   if (elem)
     {
     fprintf(fp, "%%gdis_elem\n");
@@ -294,7 +305,7 @@ unique = NULL;
 /* go through all atoms */
 for (list=model->cores ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
 
 /* determine if current atom type is in the list */
   found=0;
@@ -310,7 +321,7 @@ for (list=model->cores ; list ; list=g_slist_next(list))
       case LABEL:
       case LABEL_NORMAL:
       case LABEL_GHOST:
-        if (g_ascii_strcasecmp(types->data, core->label) == 0)
+        if (g_ascii_strcasecmp(types->data, core->atom_label) == 0)
           found++;
         break;
       }
@@ -327,15 +338,15 @@ for (list=model->cores ; list ; list=g_slist_next(list))
         unique = g_slist_prepend(unique, GINT_TO_POINTER(core->atom_code));
         break;
       case LABEL:
-        unique = g_slist_prepend(unique, g_strdup(core->label));
+        unique = g_slist_prepend(unique, g_strdup(core->atom_label));
         break;
       case LABEL_NORMAL:
         if (!core->ghost)
-          unique = g_slist_prepend(unique, g_strdup(core->label));
+          unique = g_slist_prepend(unique, g_strdup(core->atom_label));
         break;
       case LABEL_GHOST:
         if (core->ghost)
-          unique = g_slist_prepend(unique, g_strdup(core->label));
+          unique = g_slist_prepend(unique, g_strdup(core->atom_label));
         break;
       }
     }
@@ -360,7 +371,8 @@ for (types=unique ; types ; types = g_slist_next(types))
 printf("\n");
 #endif
 
-return(unique);
+/* return reversed list (preserves order) */
+return(g_slist_reverse(unique));
 }
 
 /***********************/
@@ -372,6 +384,7 @@ struct elem_pak elem_data;
 
 get_elem_data(core->atom_code, &elem_data, model);
 ARR3SET(core->colour, elem_data.colour);
+VEC3MUL(core->colour, 65535.0);
 if (core->ghost)
   core->colour[3] = 0.5;
 else
@@ -403,13 +416,50 @@ for (list=model->cores ; list ; list=g_slist_next(list))
   init_atom_charge(list->data, model);
 }
 
+/*******************************************/
+/* net charge on atom (ie including shell) */
+/*******************************************/
+gdouble atom_charge(struct core_pak *core)
+{
+gdouble q;
+struct shel_pak *shell;
+
+q = core->charge;
+if (core->shell)
+  {
+  shell = core->shell;
+  q += shell->charge;
+  }
+return(q);
+}
+
+/********************************************/
+/* initialize element properties for a core */
+/********************************************/
+void elem_init(struct core_pak *core, struct model_pak *model)
+{
+struct elem_pak elem;
+
+g_assert(model != NULL);
+g_assert(core != NULL);
+
+/* NB: assumes core->atom_code is the atomic number */
+
+get_elem_data(core->atom_code, &elem, model);
+core->atom_code = elem.number;
+core->bond_cutoff = elem.cova;
+core->charge = elem.charge;
+ARR3SET(core->colour, elem.colour);
+VEC3MUL(core->colour, 65535.0);
+}
+
 /*********************************************************/
 /* compute net charge and dipole (if model is a surface) */
 /*********************************************************/
 #define DEBUG_CALC_EMP 0
 void calc_emp(struct model_pak *data)
 {
-gdouble zdip, qsum, x[3];
+gdouble qsum, x[3], p[3];
 GSList *list;
 struct core_pak *core;
 struct shel_pak *shell;
@@ -419,7 +469,8 @@ g_assert(data != NULL);
 
 /* TODO - speed up by getting to a local var the elem data for unique atoms */
 /* cores & shell dipole calc */
-zdip=qsum=0.0;
+qsum=0.0;
+VEC3SET(p, 0.0, 0.0, 0.0);
 for (list=data->cores ; list ; list=g_slist_next(list))
   {
   core = list->data;
@@ -430,21 +481,26 @@ for (list=data->cores ; list ; list=g_slist_next(list))
   if (core->status & DELETED)
     continue;
 
-/* core contribution */
   ARR3SET(x, core->x);
   vecmat(data->latmat, x);
 
-  zdip += core->charge * x[2];
+/* dipole */
+  VEC3MUL(x, core->charge);
+  ARR3ADD(p, x);
+/* monopole */
   qsum += core->charge;
 
 /* NB: add shell constribution at the core's z location */
   if (core->shell)
     {
     shell = core->shell;
-
     ARR3SET(x, shell->x);
     vecmat(data->latmat, x);
-    zdip += shell->charge * x[2];
+
+/* dipole */
+    VEC3MUL(x, shell->charge);
+    ARR3ADD(p, x);
+/* monopole */
     qsum += shell->charge;
     }
   }
@@ -453,7 +509,7 @@ for (list=data->cores ; list ; list=g_slist_next(list))
 printf("surface: %f %f %f  (%f)\n", 
 data->surface.miller[0], data->surface.miller[1], data->surface.miller[2], 
 data->surface.shift);
-printf("net dipole: %e\n", zdip);
+P3VEC("dipole: ", p);
 printf("sum charge: %e\n", qsum);
 if (qsum*qsum > FRACTION_TOLERANCE)
   printf("Warning: your model has a net charge = %f\n", qsum);
@@ -461,7 +517,7 @@ if (qsum*qsum > FRACTION_TOLERANCE)
 
 /* only surfaces get the z dipole */
 if (data->periodic == 2)
-  data->gulp.sdipole = zdip;
+  data->gulp.sdipole = p[2];
 
 data->gulp.qsum = qsum;
 }
@@ -528,7 +584,7 @@ printf("Looking for [%s]...", elem1);
 /* attempt to match atom type with database */
 j=0;
 /* FIXME - elim dependence on const */
-for(i=0 ; i<sysenv.num_elements ; i++)
+for(i=1 ; i<sysenv.num_elements ; i++)
   {
 /* only compare if lengths match */
   n = strlen(elements[i].symbol);
@@ -617,49 +673,38 @@ return(elem_symbol_test(input));
 #define DEBUG_GET_ELEM 0
 gint get_elem_data(gint code, struct elem_pak *elem, struct model_pak *model)
 {
-gint i;
-struct elem_pak *elem_data;
 GSList *list;
+struct elem_pak *elem_data;
 
-/* TODO - seek in sysenv (cope with data == NULL) */
-
-/* search the model exceptions database */
+/* search local exceptions */
 if (model)
   {
-#if DEBUG_GET_ELEM
-printf("Searching %d model items items for code = %d.\n",model->num_elem,code);
-#endif
-  for (i=model->num_elem ; i-- ; )
+  for (list=model->elements ; list ; list=g_slist_next(list))
     {
-    if (code == (model->elements+i)->number)
+    elem_data = list->data;
+    if (code == elem_data->number)
       {
-      memcpy(elem, model->elements+i, sizeof(struct elem_pak));
+#if DEBUG_GET_ELEM
+printf("Retrieving local exception.\n");
+#endif
+      memcpy(elem, elem_data, sizeof(struct elem_pak));
       return(0);
       }
     }
-#if DEBUG_GET_ELEM
-printf("Not found.\n");
-#endif
   }
-
-/* search the global exceptions database */
-#if DEBUG_GET_ELEM
-printf("Searching linked list for code = %d.\n",code);
-#endif
-list = sysenv.elements;
-while(list != NULL)
+/* search global exceptions */
+for (list=sysenv.elements ; list ; list=g_slist_next(list))
   {
-  elem_data = (struct elem_pak *) list->data;
+  elem_data = list->data;
   if (code == elem_data->number)
     {
+#if DEBUG_GET_ELEM
+printf("Retrieving global exception.\n");
+#endif
     memcpy(elem, elem_data, sizeof(struct elem_pak));
     return(0);
     }
-  list = g_slist_next(list);
   }
-#if DEBUG_GET_ELEM
-printf("Not found.\n");
-#endif
 
 /* return from the standard database */
 if (code >= 0 && code < sysenv.num_elements)
@@ -671,7 +716,6 @@ printf("Retrieving default.\n");
   return(0);
   }
 
-/* done - no match */
 #if DEBUG_GET_ELEM
 printf("ERROR: bad element code.\n");
 #endif
@@ -684,8 +728,8 @@ return(1);
 #define DEBUG_PUT_ELEM_DATA 0
 void put_elem_data(struct elem_pak *elem, struct model_pak *model)
 {
-gint i, ix=0, flag=0;
-GSList *list;
+gint flag, create;
+GSList *list, *elements;
 struct elem_pak *elem_data;
 struct core_pak *core;
 
@@ -704,188 +748,101 @@ printf("charge: %f\n", elem->charge);
 printf("colour: %f %f %f\n", elem->colour[0], elem->colour[1], elem->colour[2]);
 #endif
 
-/* FIXME - crude hack for element data being passed in range 0.0 - 1.0 */
-/* (this happens in gperiodic - and prob should be default) */
-if (VEC3MAGSQ(elem->colour) < 3.1)
+/* global or local database */
+if (model)
   {
-  VEC3MUL(elem->colour, 65535.0);
+  elements = model->elements;
+  flag = TRUE;
+  }
+else
+  {
+  elements = sysenv.elements;
+  flag = FALSE;
   }
 
-/* if data = NULL (ie no model) -> global -> put in sysenv list */
-if (model == NULL)
+/* replace element if exists */
+create = TRUE;
+for (list=elements ; list ; list=g_slist_next(list))
   {
-/* search for existing item */
-  list = sysenv.elements;
-  while(list != NULL)
-    {
-    elem_data = (struct elem_pak *) list->data;
-    if (elem_data->number == elem->number)
-      {
-#if DEBUG_PUT_ELEM_DATA
-printf("Modifying global database.\n");
-#endif
-/* if found, overwrite values */
-      memcpy(elem_data, elem, sizeof(struct elem_pak));
-      flag++;
-      break;
-      }
-    list = g_slist_next(list);
-    } 
-/* if not found, add element to the list */
-  if (!flag)
+  elem_data = list->data;
+  if (elem_data->number == elem->number)
     {
 #if DEBUG_PUT_ELEM_DATA
-printf("Adding to global database.\n");
+printf("replacing existing...\n");
 #endif
-    elem_data = g_malloc(sizeof(struct elem_pak));
     memcpy(elem_data, elem, sizeof(struct elem_pak));
-    sysenv.elements = g_slist_prepend(sysenv.elements, elem_data);
+    create = FALSE;
+    break;
     }
   }
-else
+
+/* create new item if it didn't exist */
+if (create)
   {
 #if DEBUG_PUT_ELEM_DATA
-printf("Modifying local database.\n");
+printf("creating new exception...\n");
 #endif
-/* replace flag (as opposed to add new) */
-ix=0;
-/* alloc if non-existent */
-if (!model->num_elem)
-  {
-  model->elements = g_malloc(sizeof(struct elem_pak));
-  model->num_elem++;
-  flag++;
-  }
-else
-  {
-/* search for existing data */
-  for (i=model->num_elem ; i-- ; )
-    {
-    if (elem->number == i)
-      {
-      flag++;
-      ix = i;
-      break;
-      }
-    }
-  }
-/* TODO - a linked list would be easier... */
-/* replace or re-alloc & add? */
+  elem_data = g_malloc(sizeof(struct elem_pak));
+  memcpy(elem_data, elem, sizeof(struct elem_pak));
+
   if (flag)
-    memcpy(model->elements+ix, elem, sizeof(struct elem_pak));
+    model->elements = g_slist_prepend(model->elements, elem_data);
   else
-    {
-    model->elements = g_renew(struct elem_pak, model->elements, model->num_elem+1);
-    memcpy(model->elements+model->num_elem, elem, sizeof(struct elem_pak));
-    model->num_elem++;
-    }
+    sysenv.elements = g_slist_prepend(sysenv.elements, elem_data);
+  }
 
 /* update atom bond_cutoff */
+if (flag)
+  {
   for (list=model->cores ; list ; list=g_slist_next(list))
     {
-    core = (struct core_pak *) list->data;
+    core = list->data;
     if (core->atom_code == elem->number)
       core->bond_cutoff = elem->cova;
     }
   }
 }
 
-/******************************************/
-/* lookup and restore core default values */
-/******************************************/
-void init_elem(struct core_pak *core, gint type, struct model_pak *model)
-{
-struct elem_pak elem;
-
-get_elem_data(core->atom_code, &elem, model);
-
-switch (type)
-  {
-  case REFRESH_COLOUR:
-    ARR3SET(core->colour, elem.colour);
-    break;
-  case REFRESH_BONDS:
-    core->bond_cutoff = elem.cova;
-    break;
-  }
-}
-
 /* globals for the atom properties dialog */
 GtkWidget *apd_label, *apd_type, *apd_charge, *apd_x, *apd_y, *apd_z;
+GtkWidget *apd_growth, *apd_region, *apd_translate;
 struct model_pak *apd_data=NULL;
 struct core_pak *apd_core=NULL;
 
 /*************************************************************/
 /* get the nth colour in a sequence (eg sample RGB spectrum) */
 /*************************************************************/
-#define MAX_COLOUR 65535
-#define RAD_CYCLE 4
-#define RAD_GRANULARITY 2.88
-#define THETA_GRANULARITY 2.88
-#define DEBUG_GET_COLOUR 0
-
 void get_colour(gdouble *colour, gint n)
 {
-gint i;
-gdouble rad, ra, theta, ct, st;
+gint byte;
+gdouble r, g, b, f;
 
-theta = (gdouble) n * 2.0 * PI / THETA_GRANULARITY;
+byte = 1 + n % 7;
 
-i = n / (gdouble) RAD_CYCLE;
-ra = i;
-ra *= 2.0*PI;
-ra /= RAD_GRANULARITY;
-rad = fabs(tbl_cos(ra));
-
-#if DEBUG_GET_COLOUR
-printf("[%2d] (%f,%f) ", n, rad, theta);
-#endif
-
-ct = rad*tbl_cos(theta);
-st = rad*tbl_sin(theta);
-
-VEC3SET(colour, 0.0, 0.0, 0.0);
-
-/* pairwise rgb */
-if (st < 0.0)
-  {
-/* blue is major */
-  colour[2] = 0.5 * (1.0 - st);
-  if (ct < 0.0)
-    {
-/* mix green */
-    colour[1] = 1.0 - fabs(0.866 + ct);
-/* red is minor */
-    if (ct > -0.5)
-      colour[0] = -0.5 * ct;
-    else
-      colour[0] = 0.0;
-    }
-  else
-    {
-/* mix red */
-    colour[0] = 1.0 - fabs(0.866 - ct);
-/* green is minor */
-    if (ct < 0.5)
-      colour[1] = 0.5 * ct;
-    else
-      colour[1] = 0.0;
-    }
-  }
+if (byte & 4)
+  r = 1.0;
 else
-  {
-/* FIXME - real max and min at ct = +/- 0.866... */
-  colour[0] = 0.5 * (1.0 + ct);
-  colour[1] = 0.5 * (1.0 - ct);
-/* no blue */
-  colour[2] = 0.0;
-  }
+  r = 0.0;
 
-#if DEBUG_GET_COLOUR
-P3VEC(" : ", colour);
-#endif
+if (byte & 2)
+  g = 1.0;
+else
+  g = 0.0;
 
-VEC3MUL(colour, 65535.0);
+if (byte & 1)
+  b = 1.0;
+else
+  b = 0.0;
+
+f = n / 7;
+f *= 0.5;
+f += 1.0;
+f = 1.0/f;
+r *= f;
+g *= f;
+b *= f;
+
+VEC3SET(colour, r, g, b);
 }
 
 /****************************************/
@@ -893,33 +850,92 @@ VEC3MUL(colour, 65535.0);
 /****************************************/
 void atom_colour_scheme(gint type, struct core_pak *core, struct model_pak *model)
 {
-struct elem_pak edata;
+gdouble t;
+struct elem_pak elem;
 
 switch (type)
   {
   case ELEM:
-    get_elem_data(core->atom_code, &edata, model);
+    get_elem_data(core->atom_code, &elem, model);
     break;
 
   case MOL:
 /* get the sequence number for the colour type */
 /* NB: core->molecule is deprec. */
-    get_colour(edata.colour, core->molecule);
+    get_colour(elem.colour, core->molecule);
     break;
 
+  case OCCUPANCY:
+    VEC3MUL(core->colour, core->sof);
+    return;
+
   case REGION:
 /* get the sequence number for the colour type */
-    get_colour(edata.colour, core->region);
+    get_colour(elem.colour, core->region);
+    break;
+
+  case GROWTH_SLICE:
+    if (core->growth)
+      {
+      VEC3SET(elem.colour, 0.5, 0.5, 0.5);
+      }
+    else
+      {
+      get_elem_data(core->atom_code, &elem, model);
+      }
+    break;
+
+  case TRANSLATE:
+    if (core->translate)
+      {
+      VEC3SET(elem.colour, 0.5, 0.5, 0.5);
+      }
+    else
+      {
+      get_elem_data(core->atom_code, &elem, model);
+      }
+    break;
+
+  case VELOCITY:
+#define BC 1.3806503
+#define AN 6.0221420
+#define TMIN 0.0
+#define TRANGE 2000.0
+
+    get_elem_data(core->atom_code, &elem, model);
+    t = elem.weight * VEC3MAGSQ(core->v) * 10.0 / (3.0*AN*BC);
+
+/*
+printf("T = %f\n", t);
+*/
+
+/* clamp */
+t -= TMIN;
+if (t > TRANGE)
+  {
+/*
+  printf("tmax = %f\n", t + TMIN);
+*/
+  t = TRANGE;
+  }
+if (t < 0.0)
+  t = 0.0;
+
+/* bright red when hot (TMAX) */
+    elem.colour[0] = t/TRANGE;
+    elem.colour[1] = 0.0;
+/* dark blue when cold */
+    elem.colour[2] = 0.5 * (1.0 - t/TRANGE);
     break;
   }
 
-/* set core colour */
-ARR3SET(core->colour, edata.colour);
+ARR3SET(core->colour, elem.colour);
+VEC3MUL(core->colour, 65535.0);
 }
 
-/****************************************/
-/* change the colour scheme for a model */
-/****************************************/
+/*************************************/
+/* set the colour scheme for a model */
+/*************************************/
 void model_colour_scheme(gint type, struct model_pak *model)
 {
 GSList *list;
@@ -940,7 +956,7 @@ for (list=model->cores ; list ; list=g_slist_next(list))
 /* selection to the specified value - even (eg) incompatible elements */
 void selection_properties_change(gint type)
 {
-gint n;
+gint n, growth, region, translate;
 gdouble charge;
 const gchar *text;
 GSList *list;
@@ -961,15 +977,21 @@ switch (type)
     n = elem_symbol_test(text);
     if (n)
       {
-      get_elem_data(n, &edata, apd_data);
-
+      get_elem_data(n, &edata, model);
 /* make sure we alow enough space for the string and the \0 */
       n = (LABEL_SIZE-1 > strlen(text)) ? strlen(text) : LABEL_SIZE-1;
       for (list=model->selection ; list ; list=g_slist_next(list))
         {
-        core = (struct core_pak *) list->data;
-/* core updates */
-        g_snprintf(core->label,n+1,"%s",text);
+        core = list->data;
+        g_free(core->atom_label);
+        core->atom_label = g_strdup(text);
+/* update atttached shell */
+        if (core->shell)
+          {
+          struct shel_pak *shell = core->shell;
+          g_free(shell->shell_label);
+          shell->shell_label = g_strdup(text);
+          }
 /* NEW - don't update element specific data if the element type was not */
 /* changed - ie the user has just made a labelling change (eg C -> C1) */
         if (edata.number != core->atom_code)
@@ -984,28 +1006,79 @@ switch (type)
       g_slist_free(model->unique_atom_list);
       model->unique_atom_list = find_unique(ELEMENT, model);
       calc_emp(model);
-      redraw_canvas(SINGLE);
       }
     break;
 
   case CHARGE:
     charge = str_to_float(gtk_entry_get_text(GTK_ENTRY(apd_charge)));
-
-/*
-printf("set selection charge: %f\n", charge);
-*/
-
     for (list=model->selection ; list ; list=g_slist_next(list))
       {
-      core = (struct core_pak *) list->data;
+      core = list->data;
 /* core updates */
       core->charge = charge;
       core->lookup_charge = FALSE;
       }
     calc_emp(model);
-    redraw_canvas(SINGLE);
+    break;
+
+  case CORE_GROWTH_SLICE:
+    growth = str_to_float(gtk_entry_get_text(GTK_ENTRY(apd_growth)));
+    growth = CLAMP(growth, 0, 1);
+    for (list=model->selection ; list ; list=g_slist_next(list))
+      {
+      core = list->data;
+      core->growth = growth;
+      if (model->colour_scheme == GROWTH_SLICE)
+        atom_colour_scheme(GROWTH_SLICE, core, model);
+      }
+    break;
+
+  case CORE_REGION:
+    region = str_to_float(gtk_entry_get_text(GTK_ENTRY(apd_region)));
+    if (region > model->region_max)
+      model->region_max = region;
+
+    for (list=model->selection ; list ; list=g_slist_next(list))
+      {
+      core = list->data;
+      core->region = region;
+      if (core->shell)
+        (core->shell)->region = region;
+
+      if (model->colour_scheme == REGION)
+        atom_colour_scheme(REGION, core, model);
+      }
+    break;
+
+  case CORE_TRANSLATE:
+    translate = str_to_float(gtk_entry_get_text(GTK_ENTRY(apd_translate)));
+    translate = CLAMP(translate, 0, 1);
+    for (list=model->selection ; list ; list=g_slist_next(list))
+      {
+      core = list->data;
+      core->translate = translate;
+      if (core->shell)
+        (core->shell)->translate = translate;
+
+      if (model->colour_scheme == TRANSLATE)
+        atom_colour_scheme(TRANSLATE, core, model);
+      }
+    break;
+
+  case CORE_FF:
+    text = gtk_entry_get_text(GTK_ENTRY(apd_type));
+    for (list=model->selection ; list ; list=g_slist_next(list))
+      {
+      core = list->data;
+      if (core->atom_type)
+        g_free(core->atom_type);
+      core->atom_type = g_strdup(text);
+      }
     break;
   }
+
+gui_refresh(GUI_MODEL_PROPERTIES);
+gui_refresh(GUI_CANVAS);
 }
 
 /*****************************/
@@ -1013,32 +1086,45 @@ printf("set selection charge: %f\n", charge);
 /*****************************/
 void atom_properties_change(GtkWidget *w, gint type)
 {
-gint n;
+gint n, growth, region, translate;
 const gchar *text;
 struct elem_pak edata;
+struct model_pak *model;
 
-if (!apd_data)
+model = sysenv.active_model;
+if (!model)
+  return;
+
+/* act on multiple atoms? */
+if (g_slist_length(model->selection) > 1)
   {
-/* NEW - allow changes if more than one atom is selected */
   selection_properties_change(type);
   return;
   }
 
-g_return_if_fail(apd_core != NULL);
+if (!apd_core)
+  return;
 
 switch(type)
   {
   case NAME:
     text = gtk_entry_get_text(GTK_ENTRY(apd_label));
-/* make sure we alow enough space for the string and the \0 */
-    n = (LABEL_SIZE-1 > strlen(text)) ? strlen(text) : LABEL_SIZE-1;
-    g_snprintf(apd_core->label,n+1,"%s",text);
+    g_free(apd_core->atom_label);
+    apd_core->atom_label = g_strdup(text);
     n = elem_symbol_test(text);
 
+/* update atttached shell */
+if (apd_core->shell)
+  {
+  struct shel_pak *shell = apd_core->shell;
+  g_free(shell->shell_label);
+  shell->shell_label = g_strdup(text);
+  }
+
 /* if recognized -> update */
     if (n)
       {
-      get_elem_data(n, &edata, apd_data);
+      get_elem_data(n, &edata, model);
 
 /* NEW - don't update element specific data if the element type was not */
 /* changed - ie the user has just made a labelling change (eg C -> C1) */
@@ -1047,93 +1133,127 @@ switch(type)
         apd_core->atom_code = n;
         apd_core->bond_cutoff = edata.cova;
         }
-      init_atom_colour(apd_core, apd_data);
-      init_atom_charge(apd_core, apd_data);
+      init_atom_colour(apd_core, model);
+      init_atom_charge(apd_core, model);
 
-      g_slist_free(apd_data->unique_atom_list);
-      apd_data->unique_atom_list = find_unique(ELEMENT, apd_data);
-      calc_emp(apd_data);
-      atom_properties_update(apd_core, apd_data);
-      redraw_canvas(SINGLE);
+      g_slist_free(model->unique_atom_list);
+      model->unique_atom_list = find_unique(ELEMENT, model);
+      calc_emp(model);
+
+/* REFRESH */
+      gui_refresh(GUI_MODEL_PROPERTIES);
       }
     break;
 
-/* FIXME - not the best name, it's really the FF type */
-  case LABEL:
+  case CORE_FF:
     text = gtk_entry_get_text(GTK_ENTRY(apd_type));
     if (apd_core->atom_type)
       g_free(apd_core->atom_type);
     apd_core->atom_type = g_strdup(text);
-    redraw_canvas(SINGLE);
     break;
 
   case CHARGE:
     text = gtk_entry_get_text(GTK_ENTRY(apd_charge));
     apd_core->charge = str_to_float(text);
     apd_core->lookup_charge = FALSE;
-    calc_emp(apd_data);
-    if (apd_data->show_charge || apd_data->show_atom_charges)
-      redraw_canvas(SINGLE);
+    calc_emp(model);
     break;
 
   case COORD_X:
     text = gtk_entry_get_text(GTK_ENTRY(apd_x));
     apd_core->x[0] = str_to_float(text);
-    calc_coords(REFRESH, apd_data);
-    redraw_canvas(SINGLE);
+    coords_compute(model);
     break;
 
   case COORD_Y:
     text = gtk_entry_get_text(GTK_ENTRY(apd_y));
     apd_core->x[1] = str_to_float(text);
-    calc_coords(REFRESH, apd_data);
-    redraw_canvas(SINGLE);
+    coords_compute(model);
     break;
 
   case COORD_Z:
     text = gtk_entry_get_text(GTK_ENTRY(apd_z));
     apd_core->x[2] = str_to_float(text);
-    calc_coords(REFRESH, apd_data);
-    redraw_canvas(SINGLE);
+    coords_compute(model);
+    break;
+
+  case CORE_GROWTH_SLICE:
+    text = gtk_entry_get_text(GTK_ENTRY(apd_growth));
+    growth = CLAMP(str_to_float(text), 0, 1);
+    apd_core->growth = growth;
+    if (model->colour_scheme == GROWTH_SLICE)
+      atom_colour_scheme(GROWTH_SLICE, apd_core, model);
+    break;
+
+  case CORE_REGION:
+    text = gtk_entry_get_text(GTK_ENTRY(apd_region));
+    region = str_to_float(text);
+    if (region > model->region_max)
+      model->region_max = region;
+    apd_core->region = region;
+    if (apd_core->shell)
+      (apd_core->shell)->region = region;
+    if (model->colour_scheme == REGION)
+      atom_colour_scheme(REGION, apd_core, model);
+    break;
+
+  case CORE_TRANSLATE:
+    text = gtk_entry_get_text(GTK_ENTRY(apd_translate));
+    translate = CLAMP(str_to_float(text), 0, 1);
+    apd_core->translate = translate;
+    if (apd_core->shell)
+      (apd_core->shell)->translate = translate;
+    if (model->colour_scheme == TRANSLATE)
+      atom_colour_scheme(TRANSLATE, apd_core, model);
     break;
 
   default:
-    g_assert_not_reached();
+    printf("Not yet modifiable...\n");
   }
+gui_refresh(GUI_CANVAS);
 }
 
 /*************************************/
 /* updates the dialog for a new atom */
 /*************************************/
-void atom_properties_update(struct core_pak *core, struct model_pak *model)
+void gui_refresh_selection(void)
 {
 gdouble q;
-gchar *element, *label, *type, *charge, *x, *y, *z;
-struct shel_pak *shell;
+gchar *element, *label, *type, *charge, *x, *y, *z, *growth, *region, *translate;
+struct core_pak *core;
+struct model_pak *model;
+
+model = sysenv.active_model;
+core = NULL;
+if (model)
+  {
+  GSList *list = model->selection;
+  if (g_slist_length(list) == 1)
+    core = list->data;
+  }
 
 if (core && model)
   {
 /* data available */
   element = g_strdup(elements[core->atom_code].symbol);
-  label = g_strdup(core->label);
+  label = g_strdup(core->atom_label);
 
   if (core->atom_type)
     type = g_strdup(core->atom_type);
   else
-    type = g_strdup("?");
+    type = g_strdup("");
 
-  q = core->charge;
-  if (core->shell)
-    {
-    shell = (struct shel_pak *) core->shell;
-    q += shell->charge;
-    }
+  q = atom_charge(core); /* Replaced by C. Fisher 2004 */
   charge = g_strdup_printf("%9.4f", q);
 
   x = g_strdup_printf("%9.4f", core->x[0]);
   y = g_strdup_printf("%9.4f", core->x[1]);
   z = g_strdup_printf("%9.4f", core->x[2]);
 
+  growth = g_strdup_printf("%d", core->growth);
+  region = g_strdup_printf("%d", core->region);
+  translate = g_strdup_printf("%d", core->translate);
+
   apd_core = core;
   }
 else
@@ -1146,6 +1266,9 @@ else
   x = g_strdup("");
   y = g_strdup("");
   z = g_strdup("");
+  growth = g_strdup("");
+  region = g_strdup("");
+  translate = g_strdup("");
   }
 
 /* prevent changes from messing up the atom_properties_change() callback */
@@ -1158,8 +1281,9 @@ gtk_entry_set_text(GTK_ENTRY(apd_charge), charge);
 gtk_entry_set_text(GTK_ENTRY(apd_x), x);
 gtk_entry_set_text(GTK_ENTRY(apd_y), y);
 gtk_entry_set_text(GTK_ENTRY(apd_z), z);
-
-/* TODO : colour? SOF? */
+gtk_entry_set_text(GTK_ENTRY(apd_growth), growth);
+gtk_entry_set_text(GTK_ENTRY(apd_region), region);
+gtk_entry_set_text(GTK_ENTRY(apd_translate), translate);
 
 apd_data = model;
 
@@ -1171,14 +1295,17 @@ g_free(charge);
 g_free(x);
 g_free(y);
 g_free(z);
+g_free(growth);
+g_free(region);
+g_free(translate);
 }
 
-/****************************************/
-/* edit the properties of a single atom */
-/****************************************/
-void atom_properties_box(GtkWidget *box)
+/*******************************************/
+/* display the properties of a single atom */
+/*******************************************/
+void gui_edit_widget(GtkWidget *box)
 {
-GtkWidget *hbox, *vbox, *entry;
+GtkWidget *frame, *hbox, *vbox, *entry;
 
 /* checks */
 g_return_if_fail(box != NULL);
@@ -1203,11 +1330,6 @@ gtk_entry_set_text(GTK_ENTRY(entry), "FF Type");
 gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
 gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
 
-entry = gtk_entry_new();
-gtk_entry_set_text(GTK_ENTRY(entry), "Charge");
-gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
-gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
-
 entry = gtk_entry_new();
 gtk_entry_set_text(GTK_ENTRY(entry), "X");
 gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
@@ -1223,10 +1345,25 @@ gtk_entry_set_text(GTK_ENTRY(entry), "Z");
 gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
 gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
 
-/*
-label = gtk_label_new("Colour:");
-gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
-*/
+entry = gtk_entry_new();
+gtk_entry_set_text(GTK_ENTRY(entry), "Charge");
+gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
+
+entry = gtk_entry_new();
+gtk_entry_set_text(GTK_ENTRY(entry), "Growth");
+gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
+
+entry = gtk_entry_new();
+gtk_entry_set_text(GTK_ENTRY(entry), "Region");
+gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
+
+entry = gtk_entry_new();
+gtk_entry_set_text(GTK_ENTRY(entry), "Translate");
+gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
 
 /* right vbox - data */
 vbox = gtk_vbox_new(TRUE, 0);
@@ -1238,9 +1375,6 @@ gtk_box_pack_start(GTK_BOX(vbox), apd_label, FALSE, FALSE, 0);
 apd_type = gtk_entry_new();
 gtk_box_pack_start(GTK_BOX(vbox), apd_type, FALSE, FALSE, 0);
 
-apd_charge = gtk_entry_new();
-gtk_box_pack_start(GTK_BOX(vbox), apd_charge, FALSE, FALSE, 0);
-
 apd_x = gtk_entry_new();
 gtk_box_pack_start(GTK_BOX(vbox), apd_x, FALSE, FALSE, 0);
 
@@ -1250,21 +1384,61 @@ gtk_box_pack_start(GTK_BOX(vbox), apd_y, FALSE, FALSE, 0);
 apd_z = gtk_entry_new();
 gtk_box_pack_start(GTK_BOX(vbox), apd_z, FALSE, FALSE, 0);
 
-/* set boxes */
-atom_properties_update(NULL, NULL);
+apd_charge = gtk_entry_new();
+gtk_box_pack_start(GTK_BOX(vbox), apd_charge, FALSE, FALSE, 0);
+
+apd_growth = gtk_entry_new();
+gtk_box_pack_start(GTK_BOX(vbox), apd_growth, FALSE, FALSE, 0);
+
+apd_region = gtk_entry_new();
+gtk_box_pack_start(GTK_BOX(vbox), apd_region, FALSE, FALSE, 0);
+
+apd_translate = gtk_entry_new();
+gtk_box_pack_start(GTK_BOX(vbox), apd_translate, FALSE, FALSE, 0);
 
 /* attach callbacks (NB: set initial data first) */
 g_signal_connect(GTK_OBJECT(apd_label), "activate",
                  GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(NAME));
 g_signal_connect(GTK_OBJECT(apd_type), "activate",
-                 GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(LABEL));
-g_signal_connect(GTK_OBJECT(apd_charge), "activate",
-                 GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(CHARGE));
+                 GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(CORE_FF));
+
 g_signal_connect(GTK_OBJECT(apd_x), "activate",
                  GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(COORD_X));
 g_signal_connect(GTK_OBJECT(apd_y), "activate",
                  GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(COORD_Y));
 g_signal_connect(GTK_OBJECT(apd_z), "activate",
                  GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(COORD_Z));
-}
 
+g_signal_connect(GTK_OBJECT(apd_charge), "activate",
+                 GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(CHARGE));
+
+g_signal_connect(GTK_OBJECT(apd_growth), "activate",
+                 GTK_SIGNAL_FUNC(atom_properties_change),
+                 GINT_TO_POINTER(CORE_GROWTH_SLICE));
+
+g_signal_connect(GTK_OBJECT(apd_region), "activate",
+                 GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(CORE_REGION));
+g_signal_connect(GTK_OBJECT(apd_translate), "activate",
+                 GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(CORE_TRANSLATE));
+
+/* CURRENT */
+frame = gtk_frame_new(NULL);
+gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);
+gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
+vbox = gtk_vbox_new(TRUE, 1);
+gtk_container_add(GTK_CONTAINER(frame), vbox);
+
+gui_button_x("Add atoms", gtk_mode_switch, (gpointer) ATOM_ADD, vbox);
+gui_button_x("Add bonds", gtk_mode_switch, (gpointer) BOND_SINGLE, vbox);
+gui_button_x("Delete bonds", gtk_mode_switch, (gpointer) BOND_DELETE, vbox);
+gui_button_x("Normal mode", gtk_mode_switch, (gpointer) FREE, vbox);
+
+frame = gtk_frame_new(NULL);
+gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);
+gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
+vbox = gtk_vbox_new(TRUE, 1);
+gtk_container_add(GTK_CONTAINER(frame), vbox);
+
+gui_button_x("Mark as ghost", select_flag_ghost, NULL, vbox);
+gui_button_x("Mark as normal", select_flag_normal, NULL, vbox);
+}
