@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -31,17 +31,6 @@ The GNU GPL can also be found at http://www.gnu.org
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#ifdef __sun
-#include <sys/dirent.h>
-#else
-  #ifdef __WIN32
-  #include <dirent.h>
-  #else
-  #include <sys/dir.h>
-  #include <sys/param.h>
-  #endif
-#endif
-
 #include "gdis.h"
 #include "coords.h"
 #include "file.h"
@@ -49,9 +38,10 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "matrix.h"
 #include "model.h"
 #include "space.h"
+#include "zone.h"
 #include "render.h"
 #include "select.h"
-#include "gtkshorts.h"
+#include "shortcuts.h"
 #include "interface.h"
 #include "dialog.h"
 #include "opengl.h"
@@ -69,7 +59,7 @@ extern struct elem_pak elements[];
 #define DEBUG_FILE_INIT 0
 void file_init(void)
 {
-GSList *list, *rlist=NULL;
+GSList *rlist=NULL;
 struct file_pak *file_data;
 
 /* NEW - build a recognized image read format list */
@@ -110,7 +100,7 @@ printf("\n");
 sysenv.file_list = NULL;
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = DATA;                           /* unique identifier */ 
 file_data->group = DATA;                        /* used to group inp/out types */
 file_data->menu = TRUE;                         /* include in menu listing */
@@ -122,7 +112,7 @@ file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = ABINIT_OUT;
 file_data->group = ABINIT;
 file_data->menu = FALSE;
@@ -136,7 +126,35 @@ file_data->read_frame = read_about_frame;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = MD_ANALYSIS;
+file_data->group = MD_ANALYSIS;
+file_data->menu = FALSE;
+file_data->label = g_strdup("Analysis");
+file_data->ext = NULL;
+file_data->ext = NULL;
+/* import/export handled in analysis as a special case */
+file_data->write_file = NULL;
+file_data->read_file = NULL;
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type  */
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = BIOGRAF;
+file_data->group = BIOGRAF;
+file_data->menu = TRUE;
+file_data->label = g_strdup("bgf");
+file_data->label = g_strdup("Biograf");                 
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "bgf");
+file_data->write_file = write_bgf;
+file_data->read_file = read_bgf;
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);  
+
+/* supported file type */
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = BIOSYM;
 file_data->group = BIOSYM;
 file_data->menu = TRUE;
@@ -151,7 +169,32 @@ file_data->read_frame = read_arc_frame;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = CASTEP;
+file_data->group = CASTEP;
+file_data->menu = TRUE;
+file_data->label = g_strdup("CASTEP");
+file_data->ext = NULL;
+file_data->write_file = NULL;
+file_data->read_file = NULL;
+file_data->read_frame = NULL;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = CASTEP_OUT;
+file_data->group = CASTEP;
+file_data->menu = FALSE;
+file_data->label = g_strdup("CASTEP output");
+file_data->ext = NULL;
+file_data->ext = g_slist_append(file_data->ext, "castep");
+file_data->write_file = NULL;
+file_data->read_file = read_castep_out;
+file_data->read_frame = read_castep_out_frame;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = CIF;
 file_data->group = CIF;
 file_data->menu = TRUE;
@@ -163,11 +206,38 @@ file_data->read_file = read_cif;
 file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
+/* NEW: monty crystal graph file format */
+/* supported file type  */
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = CRYSTAL_GRAPH;
+file_data->group = CRYSTAL_GRAPH;
+file_data->menu = TRUE;
+file_data->label = g_strdup("cgf");
+file_data->label = g_strdup("Crystal Graph Files");                 
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "cgf");
+file_data->write_file = write_cgf;
+file_data->read_file = read_cgf;
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = CSSR;
+file_data->group = CSSR;
+file_data->menu = TRUE;
+file_data->label = g_strdup("CSSR");
+file_data->ext = NULL;
+file_data->ext = g_slist_append(file_data->ext, "cssr");
+file_data->write_file = write_cssr;
+file_data->read_file = read_cssr;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = DIFFAX_INP;
 file_data->group = DIFFAX_INP;
-file_data->menu = TRUE;
+file_data->menu = FALSE;
 file_data->label = g_strdup("DIFFaX");
 file_data->ext = NULL;
 file_data->ext = g_slist_prepend(file_data->ext, "dfx");
@@ -177,7 +247,47 @@ file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = DLP;
+file_data->group = DLP;
+file_data->menu = TRUE;
+file_data->label = g_strdup("DLP");
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "dlp");
+file_data->write_file = write_dlp;
+file_data->read_file = read_dlp;
+file_data->read_frame = NULL;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = DMOL_INPUT;
+file_data->group = DMOL_INPUT;
+file_data->menu = TRUE;
+file_data->label = g_strdup("DMOL");
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "dmol");
+file_data->write_file = write_dmol;
+file_data->read_file = read_dmol;
+file_data->read_frame = NULL;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = DLPOLY;
+file_data->group = DLPOLY;
+file_data->menu = TRUE;
+file_data->label = g_strdup("DL_POLY");
+file_data->ext = NULL;
+file_data->ext = g_slist_append(file_data->ext, "");
+file_data->ext = g_slist_append(file_data->ext, "dlpoly");
+file_data->write_file = write_dlpoly;
+file_data->read_file = read_dlpoly;
+file_data->read_frame = read_dlpoly_frame;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = GAMESS;
 file_data->group = GAMESS;
 file_data->menu = TRUE;
@@ -190,7 +300,7 @@ file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = GAMESS_OUT;
 file_data->group = GAMESS;
 file_data->menu = FALSE;
@@ -204,21 +314,33 @@ file_data->read_frame = read_gms_out_frame;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = MD_ANALYSIS;
-file_data->group = MD_ANALYSIS;
-file_data->menu = FALSE;
-file_data->label = g_strdup("Analysis");
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = GAUSS;
+file_data->group = GAUSS;
+file_data->menu = TRUE;
+file_data->label = g_strdup("GAUSSIAN");
 file_data->ext = NULL;
-file_data->ext = NULL;
-/* import/export handled in analysis as a special case */
-file_data->write_file = NULL;
+file_data->ext = g_slist_append(file_data->ext, "com");
+file_data->write_file = write_gauss;
 file_data->read_file = NULL;
-file_data->read_frame = NULL;                   /* frame reading */
+file_data->read_frame = NULL;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = GAUSS_OUT;
+file_data->group = GAUSS;
+file_data->menu = FALSE;
+file_data->label = g_strdup("GAUSSIAN output");
+file_data->ext = NULL;
+file_data->ext = g_slist_append(file_data->ext, "log");
+file_data->write_file = NULL;
+file_data->read_file = read_gauss_out;
+file_data->read_frame = read_gauss_out_frame;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = MORPH;
 file_data->group = MORPH;
 file_data->menu = TRUE;
@@ -231,7 +353,20 @@ file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = GEOMVIEW_OFF;
+file_data->group = GEOMVIEW_OFF;
+file_data->menu = TRUE;
+file_data->label = g_strdup("Geomview OFF");
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "off");
+file_data->write_file = NULL;
+file_data->read_file = read_off;
+file_data->read_frame = NULL;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = GULP;
 file_data->group = GULP;
 file_data->menu = TRUE;
@@ -245,7 +380,7 @@ file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = GULPOUT;
 file_data->group = GULP;
 file_data->menu = FALSE;
@@ -259,7 +394,7 @@ file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = MARVIN;
 file_data->group = MARVIN;
 file_data->menu = TRUE;
@@ -274,7 +409,7 @@ file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = MVNOUT;
 file_data->group = MARVIN;
 file_data->menu = FALSE;
@@ -289,7 +424,7 @@ file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = NWCHEM;
 file_data->group = NWCHEM;
 file_data->menu = TRUE;
@@ -302,10 +437,10 @@ file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = NWCHEM_OUT;
 file_data->group = NWCHEM;
-file_data->menu = TRUE;
+file_data->menu = FALSE;
 file_data->label = g_strdup("NWChem output");
 file_data->ext = NULL;
 file_data->ext = g_slist_prepend(file_data->ext, "nwout");
@@ -316,33 +451,7 @@ file_data->read_frame = read_nwout_frame;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = PICTURE;
-file_data->group = PICTURE;
-file_data->menu = FALSE;
-/* TODO - check supported image types with glib - add to string */
-file_data->label = g_strdup("Picture (jpg)");
-file_data->ext = rlist;
-file_data->write_file = NULL;
-file_data->read_file = NULL;
-file_data->read_frame = NULL;                   /* frame reading */
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-/* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = RIETICA;
-file_data->group = RIETICA;
-file_data->menu = TRUE;
-file_data->label = g_strdup("Rietica");
-file_data->ext = NULL;
-file_data->ext = g_slist_prepend(file_data->ext, "inp");
-file_data->write_file = NULL;
-file_data->read_file = read_rietica;
-file_data->read_frame = NULL;                   /* frame reading */
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-/* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = PDB;
 file_data->group = PDB;
 file_data->menu = TRUE;
@@ -355,57 +464,20 @@ file_data->read_frame = read_pdb_frame;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = CASTEP;
-file_data->group = CASTEP;
-file_data->menu = TRUE;
-file_data->label = g_strdup("CASTEP");
-file_data->ext = NULL;
-file_data->write_file = NULL;
-file_data->read_file = NULL;
-file_data->read_frame = NULL;
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-/* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = CASTEP_OUT;
-file_data->group = CASTEP;
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = PICTURE;
+file_data->group = PICTURE;
 file_data->menu = FALSE;
-file_data->label = g_strdup("CASTEP output");
-file_data->ext = NULL;
-file_data->ext = g_slist_append(file_data->ext, "castep");
-file_data->write_file = NULL;
-file_data->read_file = read_castep_out;
-file_data->read_frame = read_castep_out_frame;
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-/* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = GAUSS;
-file_data->group = GAUSS;
-file_data->menu = TRUE;
-file_data->label = g_strdup("GAUSSIAN");
-file_data->ext = NULL;
+/* TODO - check supported image types with glib - add to string */
+file_data->label = g_strdup("Picture (jpg)");
+file_data->ext = rlist;
 file_data->write_file = NULL;
 file_data->read_file = NULL;
 file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = GAUSS_OUT;
-file_data->group = GAUSS;
-file_data->menu = FALSE;
-file_data->label = g_strdup("GAUSSIAN output");
-file_data->ext = NULL;
-file_data->ext = g_slist_append(file_data->ext, "log");
-file_data->write_file = NULL;
-file_data->read_file = read_gauss_out;
-file_data->read_frame = read_gauss_out_frame;
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-/* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = POVRAY;
 file_data->group = POVRAY;
 file_data->menu = FALSE;
@@ -418,7 +490,46 @@ file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = PROJECT;
+file_data->group = PROJECT;
+file_data->menu = FALSE;
+file_data->label = g_strdup("Project");
+file_data->ext = NULL;
+file_data->ext = g_slist_append(file_data->ext, "pcf");
+file_data->write_file = NULL;
+file_data->read_file = project_read;
+file_data->read_frame = NULL;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = CEL;
+file_data->group = CEL;
+file_data->menu = TRUE;
+file_data->label = g_strdup("PowderCell");
+file_data->ext = NULL;
+file_data->ext = g_slist_append(file_data->ext, "cel");
+file_data->write_file = NULL;
+file_data->read_file = read_cel;
+file_data->read_frame = NULL;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = g_malloc(sizeof(struct file_pak));
+file_data->id = RIETICA;
+file_data->group = RIETICA;
+file_data->menu = TRUE;
+file_data->label = g_strdup("Rietica");
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "inp");
+file_data->write_file = NULL;
+file_data->read_file = read_rietica;
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = FDF;
 file_data->group = FDF;
 file_data->menu = TRUE;
@@ -431,7 +542,7 @@ file_data->read_frame = NULL;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = SIESTA_OUT;
 file_data->group = FDF;
 file_data->menu = FALSE;
@@ -445,7 +556,7 @@ file_data->read_frame = read_sout_frame;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = XML;
 file_data->group = XML;
 file_data->menu = TRUE;
@@ -458,7 +569,7 @@ file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = XTL;
 file_data->group = XTL;
 file_data->menu = TRUE;
@@ -471,41 +582,22 @@ file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data = g_malloc(sizeof(struct file_pak));
 file_data->id = XYZ;
 file_data->group = XYZ;
 file_data->menu = TRUE;
 file_data->label = g_strdup("XYZ");
 file_data->ext = NULL;
+/*
+file_data->ext = g_slist_append(file_data->ext, "ani");
+file_data->ext = g_slist_append(file_data->ext, "xmol");
+*/
 file_data->ext = g_slist_append(file_data->ext, "xyz");
 file_data->write_file = write_xyz;
 file_data->read_file = read_xyz;
-file_data->read_frame = NULL;                   /* frame reading */
+file_data->read_frame = read_xyz_frame;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
-/* supported file type */
-/*
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = BABEL;
-file_data->group = DATA;
-file_data->menu = TRUE;
-file_data->label = g_strdup("Filterable types");
-file_data->ext = NULL;
-file_data->write_file = NULL;
-file_data->read_file = NULL;
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = AUTO;
-file_data->group = BABEL;
-file_data->menu = TRUE;
-file_data->label = g_strdup("auto select filter");
-file_data->ext = NULL;
-file_data->write_file = NULL;
-file_data->read_file = NULL;
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-*/
-
 sysenv.file_list = g_slist_reverse(sysenv.file_list);
 }
 
@@ -542,7 +634,9 @@ dir = g_dir_open(path, 0, NULL);
 name = g_dir_read_name(dir);
 while (name)
   {
-  files = g_slist_prepend(files, g_strdup(name));
+  if (g_ascii_strncasecmp(".", name, 1) != 0)
+    files = g_slist_prepend(files, g_strdup(name));
+
   name = g_dir_read_name(dir);
   }
 g_dir_close(dir);
@@ -553,6 +647,39 @@ if (sort)
 return(files);
 }
 
+/*******************************************************/
+/* allocate a new string containing the file extension */
+/*******************************************************/
+gchar *file_extension_get(const gchar *name)
+{
+gint i,n;
+gchar *ext;
+
+if (!name)
+  return(NULL);
+
+/* search for '.' character in reverse order */
+i = n = strlen(name);
+/* minimum size, avoids troublesome cases (eg "..") */
+if (n > 2)
+  {
+  for (i=n-1 ; i-- ; )
+    {
+    if (name[i] == '.')
+      {
+      ext = g_strndup(name+i+1, n-i-1);
+/* ignore any .# extension */
+      if (!str_is_float(ext))
+        return(ext);
+      g_free(ext);
+/* mark the .# as the new end */
+      n = i;
+      }
+    }
+  }
+return(NULL);
+}
+
 /************************************************/
 /* routine to determine if a file is recognized */
 /************************************************/
@@ -572,19 +699,28 @@ g_return_val_if_fail(ptr != NULL, NULL);
 switch(type)
   {
   case BY_LABEL:
-    text = (gchar *) ptr;
+    text = g_strdup(ptr);
+
 #if DEBUG_GET_FILE_INFO
 printf("Searching for type [%s]\n", text);
 #endif
     break;
 
   case BY_EXTENSION:
-/* get file extension (if any) as separate string for matching purposes */
-    ext = find_char((gchar *) ptr, '.', LAST);
-    if (!ext)
+/* fix to get around the quotes needed to properly pass win32 filesnames with spaces */
+#if __WIN32
+    {
+    gchar *tmp = g_shell_unquote(ptr, NULL);
+    text = file_extension_get(tmp);
+    g_free(tmp); 
+    }
+#else
+    text = file_extension_get(ptr);
+#endif
+
+    if (!text)
       return(NULL);
-    else
-      text = ++ext; /* skip the '.' */
+
 #if DEBUG_GET_FILE_INFO
 printf("Searching for extension [%s]\n", text);
 #endif
@@ -599,29 +735,29 @@ printf("Searching for code [%d]\n", code);
   }
 
 /* search */
-file = sysenv.file_list;
-while(file != NULL)
+for (file=sysenv.file_list ; file ; file=g_slist_next(file))
   {
-  file_data = (struct file_pak *) file->data;
+  file_data = file->data;
 
-  switch(type)
+  switch (type)
     {
 /* compare to all extensions in list */
     case BY_EXTENSION: 
 /* go through all extensions listed under this file type */
-      ext_list = file_data->ext;
-      while(ext_list != NULL)
+      for (ext_list=file_data->ext ; ext_list ; ext_list=g_slist_next(ext_list))
         {
-        ext = (gchar *) ext_list->data;
+        ext = ext_list->data;
         if (strlen(text) == strlen(ext))
+          {
           if (g_ascii_strcasecmp(text, ext) == 0)
             {
 #if DEBUG_GET_FILE_INFO
 printf("Matched: %s\n", file_data->label);
 #endif
+            g_free(text);
             return(file_data);
             }
-        ext_list = g_slist_next(ext_list);
+          }
         }
       break;
 
@@ -634,6 +770,7 @@ printf("Matched: %s\n", file_data->label);
 #if DEBUG_GET_FILE_INFO
 printf("Matched: %s\n", file_data->label);
 #endif
+        g_free(text);
         return(file_data);
         }
       break;
@@ -652,54 +789,57 @@ printf("Matched: %s\n", file_data->label);
     default:
       printf("get_file_info() error: bad search type.\n");
     }
-
-  file = g_slist_next(file);
   }
 
+if (text)
+  g_free(text);
+
 return(NULL);
 }
 
 /**************************/
 /* detect valid filetypes */
 /**************************/
-gint file_extension_valid(gchar *name)
+gint file_extension_valid(const gchar *name)
 {
-gint len;
-gchar *ext, *text;
+gchar *ext;
 GSList *item, *list;
 struct file_pak *file;
 
 /* locate the extension */
+ /*FIXME: can find_char be replaced by standard strchr()/strrchr() functions?
+   - MW */
 ext = find_char(name, '.', LAST);
-if (!ext)
-  return(FALSE);
-else
-  ext++; /* skip the '.' */
-len = strlen(ext);
+if (ext)
+  ext++;
 
-/* compare against all types, since we now permit duplicate extensions */ 
-for (list=sysenv.file_list ; list ; list=g_slist_next(list))
+if (sysenv.file_type == DATA) 
+  /* compare against all extension types - any match is allowed */
   {
-  file = list->data;
-
-  for (item=file->ext ; item ; item=g_slist_next(item))
+  if (!ext) /* files without extensions are matched only in strict case */
+    return(FALSE);  
+  for (list=sysenv.file_list ; list ; list=g_slist_next(list))
     {
-    text = item->data;
-    if (strlen(text) == len)
-      {
-      if (g_ascii_strcasecmp(text, ext) == 0)
-        {
-        if (sysenv.file_type == DATA)
-          return(TRUE);
-        else
-          {
-          if (sysenv.file_type == file->group)
-            return(TRUE);
-          }
-        }
-      }
+    file = list->data; 
+    for (item=file->ext ; item ; item=g_slist_next(item))
+      if (g_ascii_strcasecmp(item->data, ext) == 0)
+        return(TRUE);
     }
   }
+else
+  /* strict case - file must match the dialog filter */
+  {
+  for (list=sysenv.file_list ; list ; list=g_slist_next(list))
+    {
+    file = list->data; 
+    if (sysenv.file_type == file->group)
+        for (item=file->ext ; item ; item=g_slist_next(item))
+            if ((ext && g_ascii_strcasecmp(item->data, ext) == 0)
+                || (!ext && strlen(item->data) == 0))
+              return(TRUE);
+    }
+  }
+
 return(FALSE);
 }
 
@@ -742,8 +882,7 @@ else
   return(NULL);
   }
 
-name = g_strdup(filename->str);
-g_string_free(filename, TRUE);
+name = g_string_free(filename, FALSE);
 
 #if DEBUG_GUN
 printf("using base: %s\n", name);
@@ -824,7 +963,8 @@ return(0);
 /* TODO - replacement routine for fgetline() */
 gchar *file_read_line(FILE *fp)
 {
-gchar c, *line;
+gboolean comment;
+gchar c;
 GString *buff;
 
 /* checks */
@@ -837,25 +977,51 @@ if (c == EOF)
 buff = g_string_new(NULL);
 while (c != EOF)
   {
-  g_string_append_c(buff, c);
+  if (c == '#')
+    comment = TRUE;
+  else
+    comment = FALSE;
+/* read a complete line */
+  while (c != EOF)
+    {
+    g_string_append_c(buff, c);
+    if (c == '\n')
+      break;
+    if (c == '\r')
+      {
+      gchar d;
 
-  if (c == '\n')
+      d = fgetc(fp);
+      if (d == '\n')
+        {
+/*
+        printf("skipping MSDOS crap.\n");
+*/
+        }
+      else
+        ungetc(d, fp);
+
+      break;
+      }
+    c = fgetc(fp);
+    }
+/* ignore comments (keep reading) */
+  if (comment)
+    {
+    g_string_assign(buff, "");
+    c = fgetc(fp);
+    }
+  else
     break;
-
-  c = fgetc(fp);
   }
 
 /* create a properly terminated line of text */
 g_string_append_c(buff, '\0');
-line = buff->str;
 
 /* free the GString, but not the text */
-g_string_free(buff, FALSE);
-
-return(line);
+return(g_string_free(buff, FALSE));
 }
 
-
 /**********************************************************/
 /* format value as a float printed to dp places plus unit */
 /**********************************************************/
@@ -883,87 +1049,10 @@ g_strfreev(buff);
 return (text_str);
 }
 
-/********************/
-/* BABEL conversion */
-/********************/
-gint read_using_babel(gchar *inpfile, struct model_pak *data)
-{
-gint status, type;
-gchar *inp, *out, *outfile;
-GString *txt; 
-struct file_pak *file_data;
-
-/* checks */
-g_return_val_if_fail(data != NULL, 1);
-g_return_val_if_fail(inpfile != NULL, 1);
-
-if (!sysenv.babel_path)
-  {
-  show_text(ERROR, "Sorry, you do not appear to have babel installed.\n");
-  return(2);
-  }
-
-/* new - guess filter type if none specified */
-if (sysenv.babel_type == AUTO)
-  {
-  file_data = get_file_info((gpointer *) inpfile, BY_EXTENSION);
-  if (!file_data)
-    return(3);
-  type = file_data->id;
-  }
-else
-  type = sysenv.babel_type;
-
-switch (type)
-  {
-/* FIXME - what about the periodic data??? */
-  case CSSR:
-    inp = g_strdup("cssr");
-    break;
-  case FDAT:
-    inp = g_strdup("fdat");
-    break;
-  case GAMOUT:
-    inp = g_strdup("gamout");
-    break;
-  case MOPAC_OUT:
-    inp = g_strdup("mopout");
-    break;
-  case PDB:
-    inp = g_strdup("pdb");
-    break;
-  default:
-    printf("babel_convert(): unknown file type.\n");
-    return(4);
-  }
-
-/* set up new filename */
-out = g_strdup("xyz");
-txt = g_string_new(NULL);
-g_string_assign(txt,inpfile);
-g_string_append(txt,".xyz");
-outfile = g_strdup(txt->str);
-/* convert & load via standard routine */
-show_text(STANDARD, "Converting with babel...\n");
-g_string_sprintf(txt,"%s -i%s %s -o%s %s >& /dev/null",
-                      sysenv.babel_path,
-                      inp, inpfile, out, outfile);
-system(txt->str);
-
-/* pass to xyz load routine */
-status = read_xyz(outfile, data);
-
-/* free & exit */
-g_string_free(txt, TRUE);
-g_free(inp);
-g_free(out);
-g_free(outfile);
-return(status);
-}
-
 /************************************************/
 /* read in a raw frame (animation and analysis) */
 /************************************************/
+#define DEBUG_READ_RAW 0
 gint read_raw_frame(FILE *fp, gint n, struct model_pak *model)
 {
 gint i, flag=99;
@@ -977,10 +1066,19 @@ g_assert(model != NULL);
 
 file = get_file_info(GINT_TO_POINTER(model->id), BY_FILE_ID);
 
+#if DEBUG_READ_RAW
+printf("reading frame: %d\n", n);
+#endif
+
 /* position the file pointer */
 if (model->id != GULP)
   {
   offset = g_list_nth_data(model->frame_list, n);
+
+#if DEBUG_READ_RAW
+printf("offset = %d\n", offset);
+#endif
+
   if (!offset)
     return(1);
   if (fsetpos(fp, offset))
@@ -992,7 +1090,7 @@ if (model->id != GULP)
   if (file->read_frame)
     flag = file->read_frame(fp, model);
   else
-    show_text(ERROR, "No animation read routine.\n");
+    gui_text_show(ERROR, "No animation read routine.\n");
   }
 else
   {
@@ -1083,6 +1181,7 @@ switch (sysenv.file_type)
 /* cases for which duplicate extensions occur */
   case RIETICA:
   case GAMESS:
+  case DLPOLY:
     file_data = get_file_info(GINT_TO_POINTER(sysenv.file_type), BY_FILE_ID);
     break;
 
@@ -1105,21 +1204,16 @@ else
   data = model_new();
   if (!data)
     {
-    show_text(ERROR, "Model memory allocation failed.\n");
+    gui_text_show(ERROR, "Model memory allocation failed.\n");
     return;
     }
   }
 
-/* group babel types together for this */
-if (file_data->group == BABEL)
-  data->id = file_data->group;
-else
-  data->id = file_data->id;
+data->id = file_data->id;
 
 /* read file if exists, else try prepending current working directory */
 if (file_data->read_file)
   {
-  property_add_ranked(1, "File Type", file_data->label, data);
   if (g_path_is_absolute(filename))
     {
     status = file_data->read_file(filename, data);
@@ -1132,18 +1226,22 @@ if (file_data->read_file)
     }
   }
 else
-  show_text(ERROR, "No read routine for this type. ");
+  gui_text_show(ERROR, "No read routine for this type. ");
+
+/* NEW - display error (if any) */
+if ((data->error_file_read)->len)
+  {
+  gui_text_show(ERROR, (data->error_file_read)->str);
+  }
 
 /* check for successful file load */
 if (status)
   {
-  show_text(ERROR, "Load failed.\n");
-#if DEBUG_FILE_LOAD
-  printf("Load failed, error code: %d\n",status);
-#endif
+  gui_text_show(ERROR, "Load failed.\n");
+
+printf("Load failed, error code: %d\n", status);
 
   model_delete(data);
-  canvas_shuffle();
   }
 else
   {
@@ -1157,10 +1255,26 @@ else
     if (data->grafted)
       continue;
 
-/* NEW */
-if (data->num_atoms > 1000)
+
+/* NEW - big model display mode exceptions */
+/* CURRENT - this doesn't work - something not set up? */
+/* TODO - automaticaly choose an appropriate zone size */
+/*
+if (data->num_atoms > 100000)
   {
-  core_render_mode_set(STICK, data->cores);
+  gpointer za;
+
+  core_render_mode_set(ZONE, data->cores);
+
+  za = zone_make(10.0, data);
+  zone_display_init(za, data);
+  zone_free(za);
+  }
+else
+*/
+  {
+  if (data->num_atoms > 1000)
+    core_render_mode_set(STICK, data->cores);
   }
 
 /* surfaces are always conv by default */
@@ -1185,6 +1299,7 @@ if (data->num_atoms > 1000)
       {
       space_make_images(CREATE, data);
       space_make_supercell(data);
+      model_prep(data);
       }
 
 /* NEW - store initial frame, as trajectory animation will overwrite */
@@ -1213,6 +1328,16 @@ tree_select_active();
 /***************/
 void file_load_dialog(void)
 {
+/* revert to data filetypes for special cases */
+switch (sysenv.file_type)
+  {
+  case MD_ANALYSIS:
+  case PICTURE:
+  case PROJECT:
+  case GEOMVIEW_OFF:
+    sysenv.file_type = DATA;
+  }
+
 file_dialog("Load file", NULL, FILE_LOAD, (gpointer) file_load, sysenv.file_type);
 }
 
@@ -1279,10 +1404,10 @@ else
 
 /* update */
 if (ret)
-  show_text(ERROR, "Save failed.\n");
+  gui_text_show(ERROR, "Save failed.\n");
 else
   {
-  show_text(STANDARD, "Saved model.\n");
+  gui_text_show(STANDARD, "Saved model.\n");
   dialog_destroy_type(FILE_SELECT);
   tree_model_refresh(model);
   redraw_canvas(SINGLE);
@@ -1325,3 +1450,16 @@ file_dialog("File save", text, FILE_SAVE, (gpointer) file_save_as,
                                                 sysenv.file_type);
 }
 
+/*************************************/
+/* get the size of a file (in bytes) */
+/*************************************/
+gint file_byte_size(const gchar *filename)
+{
+struct stat buff;
+
+stat(filename, &buff);
+
+return(buff.st_size);
+}
+
+
