@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -20,30 +20,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 The GNU GPL can also be found at http://www.gnu.org
 */
 
-#include <math.h>
 #include <stdio.h>
 #include <string.h>
-#include <strings.h>
-#include <ctype.h>
-#include <stdlib.h>
-#include <time.h>
-#include <unistd.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-
-#ifdef __sun
-#include <sys/dirent.h>
-#else
-#ifdef __WIN32
-#include <dirent.h>
-#else
-#include <sys/dir.h>
-#include <sys/param.h>
-#endif
-#endif
 
 #include "gdis.h"
 #include "coords.h"
+#include "model.h"
 #include "file.h"
 #include "parse.h"
 #include "matrix.h"
@@ -195,7 +177,7 @@ fclose(fp);
 
 /* post read setup */
 model->cores = g_slist_reverse(model->cores);
-prep_model(model);
+model_prep(model);
 
 return(0);
 }
