@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Andrew Lloyd Rohl
 
-andrew@power.curtin.edu.au
+andrew@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,27 +23,10 @@ The GNU GPL can also be found at http://www.gnu.org
 #include <math.h>
 #include <stdio.h>
 #include <string.h>
-#include <strings.h>
-#include <ctype.h>
-#include <stdlib.h>
-#include <time.h>
-#include <unistd.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-
-#ifdef __sun
-#include <sys/dirent.h>
-#else
-#ifdef __WIN32
-#include <dirent.h>
-#else
-#include <sys/dir.h>
-#include <sys/param.h>
-#endif
-#endif
 
 #include "gdis.h"
 #include "coords.h"
+#include "model.h"
 #include "file.h"
 #include "parse.h"
 #include "matrix.h"
@@ -60,8 +43,9 @@ gint write_pdb(gchar *filename, struct model_pak *model)
 {
 gdouble c;
 gdouble x[3];
-GSList *list;
-struct core_pak *core;
+GSList *list, *list2;
+struct bond_pak *bond;
+struct core_pak *core, *core2;
 gint i = 0;
 FILE *fp;
 
@@ -100,12 +84,22 @@ if (model->periodic)
     c =  2.0*model->rmax;
   else
     c = model->pbc[2];
+
+if (model->sginfo.spacename)
   fprintf(fp,"CRYST1%9.3f%9.3f%9.3f%7.2f%7.2f%7.2f %-11s%4d\n",
               model->pbc[0], model->pbc[1], c,
               180.0*model->pbc[3]/PI,
               180.0*model->pbc[4]/PI,
               180.0*model->pbc[5]/PI,
               model->sginfo.spacename, 0);
+else
+  fprintf(fp,"CRYST1%9.3f%9.3f%9.3f%7.2f%7.2f%7.2f %-11s%4d\n",
+              model->pbc[0], model->pbc[1], c,
+              180.0*model->pbc[3]/PI,
+              180.0*model->pbc[4]/PI,
+              180.0*model->pbc[5]/PI,
+              "P1", 0);
+
   }
 
 /*
@@ -135,25 +129,78 @@ COLUMNS        DATA TYPE       FIELD         DEFINITION
 
 for (list=model->cores ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
-  if (core->status & DELETED)
-    continue;
+  core = list->data;
 
 /* everything is cartesian after latmat mult */
   ARR3SET(x, core->x);
   vecmat(model->latmat, x);
   i++;
-  fprintf(fp,"ATOM  %5d %-4s %-3s  %4d    %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s%-2s\n",
-              i, core->label, core->res_name, core->res_no, x[0], x[1], x[2], core->sof, 0.0, "", elements[core->atom_code].symbol, "");
+  fprintf(fp,"ATOM  %5d %-4s %-3s %1s %3d    %8.3f%8.3f%8.3f%6.2f%6.2f      %-4s%2s%-2s\n",
+              i, core->atom_label, core->res_name, core->chain, core->res_no, x[0], x[1], x[2], core->sof, 0.0, "", elements[core->atom_code].symbol, "");
   }
 
+
+
+/* CURRENT */
+/* connectivity */
+i=1;
+for (list=model->cores ; list ; list=g_slist_next(list))
+  {
+  core = list->data;
+
+  if (core->bonds)
+    {
+fprintf(fp, "CONECT");
+
+fprintf(fp, " %4d", i);
+
+    for (list2=core->bonds ; list2 ; list2=g_slist_next(list2))
+      {
+      bond = list2->data;
+
+      if (connect_split(bond))
+        continue;
+
+      if (core == bond->atom1)
+        core2 = bond->atom2;
+      else
+        core2 = bond->atom1;
+
+fprintf(fp, " %4d", 1+g_slist_index(model->cores, core2));
+
+      }
+
+fprintf(fp, "\n");
+
+    }
+  i++;
+  }
+
+
 fclose(fp);
 return(0);
 }
 
-/*************************************************/
+/***********************************************/
+/* read a line of FORTRAN output padding to 80 */
+/* characters (with nulls) if required         */
+/***********************************************/
+gint fort_read_line(FILE *fp, gchar **line)
+{
+gint ret_val;
+gchar the_line[LINELEN];
+
+ret_val = fgetline(fp, the_line);
+if (!ret_val)
+  {
+  *line = g_strndup(the_line, 80);
+  }
+return(ret_val);
+}
+
+/**********************************************/
 /* read a gint from a line of FORTRAN output  */
-/*************************************************/
+/**********************************************/
 gint fort_read_gint(char *line, gint start_col, gint end_col)
 {
 gchar *field;
@@ -201,14 +248,14 @@ return(field);
 /*************************************************/
 gint read_pdb_block(FILE *fp, struct model_pak *data)
 {
-gchar line[LINELEN], *atom_name, *res_name;
+gchar *line, *atom_name, *res_name, *element;
 struct core_pak *core;
 GSList *clist;
 
 clist = data->cores;
 
 /* loop while there's data */
-while (!fgetline(fp, line))
+while (!fort_read_line(fp, &line))
   {
   /* 3-D structure? */
   if (g_ascii_strncasecmp("CRYST1", line, 6) == 0)
@@ -270,12 +317,18 @@ while (!fgetline(fp, line))
     atom_name = fort_read_string(line, 13, 16);
     if (clist)
       {
-      core = (struct core_pak *) clist->data;
+      core = clist->data;
       clist = g_slist_next(clist);
       }
     else
       {
-      core = new_core(atom_name, data);
+      element = fort_read_string(line, 77, 78);
+      element = g_strchug(element);
+/*    printf("\"%s\"\n", element);*/
+      if (*element != '\0')
+        core = core_new(element, atom_name, data);
+      else
+        core = new_core(atom_name, data);
       data->cores = g_slist_append(data->cores, core);
       }
     g_free(atom_name);
@@ -284,7 +337,9 @@ while (!fgetline(fp, line))
     res_name = fort_read_string(line, 18, 20);
     core->res_name = g_strdup(res_name);
     g_free(res_name);
-    
+  
+    core->chain = fort_read_string(line, 22, 22);
+
     core->res_no = fort_read_gint(line, 23, 26);
     
     core->x[0] = fort_read_gdouble(line, 31, 38);
@@ -302,6 +357,8 @@ while (!fgetline(fp, line))
       1 -  6         Record name      "ENDMDL"
     */
     return(0);
+  
+  g_free(line);
   }
   return(-1);
 }
@@ -332,6 +389,7 @@ g_return_val_if_fail(filename != NULL, 2);
 /* initialisers */
 data->periodic = 0;
 data->protein = TRUE;
+data->fractional = FALSE; 
 
 fp = fopen(filename, "rt");
 if (!fp)
@@ -353,7 +411,7 @@ data->basename = strdup_basename(filename);
 data->num_frames = data->cur_frame = frame;
 data->cur_frame--;
 
-prep_model(data);
+model_prep(data);
 return(0);
 }
 
