@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -29,14 +29,16 @@ The GNU GPL can also be found at http://www.gnu.org
 
 #include "gdis.h"
 #include "coords.h"
+#include "model.h"
 #include "edit.h"
 #include "file.h"
 #include "render.h"
 #include "matrix.h"
+#include "quaternion.h"
 #include "measure.h"
 #include "numeric.h"
 #include "opengl.h"
-#include "gtkshorts.h"
+#include "shortcuts.h"
 #include "interface.h"
 #include "dialog.h"
 
@@ -68,7 +70,7 @@ g_assert(model != NULL);
 
 for (list = model->frame_data_list ; list ; list=g_slist_next(list))
   {
-  frame = (struct frame_pak *) list->data;
+  frame = list->data;
 
   free_slist(frame->core_list);
   free_slist(frame->shell_list);
@@ -104,7 +106,7 @@ g_assert(model->animation == TRUE);
     {
 /* NB: frames start at 1 */
     read_raw_frame(fp, i+1, model);
-    make_latmat(model);
+    matrix_lattice_init(model);
 
     frame = g_malloc(sizeof(struct frame_pak *));
     memcpy(frame->latmat, model->latmat, 9*sizeof(gdouble));
@@ -149,33 +151,19 @@ return(0);
 /***********************************/
 /* retrieve the nth transformation */
 /***********************************/
-/* TODO - place elsewhere? */
 gint read_transform(gint n, struct model_pak *model)
 {
-struct transform_pak *transform;
+gpointer camera;
 
 g_assert(model != NULL);
+g_assert(model->camera != NULL);
 
-/* TODO - if n = 0 -> do a reset of model coords */
-if (!n)
-  return(0);
+camera = g_slist_nth_data(model->transform_list, n);
 
-transform = g_slist_nth_data(model->transform_list, n-1);
+g_assert(camera != NULL);
 
-g_assert(transform != NULL);
+model->camera = camera;
 
-switch (transform->id)
-  {
-  case ROTATION:
-    memcpy(model->rotmat, transform->matrix, 9*sizeof(gdouble));
-    model->offset[0] = transform->vector[0];
-    model->offset[1] = transform->vector[1];
-    model->scale = transform->scalar;
-    break;
-
-  default:
-    g_assert_not_reached();
-  }
 return(0);
 }
 
@@ -186,43 +174,46 @@ return(0);
 gint read_frame(FILE *fp, gint n, struct model_pak *model)
 {
 gint status;
-gdouble scale, vec[3], mat[9];
+gdouble rmax, v1[3], v2[3];
+gpointer camera=NULL;
 GString *err_text;
 
 g_assert(model != NULL);
 
+rmax = model->rmax;
+
 /* conventional or transformation style animation */
 if (model->transform_list)
   {
 /* NEW - process transformation list as an animation */
-status = read_transform(n, model);
+  status = read_transform(n, model);
   }
 else
   {
 g_assert(fp != NULL);
 
-/* preserve the initial orientation & translation */
-memcpy(mat, model->rotmat, 9*sizeof(gdouble));
-ARR3SET(vec, model->offset);
-scale = model->scale;
+ARR3SET(v1, model->centroid);
+ARR3SET(v2, model->offset);
 
 status = read_raw_frame(fp, n, model);
 if (status)
   {
   err_text = g_string_new("");
   g_string_printf(err_text, "Error reading frame: %d\n", n);
-  show_text(ERROR, err_text->str);
+  gui_text_show(ERROR, err_text->str);
   g_string_free(err_text, TRUE);
   return(1);
   }
 
-/* setup */
-prep_model(model);
+/* setup (have to save camera) */
+camera = camera_dup(model->camera);
+model_prep(model);
+model->camera = camera;
+g_free(model->camera_default);
+model->camera_default = camera;
 
-/* restore the initial orientation and offset */
-memcpy(model->rotmat, mat, 9*sizeof(gdouble));
-ARR3SET(model->offset, vec);
-model->scale = scale;
+ARR3SET(model->centroid, v1);
+ARR3SET(model->offset, v2);
   }
 
 /* apply desired constraint */
@@ -231,28 +222,18 @@ if (model->periodic && !model->anim_fix)
   switch (model->anim_confine)
     {
     case PBC_CONFINE_ATOMS:
-      pbc_constrain_atoms(model);
-      init_objs(REDO_COORDS, model);
-      calc_bonds(model);
-      calc_mols(model);
-      break;
+      coords_confine_cores(model->cores, model);
 
     case PBC_CONFINE_MOLS:
-      pbc_constrain_mols(model);
-      init_objs(REDO_COORDS, model);
+      coords_compute(model);
+      connect_bonds(model);
+      connect_molecules(model);
       break;
-
-    case UNFRAGMENT_MOLS:
-      unfragment(model);
-      init_objs(REDO_COORDS, model);
-      break;
-
-    default:
-      init_objs(REDO_COORDS, model);
     }
   }
-else
-  init_objs(REDO_COORDS, model);
+
+if (model->anim_noscale)
+  model->rmax = rmax;
 
 return(0);
 }
@@ -279,8 +260,8 @@ if (!model->transform_list)
 read_frame(fp, model->cur_frame, model);
 
 meas_graft_model(model);
-property_widget_redraw();
-symmetry_widget_redraw();
+
+gui_active_refresh();
 
 redraw_canvas(SINGLE);
 
@@ -295,15 +276,19 @@ if (!model->transform_list)
 gint anim_next_frame(struct model_pak *model)
 {
 gulong time;
-gchar *text;
+gchar *text, *name;
 
 g_assert(model != NULL);
 
 /* test if should we return to the start */
+/*
 model->cur_frame++;
+*/
+model->cur_frame += model->anim_step;
+
 if (model->cur_frame == model->num_frames)
   if (model->anim_loop)
-    model->cur_frame = 0;
+    model->cur_frame = 1;
 
 /* continue until we run out of frames (or a stop is flagged) */
 if (model->cur_frame < model->num_frames && model->animating)
@@ -316,14 +301,13 @@ time = mytimer();
 
 /* if a dialog exists - update via the current frame spinner */
   if (dialog_exists(ANIM, model))
-    gtksh_relation_update(model);
+    gui_relation_update(model);
   else
     {
 /* otherwise, update manually */
     read_frame(model->afp, model->cur_frame, model);
     meas_graft_model(model);
-    property_widget_redraw();
-    symmetry_widget_redraw();
+    gui_active_refresh();
     redraw_canvas(SINGLE);
     }
 
@@ -331,20 +315,20 @@ time = mytimer();
 time = mytimer() - time;
 model->redraw_cumulative += time;
 
-
-
 /* NEW - render to file */
   if (sysenv.render.animate)
     {
-    text = g_strdup_printf("%s_%06d.pov",
-           sysenv.render.animate_file, model->cur_frame);
-    write_povray(text, model);
+    text = g_strdup_printf("%s_%06d.pov", sysenv.render.animate_file, model->cur_frame);
+    name = g_build_filename(sysenv.cwd, text, NULL);
+
+    write_povray(name, model);
 
 /* NB: added this as jago keeps locking up on multi-frame renders */
-    if (!sysenv.render.no_exec_povray)
-      exec_pov(text);
+    if (!sysenv.render.no_povray_exec)
+      povray_exec(name);
 
     g_free(text);
+    g_free(name);
     }
 
   return(TRUE);
@@ -359,7 +343,7 @@ model->animating = FALSE;
 model->cur_frame--;
 
 /* create movie? */
-if (sysenv.render.animate && !sysenv.render.no_exec_povray)
+if (sysenv.render.animate && !sysenv.render.no_povray_exec)
   {
   text = NULL; 
   switch (sysenv.render.animate_type)
@@ -372,17 +356,18 @@ if (sysenv.render.animate && !sysenv.render.no_exec_povray)
       break;
 
     case ANIM_MPEG:
-      g_strdup_printf("%s -quality %d -delay %d %s_*.tga %s.mpg",
-                      sysenv.convert_path, (gint) sysenv.render.mpeg_quality,
-                      (gint) sysenv.render.delay,
-                      sysenv.render.animate_file, sysenv.render.animate_file);
+      text = g_strdup_printf("%s -quality %d -delay %d %s_*.tga %s.mpg",
+                             sysenv.convert_path, (gint) sysenv.render.mpeg_quality,
+                             (gint) sysenv.render.delay,
+                             sysenv.render.animate_file, sysenv.render.animate_file);
       break;
     }
+
   if (text)
     {
     system(text);
     g_free(text);
-    show_text(DEFAULT, "Completed movie creation.\n");
+    gui_text_show(DEFAULT, "Completed movie creation.\n");
     }
   }
 
@@ -401,20 +386,6 @@ if (sysenv.render.no_keep_tempfiles)
 return(FALSE);
 }
 
-/*
-gint anim_next_frame(struct model_pak *model)
-{
-if (model->cur_frame < model->num_frames-1 && model->animating)
-  {
-  model->cur_frame++;
-  gtksh_relation_update(model);
-  return(TRUE);
-  }
-
-return(FALSE);
-}
-*/
-
 /**********************************/
 /* set current frame to beginning */
 /**********************************/
@@ -425,9 +396,9 @@ struct model_pak *model;
 model = dialog_model(data);
 g_assert(model != NULL);
 
-model->cur_frame = 0;
+model->cur_frame = 1;
 
-gtksh_relation_update(model);
+gui_relation_update(model);
 }
 
 /****************************/
@@ -442,7 +413,7 @@ g_assert(model != NULL);
 
 model->cur_frame = model->num_frames-1;
 
-gtksh_relation_update(model);
+gui_relation_update(model);
 }
 
 /**********************/
@@ -455,10 +426,10 @@ struct model_pak *model;
 model = dialog_model(data);
 g_assert(model != NULL);
 
-if (model->cur_frame > 0)
+if (model->cur_frame > 1)
   {
   model->cur_frame--;
-  gtksh_relation_update(model);
+  gui_relation_update(model);
   }
 }
 
@@ -475,7 +446,7 @@ g_assert(model != NULL);
 if (model->cur_frame < model->num_frames-1)
   {
   model->cur_frame++;
-  gtksh_relation_update(model);
+  gui_relation_update(model);
   }
 }
 
@@ -502,7 +473,7 @@ if (!model->transform_list)
   model->afp = fopen(model->filename, "r");
   if (!model->afp)
     {
-    show_text(ERROR, "Failed to open animation stream.\n");
+    gui_text_show(ERROR, "Failed to open animation stream.\n");
     return;
     }
   }
@@ -545,10 +516,6 @@ void mol_pbc(struct model_pak *model)
 {
 model->anim_confine = PBC_CONFINE_MOLS;
 }
-void unfrag_pbc(struct model_pak *model)
-{
-model->anim_confine = UNFRAGMENT_MOLS;
-}
 void no_pbc(struct model_pak *model)
 {
 model->anim_confine = PBC_CONFINE_NONE;
@@ -566,9 +533,9 @@ model->animating = FALSE;
 /***********************/
 /* configure animation */
 /***********************/
-void animate_setup_widget(void)
+void gui_animate_dialog(void)
 {
-gchar *tmp;
+gchar *tmp, *title;
 gpointer dialog;
 GtkWidget *window, *frame, *vbox, *hbox, *hbox2, *anim_box;
 GtkWidget *notebook, *page, *button, *label, *entry, *scale;
@@ -588,10 +555,12 @@ anim_read_all_frames(data);
 */
 
 /* prevent recording whilst in animation */
-switch_mode(FREE);
+gui_mode_switch(FREE);
 
 /* dialog setup */
-dialog = dialog_request(ANIM, "Animation", NULL, animate_cleanup, data);
+title = g_strdup_printf("Animation: %s", data->basename);
+dialog = dialog_request(ANIM, title, NULL, animate_cleanup, data);
+g_free(title);
 if (!dialog)
   return;
 window = dialog_window(dialog);
@@ -630,13 +599,17 @@ label = gtk_label_new(tmp);
 gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
 /* desired delay */
-gtksh_direct_spin("Animation delay", &data->anim_speed,
+gui_direct_spin("Animation delay", &data->anim_speed,
                   1.0, 40.0, 1.0, NULL, NULL, vbox);
 
-gtksh_direct_check("Don't recalculate connectivity", &data->anim_fix, NULL, NULL, vbox);
-gtksh_direct_check("Loop", &data->anim_loop, NULL, NULL, vbox);
+gui_direct_spin("Animation step size ", &data->anim_step,
+                  1.0, data->num_frames, 1.0, NULL, NULL, vbox);
 
+gui_direct_check("Don't recalculate connectivity", &data->anim_fix, NULL, NULL, vbox);
 
+gui_direct_check("Don't recalculate scale", &data->anim_noscale, NULL, NULL, vbox);
+
+gui_direct_check("Loop", &data->anim_loop, NULL, NULL, vbox);
 
 
 /* page 2 */
@@ -663,18 +636,11 @@ button = add_radio_button("Confine mols to PBC", (gpointer) mol_pbc, data);
 if (data->anim_confine == PBC_CONFINE_MOLS)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
 
-button = add_radio_button("Confine unfragmented mols to PBC", 
-                                (gpointer) unfrag_pbc, data);
-if (data->anim_confine == UNFRAGMENT_MOLS)
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
-
 button = add_radio_button("Cell confinement off", (gpointer) no_pbc, data);
 if (data->anim_confine == PBC_CONFINE_NONE)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
 
 
-
-
 /* page 3 */
 page = gtk_vbox_new(FALSE, PANEL_SPACING);
 label = gtk_label_new("Rendering");
@@ -690,7 +656,7 @@ gtk_container_set_border_width(GTK_CONTAINER(vbox), PANEL_SPACING);
 
 
 anim_box = gtk_vbox_new(TRUE, 0);
-gtksh_direct_check("Create movie", &sysenv.render.animate,
+gui_direct_check("Create movie", &sysenv.render.animate,
                    anim_render, anim_box, vbox);
 gtk_box_pack_end(GTK_BOX(vbox), anim_box, TRUE, TRUE, 0);
 
@@ -718,10 +684,10 @@ gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, TRUE, 0);
 g_object_set_data(G_OBJECT(button), "id", (gpointer) ANIM_MPEG);
 
 /* movie creation parameters */
-gtksh_direct_spin("MPEG quality",
+gui_direct_spin("MPEG quality",
                   &sysenv.render.mpeg_quality, 1, 100, 1,
                   NULL, NULL, vbox);
-gtksh_direct_spin("Delay (ms)",
+gui_direct_spin("Delay (ms)",
                   &sysenv.render.delay, 0, 100, 5,
                   NULL, NULL, vbox);
 
@@ -741,10 +707,10 @@ g_object_set_data(G_OBJECT(entry), "id", (gpointer) ANIM_NAME);
 
 /* misc options */
 /*
-gtksh_direct_check("Create povray input files then stop",
-                   &sysenv.render.no_exec_povray,
+gui_direct_check("Create povray input files then stop",
+                   &sysenv.render.no_povray_exec,
                    NULL, NULL, vbox);
-gtksh_direct_check("Delete intermediate input/image files",
+gui_direct_check("Delete intermediate input/image files",
                    &sysenv.render.no_keep_tempfiles,
                    NULL, NULL, vbox);
 */
@@ -762,7 +728,7 @@ gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), frame, FALSE, FALSE, 0);
 vbox = gtk_vbox_new(FALSE, PANEL_SPACING);
 gtk_container_add(GTK_CONTAINER(frame), vbox);
 
-scale = gtksh_direct_hscale(0, data->num_frames-1, 1, &data->cur_frame, select_frame, data, vbox);
+scale = gui_direct_hscale(0, data->num_frames-1, 1, &data->cur_frame, select_frame, data, vbox);
 gtk_range_set_update_policy(GTK_RANGE(scale), GTK_UPDATE_DISCONTINUOUS);
 
 /* control buttons */
@@ -771,34 +737,36 @@ gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, PANEL_SPACING);
 hbox = gtk_hbox_new(TRUE, PANEL_SPACING);
 gtk_box_pack_start(GTK_BOX(hbox2), hbox, TRUE, FALSE, 0);
 
-gtksh_icon_button("GDIS_REWIND", NULL,
+gui_icon_button("GDIS_REWIND", NULL,
                   anim_rewind, dialog,
                   hbox);
 
-gtksh_icon_button("GDIS_STEP_BACKWARD", NULL,
+gui_icon_button("GDIS_STEP_BACKWARD", NULL,
                   anim_step_backward, dialog,
                   hbox);
 
-gtksh_icon_button("GDIS_PLAY", NULL,
+gui_icon_button("GDIS_PLAY", NULL,
                   anim_start, dialog,
                   hbox);
 
-gtksh_icon_button("GDIS_PAUSE", NULL,
+gui_icon_button("GDIS_PAUSE", NULL,
                    anim_stop, dialog,
                   hbox);
 
-gtksh_icon_button("GDIS_STEP_FORWARD", NULL,
+gui_icon_button("GDIS_STEP_FORWARD", NULL,
                   anim_step_forward, dialog,
                   hbox);
 
-gtksh_icon_button("GDIS_FASTFORWARD", NULL,
+gui_icon_button("GDIS_FASTFORWARD", NULL,
                   anim_fastforward, dialog,
                   hbox);
 
-gtksh_stock_button(GTK_STOCK_CLOSE, dialog_destroy, dialog,
+gui_stock_button(GTK_STOCK_CLOSE, dialog_destroy, dialog,
                    GTK_DIALOG(window)->action_area);
 
 /* display the dialog */
 gtk_widget_show_all(window);
+
+gui_relation_update(data);
 }
 
