@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -38,13 +38,15 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "file.h"
 #include "matrix.h"
 #include "parse.h"
-#include "gtkshorts.h"
+#include "shortcuts.h"
 #include "interface.h"
 #include "dialog.h"
 
 /* top level data structure */
 extern struct sysenv_pak sysenv;
 
+extern GtkWidget *window;
+
 GSList *dir_list=NULL;
 GtkListStore *file_path_ts=NULL, *file_name_ts=NULL;
 GtkWidget *file_path_tv=NULL, *file_name_tv=NULL;
@@ -59,6 +61,7 @@ struct dialog_pak
 gint type;
 gpointer model;
 gpointer data;
+GHashTable *children;
 void (*refresh)(gpointer);
 void (*cleanup)(gpointer);
 GtkWidget *window;
@@ -67,9 +70,10 @@ GtkWidget *window;
 /************************************/
 /* callback for destroying a dialog */
 /************************************/
-void destroy_widget(GtkWidget *w, GtkWidget *target)
+void destroy_widget(GtkWidget *w, gpointer target)
 {
-gtk_widget_destroy(target);
+if (GTK_IS_WIDGET(target))
+  gtk_widget_destroy(target);
 }
 
 /*******************************/
@@ -89,6 +93,51 @@ struct dialog_pak *dialog = data;
 return(dialog->model);
 }
 
+/************************************/
+/* dialog data extraction primitive */
+/************************************/
+gpointer dialog_child_get(gpointer data, const gchar *key)
+{
+struct dialog_pak *dialog = data;
+gpointer child = NULL;
+
+if (dialog->children)
+  child = g_hash_table_lookup(dialog->children, key);
+
+return(child);
+}
+
+/**********************************/
+/* dialog data addition primitive */
+/**********************************/
+/* used for associating a label with a child widget pointer within a dialog */
+void dialog_child_set(gpointer data, const gchar *key, gpointer value)
+{
+struct dialog_pak *dialog = data;
+
+if (!dialog->children)
+  dialog->children = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
+
+/* use replace, so old g_strdup() key gets freed */
+g_hash_table_replace(dialog->children, g_strdup(key), value);
+}
+
+/******************/
+/* debugging dump */
+/******************/
+void dialog_dump(void)
+{
+gint i=0;
+struct dialog_pak *dialog;
+GSList *list;
+
+for (list=sysenv.dialog_list ; list ; list=g_slist_next(list))
+  {
+  dialog = list->data;
+  printf("[%d] model: %p, type: %d\n", i++, dialog->model, dialog->type);
+  }
+}
+
 /********************************************/
 /* cleanup when a dialog has been destroyed */
 /********************************************/
@@ -104,6 +153,9 @@ printf("freeing dialog : %p...\n", data);
 if (dialog->cleanup)
   dialog->cleanup(dialog->model);
 
+if (dialog->children)
+  g_hash_table_destroy(dialog->children); 
+
 sysenv.dialog_list = g_slist_remove(sysenv.dialog_list, dialog);
 g_free(dialog);
 }
@@ -187,12 +239,36 @@ for (list=sysenv.dialog_list ; list ; list=g_slist_next(list))
   {
   dialog = list->data;
 
-  if (dialog->model == model && dialog->type == type)
-    return(TRUE);
+  if (model)
+    {
+    if (dialog->model == model && dialog->type == type)
+      return(TRUE);
+    }
+  else
+    {
+    if (dialog->type == type)
+      return(TRUE);
+    }
   }
 return(FALSE);
 }
 
+/******************************/
+/* update all current dialogs */ 
+/******************************/
+void dialog_refresh_all(void)
+{
+GSList *list;
+struct dialog_pak *dialog;
+
+for (list=sysenv.dialog_list ; list ; list=g_slist_next(list))
+  {
+  dialog = list->data;
+  if (dialog->refresh)
+    dialog->refresh(NULL);
+  }
+}
+
 /***********************************/
 /* update dialog of specified type */
 /***********************************/
@@ -206,7 +282,8 @@ for (list=sysenv.dialog_list ; list ; list=g_slist_next(list))
   dialog = list->data;
 
   if (dialog->type == type)
-    dialog->refresh(NULL);
+    if (dialog->refresh)
+      dialog->refresh(NULL);
   }
 }
 
@@ -223,7 +300,8 @@ for (list=sysenv.dialog_list ; list ; list=g_slist_next(list))
   dialog = list->data;
 
   if (dialog->model == model)
-    dialog->refresh(NULL);
+    if (dialog->refresh)
+      dialog->refresh(NULL);
   }
 }
 
@@ -249,13 +327,16 @@ for (list=sysenv.dialog_list ; list ; list=g_slist_next(list))
   {
   dialog = list->data;
 
-/* one instanace only dialogs (eg because of globals) */
+/* allow only one instance of these dialogs (eg because of globals) */
   switch (type)
     {
+    case CREATOR:
+    case DOCKING:
     case GENSURF:
+    case SURF:
       if (type == dialog->type)
         {
-        show_text(WARNING, "Sorry, only one instance of this dialog is allowed.\n");
+        gdk_window_show((dialog->window)->window);
         return(NULL);
         }
     }
@@ -269,7 +350,7 @@ printf(" - Exists...\n");
 /* dialog already open - raise it to the fore & select assoc. model */
     gdk_window_show((dialog->window)->window);
     if (model)
-      pick_model(model);
+      gui_model_select(model);
     return(NULL);
     }
   }
@@ -283,6 +364,7 @@ sysenv.dialog_list = g_slist_prepend(sysenv.dialog_list, dialog);
 dialog->type = type;
 dialog->model = model;
 dialog->data = NULL;
+dialog->children = NULL;
 dialog->refresh = refresh;
 dialog->cleanup = cleanup;
 
@@ -344,7 +426,7 @@ free_slist(dir_list);
 dir_list = get_dir_list(sysenv.cwd, TRUE);
 if (!dir_list)
   {
-  show_text(ERROR, "No directory listing; check your permissions.\n");
+  gui_text_show(ERROR, "No directory listing; check your permissions.\n");
   gtk_list_store_append(file_path_ts, &iter);
   gtk_list_store_set(file_path_ts, &iter, FILE_PATH, "..", -1);
   }
@@ -376,7 +458,8 @@ printf("[%s] : %d\n",full,buff.st_mode);
       gtk_list_store_set(file_name_ts, &iter, FILE_NAME, name, -1);
       }
 
-#if OLD_GOK
+#if OLD_GOK /* FIXME: can it be deleted? I haven't changed this code 
+               below to handle empty extensions */
     file_data = get_file_info((gpointer *) list->data, BY_EXTENSION);
     if (file_data)
       {
@@ -645,7 +728,6 @@ if (!dialog)
   return;
 window = dialog_window(dialog);
 
-
 /* make and set up the dialog window */
 gtk_window_set_title(GTK_WINDOW(window), title);
 gtk_window_set_default_size(GTK_WINDOW(window), 600, 400);
@@ -676,14 +758,6 @@ if (secondary_handler == (gpointer) file_save_as)
     }
   }
 
-/* option to auto unfragment */
-if (secondary_handler == (gpointer) file_load)
-  {
-  hbox = gtk_hbox_new(FALSE,0);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), hbox,FALSE,FALSE,0);
-  gtksh_direct_check("Auto unfragment", &sysenv.unfragment, NULL, NULL, hbox);
-  }
-
 /* SECOND ROW - sub directory & file listings */
 hbox = gtk_hbox_new(FALSE, PANEL_SPACING);
 gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), hbox, TRUE, TRUE, 0);
@@ -753,9 +827,9 @@ elist = NULL;
 flist = sysenv.file_list;
 while(flist != NULL)
   {
-  fdata = (struct file_pak *) flist->data;
+  fdata = flist->data;
 /* include in menu? */
-  if (fdata->menu && fdata->group != BABEL)
+  if (fdata->menu)
     elist = g_list_append(elist, fdata->label);
   flist = g_slist_next(flist);
   }
@@ -779,17 +853,17 @@ gtk_widget_set_size_request(combo, 6*sysenv.gtk_fontsize, -1);
 switch (type)
   {
   case FILE_LOAD:
-    gtksh_stock_button(GTK_STOCK_OPEN, file_event_handler, secondary_handler, 
+    gui_stock_button(GTK_STOCK_OPEN, file_event_handler, secondary_handler, 
                        GTK_DIALOG(window)->action_area);
     break;
 
   case FILE_SAVE:
-    gtksh_stock_button(GTK_STOCK_SAVE, file_event_handler, secondary_handler, 
+    gui_stock_button(GTK_STOCK_SAVE, file_event_handler, secondary_handler, 
                        GTK_DIALOG(window)->action_area);
     break;
   }
 
-gtksh_stock_button(GTK_STOCK_CANCEL, dialog_destroy, dialog,
+gui_stock_button(GTK_STOCK_CANCEL, dialog_destroy, dialog,
                    GTK_DIALOG(window)->action_area);
 
 /* all done */
@@ -803,79 +877,59 @@ update_file_pane();
 /*************************************************/
 void set_colour_widget(GtkWidget *cs, GtkWidget *w)
 {
-GdkColor col;
-GtkStyle *style;
+GdkColor colour;
 
-gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(cs), &col);
-
-style = gtk_style_copy(gtk_widget_get_style(w));
-style->bg[0].red   = col.red;
-style->bg[0].green = col.green;
-style->bg[0].blue  = col.blue;
-gtk_widget_set_style(GTK_WIDGET(w), style);
+if (GTK_IS_WIDGET(cs))
+  gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(cs), &colour);
+if (GTK_IS_WIDGET(w))
+  gtk_widget_modify_bg(w, GTK_STATE_NORMAL, &colour);
 }
 
 /***************************************/
 /* callback to update the colour value */
 /***************************************/
-void set_colour_value(GtkWidget *cs, gdouble *colour)
+void set_colour_value(GtkWidget *cs, gdouble *x)
 {
-GdkColor col;
+GdkColor colour;
 
-gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(cs), &col);
+gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(cs), &colour);
 
-colour[0] = col.red;
-colour[1] = col.green;
-colour[2] = col.blue;
-VEC3MUL(colour, 1.0/65535.0);
+x[0] = colour.red;
+x[1] = colour.green;
+x[2] = colour.blue;
+VEC3MUL(x, 1.0/65535.0);
 }
 
-/*******************************************************************/
-/* create a colour selection dialog that automatically sets colour */
-/*******************************************************************/
-void colour_dialog(gchar *title, GtkWidget *w, gpointer colour,
-                                   gpointer func, gpointer arg)
+/**********************************/
+/* create a colour editing dialog */
+/**********************************/
+void dialog_colour_new(GtkWidget *w, gdouble *colour)
 {
 GtkWidget *csd;
 
 /* create colour selection dialog */
-csd = gtk_color_selection_dialog_new(title);
+csd = gtk_color_selection_dialog_new("Edit colour");
 
 /* set the initial colour */
-gtk_color_selection_set_color
-  (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(csd)->colorsel), colour);
+gtk_color_selection_set_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(csd)->colorsel), colour);
 
 /* value colour update */
-g_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(csd)->colorsel),
-                 "color_changed",
-                  GTK_SIGNAL_FUNC(set_colour_value), (gpointer) colour);
-
+g_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(csd)->colorsel), "color_changed",
+                 GTK_SIGNAL_FUNC(set_colour_value), (gpointer) colour);
 /* widget colour update */
 if (w)
-  g_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(csd)->colorsel),
-                   "color_changed",
-                    GTK_SIGNAL_FUNC(set_colour_widget), (gpointer) w);
+  g_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(csd)->colorsel), "color_changed",
+                   GTK_SIGNAL_FUNC(set_colour_widget), (gpointer) w);
 
-/* call user's function */
-if (func)
-  g_signal_connect_after(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(csd)->colorsel),
-                        "color_changed",
-                         GTK_SIGNAL_FUNC(func), (gpointer) arg);
-
-/* default buttons */
 g_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(csd)->ok_button), "clicked",
                  GTK_SIGNAL_FUNC(destroy_widget), (gpointer) csd);
 
 gtk_widget_hide(GTK_COLOR_SELECTION_DIALOG(csd)->cancel_button);
 
 gtk_widget_show(csd);
-}
 
-/********************************************/
-/* simple callback suited the colour dialog */
-/********************************************/
-void modify_colour_dialog(GtkWidget *w, gdouble *colour)
-{
-colour_dialog("Edit Colour", w, colour, NULL, NULL);
+/*
+g_signal_connect(GTK_OBJECT(w), "destroy", GTK_SIGNAL_FUNC(destroy_widget), csd);
+*/
 }
 
