@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -40,7 +40,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "numeric.h"
 #include "interface.h"
 #include "dialog.h"
-#include "gtkshorts.h"
+#include "shortcuts.h"
 #include "opengl.h"
 
 /* main pak structures */
@@ -76,7 +76,7 @@ dhkl = 999999999.9;
 list1=list;
 while (list1)
   {
-  plane = (struct plane_pak *) list1->data;
+  plane = list1->data;
   list1 = g_slist_next(list1);
 
 /* determine if this is a repeated plane */
@@ -194,7 +194,7 @@ GSList *list;
 list = g_hash_table_lookup(sysenv.sfc_table, elements[core->atom_code].symbol);
 if (!list)
   {
-  printf("No sfc found for [%s], fudging...\n", core->label);
+  printf("No sfc found for [%s], fudging...\n", core->atom_label);
   return((gdouble) core->atom_code);
   }
 
@@ -225,7 +225,7 @@ switch (type)
     break;
 
   default:
-    printf("Unsuported radiation type, fudging...\n");
+    printf("Unsupported radiation type, fudging...\n");
     sfc = core->atom_code;
     break;
   }
@@ -244,7 +244,7 @@ return(sfc);
 void diff_calc_spectrum(GSList *list, struct model_pak *model)
 {
 gint i, n;
-gint cur_peak, old_peak, pop_flag;
+gint cur_peak, old_peak;
 gdouble angle, sol, f, g, lpf, dr;
 gdouble c1, sqrt_c1, sqrt_pi, sina, cosa, sin2a, cos2a, tana, fwhm, bf, intensity;
 gdouble *spectrum;
@@ -274,7 +274,7 @@ printf("----------------------------------------------------------------------\n
 old_peak = -1;
 for (item=list ; item ; item=g_slist_next(item))
   {
-  plane = (struct plane_pak *) item->data;
+  plane = item->data;
 
 /* structure factor squared */
   f = plane->f[0]*plane->f[0] + plane->f[1]*plane->f[1];
@@ -301,11 +301,11 @@ for (item=list ; item ; item=g_slist_next(item))
 /* determine if the current peak lies on the same bin as the previous peak - this */
 /* is the peak overlap problem (ie adding intensities over-emphasizes such peaks) */
   dr = (R2D*angle - model->diffract.theta[0]) / model->diffract.theta[2];
+/*
   cur_peak = (gint) dr;
-  if (cur_peak == old_peak)
-    pop_flag = TRUE;
-  else
-    pop_flag = FALSE;
+*/
+  cur_peak = nearest_int(dr);
+
   old_peak = cur_peak;
 
 /* fill out the spectrum with the current peak */
@@ -319,11 +319,12 @@ for (item=list ; item ; item=g_slist_next(item))
 */
 
 /* quantized distance to peak */
-dr = (gdouble ) (i - cur_peak);
-dr *= model->diffract.theta[2];
+    dr = (gdouble) (i - cur_peak);
+    dr *= model->diffract.theta[2];
 
 /* compute broadening factor - peak contribution at current spectrum interval */
-    bf = 1.0;
+    bf = 1.0 / model->diffract.theta[2];
+
     if (fabs(fwhm) > 0.00001)
       {
       switch (model->diffract.broadening)
@@ -355,17 +356,9 @@ dr *= model->diffract.theta[2];
 
 /* compute intensity for the current bin */
     intensity = plane->multiplicity*lpf*f*bf;
-
-/* if we have the peak overlap problem - replace intensity (if greater) */
-/* otherwise - normal intensity summation */
-    if (pop_flag)
-      {
-      if (intensity > spectrum[i])
-        spectrum[i] = intensity;
-      }
-    else
-      spectrum[i] += intensity;
+    spectrum[i] += intensity;
     }
+
 #if DEBUG_CALC_SPECTRUM
   printf("[%2d %2d %2d] : %2d : %6.4f : %6.3f : %6.3f : %11.4f : %11.4f\n",
           plane->index[0], plane->index[1], plane->index[2], plane->multiplicity,
@@ -420,7 +413,7 @@ else
 fp = fopen(gtk_entry_get_text(GTK_ENTRY(diff_filename)), "at");
 if (!fp)
   {
-  show_text(ERROR, "Failed to open file for raw spectrum data.\n");
+  gui_text_show(ERROR, "Failed to open file for raw spectrum data.\n");
   return;
   }
 for (i=0 ; i<n ; i++)
@@ -451,14 +444,14 @@ if (!model)
   return(1);
 if (model->periodic != 3)
   {
-  show_text(WARNING, "Your model is not 3D periodic.\n");
+  gui_text_show(WARNING, "Your model is not 3D periodic.\n");
   return(1);
   }
 
 /* checks */
 if (model->diffract.theta[0] >= model->diffract.theta[1])
   {
-  show_text(ERROR, "Invalid 2theta range.\n");
+  gui_text_show(ERROR, "Invalid 2theta range.\n");
   return(2);
   }
 
@@ -495,12 +488,12 @@ if (model->diffract.wavelength < 0.5)
   if (model->diffract.theta[1] > tmp)
     {
     model->diffract.theta[1] = tmp;
-    show_text(WARNING, "Your maximum theta value is too large.\n");
+    gui_text_show(WARNING, "Your maximum theta value is too large.\n");
     }
   if (model->diffract.theta[0] > tmp)
     {
     model->diffract.theta[1] = tmp;
-    show_text(WARNING, "Your minimum theta value is too large.\n");
+    gui_text_show(WARNING, "Your minimum theta value is too large.\n");
     }
   }
 
@@ -518,7 +511,7 @@ if (diff_all_frames)
   {
   if (model->animating)
     {
-    show_text(ERROR, "Can't do multiple animations.\n");
+    gui_text_show(ERROR, "Can't do multiple animations.\n");
     return(1);
     }
   if (!model->transform_list)
@@ -526,7 +519,7 @@ if (diff_all_frames)
     model->afp = fopen(model->filename, "r");
     if (!model->afp)
       {
-      show_text(ERROR, "Failed to open animation stream.\n");
+      gui_text_show(ERROR, "Failed to open animation stream.\n");
       return(1);
       }
     }
@@ -543,7 +536,7 @@ for (;;)
 
   for (item=list ; item ; item=g_slist_next(item))
     {
-    plane = (struct plane_pak *) item->data;
+    plane = item->data;
 
 /* sin(angle) / lambda */
     sol = 0.5/plane->dhkl;
@@ -558,7 +551,7 @@ for (;;)
 
     for (clist=model->cores ; clist ; clist=g_slist_next(clist))
       {
-      core = (struct core_pak *) clist->data;
+      core = clist->data;
 
 /* hkl * reciprocal lattice */
       ARR3SET(rdv, plane->index);
@@ -656,7 +649,7 @@ plane_min = NULL;
 /* Dhkl difference based search */
   for (item=list ; item ; item=g_slist_next(item))
     {
-    plane = (struct plane_pak *) item->data;
+    plane = item->data;
 
     d = fabs(plane->dhkl - dhkl);
     if (d < dmin)
@@ -699,7 +692,7 @@ else
 /****************************/
 /* diffraction setup dialog */
 /****************************/
-void diffract_dialog(void)
+void gui_diffract_dialog(void)
 {
 gpointer dialog;
 GtkWidget *window, *frame, *hbox, *hbox2, *vbox, *label;
@@ -711,12 +704,12 @@ if (!data)
   return;
 if (data->periodic != 3)
   {
-  show_text(ERROR, "Your model is not 3D periodic.\n");
+  gui_text_show(ERROR, "Your model is not 3D periodic.\n");
   return;
   }
 
 /* request a dialog */
-dialog = dialog_request(DIFFAX, "Power Diffraction", NULL, NULL, data);
+dialog = dialog_request(DIFFAX, "Powder Diffraction", NULL, NULL, data);
 if (!dialog)
   return;
 window = dialog_window(dialog);
@@ -776,7 +769,7 @@ gtk_box_pack_end(GTK_BOX(hbox), diff_broadening, FALSE, FALSE, PANEL_SPACING);
 
 hbox = gtk_hbox_new(FALSE, 0);
 gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, PANEL_SPACING);
-gtksh_auto_spin("Mixing parameter ", &data->diffract.asym, 0.0, 1.0, 0.01, NULL, NULL, hbox);
+gui_auto_spin("Mixing parameter ", &data->diffract.asym, 0.0, 1.0, 0.01, NULL, NULL, hbox);
 
 /* split pane */
 hbox2 = gtk_hbox_new(TRUE, 0);
@@ -791,15 +784,15 @@ gtk_container_add(GTK_CONTAINER(frame), vbox);
 
 hbox = gtk_hbox_new(FALSE, 0);
 gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, PANEL_SPACING);
-gtksh_auto_spin(" 2Theta min", &data->diffract.theta[0], 0.0, 160.0, 5.0, NULL, NULL, hbox);
+gui_auto_spin(" 2Theta min", &data->diffract.theta[0], 0.0, 160.0, 0.1, NULL, NULL, hbox);
 
 hbox = gtk_hbox_new(FALSE, 0);
 gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, PANEL_SPACING);
-gtksh_auto_spin(" 2Theta max", &data->diffract.theta[1], 0.0, 170.0, 5.0, NULL, NULL, hbox);
+gui_auto_spin(" 2Theta max", &data->diffract.theta[1], 0.0, 170.0, 0.1, NULL, NULL, hbox);
 
 hbox = gtk_hbox_new(FALSE, 0);
 gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, PANEL_SPACING);
-gtksh_auto_spin(" 2Theta step", &data->diffract.theta[2], 0.01, 1.0, 0.01, NULL, NULL, hbox);
+gui_auto_spin(" 2Theta step", &data->diffract.theta[2], 0.01, 1.0, 0.01, NULL, NULL, hbox);
 
 
 /* U V W broadening parameters */
@@ -811,15 +804,15 @@ gtk_container_add(GTK_CONTAINER(frame), vbox);
 
 hbox = gtk_hbox_new(FALSE, 0);
 gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, PANEL_SPACING);
-gtksh_auto_spin(" U ", &data->diffract.u, -9.9, 9.9, 0.05, NULL, NULL, hbox);
+gui_auto_spin(" U ", &data->diffract.u, -9.9, 9.9, 0.05, NULL, NULL, hbox);
 
 hbox = gtk_hbox_new(FALSE, 0);
 gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, PANEL_SPACING);
-gtksh_auto_spin(" V ", &data->diffract.v, -9.9, 9.9, 0.05, NULL, NULL, hbox);
+gui_auto_spin(" V ", &data->diffract.v, -9.9, 9.9, 0.05, NULL, NULL, hbox);
 
 hbox = gtk_hbox_new(FALSE, 0);
 gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, PANEL_SPACING);
-gtksh_auto_spin(" W ", &data->diffract.w, -9.9, 9.9, 0.05, NULL, NULL, hbox);
+gui_auto_spin(" W ", &data->diffract.w, -9.9, 9.9, 0.05, NULL, NULL, hbox);
 
 
 /* output stuff */
@@ -851,16 +844,16 @@ else
 */
 
 if (data->animation)
-  gtksh_direct_check("Calculate for all frames (output file only)", &diff_all_frames, NULL, NULL, vbox);
+  gui_direct_check("Calculate for all frames (output file only)", &diff_all_frames, NULL, NULL, vbox);
   else
     diff_all_frames = FALSE;
 
 /* terminating buttons */
-gtksh_stock_button(GTK_STOCK_EXECUTE,
+gui_stock_button(GTK_STOCK_EXECUTE,
                    diff_calc, NULL, 
                    GTK_DIALOG(window)->action_area);
 
-gtksh_stock_button(GTK_STOCK_CLOSE,
+gui_stock_button(GTK_STOCK_CLOSE,
                    dialog_destroy, dialog,
                    GTK_DIALOG(window)->action_area);
 
