@@ -1,8 +1,8 @@
 /*
 Copyright (C) 2003 by Andrew Lloyd Rohl and Sean David Fleming
 
-andrew@power.curtin.edu.au
-sean@power.curtin.edu.au
+andrew@ivec.org
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -24,20 +24,47 @@ The GNU GPL can also be found at http://www.gnu.org
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <time.h>
 #include <string.h>
+#include <time.h>
+#include <unistd.h>
 
 #include "gdis.h"
+#include "gamess.h"
 #include "coords.h"
 #include "file.h"
 #include "task.h"
 #include "matrix.h"
+#include "parse.h"
 #include "spatial.h"
-#include "gtkshorts.h"
+#include "shortcuts.h"
+#include "dialog.h"
 #include "interface.h"
 #include "opengl.h"
 
-extern struct sysenv_pak sysenv;
+struct basis_pak basis_sets[] = {
+{"User Defined", GMS_USER, 0},
+{"MNDO", GMS_MNDO, 0},
+{"AM1", GMS_AM1, 0},
+{"PM3", GMS_PM3, 0},
+{"MINI", GMS_MINI, 0},
+{"MIDI", GMS_MIDI, 0},
+{"STO-2G", GMS_STO, 2},
+{"STO-3G", GMS_STO, 3},
+{"STO-4G", GMS_STO, 4},
+{"STO-5G", GMS_STO, 5},
+{"STO-6G", GMS_STO, 6},
+{"3-21G", GMS_N21, 3},
+{"6-21G", GMS_N21, 6},
+{"4-31G", GMS_N31, 4},
+{"5-31G", GMS_N31, 5},
+{"6-31G", GMS_N31, 6},
+{"6-311G", GMS_N311, 6},
+{"DZV", GMS_DZV, 0},
+{"DH", GMS_DH, 0},
+{"TZV", GMS_TZV, 0},
+{"MC", GMS_MC, 0},
+{NULL, 0, 0}
+};
 
 #define CONTROL_TAB_LABEL " Control "
 #define BASIS_TAB_LABEL " Basis Set "
@@ -78,37 +105,7 @@ enum {
   HEAVY_DIFFUSE, HYDROGEN_DIFFUSE, NSTEP, GAMESS_TITLE, GAMESS_ENERGY, INPUTFILE, MODELNAME
 };
 
-struct basis_pak
-{
-gchar *label;
-GMSBasisType basis;
-gint ngauss;
-};
-
-struct basis_pak basis_sets[] = {
-{"User Defined", GMS_USER, 0},
-{"MNDO", GMS_MNDO, 0},
-{"AM1", GMS_AM1, 0},
-{"PM3", GMS_PM3, 0},
-{"MINI", GMS_MINI, 0},
-{"MIDI", GMS_MIDI, 0},
-{"STO-2G", GMS_STO, 2},
-{"STO-3G", GMS_STO, 3},
-{"STO-4G", GMS_STO, 4},
-{"STO-5G", GMS_STO, 5},
-{"STO-6G", GMS_STO, 6},
-{"3-21G", GMS_N21, 3},
-{"6-21G", GMS_N21, 6},
-{"4-31G", GMS_N31, 4},
-{"5-31G", GMS_N31, 5},
-{"6-31G", GMS_N31, 6},
-{"6-311G", GMS_N311, 6},
-{"DZV", GMS_DZV, 0},
-{"DH", GMS_DH, 0},
-{"TZV", GMS_TZV, 0},
-{"MC", GMS_MC, 0},
-{NULL, 0, 0}
-};
+extern struct sysenv_pak sysenv;
 
 /*********************/
 /* run a gamess file */
@@ -116,21 +113,75 @@ struct basis_pak basis_sets[] = {
 #define DEBUG_EXEC_GAMESS 0
 gint exec_gamess(gchar *input)
 {
-gchar *cmd;
+gchar *cmd, hostname[256];
 
 /* checks */
 if (!sysenv.gamess_path)
   return(-1);
 
-cmd = g_strdup_printf("%s %s", sysenv.gamess_path, input);
+/* NEW - acquire hostname for GAMESS job submission */
+/*
+if (gethostname(hostname, sizeof(hostname)))
+*/
+  sprintf(hostname, "localhost");
+
+#if __WIN32
+{
+gchar *basename, *bat, *fubar, *fname, *temp, *tmp, *dir, *inp, *out;
+FILE *fp;
+
+/* setup variables */
+basename = strdup_basename(input);
+fname = g_strdup_printf("%s.F05", basename);
+fubar = g_build_filename(sysenv.gamess_path, "scratch", fname, NULL);
+temp = g_build_filename(sysenv.gamess_path, "temp", basename, NULL);
+dir = g_build_filename(sysenv.gamess_path, NULL);
+inp = g_build_filename(sysenv.cwd, input, NULL);
+out = g_build_filename(sysenv.cwd, basename, NULL);
+
+/* create a batch file to run GAMESS */
+bat = gun("bat");
+fp = fopen(bat, "wt");
+fprintf(fp, "@echo off\n");
+fprintf(fp, "echo Running GAMESS using 1 CPU, job: %s\n", input);
+
+/* remove old crap in the temp directory */
+fprintf(fp, "del %s.*\n", temp);
+
+/* put the input file in the scratch directory */
+fprintf(fp, "copy \"%s\" \"%s\"\n", inp, fubar);
+
+/* run the execution script */
+fprintf(fp, "cd %s\n", sysenv.gamess_path);
+fprintf(fp, "csh -f runscript.csh %s 04 1 %s %s > \"%s.gmot\"\n",
+             basename, dir, hostname, out);
+fprintf(fp, "echo GAMESS job completed\n");
+fclose(fp);
+
+tmp = g_build_filename(sysenv.cwd, bat, NULL);
+cmd = g_strdup_printf("\"%s\"", tmp);
+
+g_free(basename);
+g_free(fname);
+g_free(fubar);
+g_free(temp);
+g_free(tmp);
+g_free(inp);
+g_free(out);
+g_free(bat);
+}
+#else
+cmd = g_strdup_printf("%s/%s %s", sysenv.gamess_path, sysenv.gamess_exe, input); 
+#endif
 
 #if DEBUG_EXEC_GAMESS
 printf("executing: [%s]\n",cmd);
+#else
+task_sync(cmd);
 #endif
 
-task_sync(cmd);
-
 g_free(cmd);
+
 return(0);
 }
 
@@ -140,9 +191,23 @@ return(0);
 #define DEBUG_EXEC_GAMESS_TASK 0
 void exec_gamess_task(struct model_pak *model)
 {
+gchar *out;
+
 /* construct output filename */
 g_free(model->gamess.out_file);
-model->gamess.out_file = g_strdup_printf("%s.gmot", model->basename);
+
+out = parse_extension_set(model->gamess.temp_file, "gmot");
+model->gamess.out_file = g_build_filename(sysenv.cwd, out, NULL);
+g_free(out);
+
+#if __WIN32
+/*
+out = g_strdup_printf("\"%s\"", model->gamess.out_file);
+*/
+out = g_shell_quote(model->gamess.out_file);
+g_free(model->gamess.out_file);
+model->gamess.out_file = out;
+#endif
 
 /* save input file and execute */
 file_save_as(model->gamess.temp_file, model);
@@ -153,19 +218,29 @@ exec_gamess(model->gamess.temp_file);
 /* process a gamess run (task) */
 /*****************************/
 #define DEBUG_PROC_GAMESS 0
-void proc_gamess_task(gpointer *ptr)
+void proc_gamess_task(gpointer ptr)
 {
+gchar *filename;
 GString *line;
 struct model_pak *data;
 
 /* TODO - model locking (moldel_ptr RO/RW etc) to prevent screw ups */
 g_return_if_fail(ptr != NULL);
-data = (struct model_pak *) ptr;
+data = ptr;
+
+/* win32 fix - which can't make up its mind if it wants to quote or not */
+filename = g_shell_unquote(data->gamess.out_file, NULL);
+
+if (!g_file_test(filename, G_FILE_TEST_EXISTS))
+  {
+  printf("Missing output file [%s]\n", filename);
+  return;
+  }
 
 if (data->gamess.run_type < GMS_OPTIMIZE && !data->animation)
   {
 /* same model (ie with current energetics dialog) so update */
-    file_load(data->gamess.out_file, data);
+    file_load(filename, data);
 /* update energy (TODO - only if successful) */
     line = g_string_new(NULL);
     if (data->gamess.have_energy)
@@ -183,25 +258,25 @@ else
 /* so that we can check if a dialog exsits to be updated */
 /* get new coords */
 /* create new model for the minimized result */
-  file_load(data->gamess.out_file, NULL);
+  file_load(filename, NULL);
   }
 
-redraw_canvas(ALL);
+g_free(filename);
 
-return;
+redraw_canvas(ALL);
 }
 
 /*************************************************/
 /* controls the use of GAMESS to optimise coords */
 /*************************************************/
 #define DEBUG_RUN_GAMESS 0
-void run_gamess(GtkWidget *w, struct model_pak *data)
+void gui_gamess_task(GtkWidget *w, struct model_pak *data)
 {
 /* checks */
 g_return_if_fail(data != NULL);
 if (!sysenv.gamess_path)
   {
-  show_text(ERROR, "GAMESS executable was not found.\n");
+  gui_text_show(ERROR, "GAMESS executable was not found.\n");
   return;
   }
 
@@ -313,10 +388,35 @@ while (basis_sets[i].label)
   }
 }
 
+/******************************/
+/* functional selection event */
+/******************************/
+void gms_functional_select(GtkWidget *w, struct model_pak *model)
+{
+const gchar *label;
+
+label = gtk_entry_get_text(GTK_ENTRY(w));
+
+if (g_strncasecmp(label, "svwn", 4) == 0)
+  {
+  model->gamess.dft_functional = SVWN;
+  return;
+  }
+if (g_strncasecmp(label, "blyp", 4) == 0)
+  {
+  model->gamess.dft_functional = BLYP;
+  return;
+  }
+if (g_strncasecmp(label, "b3ly", 4) == 0)
+  {
+  model->gamess.dft_functional = B3LYP;
+  return;
+  }
+}
+
 /***************************/
 /* text handling callbacks */
 /***************************/
-
 gint event_pol_text_fields(GtkWidget *w, gpointer *obj)
 {
 gint id;
@@ -349,12 +449,12 @@ return(FALSE);
 /*********************************/
 /* GAMESS job setup/results page */
 /*********************************/
-void gamess_widget(GtkWidget *box, struct model_pak *data)
+void gui_gamess_widget(GtkWidget *box, struct model_pak *data)
 {
 gint i, j;
 GString *line;
 GList *list;
-GtkWidget *hbox, *vbox, *vbox1, *vbox2, *vbox3, *page;
+GtkWidget *hbox, *vbox, *vbox1, *vbox2, *vbox3, *vbox4, *page;
 GtkWidget *frame, *button, *label, *entry, *notebook;
 GtkWidget *combo;
 
@@ -365,13 +465,6 @@ g_assert(data != NULL);
 /* string manipulation scratchpad */
 line = g_string_new(NULL);
 
-/* print method blurb */
-vbox = gtk_vbox_new(TRUE, 0);
-gtk_box_pack_start(GTK_BOX(box), vbox, FALSE, FALSE, 0);
-g_string_sprintf(line,"\nQuantum mechanics method\nGAMESS GUI\n");
-label = gtk_label_new(line->str);
-gtk_container_add(GTK_CONTAINER(vbox), label);
-
 /* create notebook */
 notebook = gtk_notebook_new();
 gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
@@ -483,9 +576,45 @@ gtk_box_pack_start(GTK_BOX(vbox2), frame, FALSE, FALSE, 0);
 gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
 vbox = gtk_vbox_new(TRUE, 0);
 gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtksh_direct_spin("Maximum iterations", &data->gamess.maxit, 1, 150, 1,
+gui_direct_spin("Maximum iterations", &data->gamess.maxit, 1, 150, 1,
                   NULL, NULL, vbox);
 
+/* frame for DFT */
+frame = gtk_frame_new(NULL);
+gtk_box_pack_start(GTK_BOX(vbox3), frame, FALSE, FALSE, 0);
+gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
+vbox = gtk_vbox_new(TRUE, 0);
+gtk_container_add(GTK_CONTAINER(frame), vbox);
+
+vbox4 = gtk_vbox_new(TRUE, 0);
+/* TODO - set sensitive stuff -> candidate for a short cut */
+gui_direct_check("DFT calculation", &data->gamess.dft, gui_checkbox_refresh, vbox4, vbox);
+gtk_box_pack_start(GTK_BOX(vbox), vbox4, FALSE, FALSE, 0);
+
+
+/* TODO - the pulldown list is another candidate for a short cut */
+hbox = gtk_hbox_new(FALSE, PANEL_SPACING);
+gtk_box_pack_start(GTK_BOX(vbox4), hbox, FALSE, FALSE, 0);
+
+label = gtk_label_new("Functional");
+gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+
+list = NULL;
+list = g_list_append(list, "SVWN (LDA)");
+list = g_list_append(list, "BLYP");
+list = g_list_append(list, "B3LYP");
+
+combo = gtk_combo_new();
+gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(combo)->entry), FALSE);
+gtk_combo_set_popdown_strings(GTK_COMBO(combo), list);
+gtk_box_pack_start(GTK_BOX(hbox), combo, FALSE, FALSE, 0);
+
+g_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->entry), "changed", 
+                 GTK_SIGNAL_FUNC(gms_functional_select), data);
+
+
+
+
 /* frame for time, memory and output */
 frame = gtk_frame_new("Run control");
 gtk_box_pack_start(GTK_BOX(vbox3), frame, FALSE, FALSE, 0);
@@ -500,13 +629,13 @@ while (run_control[i].label)
   switch (run_control[i].id)
     {
     case TIMLIM:
-      gtksh_direct_spin(run_control[i].label,
+      gui_direct_spin(run_control[i].label,
                         &data->gamess.time_limit, 0, 30000, 10,
                         NULL, NULL, vbox);
       break;
 
     case MWORDS:
-      gtksh_direct_spin(run_control[i].label,
+      gui_direct_spin(run_control[i].label,
                         &data->gamess.mwords, 1, 150, 1,
                         NULL, NULL, vbox);
       break;
@@ -518,7 +647,7 @@ while (run_control[i].label)
   }
 
 /* wide output button */
-gtksh_direct_check("Wide output", &data->gamess.wide_output, NULL, NULL, vbox);
+gui_direct_check("Wide output", &data->gamess.wide_output, NULL, NULL, vbox);
 
 /* frame for charge and multiplicity */
 frame = gtk_frame_new("Electronic configuration");
@@ -534,13 +663,13 @@ while (econfig[i].label)
   switch (econfig[i].id)
     {
     case TOTAL_Q:
-      gtksh_direct_spin(econfig[i].label,
+      gui_direct_spin(econfig[i].label,
                         &data->gamess.total_charge, -5, 5, 1,
                         NULL, NULL, vbox);
       break;
 
     case MULTIPLICITY:
-      gtksh_direct_spin(econfig[i].label,
+      gui_direct_spin(econfig[i].label,
                         &data->gamess.multiplicity, 1, 5, 1,
                         NULL, NULL, vbox);
       break;
@@ -605,15 +734,15 @@ while (pol_funcs[i].label)
   switch (pol_funcs[i].id)
     {
     case NUM_P:
-      gtksh_direct_spin(pol_funcs[i].label, &data->gamess.num_p, 0, 3, 1,
+      gui_direct_spin(pol_funcs[i].label, &data->gamess.num_p, 0, 3, 1,
                         NULL, NULL, vbox);
       break;
     case NUM_D:
-      gtksh_direct_spin(pol_funcs[i].label, &data->gamess.num_d, 0, 3, 1,
+      gui_direct_spin(pol_funcs[i].label, &data->gamess.num_d, 0, 3, 1,
                         NULL, NULL, vbox);
       break;
     case NUM_F:
-      gtksh_direct_spin(pol_funcs[i].label, &data->gamess.num_f, 0, 1, 1,
+      gui_direct_spin(pol_funcs[i].label, &data->gamess.num_f, 0, 1, 1,
                         NULL, NULL, vbox);
       break;
     default:
@@ -628,9 +757,9 @@ gtk_box_pack_start(GTK_BOX(vbox2), frame, FALSE, FALSE, 0);
 gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
 vbox = gtk_vbox_new(TRUE, 0);
 gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtksh_direct_check("Heavy atoms (s & p)", &data->gamess.have_heavy_diffuse,
+gui_direct_check("Heavy atoms (s & p)", &data->gamess.have_heavy_diffuse,
                     NULL, NULL, vbox);
-gtksh_direct_check("Hydrogen (s only)", &data->gamess.have_hydrogen_diffuse,
+gui_direct_check("Hydrogen (s only)", &data->gamess.have_hydrogen_diffuse,
                     NULL, NULL, vbox);
 
 /* frame for optimisation settings */
@@ -678,7 +807,7 @@ gtk_container_add(GTK_CONTAINER(frame), vbox);
 /* optimization cycles */
 hbox = gtk_hbox_new(FALSE, 0);
 gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-gtksh_direct_spin("Maximum cycles", &data->gamess.nstep,
+gui_direct_spin("Maximum cycles", &data->gamess.nstep,
                   0, 500, 10, NULL, NULL, hbox);
 
 
@@ -737,8 +866,6 @@ if (g_ascii_strncasecmp(data->gamess.title, "none", 4) == 0)
   }
 
 /* temporary GAMESS input filename */
-label = gtk_label_new("Structure name");
-gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
 label = gtk_label_new("Title");
 gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
 label = gtk_label_new("Total energy (Hartrees)");
@@ -748,13 +875,6 @@ gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
 vbox = gtk_vbox_new(TRUE, 2);
 gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
 
-entry = gtk_entry_new_with_max_length(LINELEN);
-gtk_entry_set_text(GTK_ENTRY(entry), data->basename);
-gtk_box_pack_start (GTK_BOX (vbox), entry, TRUE, TRUE, 0);
-g_signal_connect(GTK_OBJECT(entry), "changed",
-                   GTK_SIGNAL_FUNC(name_entry_changed),
-                   (gpointer) data);
-
 entry = gtk_entry_new_with_max_length(LINELEN);
 gtk_entry_set_text(GTK_ENTRY(entry), data->gamess.title);
 gtk_box_pack_start (GTK_BOX (vbox), entry, TRUE, TRUE, 0);
@@ -776,6 +896,60 @@ gtk_entry_set_editable(GTK_ENTRY(data->gamess.energy_entry), FALSE);
 /* done */
 gtk_widget_show_all(box);
 
+if (data->gamess.dft)
+  gtk_widget_set_sensitive(vbox4, TRUE);
+else
+  gtk_widget_set_sensitive(vbox4, FALSE);
+
 g_string_free(line, TRUE);
 }
 
+/****************************************/
+/* run the approriate energetics method */
+/****************************************/
+void gamess_execute(GtkWidget *w, gpointer data)
+{
+struct model_pak *model = data;
+
+g_assert(model != NULL);
+
+gui_gamess_task(NULL, model);
+}
+
+/****************************/
+/* GAMESS energetics dialog */
+/****************************/
+void gamess_dialog(void)
+{
+gpointer dialog;
+GtkWidget *window, *frame, *vbox;
+struct model_pak *model;
+
+model = sysenv.active_model;
+if (!model)
+  return;
+
+/* request an energetics dialog */
+dialog = dialog_request(GAMESS, "GAMESS configuration", NULL, NULL, NULL);
+if (!dialog)
+  return;
+window = dialog_window(dialog);
+
+frame = gtk_frame_new(NULL);
+gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), frame, FALSE, FALSE, 0);
+gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
+
+vbox = gtk_vbox_new(FALSE,0);
+gtk_container_add(GTK_CONTAINER(frame), vbox);
+gui_gamess_widget(vbox, model);
+
+/* terminating button */
+gui_stock_button(GTK_STOCK_EXECUTE, gamess_execute, model,
+                   GTK_DIALOG(window)->action_area);
+
+gui_stock_button(GTK_STOCK_CLOSE, dialog_destroy, dialog,
+                   GTK_DIALOG(window)->action_area);
+
+gtk_widget_show_all(window);
+}
+
