@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -35,9 +35,11 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "edit.h"
 #include "matrix.h"
 #include "opengl.h"
+#include "parse.h"
 #include "render.h"
+#include "spatial.h"
 #include "select.h"
-#include "gtkshorts.h"
+#include "shortcuts.h"
 #include "interface.h"
 #include "dialog.h"
 #include "measure.h"
@@ -45,10 +47,12 @@ The GNU GPL can also be found at http://www.gnu.org
 extern struct sysenv_pak sysenv;
 extern struct elem_pak elements[];
 
-gint selected_label = -1;
-GtkWidget *label_list, *start_spin, *stop_spin;
-GtkWidget *match1, *match2, *match3, *match4, *search;
+gdouble measure_min_12 = 0.1, measure_max_12 = 2.0;
+gdouble measure_min_23 = 0.1, measure_max_23 = 2.0;
+gdouble measure_min_a = 0.1, measure_max_a = 180.0;
 GtkWidget *meas_tv;
+GtkWidget *match1, *match2, *match3, *match4, *search, *entry;
+GtkWidget *spin1, *spin2, *spin3, *spin4, *spin5, *spin6;
 GtkTreeStore *meas_ts=NULL;
 
 enum {MEAS_NAME, MEAS_ATOMS, MEAS_VALUE, MEAS_MODEL, MEAS_POINTER, MEAS_NCOLS};
@@ -140,6 +144,43 @@ if (target != model)
 return(1);
 }
 
+/**************************************/
+/* locate a single measurement's iter */
+/**************************************/
+void measure_iter_find(GtkTreeIter *iter, gpointer measure, struct model_pak *model)
+{
+/* TODO */
+}
+
+/*******************************/
+/* update a single measurement */
+/*******************************/
+void measure_update_selected(gpointer measure, struct model_pak *model)
+{
+gchar *value;
+GtkTreeIter iter;
+GtkTreeModel *treemodel;
+GtkTreeSelection *selection;
+
+/* checks */
+treemodel = meas_treemodel();
+if (!treemodel)
+  return;
+
+/* get selected iterator */
+if (!(selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(meas_tv))))
+  return;
+if (!gtk_tree_selection_get_selected(selection, &treemodel, &iter))
+  return;
+
+
+value = g_strdup(measure_value_get(measure));
+
+gtk_tree_store_set(meas_ts, &iter, MEAS_VALUE, value, -1);
+
+g_free(value);
+}
+
 /******************************/
 /* graft a single measurement */
 /******************************/
@@ -344,16 +385,13 @@ redraw_canvas(SINGLE);
 void info_bond(GtkWidget *w, gint x, gint y, struct model_pak *model)
 {
 gpointer measure;
-GSList *list;
 struct core_pak *core[2];
 struct bond_pak *bond;
 
 /* seek */
-list = gl_seek_bond(w, x, y, model);
-if (!list)
+bond = gl_seek_bond(x, y, model);
+if (!bond)
   return;
-else
-  bond = list->data;
 
 /* FIXME - have to ignore hydrogen bonds to stop dodgy bond measurements being drawn */
 if (bond->type == BOND_HBOND)
@@ -376,13 +414,11 @@ redraw_canvas(SINGLE);
 /*****************/
 void info_dist(gint x, gint y, struct model_pak *model)
 {
-gdouble r[3];
 gpointer measure;
 static struct core_pak *core[2];
 
 /* attempt to find core */
-pixel_coord_map(x, y, r, model);
-if (!(core[model->state] = seek_coord3d(r, model)))
+if (!(core[model->state] = gl_seek_core(x, y, model)))
   return;
 
 /* are we looking at the 1st or the 2nd call? */
@@ -423,13 +459,11 @@ redraw_canvas(SINGLE);
 /******************/
 void info_angle(gint x, gint y, struct model_pak *model)
 {
-gdouble r[3];
 gpointer measure;
 static struct core_pak *core[3];
 
 /* attempt to find core */
-pixel_coord_map(x, y, r, model);
-if (!(core[model->state] = seek_coord3d(r, model)))
+if (!(core[model->state] = gl_seek_core(x, y, model)))
   return;
 
 /* are we looking at the 1st or the 2nd call? */
@@ -471,13 +505,11 @@ redraw_canvas(SINGLE);
 /****************************/
 void info_torsion(gint x, gint y, struct model_pak *model)
 {
-gdouble r[3];
 gpointer measure;
 static struct core_pak *core[4];
 
 /* attempt to find core */
-pixel_coord_map(x, y, r, model);
-if (!(core[model->state] = seek_coord3d(r, model)))
+if (!(core[model->state] = gl_seek_core(x, y, model)))
   return;
 
 /* what stage are we at? */
@@ -543,7 +575,7 @@ if (i == a)
 /* user has put in something like H1 - compare this with the atom label */
   if (g_ascii_strcasecmp(label1, elements[i].symbol) != 0)
     {
-    if (g_ascii_strcasecmp(core1->label, label1) == 0)
+    if (g_ascii_strcasecmp(core1->atom_label, label1) == 0)
       mask |= 1;
     }
   else
@@ -554,7 +586,7 @@ if (j == a)
   {
   if (g_ascii_strcasecmp(label1, elements[j].symbol) != 0)
     {
-    if (g_ascii_strcasecmp(core2->label, label1) == 0)
+    if (g_ascii_strcasecmp(core2->atom_label, label1) == 0)
       mask |= 2;
     }
   else
@@ -565,7 +597,7 @@ if (i == b)
   {
   if (g_ascii_strcasecmp(label2, elements[i].symbol) != 0)
     {
-    if (g_ascii_strcasecmp(core1->label, label2) == 0)
+    if (g_ascii_strcasecmp(core1->atom_label, label2) == 0)
       mask |= 4;
     }
   else
@@ -576,7 +608,7 @@ if (j == b)
   {
   if (g_ascii_strcasecmp(label2, elements[j].symbol) != 0)
     {
-    if (g_ascii_strcasecmp(core2->label, label2) == 0)
+    if (g_ascii_strcasecmp(core2->atom_label, label2) == 0)
       mask |= 8;
     }
   else
@@ -626,16 +658,16 @@ return(1);
 /* geometry search - bonds */
 /***************************/
 #define DEBUG_BOND_SEARCH 0
-void cb_bond_search(gint type, gdouble r1, gdouble r2, struct model_pak *model)
+void measure_bond_setup(struct model_pak *model)
 {
 const gchar *label[2];
 
+/* checks */
 g_assert(model != NULL);
 
 label[0] = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(match1)->entry));
 label[1] = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(match2)->entry));
-
-measure_bond_search(label, r1, r2, model);
+measure_bond_search(label, measure_min_12, measure_max_12, model);
 meas_graft_model(model);
 }
 
@@ -643,7 +675,7 @@ meas_graft_model(model);
 /* geometry search - distances */
 /*******************************/
 #define DEBUG_DIST_SEARCH 0
-void cb_dist_search(gint type, gdouble r1, gdouble r2, gint inter, struct model_pak *model)
+void measure_dist_setup(gint type, struct model_pak *model)
 {
 const gchar *label[2];
 
@@ -653,19 +685,16 @@ g_assert(model != NULL);
 /* get pattern matching entries */
 label[0] = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(match1)->entry));
 label[1] = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(match2)->entry));
+measure_distance_search(label, type, measure_min_12, measure_max_12, model);
 
-if (inter)
-  measure_inter_search(label, r1, r2, model);
-else
-  measure_dist_search(label, r1, r2, model);
 meas_graft_model(model);
 }
 
-/****************************/
-/* geometry search - angles */
-/****************************/
+/*********************************/
+/* geometry search - bond angles */
+/*********************************/
 #define DEBUG_ANGLE_SEARCH 0
-void cb_angle_search(gint type, gdouble a1, gdouble a2, struct model_pak *model)
+void measure_bangle_setup(struct model_pak *model)
 {
 const gchar *label[3];
 
@@ -675,59 +704,66 @@ g_assert(model != NULL);
 label[0] = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(match1)->entry));
 label[1] = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(match2)->entry));
 label[2] = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(match3)->entry));
+measure_bangle_search(label, measure_min_a, measure_max_a, model);
+meas_graft_model(model);
+}
 
-measure_angle_search(label, a1, a2, model);
+/********************************/
+/* geometry search - any angles */
+/********************************/
+void measure_angle_setup(struct model_pak *model)
+{
+gdouble measure_range[6];
+const gchar *label[3];
+
+/* checks */
+g_assert(model != NULL);
+
+/* get pattern matching entries */
+label[0] = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(match1)->entry));
+label[1] = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(match2)->entry));
+label[2] = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(match3)->entry));
+
+VEC2SET(&measure_range[0], measure_min_12, measure_max_12);
+VEC2SET(&measure_range[2], measure_min_23, measure_max_23);
+VEC2SET(&measure_range[4], measure_min_a, measure_max_a);
+measure_angle_search(label, measure_range, model);
 meas_graft_model(model);
 }
 
 /******************************/
 /* geometry search - torsions */
 /******************************/
-void cb_dihedral_search(struct model_pak *data, gdouble a1, gdouble a2, gint type)
+void measure_dihedral_setup(struct model_pak *data)
 {
-type &= 15;
 printf("Not implemented yet!\n");
 }
 
 /********************************************/
 /* geometry search callback - general setup */
 /********************************************/
-void geom_search(void)
+void measure_search_setup(void)
 {
-gint match_mask=0;
-gdouble x1, x2;
-const gchar *tmp;
-struct model_pak *data;
+const gchar *text;
+struct model_pak *model;
 
-data = sysenv.active_model;
-if (!data)
+/* checks */
+model = sysenv.active_model;
+if (!model)
   return;
 
-/* and if switches back to dist/bond search, change to 2.0 (ie angs) */
-x1 = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(start_spin));
-x2 = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(stop_spin));
-
-/* setup the atom matching requirements */
-tmp = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(match1)->entry));
-if (g_ascii_strncasecmp(tmp, "Select", 6) == 0)
-  match_mask |= 1;
-tmp = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(match2)->entry));
-if (g_ascii_strncasecmp(tmp, "Select", 6) == 0)
-  match_mask |= 2;
-tmp = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(match3)->entry));
-if (g_ascii_strncasecmp(tmp, "Select", 6) == 0)
-  match_mask |= 4;
-
 /* get the measurement type (ensure match mask ignores irrelevant bits) */
-tmp = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(search)->entry));
-if (g_ascii_strncasecmp(tmp, "Bonds", 5) == 0)
-  cb_bond_search(match_mask, x1, x2, data);
-if (g_ascii_strncasecmp(tmp, "Distance", 8) == 0)
-  cb_dist_search(match_mask, x1, x2, 0, data);
-if (g_ascii_strncasecmp(tmp, "Intermolecular", 14) == 0)
-  cb_dist_search(match_mask, x1, x2, 1, data);
-if (g_ascii_strncasecmp(tmp, "Angles", 6) == 0)
-  cb_angle_search(match_mask, x1, x2, data);
+text = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(search)->entry));
+if (g_ascii_strncasecmp(text, "Bonds", 5) == 0)
+  measure_bond_setup(model);
+if (g_ascii_strncasecmp(text, "Distance", 8) == 0)
+  measure_dist_setup(MEASURE_DISTANCE, model);
+if (g_ascii_strncasecmp(text, "Intermolecular", 14) == 0)
+  measure_dist_setup(MEASURE_INTER, model);
+if (g_ascii_strncasecmp(text, "Bond Angles", 11) == 0)
+  measure_bangle_setup(model);
+if (g_ascii_strncasecmp(text, "Angles", 6) == 0)
+  measure_angle_setup(model);
 
 redraw_canvas(SINGLE);
 }
@@ -744,12 +780,228 @@ meas_tv = NULL;
 meas_ts = NULL;
 }
 
+/**************************************/
+/* measurement commit change callback */
+/**************************************/
+#define DEBUG_MEASURE_VALUE_CHANGED 0
+void measure_value_changed(GtkWidget *w, gpointer dummy)
+{
+gdouble old, new, delta;
+gdouble v[3], o[3], mat[9];
+gpointer measure;
+GSList *list, *list1, *list2;
+GtkTreeSelection *selection;
+GtkTreeModel *treemodel;
+GtkTreeIter iter;
+struct core_pak *core, *core1, *core2, *core3, *core4;
+struct shel_pak *shel;
+struct model_pak *model;
+
+/* CURRENT */
+
+selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(meas_tv));
+
+if (gtk_tree_selection_get_selected(selection, &treemodel, &iter))
+  {
+  gtk_tree_model_get(treemodel, &iter, MEAS_MODEL, &model, MEAS_POINTER, &measure, -1);
+  if (measure && model)
+    {
+#if DEBUG_MEASURE_VALUE_CHANGED
+printf("old value: [%s]\n", measure_value_get(measure));
+printf("new value: [%s]\n", gtk_entry_get_text(GTK_ENTRY(entry)));
+#endif
+
+old = str_to_float(measure_value_get(measure));
+new = str_to_float(gtk_entry_get_text(GTK_ENTRY(entry)));
+delta = new - old;
+
+    switch (measure_type_get(measure))
+      {
+      case MEASURE_BOND:
+
+/* TODO - check that atoms are bonded */
+
+list1 = measure_cores_get(measure);
+core1 = g_slist_nth_data(list1, 0);
+core2 = g_slist_nth_data(list1, 1);
+
+ARR3SET(v, core2->x);
+ARR3SUB(v, core1->x);
+
+/* TODO - get the PBC adjusted offset vector */
+/*
+ARR3SUB(v, bond->offset);
+*/
+
+vecmat(model->latmat, v);
+normalize(v, 3);
+
+VEC3MUL(v, delta);
+
+vecmat(model->ilatmat, v);
+
+
+#if DEBUG_MEASURE_VALUE_CHANGED
+P3VEC("offset: ", v);
+printf("[%s] - [%s]\n", core1->atom_label, core2->atom_label);
+#endif
+
+connect_fragment_init(model);
+
+list2 = connect_fragment_get(core1, core2, model);
+
+#if DEBUG_MEASURE_VALUE_CHANGED
+printf("Adjusting position of %d atoms.\n", g_slist_length(list2));
+#endif
+
+for (list=list2 ; list ; list=g_slist_next(list))
+  {
+  core = list->data;
+
+/* adjust core position via delta scaled vector */
+/* NB: the vector needs to take PBCs into account */
+
+#if DEBUG_MEASURE_VALUE_CHANGED
+printf(" + [%s]\n", core->atom_label);
+#endif
+
+  ARR3ADD(core->x, v);
+  }
+
+        break;
+
+      case MEASURE_ANGLE:
+
+list1 = measure_cores_get(measure);
+
+core = g_slist_nth_data(list1, 0);
+core1 = g_slist_nth_data(list1, 1);
+core2 = g_slist_nth_data(list1, 2);
+
+compute_normal(v, core->x, core1->x, core2->x);
+
+/* cartesian offset to move middle core to origin */
+ARR3SET(o, core1->x);
+vecmat(model->latmat, o);
+
+/* convert to radians and build an appropriate rotation matrix */
+delta *= D2R;
+
+matrix_v_rotation(mat, v, delta);
+
+connect_fragment_init(model);
+
+list2 = connect_fragment_get(core1, core2, model);
+
+#if DEBUG_MEASURE_VALUE_CHANGED
+printf("Adjusting position of %d atoms.\n", g_slist_length(list2));
+#endif
+
+for (list=list2 ; list ; list=g_slist_next(list))
+  {
+  core = list->data;
+
+/* transform */
+  vecmat(model->latmat, core->x);
+  ARR3SUB(core->x, o);
+  vecmat(mat, core->x);
+  ARR3ADD(core->x, o);
+  vecmat(model->ilatmat, core->x);
+
+/* assoc. shell? */
+  if (core->shell)
+    {
+    shel = core->shell;
+
+/* transform */
+    vecmat(model->latmat, shel->x);
+    ARR3SUB(core->x, o);
+    vecmat(mat, shel->x);
+    ARR3ADD(core->x, o);
+    vecmat(model->ilatmat, shel->x);
+    }
+  }
+
+        break;
+
+
+      case MEASURE_TORSION:
+
+list1 = measure_cores_get(measure);
+core2 = g_slist_nth_data(list1, 1);
+core3 = g_slist_nth_data(list1, 2);
+core4 = g_slist_nth_data(list1, 3);
+
+ARR3SET(v, core3->x);
+ARR3SUB(v, core2->x);
+normalize(v, 3);
+
+/* cartesian offset to move middle core to origin */
+ARR3SET(o, core3->x);
+vecmat(model->latmat, o);
+
+/* convert to radians and build an appropriate rotation matrix */
+delta *= -D2R;
+
+matrix_v_rotation(mat, v, delta);
+
+connect_fragment_init(model);
+
+list2 = connect_fragment_get(core3, core4, model);
+
+for (list=list2 ; list ; list=g_slist_next(list))
+  {
+  core = list->data;
+
+/* transform */
+  vecmat(model->latmat, core->x);
+  ARR3SUB(core->x, o);
+  vecmat(mat, core->x);
+  ARR3ADD(core->x, o);
+  vecmat(model->ilatmat, core->x);
+
+/* assoc. shell? */
+  if (core->shell)
+    {
+    shel = core->shell;
+
+/* transform */
+    vecmat(model->latmat, shel->x);
+    ARR3SUB(core->x, o);
+    vecmat(mat, shel->x);
+    ARR3ADD(core->x, o);
+    vecmat(model->ilatmat, shel->x);
+    }
+  }
+
+        break;
+
+
+
+      default:
+        gui_text_show(WARNING, "Sorry, can't adjust this measurement.\n");
+        return;
+
+      }
+    }
+  }
+
+/* update */
+coords_compute(model);
+connect_refresh(model);
+
+measure_update_single(measure, model);
+measure_update_selected(measure, model);
+
+gui_refresh(GUI_CANVAS);
+}
+
 /*****************************/
 /* manually add measurements */
 /*****************************/
 void meas_manual_page(GtkWidget *box)
 {
-GtkWidget *frame, *vbox;
+GtkWidget *frame, *vbox, *hbox, *label;
 
 /* Frame - type */
 frame = gtk_frame_new (NULL);
@@ -760,14 +1012,33 @@ vbox = gtk_vbox_new(TRUE, PANEL_SPACING);
 gtk_container_add(GTK_CONTAINER(frame), vbox);
 gtk_container_set_border_width(GTK_CONTAINER(vbox), PANEL_SPACING);
 
-gtksh_button_x("Measure distances",
-               gtk_switch_mode, GINT_TO_POINTER(DIST_INFO), vbox);
-gtksh_button_x("Measure bonds",
-               gtk_switch_mode, GINT_TO_POINTER(BOND_INFO), vbox);
-gtksh_button_x("Measure angles",
-               gtk_switch_mode, GINT_TO_POINTER(ANGLE_INFO), vbox);
-gtksh_button_x("Measure torsions",
-               gtk_switch_mode, GINT_TO_POINTER(DIHEDRAL_INFO), vbox);
+gui_button_x("Measure distances",
+               gtk_mode_switch, GINT_TO_POINTER(DIST_INFO), vbox);
+gui_button_x("Measure bonds",
+               gtk_mode_switch, GINT_TO_POINTER(BOND_INFO), vbox);
+gui_button_x("Measure angles",
+               gtk_mode_switch, GINT_TO_POINTER(ANGLE_INFO), vbox);
+gui_button_x("Measure torsions",
+               gtk_mode_switch, GINT_TO_POINTER(DIHEDRAL_INFO), vbox);
+
+frame = gtk_frame_new (NULL);
+gtk_box_pack_start(GTK_BOX(box),frame,FALSE,FALSE,0); 
+gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
+
+vbox = gtk_vbox_new(FALSE, PANEL_SPACING);
+gtk_container_add(GTK_CONTAINER(frame), vbox);
+gtk_container_set_border_width(GTK_CONTAINER(vbox), PANEL_SPACING);
+
+hbox = gtk_hbox_new(FALSE, PANEL_SPACING);
+gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,0); 
+
+label = gtk_label_new("Measurement value");
+gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0); 
+
+entry = gtk_entry_new();
+gtk_box_pack_end(GTK_BOX(hbox),entry,FALSE,FALSE,0); 
+
+g_signal_connect(GTK_OBJECT(entry), "activate", GTK_SIGNAL_FUNC(measure_value_changed), NULL);
 }
 
 /*******************************************/
@@ -778,16 +1049,29 @@ void search_type_changed(GtkWidget *w, gpointer *data)
 const gchar *tmp;
 
 tmp = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(search)->entry));
+
+/* default atom pair matching */
+gtk_widget_set_sensitive(GTK_WIDGET(match3), FALSE);
+gtk_widget_set_sensitive(GTK_WIDGET(spin3), FALSE);
+gtk_widget_set_sensitive(GTK_WIDGET(spin4), FALSE);
+gtk_widget_set_sensitive(GTK_WIDGET(spin5), FALSE);
+gtk_widget_set_sensitive(GTK_WIDGET(spin6), FALSE);
+
 if (g_ascii_strncasecmp(tmp, "Angles", 6) == 0)
   {
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(stop_spin), 180.0);
   gtk_widget_set_sensitive(GTK_WIDGET(match3), TRUE);
+  gtk_widget_set_sensitive(GTK_WIDGET(spin3), TRUE);
+  gtk_widget_set_sensitive(GTK_WIDGET(spin4), TRUE);
+  gtk_widget_set_sensitive(GTK_WIDGET(spin5), TRUE);
+  gtk_widget_set_sensitive(GTK_WIDGET(spin6), TRUE);
   }
-else
+if (g_ascii_strncasecmp(tmp, "Bond Angles", 11) == 0)
   {
-/* TODO - if previous mode was ANGLE change, else leave alone */
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(stop_spin), 2.0);
-  gtk_widget_set_sensitive(GTK_WIDGET(match3), FALSE);
+  gtk_widget_set_sensitive(GTK_WIDGET(match3), TRUE);
+  gtk_widget_set_sensitive(GTK_WIDGET(spin3), TRUE);
+  gtk_widget_set_sensitive(GTK_WIDGET(spin4), TRUE);
+  gtk_widget_set_sensitive(GTK_WIDGET(spin5), TRUE);
+  gtk_widget_set_sensitive(GTK_WIDGET(spin6), TRUE);
   }
 }
 
@@ -796,22 +1080,39 @@ else
 /***************/
 void meas_search_page(GtkWidget *box)
 {
-GtkWidget *frame, *vbox, *hbox, *table, *button, *label;
-GtkAdjustment *adj;
+GtkWidget *frame, *hbox, *table, *button, *label;
 GList *match_list=NULL, *search_list=NULL;
 
-/* Frame - type */
+/* main frame */
 frame = gtk_frame_new (NULL);
 gtk_box_pack_start(GTK_BOX(box),frame,FALSE,FALSE,0); 
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
+gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING/2);
+hbox = gtk_hbox_new(FALSE, 0);
+gtk_container_add(GTK_CONTAINER(frame), hbox);
 
-vbox = gtk_vbox_new(FALSE, 0);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-gtk_container_set_border_width(GTK_CONTAINER(vbox), PANEL_SPACING);
+search_list = g_list_append(search_list, "Bonds");
+search_list = g_list_append(search_list, "Distances");
+search_list = g_list_append(search_list, "Intermolecular");
+search_list = g_list_append(search_list, "Bond Angles");
+search_list = g_list_append(search_list, "Angles");
 
-/* atom matching options */
+search = gtk_combo_new();
+gtk_box_pack_start(GTK_BOX(hbox),search,FALSE,FALSE,0); 
+gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(search)->entry), "Bonds");
+gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(search)->entry), FALSE);
+gtk_combo_set_popdown_strings(GTK_COMBO(search), search_list);
+g_signal_connect(GTK_OBJECT(GTK_COMBO(search)->entry), "changed", 
+                 GTK_SIGNAL_FUNC(search_type_changed), (gpointer *) search);
+
+button = gui_icon_button(GTK_STOCK_FIND, "Search ", measure_search_setup, NULL, NULL);
+gtk_box_pack_end(GTK_BOX(hbox),button,FALSE,FALSE,0); 
+
+/* setup frame */
+frame = gtk_frame_new(NULL);
+gtk_box_pack_start(GTK_BOX(box),frame,FALSE,FALSE,0); 
+gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING/2);
 table = gtk_table_new(3, 4, FALSE);
-gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
+gtk_container_add(GTK_CONTAINER(frame), table);
 
 /* labels for top row */
 label = gtk_label_new("Atom 1");
@@ -824,10 +1125,6 @@ gtk_table_attach_defaults(GTK_TABLE(table),label,2,3,0,1);
 /* construct combo options */
 match_list = g_list_append(match_list, "Any");
 match_list = g_list_append(match_list, "Selected");
-search_list = g_list_append(search_list, "Bonds");
-search_list = g_list_append(search_list, "Distances");
-search_list = g_list_append(search_list, "Intermolecular");
-search_list = g_list_append(search_list, "Angles");
 
 /* atom match combo boxes */
 match1 = gtk_combo_new();
@@ -845,48 +1142,36 @@ gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(match3)->entry), "Any");
 gtk_combo_set_popdown_strings(GTK_COMBO(match3), match_list);
 gtk_table_attach_defaults(GTK_TABLE(table),match3,2,3,1,2);
 
-/* labels for second row */
-label = gtk_label_new("Search type");
-gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,2,3);
-label = gtk_label_new("Search range");
-gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,2,3);
-
-/* search type combo box */
-search = gtk_combo_new();
-gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(search)->entry), "Bonds");
-gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(search)->entry), FALSE);
-gtk_combo_set_popdown_strings(GTK_COMBO(search), search_list);
-gtk_table_attach_defaults(GTK_TABLE(table),search,0,1,3,4);
-
-/* CURRENT */
-g_signal_connect(GTK_OBJECT(GTK_COMBO(search)->entry), "changed", 
-                 GTK_SIGNAL_FUNC(search_type_changed), (gpointer *) search);
-
-
-/* stop distance spinner */
+/* measurement ranges */
+label = gtk_label_new(" 1 - 2 cutoffs ");
+gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
 hbox = gtk_hbox_new(TRUE, 0);
+gtk_table_attach_defaults(GTK_TABLE(table),hbox,0,1,3,4);
+spin1 = gui_direct_spin(NULL, &measure_min_12, 0.1, 100.0, 0.1, NULL, NULL, NULL);
+gtk_box_pack_start(GTK_BOX(hbox), spin1, TRUE, TRUE, 0);
+spin2 = gui_direct_spin(NULL, &measure_max_12, 0.1, 100.0, 0.1, NULL, NULL, NULL);
+gtk_box_pack_start(GTK_BOX(hbox), spin2, TRUE, TRUE, 0);
 
-/* start value */
-adj = (GtkAdjustment *) gtk_adjustment_new
-      (0.1, 0.0, 360.0, 0.1, 1.0, 0);
-start_spin = gtk_spin_button_new (adj, 0.01, 2);
-gtk_spin_button_set_wrap(GTK_SPIN_BUTTON (start_spin), FALSE);
-gtk_box_pack_start(GTK_BOX(hbox), start_spin, TRUE, TRUE, 0);
-
-/* stop value */
-adj = (GtkAdjustment *) gtk_adjustment_new
-      (2.0, 0.0, 360.0, 0.1, 1.0, 0);
-stop_spin = gtk_spin_button_new (adj, 0.01, 2);
-gtk_spin_button_set_wrap(GTK_SPIN_BUTTON (stop_spin), FALSE);
-gtk_box_pack_start(GTK_BOX(hbox), stop_spin, TRUE, TRUE, 0);
+label = gtk_label_new(" 2 - 3 cutoffs ");
+gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 2, 3);
+hbox = gtk_hbox_new(TRUE, 0);
 gtk_table_attach_defaults(GTK_TABLE(table),hbox,1,2,3,4);
+spin3 = gui_direct_spin(NULL, &measure_min_23, 0.1, 100.0, 0.1, NULL, NULL, NULL);
+gtk_box_pack_start(GTK_BOX(hbox), spin3, TRUE, TRUE, 0);
+spin4 = gui_direct_spin(NULL, &measure_max_23, 0.1, 100.0, 0.1, NULL, NULL, NULL);
+gtk_box_pack_start(GTK_BOX(hbox), spin4, TRUE, TRUE, 0);
 
-/* search button */
-button = gtksh_stock_button(GTK_STOCK_FIND, geom_search, NULL, NULL);
-gtk_table_attach_defaults(GTK_TABLE(table),button,2,3,3,4);
+label = gtk_label_new(" Angle cutoffs ");
+gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 2, 3);
+hbox = gtk_hbox_new(TRUE, 0);
+gtk_table_attach_defaults(GTK_TABLE(table),hbox,2,3,3,4);
+spin5 = gui_direct_spin(NULL, &measure_min_a, 0.0, 180.0, 5.0, NULL, NULL, NULL);
+gtk_box_pack_start(GTK_BOX(hbox), spin5, TRUE, TRUE, 0);
+spin6 = gui_direct_spin(NULL, &measure_max_a, 0.0, 180.0, 5.0, NULL, NULL, NULL);
+gtk_box_pack_start(GTK_BOX(hbox), spin6, TRUE, TRUE, 0);
 
 /* I don't like doing this, but the default size is */
-/* much larger than it needs to be, wasting too much space */
+/* much larger than it needs to be & wastes too much space */
 gtk_widget_set_size_request(match1, 13*sysenv.gtk_fontsize, -1);
 gtk_widget_set_size_request(match2, 13*sysenv.gtk_fontsize, -1);
 gtk_widget_set_size_request(match3, 13*sysenv.gtk_fontsize, -1);
@@ -897,6 +1182,10 @@ gtk_table_set_col_spacings(GTK_TABLE(table), PANEL_SPACING);
 
 /* default measurement is bonds */
 gtk_widget_set_sensitive(GTK_WIDGET(match3), FALSE);
+gtk_widget_set_sensitive(GTK_WIDGET(spin3), FALSE);
+gtk_widget_set_sensitive(GTK_WIDGET(spin4), FALSE);
+gtk_widget_set_sensitive(GTK_WIDGET(spin5), FALSE);
+gtk_widget_set_sensitive(GTK_WIDGET(spin6), FALSE);
 }
 
 /***********************************/
@@ -932,19 +1221,39 @@ GtkTreeIter iter;
 gpointer measure;
 struct model_pak *model;
 
+gtk_entry_set_text(GTK_ENTRY(entry), "");
+
 if (gtk_tree_selection_get_selected(selection, &treemodel, &iter))
   {
   gtk_tree_model_get(treemodel, &iter, MEAS_MODEL, &model, -1);
   gtk_tree_model_get(treemodel, &iter, MEAS_POINTER, &measure, -1);
   if (measure && model)
+    {
     measure_select(measure, model);
+
+    gtk_entry_set_text(GTK_ENTRY(entry), measure_value_get(measure));
+
+/* CURRENT - only allow adjustments to supported editables */
+    switch (measure_type_get(measure))
+      {
+      case MEASURE_BOND:
+      case MEASURE_ANGLE:
+      case MEASURE_TORSION:
+        gtk_widget_set_sensitive(GTK_WIDGET(entry), TRUE);
+        break;
+
+      default:
+        gtk_widget_set_sensitive(GTK_WIDGET(entry), FALSE);
+        break;
+      }
+    }
   }
 }
 
 /**************************/
 /* geometric measurements */
 /**************************/
-void geom_info(void)
+void gui_measure_dialog(void)
 {
 gint i;
 gpointer dialog;
@@ -962,7 +1271,7 @@ dialog = dialog_request(GEOMETRY, "Measurements", NULL, meas_cleanup, NULL);
 if (!dialog)
   return;
 window = dialog_window(dialog);
-gtk_window_set_default_size(GTK_WINDOW(window), 250, 450);
+gtk_window_set_default_size(GTK_WINDOW(window), 250, 550);
 
 /* notebook */
 notebook = gtk_notebook_new();
@@ -985,7 +1294,7 @@ meas_search_page(vbox);
 /* measurements viewing pane */
 frame = gtk_frame_new ("Label list");
 gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox),frame,TRUE,TRUE,0); 
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
+gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING/2);
 vbox = gtk_vbox_new(FALSE, 0);
 gtk_container_add(GTK_CONTAINER(frame), vbox);
 gtk_container_set_border_width(GTK_CONTAINER(GTK_BOX(vbox)), PANEL_SPACING);
@@ -1026,12 +1335,12 @@ for (i=0 ; i<=MEAS_VALUE ; i++)
 hbox = gtk_hbox_new(TRUE, 10);
 gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, PANEL_SPACING);
 
-gtksh_button(" Select ", measure_select_all, NULL, hbox, TT);
-gtksh_button(" Delete ", meas_prune_selected, NULL, hbox, TT);
-gtksh_button(" Dump ", meas_dump_all, NULL, hbox, TT);
+gui_button(" Select ", measure_select_all, NULL, hbox, TT);
+gui_button(" Delete ", meas_prune_selected, NULL, hbox, TT);
+gui_button(" Dump ", meas_dump_all, NULL, hbox, TT);
 
 /* terminating button */
-gtksh_stock_button(GTK_STOCK_CLOSE, dialog_destroy, dialog,
+gui_stock_button(GTK_STOCK_CLOSE, dialog_destroy, dialog,
                    GTK_DIALOG(window)->action_area);
 
 /* done */
