@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -22,10 +22,13 @@ The GNU GPL can also be found at http://www.gnu.org
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <math.h>
 
 #include "gdis.h"
 #include "dialog.h"
+#include "interface.h"
+#include "gui_image.h"
 
 #include "go.xpm"
 #include "pause.xpm"
@@ -41,7 +44,8 @@ extern struct sysenv_pak sysenv;
 extern GtkWidget *window;
 
 enum {AUTO_CHECK, AUTO_SPIN, AUTO_RANGE,
-      AUTO_TEXT_LABEL, AUTO_INT_LABEL, AUTO_FLOAT_LABEL};
+      AUTO_TEXT_ENTRY, AUTO_TEXT_LABEL,
+      AUTO_INT_LABEL, AUTO_FLOAT_LABEL};
 
 struct relation_pak 
 {
@@ -59,14 +63,14 @@ GtkWidget *widget;
 gint type;
 };
 
-GSList *gtksh_relation_list=NULL;
+GSList *gui_relation_list=NULL;
 
 #define DEBUG_RELATION 0
 
 /**************************************/
 /* auto update widget handling events */
 /**************************************/
-void gtksh_relation_destroy(GtkWidget *w, gpointer dummy)
+void gui_relation_destroy(GtkWidget *w, gpointer dummy)
 {
 GSList *list;
 struct relation_pak *relation;
@@ -76,10 +80,10 @@ printf("destroying widget (%p)", w);
 #endif
 
 /* find appropriate relation */
-list = gtksh_relation_list;
+list = gui_relation_list;
 while (list)
   {
-  relation = (struct relation_pak *) list->data;
+  relation = list->data;
   list = g_slist_next(list);
 
 /* free associated data */
@@ -89,7 +93,7 @@ while (list)
 printf(" : relation (%p)", relation);
 #endif
     g_free(relation);
-    gtksh_relation_list = g_slist_remove(gtksh_relation_list, relation);
+    gui_relation_list = g_slist_remove(gui_relation_list, relation);
     }
   }
 #if DEBUG_RELATION
@@ -100,7 +104,7 @@ printf("\n");
 /***************************************************/
 /* set a value according to supplied widget status */
 /***************************************************/
-void gtksh_relation_set_value(GtkWidget *w, struct model_pak *model)
+void gui_relation_set_value(GtkWidget *w, struct model_pak *model)
 {
 gpointer value;
 GSList *list;
@@ -112,11 +116,11 @@ g_assert(w != NULL);
 if (!model)
   model = sysenv.active_model;
 
-/* find appropriate relation */
-list = gtksh_relation_list;
+/* find appropriate relation (direct -> widget, else model) */
+list = gui_relation_list;
 while (list)
   {
-  relation = (struct relation_pak *) list->data;
+  relation = list->data;
   list = g_slist_next(list);
 
   if (relation->widget != w)
@@ -155,47 +159,40 @@ printf("model %p : relation %p : setting variable to %f\n",
     case AUTO_RANGE:
       *((gint *) value) = gtk_range_get_value(GTK_RANGE(w));
       break;
+
+    case AUTO_TEXT_ENTRY:
+/* CURRENT */
+      g_free(*((gchar **) value));
+      *((gchar **) value) = g_strdup(gtk_entry_get_text(GTK_ENTRY(w)));
+      break;
+
     }
   }
 }
 
-/*****************************/
-/* called for a model switch */
-/*****************************/
-/* ie sets all widget states according to related variable */
-/* NB: pass NULL to get everything updated */
-void gtksh_relation_update(struct model_pak *model)
+/***************************************************/
+/* updates dependant widget with new variable data */
+/***************************************************/
+void gui_relation_update_widget(gpointer value)
 {
 gchar *text;
 GSList *list;
 struct relation_pak *relation;
-gpointer value;
 
 /* loop over all existing relations */
-for (list=gtksh_relation_list ; list ; list=g_slist_next(list))
+for (list=gui_relation_list ; list ; list=g_slist_next(list))
   {
-  relation = (struct relation_pak *) list->data;
+  relation = list->data;
 
   g_assert(relation != NULL);
 
-/* update test */
-  if (relation->direct)
-    {
-    if (model)
-      if (model != relation->model)
-        continue;
-    value = relation->variable;
-    }
-  else
-    {
-    if (model)
-      value = (gpointer) model + relation->offset;
-    else
-      continue;
-    }
+  if (!relation->direct)
+    continue;
+  if (relation->variable != value)
+    continue;
 
 #if DEBUG_RELATION
-printf("model %p : relation %p : type %d\n", model, relation, relation->type);
+printf("relation %p : type %d : updating widget\n", relation, relation->type);
 #endif
 
 /* synchronize widget and variable */
@@ -219,6 +216,99 @@ printf("model %p : relation %p : type %d\n", model, relation, relation->type);
       gtk_range_set_value(GTK_RANGE(relation->widget), *((gint *) value));
       break;
 
+    case AUTO_TEXT_ENTRY:
+/* CURRENT */
+      text = *((gchar **) value);
+      if (text)
+        gtk_entry_set_text(GTK_ENTRY(relation->widget), text);
+      break;
+
+    case AUTO_TEXT_LABEL:
+      gtk_label_set_text(GTK_LABEL(relation->widget), *((gchar **) value));
+      break;
+
+    case AUTO_INT_LABEL:
+      text = g_strdup_printf("%d", *((gint *) value));
+      gtk_label_set_text(GTK_LABEL(relation->widget), text);
+      g_free(text);
+      break;
+
+    case AUTO_FLOAT_LABEL:
+      text = g_strdup_printf("%.4f", *((gdouble *) value));
+      gtk_label_set_text(GTK_LABEL(relation->widget), text);
+      g_free(text);
+      break;
+    }
+  }
+}
+
+/*****************************/
+/* called for a model switch */
+/*****************************/
+/* ie sets all widget states according to related variable */
+/* NB: pass NULL to get everything updated */
+void gui_relation_update(gpointer data)
+{
+gchar *text;
+GSList *list;
+struct relation_pak *relation;
+gpointer value;
+
+/* loop over all existing relations */
+for (list=gui_relation_list ; list ; list=g_slist_next(list))
+  {
+  relation = list->data;
+
+  g_assert(relation != NULL);
+
+/* update test */
+  if (relation->direct)
+    {
+    if (data)
+      if (data != relation->model)
+        continue;
+    value = relation->variable;
+    }
+  else
+    {
+    if (data)
+      value = data + relation->offset;
+    else
+      continue;
+    }
+
+#if DEBUG_RELATION
+printf("relation %p : type %d : model change\n", relation, relation->type);
+#endif
+
+/* synchronize widget and variable */
+  switch (relation->type)
+    {
+    case AUTO_CHECK:
+      if (*((gint *) value))
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(relation->widget),
+                                     TRUE);
+      else
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(relation->widget),
+                                     FALSE);
+      break;
+
+    case AUTO_SPIN:
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(relation->widget),
+                                *((gdouble *) value));
+      break;
+
+    case AUTO_RANGE:
+      gtk_range_set_value(GTK_RANGE(relation->widget), *((gint *) value));
+      break;
+
+    case AUTO_TEXT_ENTRY:
+/* CURRENT */
+      text = *((gchar **) value);
+      if (text)
+        gtk_entry_set_text(GTK_ENTRY(relation->widget), text);
+      break;
+
     case AUTO_TEXT_LABEL:
       gtk_label_set_text(GTK_LABEL(relation->widget), *((gchar **) value));
       break;
@@ -242,7 +332,7 @@ printf("model %p : relation %p : type %d\n", model, relation, relation->type);
 /* create a new variable to widget correspondance */
 /**************************************************/
 void
-gtksh_relation_submit(GtkWidget        *widget,
+gui_relation_submit(GtkWidget        *widget,
                       gint              type,
                       gpointer          value,
                       gboolean          direct,
@@ -254,7 +344,10 @@ struct relation_pak *relation;
 relation = g_malloc(sizeof(struct relation_pak));
 relation->direct = direct;
 relation->type = type;
-relation->offset = value - (gpointer) model;
+if (model)
+  relation->offset = value - (gpointer) model;
+else
+  relation->offset = 0;
 relation->variable = value;
 relation->model = model;
 relation->widget = widget;
@@ -294,7 +387,7 @@ switch (relation->type)
     break;
   }
 
-gtksh_relation_list = g_slist_prepend(gtksh_relation_list, relation);
+gui_relation_list = g_slist_prepend(gui_relation_list, relation);
 }
 
 /*****************************************/
@@ -323,8 +416,8 @@ return(button);
 /*********************************/
 /* relative updated check button */
 /*********************************/
-/* NB: assumes switch_model() will call gtksh_relation_update() */
-GtkWidget *gtksh_auto_check(gchar *label, gpointer cb, gpointer arg,
+/* NB: assumes gui_mode_switchl() will call gui_relation_update() */
+GtkWidget *gui_auto_check(gchar *label, gpointer cb, gpointer arg,
                             gint *state, GtkWidget *box)
 {
 GtkWidget *button;
@@ -335,13 +428,13 @@ g_assert(model != NULL);
 
 /* create the button */
 button = gtk_check_button_new_with_label(label);
-gtksh_relation_submit(button, AUTO_CHECK, state, FALSE, model);
+gui_relation_submit(button, AUTO_CHECK, state, FALSE, model);
 if (box)
   gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
 
 /* callback to set the variable to match the widget */
 g_signal_connect(GTK_OBJECT(button), "clicked",
-                 GTK_SIGNAL_FUNC(gtksh_relation_set_value), NULL);
+                 GTK_SIGNAL_FUNC(gui_relation_set_value), NULL);
 
 /* callback to do (user defined) update tasks */
 if (cb)
@@ -349,16 +442,28 @@ if (cb)
 
 /* callback to remove the variable <-> widget relation */
 g_signal_connect(GTK_OBJECT(button), "destroy",
-                 GTK_SIGNAL_FUNC(gtksh_relation_destroy), NULL);
+                 GTK_SIGNAL_FUNC(gui_relation_destroy), NULL);
 
 return(button);
 }
 
+/*************************************************/
+/* activate/deactivate based on a checkbox state */
+/*************************************************/
+/* TODO - shortcut for setup up one of these widgets */
+void gui_checkbox_refresh(GtkWidget *w, GtkWidget *box)
+{
+if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
+  gtk_widget_set_sensitive(box, TRUE);
+else
+  gtk_widget_set_sensitive(box, FALSE);
+}
+
 /*********************************/
 /* directly updated check button */
 /*********************************/
-/* NB: assumes switch_model() will call gtksh_relation_update() */
-GtkWidget *gtksh_direct_check(gchar *label, gint *state,
+/* NB: assumes gui_mode_switch() will call gui_relation_update() */
+GtkWidget *gui_direct_check(gchar *label, gint *state,
                               gpointer cb, gpointer arg,
                               GtkWidget *box)
 {
@@ -369,18 +474,18 @@ model = sysenv.active_model;
 
 /* create the button */
 button = gtk_check_button_new_with_label(label);
-gtksh_relation_submit(button, AUTO_CHECK, state, TRUE, model);
+gui_relation_submit(button, AUTO_CHECK, state, TRUE, model);
 if (box)
   gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
 
 
 /* callback to set the variable to match the widget */
 g_signal_connect(GTK_OBJECT(button), "clicked",
-                 GTK_SIGNAL_FUNC(gtksh_relation_set_value), model);
+                 GTK_SIGNAL_FUNC(gui_relation_set_value), model);
 
 /* callback to remove the variable <-> widget relation */
 g_signal_connect(GTK_OBJECT(button), "destroy",
-                 GTK_SIGNAL_FUNC(gtksh_relation_destroy), NULL);
+                 GTK_SIGNAL_FUNC(gui_relation_destroy), NULL);
 
 /* callback to do (user defined) update tasks */
 if (cb)
@@ -392,7 +497,7 @@ return(button);
 /****************************/
 /* a simple labelled button */
 /****************************/
-GtkWidget *gtksh_button(gchar *txt, gpointer cb, gpointer arg, GtkWidget *w, gint mask)
+GtkWidget *gui_button(gchar *txt, gpointer cb, gpointer arg, GtkWidget *w, gint mask)
 {
 gint fill1, fill2;
 GtkWidget *button;
@@ -424,7 +529,9 @@ return(button);
 /************************************/
 /* text label plus an X push button */
 /************************************/
-void gtksh_button_x(gchar *text, gpointer cb, gpointer arg, GtkWidget *box)
+GtkWidget *gui_button_x(gchar *text,
+                          gpointer cb, gpointer arg,
+                          GtkWidget *box)
 {
 GtkWidget *hbox, *button, *label, *image;
 GdkPixmap *pixmap;
@@ -460,12 +567,14 @@ if (box)
 
 if (cb)
   g_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(cb), arg);
+
+return(button);
 }
 
 /****************************************/
 /* text label plus labelled push button */
 /****************************************/
-void gtksh_button_label(gchar *t1, gchar *t2, gpointer cb, gpointer arg, GtkWidget *box)
+void gui_button_label(gchar *t1, gchar *t2, gpointer cb, gpointer arg, GtkWidget *box)
 {
 GtkWidget *hbox, *button, *label;
 
@@ -597,7 +706,7 @@ return(spin);
 /************************************************************/
 /* as above, but automatically attaches spinner to callback */
 /************************************************************/
-GtkWidget *gtksh_new_spin(gchar *text, gdouble x0, gdouble x1, gdouble dx,
+GtkWidget *gui_new_spin(gchar *text, gdouble x0, gdouble x1, gdouble dx,
                           gpointer callback, GtkWidget *box)
 {
 GtkWidget *hbox, *label, *spin;
@@ -628,8 +737,8 @@ return(spin);
 /* automatically updated spinner */
 /*********************************/
 /* current model relative correlation between value and spinner */
-/* NB: assumes switch_model() will call gtksh_relation_update() */
-GtkWidget *gtksh_auto_spin(gchar *text, gdouble *value,
+/* NB: assumes gui_mode_switchl() will call gui_relation_update() */
+GtkWidget *gui_auto_spin(gchar *text, gdouble *value,
                            gdouble min, gdouble max, gdouble step,
                            gpointer callback, gpointer arg, GtkWidget *box)
 {
@@ -643,15 +752,15 @@ g_assert(model != NULL);
 spin = new_spinner(text, min, max, step, callback, arg, box);
 
 /* set up a relationship */
-gtksh_relation_submit(spin, AUTO_SPIN, value, FALSE, model);
+gui_relation_submit(spin, AUTO_SPIN, value, FALSE, model);
 
 /* callback to set the variable to match the widget */
 g_signal_connect(GTK_OBJECT(spin), "value-changed",
-                (gpointer) gtksh_relation_set_value, NULL);
+                (gpointer) gui_relation_set_value, NULL);
 
 /* callback to remove the variable <-> widget relation */
 g_signal_connect(GTK_OBJECT(spin), "destroy",
-                 GTK_SIGNAL_FUNC(gtksh_relation_destroy), NULL);
+                 GTK_SIGNAL_FUNC(gui_relation_destroy), NULL);
 
 return(spin);
 }
@@ -660,7 +769,7 @@ return(spin);
 /* automatically updated spinner */
 /*********************************/
 /* direct correlation between value and spinner */
-GtkWidget *gtksh_direct_spin(gchar *text, gdouble *value,
+GtkWidget *gui_direct_spin(gchar *text, gdouble *value,
                              gdouble min, gdouble max, gdouble step,
                              gpointer callback, gpointer arg, GtkWidget *box)
 {
@@ -675,6 +784,7 @@ model = sysenv.active_model;
 spin = new_spinner(text, min, max, step, NULL, NULL, box);
 
 /* HACK - cope with GTK underestimating the size needed to display spin values */
+/* TODO - examine sig fig of *value to get dp */
 digits = log10(step);
 if (digits < 0.0)
   {
@@ -692,15 +802,15 @@ gtk_widget_set_size_request(spin, sysenv.gtk_fontsize*5, -1);
 */
 
 /* set up a relationship */
-gtksh_relation_submit(spin, AUTO_SPIN, value, TRUE, model);
+gui_relation_submit(spin, AUTO_SPIN, value, TRUE, model);
 
 /* callback to set the variable to match the widget */
 g_signal_connect(GTK_OBJECT(spin), "value-changed",
-                 GTK_SIGNAL_FUNC(gtksh_relation_set_value), model);
+                 GTK_SIGNAL_FUNC(gui_relation_set_value), model);
 
 /* callback to remove the variable <-> widget relation */
 g_signal_connect(GTK_OBJECT(spin), "destroy",
-                 GTK_SIGNAL_FUNC(gtksh_relation_destroy), NULL);
+                 GTK_SIGNAL_FUNC(gui_relation_destroy), NULL);
 
 /* connect after, so all updates done before user callback is invoked */
 if (callback)
@@ -713,7 +823,7 @@ return(spin);
 /**************************/
 /* auto update slider bar */
 /**************************/
-GtkWidget *gtksh_direct_hscale(gdouble min, gdouble max, gdouble step,
+GtkWidget *gui_direct_hscale(gdouble min, gdouble max, gdouble step,
                                gpointer value, gpointer func, gpointer arg,
                                                             GtkWidget *box)
 {
@@ -729,11 +839,11 @@ gtk_range_set_update_policy(GTK_RANGE(hscale), GTK_UPDATE_CONTINUOUS);
 if (box)
   gtk_box_pack_start(GTK_BOX(box), hscale, TRUE, TRUE, 0);
 
-gtksh_relation_submit(hscale, AUTO_RANGE, value, TRUE, model);
+gui_relation_submit(hscale, AUTO_RANGE, value, TRUE, model);
 
 /* callback to set the variable to match the widget */
 g_signal_connect(GTK_OBJECT(hscale), "value_changed",
-                 GTK_SIGNAL_FUNC(gtksh_relation_set_value), model);
+                 GTK_SIGNAL_FUNC(gui_relation_set_value), model);
 
 /* user callback */
 if (func)
@@ -742,15 +852,60 @@ if (func)
 
 /* callback to remove the variable <-> widget relation */
 g_signal_connect(GTK_OBJECT(hscale), "destroy",
-                GTK_SIGNAL_FUNC(gtksh_relation_destroy), NULL);
+                GTK_SIGNAL_FUNC(gui_relation_destroy), NULL);
 
 return(hscale);
 }
 
+/**************************************************/
+/* convenience function for labelled text entries */
+/**************************************************/
+GtkWidget *gui_text_entry(gchar *text, gchar **value,
+                          gint edit, gint pack, GtkWidget *box)
+{
+GtkWidget *hbox, *label, *entry;
+
+g_assert(box != NULL);
+
+hbox = gtk_hbox_new(FALSE, 0);
+gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
+if (text)
+  {
+  label = gtk_label_new(text);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+  }
+entry = gtk_entry_new();
+
+if (value)
+  {
+  if (*value)
+    gtk_entry_set_text(GTK_ENTRY(entry), *value);
+  }
+
+gtk_entry_set_editable(GTK_ENTRY(entry), edit);
+
+if (value)
+  gui_relation_submit(entry, AUTO_TEXT_ENTRY, value, TRUE, NULL);
+
+gtk_box_pack_end(GTK_BOX(hbox), entry, pack, TRUE, 0);
+
+if (value)
+  {
+  if (edit)
+    g_signal_connect(GTK_OBJECT(entry), "changed",
+                     GTK_SIGNAL_FUNC(gui_relation_set_value), NULL);
+
+  g_signal_connect(GTK_OBJECT(entry), "destroy",
+                   GTK_SIGNAL_FUNC(gui_relation_destroy), NULL);
+  }
+
+return(entry);
+}
+
 /**************************/
 /* auto update text label */
 /**************************/
-GtkWidget *gtksh_auto_text_label(gchar **text)
+GtkWidget *gui_auto_text_label(gchar **text)
 {
 GtkWidget *label;
 struct model_pak *model;
@@ -764,11 +919,11 @@ if (*text)
 else
   label = gtk_label_new("null");
 
-gtksh_relation_submit(label, AUTO_TEXT_LABEL, text, FALSE, model);
+gui_relation_submit(label, AUTO_TEXT_LABEL, text, FALSE, model);
 
 /* callback to remove the variable <-> widget relation */
 g_signal_connect(GTK_OBJECT(label), "destroy",
-                 GTK_SIGNAL_FUNC(gtksh_relation_destroy), NULL);
+                 GTK_SIGNAL_FUNC(gui_relation_destroy), NULL);
 
 return(label);
 }
@@ -776,7 +931,7 @@ return(label);
 /**************************/
 /* auto update text label */
 /**************************/
-GtkWidget *gtksh_auto_int_label(gint *value)
+GtkWidget *gui_auto_int_label(gint *value)
 {
 gchar *text;
 GtkWidget *label;
@@ -789,11 +944,11 @@ text = g_strdup_printf("%d", *value);
 label = gtk_label_new(text);
 g_free(text);
 
-gtksh_relation_submit(label, AUTO_INT_LABEL, value, FALSE, model);
+gui_relation_submit(label, AUTO_INT_LABEL, value, FALSE, model);
 
 /* callback to remove the variable <-> widget relation */
 g_signal_connect(GTK_OBJECT(label), "destroy",
-                 GTK_SIGNAL_FUNC(gtksh_relation_destroy), NULL);
+                 GTK_SIGNAL_FUNC(gui_relation_destroy), NULL);
 
 return(label);
 }
@@ -801,7 +956,7 @@ return(label);
 /**************************/
 /* auto update text label */
 /**************************/
-GtkWidget *gtksh_auto_float_label(gdouble *value)
+GtkWidget *gui_auto_float_label(gdouble *value)
 {
 gchar *text;
 GtkWidget *label;
@@ -814,11 +969,11 @@ text = g_strdup_printf("%.4f", *value);
 label = gtk_label_new(text);
 g_free(text);
 
-gtksh_relation_submit(label, AUTO_FLOAT_LABEL, value, FALSE, model);
+gui_relation_submit(label, AUTO_FLOAT_LABEL, value, FALSE, model);
 
 /* callback to remove the variable <-> widget relation */
 g_signal_connect(GTK_OBJECT(label), "destroy",
-                 GTK_SIGNAL_FUNC(gtksh_relation_destroy), NULL);
+                 GTK_SIGNAL_FUNC(gui_relation_destroy), NULL);
 
 return(label);
 }
@@ -827,7 +982,7 @@ return(label);
 /* create a stock GTK button */
 /*****************************/
 GtkWidget * 
-gtksh_stock_button(const gchar *id, gpointer cb, gpointer arg, GtkWidget *box)
+gui_stock_button(const gchar *id, gpointer cb, gpointer arg, GtkWidget *box)
 {
 GtkWidget *button;
 
@@ -845,11 +1000,12 @@ return(button);
 /* create a button with a stock icon & custom text */
 /***************************************************/
 GtkWidget * 
-gtksh_icon_button(const gchar *id, const gchar *text,
+gui_icon_button(const gchar *id, const gchar *text,
                   gpointer cb, gpointer arg,
                   GtkWidget *box)
 {
 gint stock = TRUE;
+gpointer data;
 GtkWidget *hbox, *button, *label, *image=NULL;
 GdkBitmap *mask;
 GdkPixmap *pixmap;
@@ -860,6 +1016,21 @@ button = gtk_button_new();
 hbox = gtk_hbox_new(FALSE, 4);
 gtk_container_add(GTK_CONTAINER(button), hbox);
 
+
+/* CURRENT */
+if (g_ascii_strncasecmp(id, "IMAGE_", 6) == 0)
+  {
+  data = image_table_lookup(id);
+  if (data)
+    {
+    image = gtk_image_new_from_pixbuf(data);
+    stock = FALSE;
+    }
+  else
+    printf("[%s] : image not found.\n", id);
+  }
+
+
 /* GDIS icons */
 if (g_ascii_strncasecmp(id, "GDIS_PAUSE", 10) == 0)
   {
@@ -958,3 +1129,201 @@ if (cb)
 return(button);
 }
 
+/*******************************************/
+/* update function for scolled text window */
+/*******************************************/
+void gui_text_window_update(GtkTextBuffer *buffer, gchar **text)
+{
+GtkTextIter start, end;
+
+gtk_text_buffer_get_bounds(buffer, &start, &end);
+g_free(*text);
+*text = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
+}
+
+/*********************************/
+/* short cut for displaying text */
+/*********************************/
+GtkWidget *gui_text_window(gchar **text, gint editable)
+{
+gint n;
+GtkWidget *swin, *view;
+GtkTextBuffer *buffer;
+
+g_assert(text != NULL);
+
+swin = gtk_scrolled_window_new(NULL, NULL);
+gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin),
+                               GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+view = gtk_text_view_new();
+gtk_text_view_set_editable(GTK_TEXT_VIEW(view), editable);
+gtk_container_add(GTK_CONTAINER(swin), view);
+buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
+/* tags??? */
+/*
+gtk_text_buffer_create_tag(buffer, "fg_blue", "foreground", "blue", NULL);  
+gtk_text_buffer_create_tag(buffer, "fg_red", "foreground", "red", NULL);
+*/
+if (*text)
+  {
+  n = strlen(*text);
+  gtk_text_buffer_insert_at_cursor(buffer, *text, n);
+  }
+
+if (editable)
+  g_signal_connect(G_OBJECT(buffer), "changed",
+                   GTK_SIGNAL_FUNC(gui_text_window_update), text);
+
+return(swin);
+}
+
+/**********************************/
+/* vbox in a frame layout shorcut */
+/**********************************/
+GtkWidget *gui_frame_vbox(const gchar *label, gint p1, gint p2, GtkWidget *box)
+{
+GtkWidget *frame, *vbox;
+
+g_assert(box != NULL);
+
+frame = gtk_frame_new(label);
+gtk_box_pack_start(GTK_BOX(box), frame, p1, p2, 0); 
+gtk_container_set_border_width(GTK_CONTAINER(frame), 0.5*PANEL_SPACING);
+vbox = gtk_vbox_new(FALSE, PANEL_SPACING);
+gtk_container_add(GTK_CONTAINER(frame), vbox);
+
+return(vbox);
+}
+
+/***************************************************************/
+/* horizontal packing function for label plus some data widget */
+/***************************************************************/
+void gui_hbox_pack(GtkWidget *box, gchar *text, GtkWidget *w, gint pack)
+{
+gint pack1, pack2;
+GtkWidget *label, *hbox;
+
+g_assert(box != NULL);
+
+/* TODO - the pack argument is intended to allow the possibility of different packing styles */
+/* TODO - switch (pack) {} */
+pack1 = TRUE;
+pack2 = TRUE;
+
+hbox = gtk_hbox_new(FALSE, 0);
+gtk_box_pack_start(GTK_BOX(box), hbox, pack1, pack2, 0);
+
+if (text)
+  {
+  label = gtk_label_new(text);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
+  gtk_box_pack_end(GTK_BOX(hbox), w, FALSE, FALSE, 0);
+  }
+if (w)
+  gtk_box_pack_end(GTK_BOX(hbox), w, TRUE, TRUE, 0);
+}
+
+/**************************************/
+/* pulldown menu convenience function */
+/**************************************/
+/* TODO - when ready - will replace the old gui_pulldown_new() function */
+gpointer gui_pd_new(GSList *list, gint active, gpointer callback, gpointer argument)
+{
+GSList *item;
+GtkTreeModel *treemodel;
+GtkTreePath *treepath;
+GtkTreeIter iter;
+GtkWidget *w = NULL;
+
+#if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 4
+/* build the widget and associated drop down list */
+w = gtk_combo_box_new_text();
+for (item=list ; item ; item=g_slist_next(item))
+  gtk_combo_box_append_text(GTK_COMBO_BOX(w), item->data);
+
+/* set the currently active item */
+treemodel = gtk_combo_box_get_model(GTK_COMBO_BOX(w));
+treepath = gtk_tree_path_new_from_indices(active, -1);
+gtk_tree_model_get_iter(treemodel, &iter, treepath);
+gtk_combo_box_set_active_iter(GTK_COMBO_BOX(w), &iter);
+
+/* attach callback (if any) */
+if (callback)
+  g_signal_connect(GTK_OBJECT(w), "changed", GTK_SIGNAL_FUNC(callback), argument);
+#endif
+
+return(w);
+}
+
+/*******************************************************/
+/* retrieve the current active pulldown menu item text */
+/*******************************************************/
+gchar *gui_pd_text(gpointer pd)
+{
+return(gtk_combo_box_get_active_text(GTK_COMBO_BOX(pd)));
+}
+
+/**************************************/
+/* pulldown menu convenience function */
+/**************************************/
+gpointer gui_pulldown_new(const gchar *text, GList *list, gint edit, GtkWidget *box)
+{
+GtkWidget *hbox, *label, *combo;
+
+combo = gtk_combo_new();
+gtk_combo_set_popdown_strings(GTK_COMBO(combo), list);
+gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(combo)->entry), edit);
+
+if (box)
+  {
+/* create the text/spinner layout */
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(box), hbox, TRUE, TRUE, 0);
+  if (text)
+    {
+    label = gtk_label_new(text);
+    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
+    gtk_box_pack_end(GTK_BOX(hbox), combo, FALSE, FALSE, 0);
+    }
+  else
+    gtk_box_pack_end(GTK_BOX(hbox), combo, TRUE, TRUE, 0);
+  }
+return(GTK_COMBO(combo)->entry);
+}
+
+/********************************/
+/* pulldown menu data retrieval */
+/********************************/
+const gchar *gui_pulldown_text(gpointer w)
+{
+if (GTK_IS_ENTRY(w))
+  return(gtk_entry_get_text(GTK_ENTRY(w)));
+return(NULL);
+}
+
+/*******************************/
+/* labelled colour editing box */
+/*******************************/
+void gui_colour_box(const gchar *text, gdouble *rgb, GtkWidget *box)
+{
+GtkWidget *hbox, *label, *button;
+GdkColor colour;
+
+hbox = gtk_hbox_new(FALSE, 0);
+gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
+if (text)
+  {
+  label = gtk_label_new(text);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+  }
+
+button = gtk_button_new_with_label("    ");
+gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+g_signal_connect(GTK_OBJECT(button), "clicked",
+                 GTK_SIGNAL_FUNC(dialog_colour_new), rgb);
+
+colour.red   = rgb[0]*65535.0;
+colour.green = rgb[1]*65535.0;
+colour.blue  = rgb[2]*65535.0;
+gtk_widget_modify_bg(button, GTK_STATE_NORMAL, &colour);
+}
