@@ -30,7 +30,8 @@
 #include "coords.h"
 #include "edit.h"
 #include "matrix.h"
-#include "gtkshorts.h"
+#include "sginfo.h"
+#include "shortcuts.h"
 #include "interface.h"
 #include "dialog.h"
 #include "opengl.h"
@@ -1446,6 +1447,7 @@ for( i = 0 ; i < ImproperAxesCount ; i++ )
 void update_symmetry(struct model_pak *data)
 {
 gint i, j , k, n, m, p;
+gdouble angle;
 GString *code;
 struct symop_pak *symops;
 
@@ -1526,7 +1528,13 @@ for (i=MaxAxisOrder ; i>=2 ; i--)
   for (m=0 ; m<NormalAxesCounts[i] ; m++)
     {
     (symops+j)->type = PAXIS;
+    angle = 2.0*G_PI / (gdouble) i;
+
+/*
     get_rot_matrix((symops+j)->matrix, PAXIS, &(NormalAxes[p++]->direction[0]), i);
+*/
+matrix_v_rotation((symops+j)->matrix, &(NormalAxes[p++]->direction[0]), angle);
+
     j++;
     }
   }
@@ -1659,12 +1667,16 @@ Atoms = g_malloc0(g_slist_length(data->cores) * sizeof(OBJECT));
 AtomsCount=0;
 for (list=data->cores ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
   if (core->status & (DELETED | HIDDEN))
     continue;
 
   Atoms[AtomsCount].type = core->atom_code;
+/*
   ARR3SET(Atoms[AtomsCount].x, core->x);
+*/
+  ARR3SET(Atoms[AtomsCount].x, core->rx);
+
   AtomsCount++;
   }
 
@@ -1677,10 +1689,13 @@ return(0);
 /*************/
 /* main call */
 /*************/
-void seek_sym(GtkWidget *w, struct model_pak *data)
+void gui_symmetry_analyse(void)
 {
+struct model_pak *data = sysenv.active_model;
+
 /* checks */
-g_return_if_fail(data != NULL);
+if (!data)
+  return;
 if (!g_slist_length(data->cores))
   return;
 
@@ -1727,38 +1742,15 @@ if (SymmetryCode)
 if (InversionCentersCount)
   g_free(InversionCenters);
 
-/* update dialog */
-update_symlist(data);
+/* update widget */
+gui_active_refresh();
 }
 
-/*********************************/
-/* display the symmetry elements */
-/*********************************/
-void update_symlist(struct model_pak *model)
-{
-gint i;
-gchar *text;
-GtkTreeIter iter;
-
-/* re-populate */
-gtk_list_store_clear(symmetry_ls);
-for (i=0 ; i<model->symmetry.num_items ; i++)
-  {
-  gtk_list_store_append(symmetry_ls, &iter);
-
-  text = *((model->symmetry.items)+i);
-
-  gtk_list_store_set(symmetry_ls, &iter, 0, text, -1);
-  }
-
-/* point group */
-gtk_label_set_text(GTK_LABEL(model->symmetry.pg_entry), model->symmetry.pg_name);
-}
-/*******************************/
-/* verify a symmetry operation */
-/*******************************/
+/*****************************/
+/* hide a symmetry operation */
+/*****************************/
 #define DEBUG_DELETE_SYMOP 0
-gint delete_symop(struct model_pak *data, gint num)
+gint gui_symmetry_hide(struct model_pak *data, gint num)
 {
 gint flag, cycle;
 gdouble *mat, vec[3], dx[3];
@@ -1774,7 +1766,7 @@ P3MAT("",mat);
 
 for (list1=data->cores ; list1 ; list1=g_slist_next(list1))
   {
-  core1 = (struct core_pak *) list1->data;
+  core1 = list1->data;
   if (core1->status & HIDDEN)
     continue;
 
@@ -1791,7 +1783,7 @@ for (list1=data->cores ; list1 ; list1=g_slist_next(list1))
     flag=0;
     for (list2=data->cores ; list2 ; list2=g_slist_next(list2))
       {
-      core2 = (struct core_pak *) list2->data;
+      core2 = list2->data;
       if (core1->atom_code != core2->atom_code)
         continue;
 
@@ -1827,145 +1819,207 @@ return(0);
 /**********************************/
 /* reduce to asymetric components */
 /**********************************/
-void unique_toggle(GtkWidget *w, struct model_pak *data)
+void gui_symmetry_toggle(void)
 {
 gint i;
 GSList *list;
 struct core_pak *core;
+struct model_pak *model = sysenv.active_model;
 
-/* toggle */
-data->asym_on ^= 1;
-if (data->asym_on)
+if (!model)
+  return;
+
+model->asym_on ^= 1;
+
+if (model->asym_on)
   {
-/* permanently remove deleted atoms */
-  delete_commit(data);
-/* cycle through */
-  for (i=data->symmetry.num_symops ; i-- ; )
-    delete_symop(data, i);
+  switch (model->periodic)
+    {
+    case 3:
+      for (list=model->cores ; list ; list=g_slist_next(list))
+        {
+        core = list->data;
+        if (!core->primary)
+          core->status |= HIDDEN;
+        }
+      break;
+
+    case 0:
+      for (i=model->symmetry.num_symops ; i-- ; )
+        gui_symmetry_hide(model, i);
+      break;
+
+    }
   }
 else
   {
-  for (list=data->cores ; list ; list=g_slist_next(list))
+/* remove hidden flag */
+  for (list=model->cores ; list ; list=g_slist_next(list))
     {
-    core = (struct core_pak *) list->data;
+    core = list->data;
     core->status &= ~HIDDEN;
     }
   }
 
-/* update - (only if atoms were hidden/deleted) */
-init_objs(REDO_COORDS, data);
-calc_bonds(data);
-calc_mols(data);
-
 redraw_canvas(SINGLE);
 }
 
-/***************************/
-/* symmetry dialog cleanup */
-/***************************/
-void symmetry_cleanup(void)
+/*******************************/
+/* symmetry information widget */
+/*******************************/
+void gui_symmetry_refresh(GtkWidget *box)
 {
-gtk_list_store_clear(symmetry_ls);
-symmetry_ls = NULL;
-symmetry_tv = NULL;
-}
-
-/*******************/
-/* symmetry dialog */
-/*******************/
-/* TODO - periodicity counterpart, ie sym elements/pt group/asym etc. */
-/* => when model is loaded, if not periodic -> run the above code */
-/* might have problems with editing/minimization resulting in diff */
-/* symmetry than prev calc'd (especially when removing sym elements) */
-void symmetry_dialog(void)
-{
-gpointer dialog;
-GtkWidget *window, *swin, *frame, *mainvbox, *vbox, *hbox;
-GtkWidget *label;
+gint i;
+gchar *value;
+GtkTreeIter iter;
 GtkCellRenderer *renderer;
 GtkTreeViewColumn *column;
-struct model_pak *data;
+static GtkListStore *ls=NULL;
+static GtkWidget *tv=NULL, *frame, *vbox;
+gchar *label[] = {"space group", "system", "atoms",
+                      "a", "b", "c", "alpha", "beta", "gamma",
+                      "surface area", "volume", NULL};
 
-/* checks */
-data = sysenv.active_model;
-if (!data)
+if (box)
+  {
+g_assert(ls == NULL);
+g_assert(tv == NULL);
+
+/* new tree list */
+  ls = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+  tv = gtk_tree_view_new_with_model(GTK_TREE_MODEL(ls));
+  gtk_box_pack_start(GTK_BOX(box), tv, TRUE, TRUE, 0);
+
+/* setup cell renderers */
+  renderer = gtk_cell_renderer_text_new();
+  column = gtk_tree_view_column_new_with_attributes(" ", renderer, "text", 0, NULL);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);
+  renderer = gtk_cell_renderer_text_new();
+  column = gtk_tree_view_column_new_with_attributes(" ", renderer, "text", 1, NULL);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);
+  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tv), FALSE);
+
+/* currently, allow nothing to be selected */
+  gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(tv)),
+                              GTK_SELECTION_NONE);
+
+/* actions */
+  frame = gtk_frame_new(NULL);
+  gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);
+  gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
+  vbox = gtk_vbox_new(TRUE, 1);
+  gtk_container_add(GTK_CONTAINER(frame), vbox);
+
+  gui_button_x("Guess Pointgroup ", gui_symmetry_analyse, NULL, vbox);
+  gui_button_x("Asymmetric unit ", gui_symmetry_toggle, NULL, vbox);
+  }
+else
+  {
+  struct model_pak *model = sysenv.active_model;
+
+g_assert(ls != NULL);
+g_assert(tv != NULL);
+
+gtk_list_store_clear(ls);
+
+if (!model)
   return;
 
-/* retrieve a suitable dialog */
-dialog = dialog_request(SYMMETRY, "Symmetry analyzer", NULL, symmetry_cleanup, data);
-if (!dialog)
-  return;
-window = dialog_window(dialog);
-gtk_window_set_default_size(GTK_WINDOW(window), 250, 250);
+/* TODO - put symmetry info in its own pulldown? */
+/* Symmetry part */
+if (model->periodic)
+  {
+/* populate with space group info only if 3D */
+  if (model->periodic == 3)
+    i=0;
+  else
+    i=3;
 
-/* main dialog vbox */
-mainvbox = gtk_vbox_new(FALSE, 0);
-gtk_container_add(GTK_CONTAINER(GTK_BOX(GTK_DIALOG(window)->vbox)),mainvbox);
-gtk_container_set_border_width (GTK_CONTAINER(mainvbox), PANEL_SPACING);
+  while (label[i])
+    {
+    value = NULL;
+    switch (i)
+      {
+      case 0:
+        value = g_strdup_printf("%s", model->sginfo.spacename);
+        break;
+      case 1:
+        value = g_strdup_printf("%s", model->sginfo.latticename);
+        break;
+/*
+    case 2:
+      value = g_strdup_printf("%d (%d)", model->num_atoms, model->num_asym);
+      break;
+*/
+      case 3:
+        value = g_strdup_printf("%8.4f", model->pbc[0]);
+        break;
+      case 4:
+        if (model->periodic > 1)
+          value = g_strdup_printf("%8.4f", model->pbc[1]);
+        break;
+      case 5:
+        if (model->periodic > 2)
+          value = g_strdup_printf("%8.4f", model->pbc[2]);
+        break;
+      case 6:
+        if (model->periodic > 2)
+          value = g_strdup_printf("%8.2f", R2D*model->pbc[3]);
+        break;
+      case 7:
+        if (model->periodic > 2)
+          value = g_strdup_printf("%8.2f", R2D*model->pbc[4]);
+        break;
+      case 8:
+        if (model->periodic > 1)
+          value = g_strdup_printf("%8.2f", R2D*model->pbc[5]);
+        break;
+      case 9:
+        if (model->periodic == 2)
+          value = g_strdup_printf("%.4f", model->area);
+        break;
+      case 10:
+        if (model->periodic == 3)
+          value = g_strdup_printf("%.2f", model->volume);
+        break;
+      } 
+/* only add if a valid value */
+    if (value)
+      {
+      gtk_list_store_append(ls, &iter);
+      gtk_list_store_set(ls, &iter, 0, label[i], -1);
+      gtk_list_store_set(ls, &iter, 1, value, -1);
+/*
+      msb_list = g_slist_prepend(msb_list, value);
+*/
+      }
+    i++;
+    }
+  }
+else
+  {
+  if (model->id == MORPH)
+    {
+    i = PG_Index(model->sginfo.pointgroup);
 
-/* model name */
-frame = gtk_frame_new(NULL);
-gtk_box_pack_start(GTK_BOX(mainvbox),frame,FALSE,FALSE,0); 
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-hbox = gtk_hbox_new(FALSE,5);
-gtk_container_add(GTK_CONTAINER(frame), hbox);
-gtk_container_set_border_width(GTK_CONTAINER(GTK_BOX(hbox)), PANEL_SPACING);
+    gtk_list_store_append(ls, &iter);
+    gtk_list_store_set(ls, &iter, 0, "point group", 1, PG_Names[i], -1);
+    }
+  else
+    {
+/* populate with symmetry info */
+    for (i=0 ; i<model->symmetry.num_items ; i++)
+      {
+      gtk_list_store_append(ls, &iter);
+      value = *((model->symmetry.items)+i);
+      gtk_list_store_set(ls, &iter, 0, value, -1);
+      }
+/* point group */
+    gtk_list_store_append(ls, &iter);
+    gtk_list_store_set(ls, &iter, 0, "point group", 1, model->symmetry.pg_name, -1);
+    }
+  }
 
-label = gtk_label_new(g_strdup_printf("Model: %s",data->basename));
-gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-
-/* main list */
-frame = gtk_frame_new("Symmetry elements");
-gtk_box_pack_start(GTK_BOX(mainvbox),frame,TRUE,TRUE,0); 
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-/* create a vbox in the frame */
-vbox = gtk_vbox_new(FALSE, 0);
-gtk_container_add (GTK_CONTAINER(frame), vbox);
-gtk_container_set_border_width (GTK_CONTAINER(GTK_BOX(vbox)), PANEL_SPACING);
-/* scrolled model pane */
-swin = gtk_scrolled_window_new(NULL, NULL);
-gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin),
-                               GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-gtk_box_pack_start(GTK_BOX(vbox), swin, TRUE, TRUE, 0);
-
-/* tree list */
-symmetry_ls = gtk_list_store_new(1, G_TYPE_STRING);
-symmetry_tv = gtk_tree_view_new_with_model(GTK_TREE_MODEL(symmetry_ls));
-gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(swin), symmetry_tv);
-renderer = gtk_cell_renderer_text_new();
-column = gtk_tree_view_column_new_with_attributes(" ", renderer, "text", 0, NULL);
-gtk_tree_view_append_column(GTK_TREE_VIEW(symmetry_tv), column);
-gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(symmetry_tv), FALSE);
-
-
-
-/* guessed point group */
-frame = gtk_frame_new(NULL);
-gtk_box_pack_start(GTK_BOX(mainvbox),frame,FALSE,FALSE,0); 
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-hbox = gtk_hbox_new(FALSE,5);
-gtk_container_add(GTK_CONTAINER(frame), hbox);
-gtk_container_set_border_width(GTK_CONTAINER(GTK_BOX(hbox)), PANEL_SPACING);
-
-label = gtk_label_new("Apparent point group: ");
-gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-
-/* make new, if none exists */
-data->symmetry.pg_entry = gtk_label_new(data->symmetry.pg_name);
-gtk_box_pack_start(GTK_BOX(hbox), data->symmetry.pg_entry, FALSE, FALSE, 0);
-
-/* terminating buttons */
-gtksh_button(" Analyse ", seek_sym, (gpointer) data,
-             (GTK_DIALOG(window)->action_area), TT);
-gtksh_button(" Condense ", unique_toggle, (gpointer) data,
-             (GTK_DIALOG(window)->action_area), TT);
-gtksh_stock_button(GTK_STOCK_CLOSE, dialog_destroy, dialog,
-                   (GTK_DIALOG(window)->action_area));
-
-
-/* done, draw and exit */
-update_symlist(data);
-gtk_widget_show_all(window);
-pick_model(data);
+  }
 }
 
