@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2000 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -39,7 +39,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "task.h"
 #include "file.h"
 #include "parse.h"
-#include "gtkshorts.h"
+#include "shortcuts.h"
 #include "interface.h"
 #include "dialog.h"
 
@@ -54,16 +54,15 @@ extern struct sysenv_pak sysenv;
 gdouble max_threads=1;
 gint task_show_running = 1;
 gint task_show_queued = 0;
-gint task_show_completed = 0;
+gint task_show_completed = 1;
 
 GtkWidget *task_list=NULL;
-GtkWidget *task_label1, *task_label2, *task_label3;
 gint selected_task = -1;
 gint task_info_pid = 0;
 gint task_thread_pid = -1;
 GArray *psarray = NULL;
 GtkListStore *task_list_ls=NULL;
-GtkWidget *task_list_tv=NULL;
+GtkWidget *task_list_tv=NULL, *task_text_view;
 char *strptime(const char *, const char *, struct tm *);
 
 /**************************/
@@ -171,19 +170,11 @@ curr_process.parent = curr_process.child = curr_process.sister = -1;
  * SG's - 'ps -Ao "pid ppid pcpu vsz etime comm"
  * 			(unfortunately, no %mem - and vsz is absolute :-(
  */
-#if DEBUG_CALC_TASK_INFO
-printf("making ps command string\n");
+
 #ifdef __sgi
 cmd = g_strdup_printf("ps -Ao \"pid ppid pcpu vsz etime comm\"");
 #else
-cmd = g_strdup_printf("ps -o \"lstart pid ppid %%cpu %%mem time ucomm\"");
-#endif
-#else
-#ifdef __sgi
-cmd = g_strdup_printf("ps -Ao \"pid ppid pcpu vsz etime comm\"");
-#else
-cmd = g_strdup_printf("ps -axo \"lstart pid ppid %%cpu %%mem time ucomm\"");
-#endif
+cmd = g_strdup_printf("ps axo \"lstart pid ppid %%cpu %%mem time ucomm\"");
 #endif
 
 /* run ps command */
@@ -191,14 +182,14 @@ fp = popen(cmd, "r");
 g_free(cmd);
 if (!fp)
   {
-  show_text(ERROR, "unable to launch ps command\n");
+  gui_text_show(ERROR, "unable to launch ps command\n");
   return;
   }
 
 /* skip title line */
 if (fgetline(fp, line))
   {
-  show_text(ERROR, "unable to read first line of output from ps command\n");
+  gui_text_show(ERROR, "unable to read first line of output from ps command\n");
   return;
   }
     
@@ -219,7 +210,7 @@ printf("%s", line);
 #endif
   if (line_no_time == NULL)
     {
-    show_text(ERROR, "file produced by ps not understood\n");
+    gui_text_show(ERROR, "file produced by ps not understood\n");
     return;
     }
   curr_process.start_time = mktime(&start_tm);
@@ -238,7 +229,7 @@ printf("%s", line);
 pclose(fp);
 
 /* Build the process hierarchy. Every process marks itself as first child */
-/* of it's parent or as sister of first child of it's parent */
+/* of it's parent or as sister of first child of its parent */
 /* algorithm taken from pstree.c by Fred Hucht */
 
 for (me = 0; me < psarray->len; me++)
@@ -332,17 +323,45 @@ printf("pid: %d pcpu: %.1f pmem: %.1f time: %s\n", tdata->pid, tdata->pcpu, tdat
 free_slist(palist);
 */
 
-/* hack for round off possibly giving >100% */
+/* prevent rounding errors giving >100% */
 if (tdata->pcpu > 100.0)
   tdata->pcpu = 100.0;
 if (tdata->pmem > 100.0)
   tdata->pmem = 100.0;
 }
 
+/**********************************************/
+/* get pointer to the currently selected task */
+/**********************************************/
+gpointer task_selected(void)
+{
+gpointer task;
+GtkTreeModel *treemodel;
+GtkTreeSelection *selection;
+GtkTreeIter iter;
+
+treemodel = gtk_tree_view_get_model(GTK_TREE_VIEW(task_list_tv));
+selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(task_list_tv));
+
+if (gtk_tree_model_get_iter_first(treemodel, &iter))
+  {
+  do
+    {
+    if (gtk_tree_selection_iter_is_selected(selection, &iter))
+      {
+      gtk_tree_model_get(treemodel, &iter, 6, &task, -1);
+      return(task);
+      }
+    }
+  while (gtk_tree_model_iter_next(treemodel, &iter));
+  }
+return(NULL);
+}
+
 /**********************************/
 /* get the currently selected row */
 /**********************************/
-gint task_list_selected(void)
+gint task_selected_row(void)
 {
 gint row=0;
 GtkTreeModel *treemodel;
@@ -370,107 +389,167 @@ return(-1);
 /*********************************/
 gint update_task_info(void)
 {
-gint i, type, row, num_rows=0;
-gchar *txt, *info[6];
+gint i, type, update, row, num_rows=0;
+gchar *txt;
 GSList *tlist=NULL;
 GtkTreeIter iter;
 GtkTreeModel *treemodel;
 GtkTreeSelection *selection;
+GtkTextMark *mark;
+GtkTextIter text_iter;
+GtkTextBuffer *buffer;
 struct task_pak *task=NULL;
+static gpointer previous_task=NULL;
 
 /* checks */
 if (!dialog_exists(TASKMAN, NULL))
   return(TRUE);
-
 if (!task_list_ls)
   return(TRUE);
 
-/* TODO - if no running or active task, don't bother to update? */
-
-/* store */
-row = task_list_selected();
-
-/* update task summary info */
-txt = g_strdup_printf("%d", g_thread_pool_get_num_threads(sysenv.thread_pool));
-gtk_label_set_text(GTK_LABEL(task_label1), txt);
-g_free(txt);
-
-txt = g_strdup_printf("%d", g_thread_pool_unprocessed(sysenv.thread_pool));
-gtk_label_set_text(GTK_LABEL(task_label2), txt);
-g_free(txt);
-
-/*
-txt = g_strdup_printf("%d", g_thread_pool_get_max_threads(sysenv.thread_pool));
-gtk_label_set_text(GTK_LABEL(task_label3), txt);
-g_free(txt);
-*/
-
-
-/* remove the old tree */
-gtk_list_store_clear(task_list_ls);
-
-for (type=RUNNING ; type<=COMPLETED ; type++)
+/* NEW - update using control file */
+task = task_selected();
+buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(task_text_view));
+if (task)
   {
-  switch (type)
+  if (previous_task != task)
     {
-    case RUNNING:
-      if (!task_show_running)
-        continue;
-      break;
-    case QUEUED:
-      if (!task_show_queued)
-        continue;
-      break;
-    case COMPLETED:
-      if (!task_show_completed)
-        continue;
-      break;
+/* complete refresh */
+    task->status_index = 0;
+    gtk_text_buffer_set_text(buffer, "", 0);
+    update = TRUE;
     }
+  else
+    update = FALSE;
 
-/* re-populate */
-for (tlist=sysenv.task_list ; tlist ; tlist=g_slist_next(tlist)) 
-  {
-  task = (struct task_pak *) tlist->data;
-
-  if (task->status != type)
-    continue;
-
-  gtk_list_store_append(task_list_ls, &iter);
-
-  switch(task->status)
+  switch (task->status)
     {
-    case QUEUED:
-      gtk_list_store_set(task_list_ls, &iter, 1, task->label, 2, "Queued", -1);
+    case RUNNING:
+/* running - always update */
+      update = TRUE;
       break;
 
-    case RUNNING:
-      calc_task_info(task);
-      info[0] = g_strdup_printf("%d", task->pid);
-      if (task->label)
-        info[1] = g_strdup(task->label);
-      else
-        info[1] = g_strdup(" Unknown ");
-      if (task->progress > 0.0)
-        info[2] = g_strdup_printf("%5.1f%%", task->progress);
-      else
-        info[2] = g_strdup("Running");
-      info[3] = g_strdup_printf("%5.1f", task->pcpu);
-      info[4] = g_strdup_printf("%5.1f", task->pmem);
-      info[5] = g_strdup_printf(" %s", task->time);
-      for (i=0 ; i<6 ; i++)
+    case KILLED:
+    case COMPLETED:
+      if (task->status_fp)
         {
-        gtk_list_store_set(task_list_ls, &iter, i, info[i], -1);
-        g_free(info[i]);
+/* completed - only update if status file is still open */
+        task->status_index = 0;
+        update = TRUE;
         }
       break;
-
-    case COMPLETED:
-      gtk_list_store_set(task_list_ls, &iter, 1, task->label, 2, "Completed", -1);
-      break;
     }
-  num_rows++;
+
+  if (update)
+    {
+/* refresh the status message and print */
+    task_status_update(task);
+    txt = (task->status_text)->str;
+    if (task->status_index >= 0)
+      {
+      i = strlen(txt+task->status_index);
+      if (i)
+        {
+/* display only most recent lines */
+/*
+        gtk_text_buffer_set_text(buffer, txt+task->status_index, i);
+*/
+
+/* append to the whole thing */
+        gtk_text_buffer_insert_at_cursor(buffer, txt+task->status_index, i);
+        task->status_index += i;
+
+/* force scroll to end of buffer by default */
+        mark = gtk_text_buffer_get_insert(buffer);
+        gtk_text_buffer_get_end_iter(buffer, &text_iter);
+        gtk_text_buffer_move_mark(buffer, mark, &text_iter);
+        gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(task_text_view),
+                                     mark, 0.0, FALSE, 0, 0);
+        }
+      }
+    }
+  previous_task = task;
+  }
+else
+  {
+  gtk_text_buffer_set_text(buffer, "", 0);
+  previous_task = NULL;
   }
 
+/* remove deleted tasks */
+/* CURRENT - NEVER allow this? so user can also look at status file of completed jobs */
+tlist = sysenv.task_list;
+while (tlist)
+  {
+  task = tlist->data;
+  tlist = g_slist_next(tlist);
+  if (task->status == REMOVED)
+    {
+    sysenv.task_list = g_slist_remove(sysenv.task_list, task);
+    task_free(task);
+    }
+  }
+
+/* clear list and re-populate according to status */
+row = task_selected_row();
+gtk_list_store_clear(task_list_ls);
+for (type=RUNNING ; type<=COMPLETED ; type++)
+  {
+/* fill out the list with current status type */
+  for (tlist=sysenv.task_list ; tlist ; tlist=g_slist_next(tlist))
+    {
+    task = tlist->data;
+    if (task->status != type)
+      continue;
+
+    gtk_list_store_append(task_list_ls, &iter);
+
+    if (task->label)
+      gtk_list_store_set(task_list_ls, &iter, 1, task->label, -1);
+    else
+      gtk_list_store_set(task_list_ls, &iter, 1, "Unknown", -1);
+
+    switch(task->status)
+      {
+      case QUEUED:
+        gtk_list_store_set(task_list_ls, &iter, 2, "Queued", -1);
+        break;
+
+      case RUNNING:
+        calc_task_info(task);
+        txt = g_strdup_printf("%d", task->pid);
+        gtk_list_store_set(task_list_ls, &iter, 0, txt, -1);
+        g_free(txt);
+        if (task->progress > 0.0)
+          txt = g_strdup_printf("%5.1f%%", task->progress);
+        else
+          txt = g_strdup("Running");
+        gtk_list_store_set(task_list_ls, &iter, 2, txt, -1);
+        g_free(txt);
+        txt = g_strdup_printf("%5.1f", task->pcpu);
+        gtk_list_store_set(task_list_ls, &iter, 3, txt, -1);
+        g_free(txt);
+        txt = g_strdup_printf("%5.1f", task->pmem);
+        gtk_list_store_set(task_list_ls, &iter, 4, txt, -1);
+        g_free(txt);
+        txt = g_strdup_printf(" %s", task->time);
+        gtk_list_store_set(task_list_ls, &iter, 5, txt, -1);
+        g_free(txt);
+        break;
+
+      case KILLED:
+        gtk_list_store_set(task_list_ls, &iter, 2, "Killed", -1);
+        break;
+
+      case COMPLETED:
+        gtk_list_store_set(task_list_ls, &iter, 2, "Completed", -1);
+        break;
+      }
+  
+/* NEW - add the task pointer itself */
+    gtk_list_store_set(task_list_ls, &iter, 6, task, -1);
+    num_rows++;
+    }
   }
 
 /* restore selected row, or the previous (if possible) if deleted  */
@@ -490,26 +569,54 @@ if (row >= 0)
 /* NEW */
 sysenv.max_threads = max_threads;
 g_thread_pool_set_max_threads(sysenv.thread_pool, sysenv.max_threads, NULL);
-/* this doesn't stop the apparent mem leak */
 g_thread_pool_stop_unused_threads();
 
 return(TRUE);
 }
 
-/**************************/
-/* stop all current tasks */
-/**************************/
-void task_remove_queued(GtkWidget *w, gpointer data)
+/***********************************/
+/* recursively kill a process tree */
+/***********************************/
+void task_kill_tree(gint child)
 {
-GSList *list;
-struct task_pak *task;
+gchar *cmd;
+struct task_pak *process;
 
-for (list=sysenv.task_list ; list ; list=g_slist_next(list))
+while (EXIST(child))
   {
-  task = list->data;
-/* TODO - should use a task mutex lock */
-  if (task->status == QUEUED)
-    task->status = KILLED;
+  process = &g_array_index(psarray, struct task_pak, child);
+/* recurse down the tree */
+  task_kill_tree(process->child);
+/* terminate process */
+  cmd = g_strdup_printf("kill TERM %d >& /dev/null", process->pid);
+  system(cmd);
+  g_free(cmd);
+/* get next process at this level */
+  child = process->sister;
+  }
+}
+
+/***********************/
+/* stop a running task */
+/***********************/
+void task_kill_running(struct task_pak *task)
+{
+gint i;
+struct task_pak *process;
+
+/* get task and bottom level process pid */
+if (task && psarray)
+  {
+  for (i=psarray->len ; i-- ; )
+    {
+    process = &g_array_index(psarray, struct task_pak, i);
+    if (process->pid == task->pid)
+      {
+      task_kill_tree(process->child);
+      break;
+      }
+    }
+  task->status = KILLED;
   }
 }
 
@@ -526,15 +633,61 @@ while (list)
   {
   task = list->data;
   list = g_slist_next(list);
-  if (task->status == COMPLETED)
+  if (task->status == COMPLETED || task->status == KILLED)
     {
-/* TODO - should use a task mutex lock */
     sysenv.task_list = g_slist_remove(sysenv.task_list, task);
-    g_free(task);
+    task_free(task);
     }
   }
 }
 
+/**********************************************/
+/* terminate selected running or queued tasks */
+/**********************************************/
+void task_kill_selected(void)
+{
+struct task_pak *task;
+
+task = task_selected();
+if (task)
+  {
+  switch (task->status)
+    {
+    case RUNNING:
+      task_kill_running(task);
+      break;
+
+    case QUEUED:
+      task->status = KILLED;
+      break;
+    }
+  }
+}
+
+/*****************************************/
+/* terminate all running or queued tasks */
+/*****************************************/
+void task_kill_all(void)
+{
+GSList *list;
+struct task_pak *task;
+
+/* remove all queued first - to ensure they aren't executed */
+for (list=sysenv.task_list ; list ; list=g_slist_next(list))
+  {
+  task = list->data;
+  if (task->status == QUEUED)
+    task->status = KILLED;
+  }
+
+for (list=sysenv.task_list ; list ; list=g_slist_next(list))
+  {
+  task = list->data;
+  if (task->status == RUNNING)
+    task_kill_running(task);
+  }
+}
+
 /***********************/
 /* task dialog cleanup */
 /***********************/
@@ -551,12 +704,9 @@ task_list_tv = NULL;
 void task_dialog(void)
 {
 gint i;
-gchar *txt;
-gchar *titles[6] = {"  PID  ", "       Job       ", "   Status   ", 
-                    "  \% CPU  ", "  \% Mem  ", "  Time  "};
+gchar *titles[6] = {"  PID  ", "       Job       ", "   Status   ", "  \% CPU  ", "  \% Mem  ", "  Time  "};
 gpointer dialog;
-GtkWidget *window, *swin, *frame;
-GtkWidget *vbox, *hbox, *label;
+GtkWidget *window, *swin, *frame, *vbox, *hbox;
 GtkCellRenderer *renderer;
 GtkTreeViewColumn *column;
 
@@ -565,51 +715,24 @@ dialog = dialog_request(TASKMAN, "Task Manager", NULL, task_cleanup, NULL);
 if (!dialog)
   return;
 window = dialog_window(dialog);
-gtk_window_set_default_size(GTK_WINDOW(window), 600, 340);
+gtk_window_set_default_size(GTK_WINDOW(window), 600, 600);
 
-/* Frame */
-frame = gtk_frame_new (NULL);
-gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox),frame,FALSE,FALSE,0); 
-gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
-vbox = gtk_vbox_new(TRUE, 0);
-gtk_container_add(GTK_CONTAINER(frame), vbox);
-
-/* create a hbox in the frame */
+/* task thread setup */
 hbox = gtk_hbox_new(FALSE, 0);
-gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-label = gtk_label_new("Number of running tasks");
-gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-txt = g_strdup_printf("%d", g_thread_pool_get_num_threads(sysenv.thread_pool));
-task_label1 = gtk_label_new(txt);
-gtk_box_pack_end(GTK_BOX(hbox), task_label1, FALSE, FALSE, 0);
-g_free(txt);
+gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), hbox, FALSE, FALSE, 0); 
+gtk_container_set_border_width(GTK_CONTAINER(hbox), PANEL_SPACING);
+gui_direct_spin("Maximum number of running tasks", &max_threads, 1, 32, 1, NULL, NULL, hbox);
 
-/* create a hbox in the frame */
-hbox = gtk_hbox_new(FALSE, 0);
-gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-label = gtk_label_new("Number of queued tasks");
-gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-txt = g_strdup_printf("%d", g_thread_pool_unprocessed(sysenv.thread_pool));
-task_label2 = gtk_label_new(txt);
-gtk_box_pack_end(GTK_BOX(hbox), task_label2, FALSE, FALSE, 0);
-g_free(txt);
+#if _WIN32
+gtk_widget_set_sensitive(GTK_WIDGET(hbox), FALSE);
+#endif
 
-/* create a hbox in the frame */
-hbox = gtk_hbox_new(FALSE, 0);
-gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-gtksh_direct_spin("Maximum number of running tasks", &max_threads, 1, 32, 1, NULL, NULL, hbox);
-
-gtksh_direct_check("Show running tasks", &task_show_running, NULL, NULL, vbox);
-gtksh_direct_check("Show queued tasks", &task_show_queued, NULL, NULL, vbox);
-gtksh_direct_check("Show completed tasks", &task_show_completed, NULL, NULL, vbox);
-
-
-/* Frame */
-frame = gtk_frame_new (NULL);
-gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox),frame,TRUE,TRUE,0); 
+/* job list frame */
+frame = gtk_frame_new(NULL);
+gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), frame, TRUE, TRUE, 0); 
 gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING);
 /* create a vbox in the frame */
-vbox = gtk_vbox_new(FALSE, 0);
+vbox = gtk_vbox_new(FALSE, PANEL_SPACING/2);
 gtk_container_add(GTK_CONTAINER(frame), vbox);
 gtk_container_set_border_width(GTK_CONTAINER(GTK_BOX(vbox)), PANEL_SPACING);
 
@@ -619,9 +742,9 @@ gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (swin),
                                GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 gtk_box_pack_start(GTK_BOX(vbox), swin, TRUE, TRUE, 0);
 
-
-task_list_ls = gtk_list_store_new(6, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
-                                     G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+task_list_ls = gtk_list_store_new(7, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
+                                     G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
+                                     G_TYPE_POINTER);
 task_list_tv = gtk_tree_view_new_with_model(GTK_TREE_MODEL(task_list_ls));
 gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(swin), task_list_tv);
 for (i=0 ; i<6 ; i++)
@@ -631,17 +754,44 @@ for (i=0 ; i<6 ; i++)
   gtk_tree_view_append_column(GTK_TREE_VIEW(task_list_tv), column);
   }
 
+/* setup the selection handler */
+/*
+{
+GtkTreeSelection *select;
+select = gtk_tree_view_get_selection(GTK_TREE_VIEW(task_list_tv));
+g_signal_connect(G_OBJECT(select), "changed",
+                 G_CALLBACK(task_selection_changed),
+                 NULL);
+}
+*/
+
+
+/* selected task status file frame */
+gtk_box_pack_start(GTK_BOX(vbox), gtk_hseparator_new(), FALSE, FALSE, 0);
+swin = gtk_scrolled_window_new(NULL, NULL);
+gtk_box_pack_start(GTK_BOX(vbox), swin, TRUE, TRUE, 0);
+gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin),
+                               GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+task_text_view = gtk_text_view_new();
+gtk_text_view_set_editable(GTK_TEXT_VIEW(task_text_view), FALSE);
+gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(task_text_view), FALSE);
+gtk_container_add(GTK_CONTAINER(swin), task_text_view);
+
+
 /* control buttons */
-/* TODO - can we stop a running thread? */
-gtksh_icon_button(GTK_STOCK_CLEAR, "Remove queued",
-                  task_remove_queued, NULL,
+gui_icon_button(GTK_STOCK_REMOVE, "Kill selected",
+                  task_kill_selected, NULL,
                   GTK_DIALOG(window)->action_area);
 
-gtksh_icon_button(GTK_STOCK_CLEAR, "Remove completed",
+gui_icon_button(GTK_STOCK_DELETE, "Kill all",
+                  task_kill_all, NULL,
+                  GTK_DIALOG(window)->action_area);
+
+gui_icon_button(GTK_STOCK_CLEAR, "Remove completed",
                   task_remove_completed, NULL,
                   GTK_DIALOG(window)->action_area);
 
-gtksh_stock_button(GTK_STOCK_CLOSE,
+gui_stock_button(GTK_STOCK_CLOSE,
                    dialog_destroy, dialog,
                    GTK_DIALOG(window)->action_area);
 
