@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -44,7 +44,9 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "opengl.h"
 #include "numeric.h"
 #include "module.h"
+#include "library.h"
 #include "interface.h"
+#include "gui_image.h"
 
 /* main data structures */
 struct sysenv_pak sysenv;
@@ -65,6 +67,9 @@ free_slist((GSList *) sfc_list);
 void sys_free(void)
 {
 g_hash_table_destroy(sysenv.sfc_table);
+g_hash_table_destroy(sysenv.library);
+g_hash_table_destroy(sysenv.manual);
+g_hash_table_destroy(sysenv.image_table);
 module_free();
 }
 
@@ -148,7 +153,12 @@ for (;;)
     }
 
 /* povray executable */
-  if (g_ascii_strncasecmp("povray_exe",*buff,10) == 0)
+  if (g_ascii_strncasecmp("povray_p", *buff, 8) == 0)
+    {
+    if (num_tokens > 1)
+      sysenv.povray_path = g_strdup(*(buff+1));
+    }
+  if (g_ascii_strncasecmp("povray_e", *buff, 8) == 0)
     {
     if (num_tokens > 1)
       {
@@ -156,14 +166,13 @@ for (;;)
       sysenv.povray_exe = g_strdup(*(buff+1));
       }
     }
-/* auto unfrag? */
-  if (g_ascii_strncasecmp("unfragment",*buff,10) == 0)
+/* animation creation tool */
+  if (g_ascii_strncasecmp("convert_p", *buff, 9) == 0)
     {
     if (num_tokens > 1)
-      sysenv.unfragment = str_to_float(*(buff+1));
+      sysenv.convert_path = g_strdup(*(buff+1));
     }
-/* animation creation executable */
-  if (g_ascii_strncasecmp("convert_exe",*buff,11) == 0)
+  if (g_ascii_strncasecmp("convert_e", *buff, 9) == 0)
     {
     if (num_tokens > 1)
       {
@@ -171,8 +180,13 @@ for (;;)
       sysenv.convert_exe = g_strdup(*(buff+1));
       }
     }
-/* image viewing executable */
-  if (g_ascii_strncasecmp("viewer_exe",*buff,10) == 0)
+/* image viewing */
+  if (g_ascii_strncasecmp("viewer_p", *buff, 8) == 0)
+    {
+    if (num_tokens > 1)
+      sysenv.viewer_path = g_strdup(*(buff+1));
+    }
+  if (g_ascii_strncasecmp("viewer_e", *buff, 8) == 0)
     {
     if (num_tokens > 1)
       {
@@ -180,8 +194,13 @@ for (;;)
       sysenv.viewer_exe = g_strdup(*(buff+1));
       }
     }
-/* gulp executable */
-  if (g_ascii_strncasecmp("gulp",*buff,4) == 0)
+/* gulp */
+  if (g_ascii_strncasecmp("gulp_p", *buff, 6) == 0)
+    {
+    if (num_tokens > 1)
+      sysenv.gulp_path = g_strdup(*(buff+1));
+    }
+  if (g_ascii_strncasecmp("gulp_e", *buff, 6) == 0)
     {
     if (num_tokens > 1)
       {
@@ -189,15 +208,6 @@ for (;;)
       sysenv.gulp_exe = g_strdup(*(buff+1));
       }
     }
-/* file conversion executable */
-  if (g_ascii_strncasecmp("babel_exe",*buff,9) == 0)
-    {
-    if (num_tokens > 1)
-      {
-      g_free(sysenv.babel_exe);
-      sysenv.babel_exe = g_strdup(*(buff+1));
-      }
-    }
 /* OpenGL drawing font */
   if (g_ascii_strncasecmp("gl_font",*buff,7) == 0)
     if (num_tokens > 1)
@@ -223,13 +233,6 @@ for (;;)
     if (num_tokens > 1)
       sysenv.apb_on = (gint) str_to_float(*(buff+1));
 
-/* rotation adjustment box */
-/*
-  if (g_ascii_strncasecmp("rab",*buff,3) == 0)
-    if (num_tokens > 1)
-      sysenv.rab_on = (gint) str_to_float(*(buff+1));
-*/
-
 /* halo type */
   if (g_ascii_strncasecmp("halo",*buff,4) == 0)
     if (num_tokens > 1)
@@ -240,6 +243,35 @@ for (;;)
     if (num_tokens > 1)
       sysenv.render.fast_rotation = (gint) str_to_float(*(buff+1));
 
+  if (g_ascii_strncasecmp(*buff, "colour_bg", 9) == 0)
+    {
+    if (num_tokens > 3)
+      {
+      sysenv.render.bg_colour[0] = str_to_float(*(buff+1));
+      sysenv.render.bg_colour[1] = str_to_float(*(buff+2));
+      sysenv.render.bg_colour[2] = str_to_float(*(buff+3));
+      }
+    }
+  if (g_ascii_strncasecmp(*buff, "colour_morph", 11) == 0)
+    {
+    if (num_tokens > 3)
+      {
+      sysenv.render.morph_colour[0] = str_to_float(*(buff+1));
+      sysenv.render.morph_colour[1] = str_to_float(*(buff+2));
+      sysenv.render.morph_colour[2] = str_to_float(*(buff+3));
+      }
+    }
+  if (g_ascii_strncasecmp("colour_rsurf", *buff, 12) == 0)
+    {
+    if (num_tokens > 3)
+      {
+      sysenv.render.rsurf_colour[0] = str_to_float(*(buff+1));
+      sysenv.render.rsurf_colour[1] = str_to_float(*(buff+2));
+      sysenv.render.rsurf_colour[2] = str_to_float(*(buff+3));
+      }
+    }
+
+/* cleanup */
   g_strfreev(buff);
   }
 
@@ -282,20 +314,44 @@ fprintf(fp,"mtb %d\n", sysenv.mtb_on);
 fprintf(fp,"mpb %d\n", sysenv.mpb_on);
 fprintf(fp,"msb %d\n", sysenv.msb_on);
 fprintf(fp,"apb %d\n", sysenv.apb_on);
-fprintf(fp,"rab %d\n", sysenv.rab_on);
-fprintf(fp,"unfragment %d\n", sysenv.unfragment);
 fprintf(fp,"halos %d\n", sysenv.render.halos);
 fprintf(fp,"fast %d\n", sysenv.render.fast_rotation);
-fprintf(fp,"povray_exe %s\n", sysenv.povray_exe);
-fprintf(fp,"convert_exe %s\n", sysenv.convert_exe);
-fprintf(fp,"viewer_exe %s\n", sysenv.viewer_exe);
-fprintf(fp,"gulp_exe %s\n", sysenv.gulp_exe);
+
+fprintf(fp,"colour_bg %f %f %f\n", sysenv.render.bg_colour[0],
+                                   sysenv.render.bg_colour[1],
+                                   sysenv.render.bg_colour[2]);
+fprintf(fp,"colour_morph %f %f %f\n", sysenv.render.morph_colour[0],
+                                      sysenv.render.morph_colour[1],
+                                      sysenv.render.morph_colour[2]);
+fprintf(fp,"colour_rsurf %f %f %f\n", sysenv.render.rsurf_colour[0],
+                                      sysenv.render.rsurf_colour[1],
+                                      sysenv.render.rsurf_colour[2]);
+
+if (sysenv.babel_path)
+  fprintf(fp,"babel_path %s\n", sysenv.babel_path);
+if (sysenv.convert_path)
+  fprintf(fp,"convert_path %s\n", sysenv.convert_path);
+if (sysenv.gulp_path)
+  fprintf(fp,"gulp_path %s\n", sysenv.gulp_path);
+if (sysenv.gamess_path)
+  fprintf(fp,"gamess_path %s\n", sysenv.gamess_path);
+if (sysenv.povray_path)
+  fprintf(fp,"povray_path %s\n", sysenv.povray_path);
+if (sysenv.viewer_path)
+  fprintf(fp,"viewer_path %s\n", sysenv.viewer_path);
+
+
 fprintf(fp,"babel_exe %s\n", sysenv.babel_exe);
+fprintf(fp,"convert_exe %s\n", sysenv.convert_exe);
+fprintf(fp,"gulp_exe %s\n", sysenv.gulp_exe);
+fprintf(fp,"gamess_exe %s\n", sysenv.gamess_exe);
+fprintf(fp,"povray_exe %s\n", sysenv.povray_exe);
+fprintf(fp,"viewer_exe %s\n", sysenv.viewer_exe);
+
+
 
 /* write the non-default element data */
 write_elem_data(fp);
-/* NEW - write the scattering factor coefficients */
-write_sfc_data(fp);
 
 fclose(fp);
 return(0);
@@ -316,6 +372,7 @@ FILE *fp;
 sysenv.max_threads = 1;
 sysenv.task_list = NULL;
 sysenv.dialog_list = NULL;
+sysenv.glconfig = NULL;
 sysenv.stereo = FALSE;
 sysenv.stereo_windowed = FALSE;
 sysenv.stereo_fullscreen = FALSE;
@@ -324,28 +381,34 @@ sysenv.canvas_rows = 1;
 sysenv.canvas_cols = 1;
 sysenv.width = START_WIDTH;
 sysenv.height = START_HEIGHT;
+sysenv.snapshot = FALSE;
+sysenv.snapshot_filename = NULL;
 sysenv.tree_width = START_WIDTH/4;
 sysenv.tree_divider = -1;
 sysenv.tray_height = 60;
 sysenv.mpane = NULL;
+sysenv.tpane = NULL;
+sysenv.fps = 40;
 sysenv.moving = FALSE;
 sysenv.roving = FALSE;
-sysenv.redraw = TRUE;
+sysenv.refresh_canvas = FALSE;
+sysenv.refresh_dialog = FALSE;
+sysenv.refresh_tree = FALSE;
+sysenv.refresh_properties = FALSE;
+sysenv.refresh_text = FALSE;
 sysenv.mtb_on = TRUE;
 sysenv.mpb_on = TRUE;
 sysenv.msb_on = TRUE;
 sysenv.apb_on = TRUE;
-sysenv.rab_on = FALSE;
 sysenv.lmb_on = FALSE;
 sysenv.pib_on = FALSE;
 /* default to single model display */
+sysenv.mal = NULL;
 sysenv.displayed[0] = -1;
 sysenv.active_model = NULL;
 sysenv.canvas = TRUE;
 sysenv.select_mode = CORE;
-sysenv.unfragment = TRUE;
 sysenv.calc_pbonds = TRUE;
-VEC3SET(sysenv.angle, 0.0, 0.0, 0.0);
 
 /* default masks */
 sysenv.file_type = DATA;
@@ -354,19 +417,30 @@ sysenv.num_elements = sizeof(elements) / sizeof(struct elem_pak);
 sysenv.elements = NULL;
 
 /* rendering setup */
-sysenv.render.type = BALL_STICK;
 sysenv.render.width = 600;
 sysenv.render.height = 600;
 sysenv.render.vp_dist = 5.0;
+sysenv.render.zone_size = 10.0;
+
+/* stereo defaults */
+sysenv.render.stereo_quadbuffer = FALSE;
+sysenv.render.stereo_use_frustum = TRUE;
+sysenv.render.stereo_eye_offset = 1.0;
+sysenv.render.stereo_parallax = 1.0;
+sysenv.render.stereo_left = TRUE;
+sysenv.render.stereo_right = TRUE;
 
 /* default light */
 light = g_malloc(sizeof(struct light_pak));
 light->type = DIRECTIONAL;
-VEC3SET(light->x, -1.0,  1.0, -1.0);
+VEC3SET(light->x, -1.0, 1.0, -1.0);
 VEC3SET(light->colour, 1.0, 1.0, 1.0);
 light->ambient = 0.2;
-light->diffuse = 0.9;
-light->specular = 0.9;
+light->diffuse = 0.8;
+light->specular = 0.7;
+/*** JJM new values for better (IMHO) lighting ***/
+light->diffuse = 0.5;
+//light->specular = 0.3;
 sysenv.render.light_list = g_slist_append(sysenv.render.light_list, light);
 sysenv.render.num_lights = 1;
 
@@ -378,8 +452,8 @@ sysenv.render.wire_model = FALSE;
 sysenv.render.wire_surface = FALSE;
 sysenv.render.shadowless = FALSE;
 sysenv.render.animate = FALSE;
-sysenv.render.no_exec_povray = FALSE;
-sysenv.render.no_keep_tempfiles = FALSE;
+sysenv.render.no_povray_exec = FALSE;
+sysenv.render.no_keep_tempfiles = TRUE;
 sysenv.render.animate_type = ANIM_GIF;
 sysenv.render.animate_file = g_strdup("movie");
 sysenv.render.atype = FALSE;
@@ -388,16 +462,17 @@ sysenv.render.morph_finish = g_strdup("F_Glass4");
 sysenv.render.ref_index = 2.5;
 sysenv.render.delay = 20.0;
 sysenv.render.mpeg_quality = 95.0;
-sysenv.render.sphere_quality = 2.0;
+sysenv.render.sphere_quality = 3.0;
 sysenv.render.cylinder_quality = 9.0;
 sysenv.render.ribbon_quality = 10.0;
-sysenv.render.halos = FALSE;
+sysenv.render.ms_grid_size = 0.5;
+sysenv.render.auto_quality = FALSE;
 sysenv.render.fast_rotation = FALSE;
-
+sysenv.render.halos = FALSE;
 sysenv.render.ambience = 0.2;
 sysenv.render.diffuse = 0.9;
 sysenv.render.specular = 0.9;
-sysenv.render.transmit = 0.9;
+sysenv.render.transmit = 1.0;
 sysenv.render.ghost_opacity = 0.5;
 sysenv.render.ball_rad = 0.3;
 sysenv.render.stick_rad = 0.1;
@@ -425,6 +500,9 @@ VEC3SET(sysenv.render.ribbon_colour, 0.0, 0.0, 1.0);
 
 /* setup directory and file pointers */
 sysenv.cwd = g_get_current_dir();
+const gchar *envdir = g_getenv("GDIS_START_DIR");
+if (envdir)
+  sysenv.cwd = (gchar *) envdir;
 
 #if DEBUG_SYS_INIT
 printf("cwd: [%s]\n", sysenv.cwd);
@@ -476,14 +554,25 @@ else
   }
 
 /* defaults */
+#if _WIN32
+sysenv.babel_exe = g_strdup("babel.exe");
+sysenv.povray_exe = g_strdup("povray.exe");
+sysenv.convert_exe = g_strdup("convert.exe");
+sysenv.viewer_exe = g_strdup("display.exe");
+sysenv.gulp_exe = g_strdup("gulp.exe");
+sysenv.gamess_exe = g_strdup("wingamess");
+#else
+sysenv.babel_exe = g_strdup("babel");
 sysenv.povray_exe = g_strdup("povray");
 sysenv.convert_exe = g_strdup("convert");
 sysenv.viewer_exe = g_strdup("display");
 sysenv.gulp_exe = g_strdup("gulp");
-sysenv.babel_exe = g_strdup("babel");
-sysenv.grace_exe = g_strdup("xmgrace");
 sysenv.gamess_exe = g_strdup("run_gms_for_gdis");
+#endif
+
+
 strcpy(sysenv.gl_fontname, GL_FONT);
+
 /* atomic scattering factor coefficients */
 sysenv.sfc_table = g_hash_table_new_full(&g_str_hash,
                                          &hash_strcmp,
@@ -510,6 +599,12 @@ else
 
 /* if failed to read gdisrc - write a new one */
 /* TODO - if overwrite old version, save/rename first? */
+sysenv.babel_path = NULL;
+sysenv.convert_path = NULL;
+sysenv.gulp_path = NULL;
+sysenv.gamess_path = NULL;
+sysenv.povray_path = NULL;
+sysenv.viewer_path = NULL;
 if (read_gdisrc())
   {
   printf("creating: %-50s ", sysenv.init_file);
@@ -522,13 +617,30 @@ else
   printf("scanning: %-50s [ok]\n", sysenv.init_file);
 
 /* get executable paths */
-sysenv.povray_path = g_find_program_in_path(sysenv.povray_exe);
-sysenv.convert_path = g_find_program_in_path(sysenv.convert_exe);
-sysenv.viewer_path = g_find_program_in_path(sysenv.viewer_exe);
-sysenv.gulp_path = g_find_program_in_path(sysenv.gulp_exe);
-sysenv.babel_path = g_find_program_in_path(sysenv.babel_exe);
-sysenv.grace_path = g_find_program_in_path(sysenv.grace_exe);
-sysenv.gamess_path = g_find_program_in_path(sysenv.gamess_exe);
+if (!sysenv.babel_path)
+  sysenv.babel_path = g_find_program_in_path(sysenv.babel_exe);
+if (!sysenv.convert_path)
+  sysenv.convert_path = g_find_program_in_path(sysenv.convert_exe);
+if (!sysenv.gulp_path)
+  sysenv.gulp_path = g_find_program_in_path(sysenv.gulp_exe);
+if (!sysenv.povray_path)
+  sysenv.povray_path = g_find_program_in_path(sysenv.povray_exe);
+if (!sysenv.viewer_path)
+  sysenv.viewer_path = g_find_program_in_path(sysenv.viewer_exe);
+
+/* HACK FIX - GAMESS path contains the path, not the path + file */
+temp = g_find_program_in_path(sysenv.gamess_exe);
+
+if (temp)
+  sysenv.gamess_path = g_path_get_dirname(temp);
+else
+#if __WIN32
+  sysenv.gamess_path = g_strdup("c:\\wingamess");
+#else
+  sysenv.gamess_path = NULL;
+#endif
+
+g_free(temp);
 
 /* tree init */
 sysenv.num_trees = 0;
@@ -536,12 +648,15 @@ sysenv.num_trees = 0;
 sysenv.select_source = NULL;
 sysenv.module_list = NULL;
 sysenv.projects = NULL;
+sysenv.manual = NULL;
+sysenv.image_table = NULL;
+sysenv.surfaces = NULL;
 
-/* file extension recognition stuff */
+/* module */
 file_init();
-
-/* numerical stuff */
+help_init();
 init_math();
+library_init();
 }
 
 /********/
@@ -571,15 +686,19 @@ if (sysenv.canvas)
   gtk_init(&argc, &argv);
   gtk_gl_init(&argc, &argv);
 
-/* functions below aren't protect by thread enter/leave */
-/* since nothing else writes to the widgets they control */
-/* task update timer */
-  g_timeout_add(1000, (GSourceFunc) &update_task_info, NULL);
-/* screen redraw timer */
-  g_timeout_add(25, (GSourceFunc) &redraw_handler, NULL);
-
 /* main interface */
-  connect_events();
+  gui_init();
+
+/* task update timer */
+/* TODO - put this in gui_widget_handler? */
+  g_timeout_add(1000, (GSourceFunc) &update_task_info, NULL);
+
+/* screen redraw timer */
+  g_timeout_add(25, (GSourceFunc) &gui_canvas_handler, NULL);
+
+/* CURRENT - gui updates done through a timer */
+/* this was done so that threads can safely schedule a gui update */
+  g_timeout_add(200, (GSourceFunc) &gui_widget_handler, NULL);
 
 /* process arguments as files to load */
   for (i=1 ; i<argc ; i++)
