@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -29,9 +29,11 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "coords.h"
 #include "edit.h"
 #include "matrix.h"
+#include "quaternion.h"
 #include "model.h"
 #include "morph.h"
 #include "opengl.h"
+#include "render.h"
 #include "select.h"
 #include "space.h"
 #include "spatial.h"
@@ -40,6 +42,42 @@ The GNU GPL can also be found at http://www.gnu.org
 /* main structure */
 extern struct sysenv_pak sysenv;
 
+/**********************************************************/
+/* adjust a rotation matrix wrt curent camera position */
+/**********************************************************/
+void matrix_camera_transform(gdouble *rot, struct model_pak *model)
+{
+gdouble q[9], qi[9], mat[9];
+
+g_assert(model != NULL);
+
+/* build component matrices */
+quat_matrix(qi, camera_q(model->camera));
+quat_matrix(q, camera_q(model->camera));
+matrix_invert(qi);
+
+/* build transformation */
+memcpy(mat, model->latmat, 9*sizeof(gdouble));
+matmat(qi, mat);
+matmat(rot, mat);
+matmat(q, mat);
+matmat(model->ilatmat, mat);
+
+memcpy(rot, mat, 9*sizeof(gdouble));
+}
+
+/**********************************************************/
+/* construct a rotation matrix wrt curent camera position */
+/**********************************************************/
+void matrix_relative_rotation(gdouble *m, gdouble a, gint type, struct model_pak *model)
+{
+g_assert(model != NULL);
+
+/* get rotation and transform according to camera */
+matrix_rotation(m, a, type);
+matrix_camera_transform(m, model);
+}
+
 /*******************************/
 /* multiply two (3x3) matrices */
 /*******************************/
@@ -50,7 +88,7 @@ gint i;
 gdouble tmp[9], *res, *ptr1, *ptr2;
 
 /* init */
-transpose(mat2);
+matrix_transpose(mat2);
 res = tmp;
 ptr1 = mat1;
 
@@ -76,8 +114,6 @@ for (i=3 ; i-- ; )
 
 /* copy result to mat2 */
 memcpy(mat2,tmp,9*sizeof(gdouble));
-
-return;
 }
 
 /*******************************/
@@ -90,7 +126,7 @@ gint i;
 gdouble tmp[16], *res, *ptr1, *ptr2;
 
 /* init */
-transpose4(mat2);
+matrix_transpose_44(mat2);
 res = tmp;
 ptr1 = mat1;
 
@@ -227,11 +263,10 @@ vptr--;
 *(++vptr) += *mptr * z;
 }
 
-/**********************/
-/* transpose matrices */
-/**********************/
-/* 4x4 */
-void transpose4(gdouble *mat)
+/*****************/
+/* transpose 4x4 */
+/*****************/
+void matrix_transpose_44(gdouble *mat)
 {
 gdouble tmp[16], *ptr=mat;
 
@@ -252,8 +287,11 @@ ptr++;
 *ptr++ = tmp[7]; 
 *ptr   = tmp[11]; 
 }
-/* 3x3 */
-void transpose(gdouble *mat)
+
+/*****************/
+/* transpose 3x3 */
+/*****************/
+void matrix_transpose(gdouble *mat)
 {
 gdouble tmp[9], *ptr=mat;
 
@@ -285,13 +323,13 @@ cof[8] = mat[0]*mat[4] - mat[1]*mat[3];
 /* new matrix inversion routine */
 /********************************/
 #define DEBUG_INVMAT 0
-gint invmat(gdouble *mat)
+gint matrix_invert(gdouble *mat)
 {
 gdouble d, id;
 gdouble cof[9];
 
 /* determinant */
-d = det(mat);
+d = matrix_determinant(mat);
 
 #if DEBUG_INVMAT
 P3MAT("input:", mat);
@@ -301,7 +339,7 @@ printf("determinant = %f\n", d);
 /* FIXME - what is a good value to use here??? */
 if (fabs(d) < 0.0001)
   {
-  show_text(ERROR, "Bad lattice matrix.");
+  gui_text_show(ERROR, "Bad lattice matrix.");
   return(1);
   }
 id = 1.0/d;
@@ -309,7 +347,7 @@ id = 1.0/d;
 /* compute co-factor matrix */
 make_cofmat(cof, mat);
 
-transpose(cof);
+matrix_transpose(cof);
 
 VEC3MUL(&cof[0], id);
 VEC3MUL(&cof[3], id);
@@ -381,12 +419,9 @@ lenprod = magnitude(vec1, dim) * magnitude(vec2, dim);
 /* get cos of the angle */
 if (lenprod > FRACTION_TOLERANCE)
   cosa = dot / lenprod;
-  
+ 
 /* enforce range */
-if (cosa < -1.0)
-  cosa = -1.0;
-if (cosa > 1.0)
-  cosa = 1.0;
+cosa = CLAMP(cosa, -1.0, 1.0);
 
 return(acos(cosa));
 }
@@ -457,7 +492,7 @@ ARR3SUB(res, tmp);
 /***********************/
 /* determinant for 3x3 */
 /***********************/
-gdouble det(gdouble *mat)
+gdouble matrix_determinant(gdouble *mat)
 {
 gdouble vec1[3], vec2[3], vec3[3], tmp[3];
 
@@ -481,13 +516,13 @@ gdouble vec[3], tmat[9];
 
 /* NB: latmat has cell vectors in columns */
 memcpy(tmat, latmat, 9*sizeof(gdouble));
-transpose(tmat);
+matrix_transpose(tmat);
 
 /* compute volume */
 crossprod(vec, &tmat[0], &tmat[3]); 
 ARR3MUL(vec, &tmat[6]);
 
-return(vec[0]+vec[1]+vec[2]);
+return(fabs(vec[0]+vec[1]+vec[2]));
 }
 
 /****************************************************/
@@ -499,7 +534,7 @@ gdouble a, b, c, tmat[9];
 
 /* NB: latmat has cell vectors in columns */
 memcpy(tmat, latmat, 9*sizeof(gdouble));
-transpose(tmat);
+matrix_transpose(tmat);
 
 /* get lengths and angle between */
 a = VEC3MAG(&tmat[0]);
@@ -510,281 +545,125 @@ c = via(&tmat[0], &tmat[3], 3);
 return(a * b * sin(c));
 }
 
-/*******************************************************************/
-/* construct matrix for (cartesian space) reflection about a plane */
-/*******************************************************************/
-/* TODO - pointer to the operation type reflection/rotation etc. */
-#define DEBUG_MAKE_SYMMAT 0
-void make_symmat(gdouble *mat,
-                 gdouble *latmat,
-                 gpointer *spatial,
-                 gint type,
-                 gdouble angle) 
+/*****************************/
+/* initialize a 3x3 identity */
+/*****************************/
+void matrix_identity(gdouble *mat)
 {
-gint n;
-gdouble proj, len;
-gdouble a[3], b[3], c[3], vec[4], tmp[4], mop[16], rot[9];
-struct vec_pak *p[4];
-struct spatial_pak *sdata;
-GSList *list;
+VEC3SET(&mat[0], 1.0, 0.0, 0.0);
+VEC3SET(&mat[3], 0.0, 1.0, 0.0);
+VEC3SET(&mat[6], 0.0, 0.0, 1.0);
+}
 
-g_return_if_fail(mat != NULL);
-g_return_if_fail(latmat != NULL);
-g_return_if_fail(spatial != NULL);
+/************************************************************/
+/* assign a transformation matrix that aligns a vector to z */
+/************************************************************/
+#define DEBUG_Z_ALIGNMENT 0
+void matrix_z_alignment(gdouble *mat, gdouble *vec)
+{
+gdouble len, proj, iproj, n[3], mop[9];
 
 /* default return matrix */
-VEC4SET(&mat[0],  1.0, 0.0, 0.0, 0.0);
-VEC4SET(&mat[4],  0.0, 1.0, 0.0, 0.0);
-VEC4SET(&mat[8],  0.0, 0.0, 1.0, 0.0);
-VEC4SET(&mat[12], 0.0, 0.0, 0.0, 1.0);
-
-/* acquire points from the spatial object */
-sdata = (struct spatial_pak *) spatial;
-n=0;
-list = (GSList *) sdata->data;
-while (list && n<4)
-  {
-  p[n] = (struct vec_pak *) list->data;
-  list = g_slist_next(list);
-  n++;
-  }
-
-/* compute orientation vector (plane normal/vector direction) */
-VEC4SET(vec, 0.0, 0.0, 0.0, 0.0);
-switch (sdata->type)
-  {
-  case SPATIAL_VECTOR:
-    g_assert(n > 1);
-/* NB: vector points from 1st to 2nd */
-    ARR3SET(a, p[0]->x);
-    ARR3SET(b, p[1]->x);
-    vecmat(latmat, a);
-    vecmat(latmat, b);
-    ARR3SET(vec, b);
-    ARR3SUB(vec, a);
-    break;
-  case SPATIAL_PLANE:
-    g_assert(n > 2);
-    ARR3SET(a, p[0]->x);
-    ARR3SET(b, p[1]->x);
-    ARR3SET(c, p[2]->x);
-    vecmat(latmat, a);
-    vecmat(latmat, b);
-    vecmat(latmat, c);
-    calc_norm(vec, a, b, c);
-    break;
-
-  default:
-    printf("Unknown spatial object.\n");
-    return;
-  }
+matrix_identity(mat);
+ARR3SET(n, vec);
 
 /* check */
-len = VEC3MAG(vec);
+len = VEC3MAG(n);
 if (len < FRACTION_TOLERANCE)
   {
-  printf("get_ref_matrix() error: bad orientation vector.\n");
+  printf("matrix_z_alignment(): bad orientation vector.\n");
   return;
   }
 
-/* constants */
-vec[0] /= len;
-vec[1] /= len;
-vec[2] /= len;
-proj = sqrt(vec[0]*vec[0] + vec[1]*vec[1]);
-
-#if DEBUG_MAKE_SYMMAT
-P3VEC("[0]    orientation vector:", vec);
-#endif
+/* normalize */
+VEC3MUL(n, 1.0/len);
+proj = sqrt(n[0]*n[0] + n[1]*n[1]);
+iproj = 1.0 / proj;
 
 /* rot z -> yz plane */
 if (proj > FRACTION_TOLERANCE)
   {
-  VEC4SET(&mop[0], vec[1]/proj, -vec[0]/proj, 0.0, 0.0);
-  VEC4SET(&mop[4], vec[0]/proj,  vec[1]/proj, 0.0, 0.0);
-  VEC4SET(&mop[8], 0.0, 0.0, 1.0, 0.0);
-  VEC4SET(&mop[12], 0.0, 0.0, 0.0, 1.0);
-  mat4mat(mop, mat);
+  VEC3SET(&mop[0], n[1]*iproj, -n[0]*iproj, 0.0);
+  VEC3SET(&mop[3], n[0]*iproj,  n[1]*iproj, 0.0);
+  VEC3SET(&mop[6], 0.0, 0.0, 1.0);
+  matmat(mop, mat);
   }
-#if DEBUG_MAKE_SYMMAT
-else
-  printf("plane is already yz\n");
-#endif
-
-#if DEBUG_MAKE_SYMMAT
-ARR3SET(tmp, vec);
-tmp[3] = 0.0;
-vec4mat(mat, tmp);
-P3VEC("[1]    orientation vector:", tmp);
-#endif
 
 /* rot x -> colinear with z */
-VEC4SET(&mop[0], 1.0, 0.0, 0.0, 0.0);
-VEC4SET(&mop[4], 0.0, vec[2], -proj, 0.0);
-VEC4SET(&mop[8], 0.0, proj, vec[2], 0.0);
-VEC4SET(&mop[12], 0.0, 0.0, 0.0, 1.0);
-mat4mat(mop, mat);
+VEC3SET(&mop[0], 1.0, 0.0, 0.0);
+VEC3SET(&mop[3], 0.0, n[2], -proj);
+VEC3SET(&mop[6], 0.0, proj, n[2]);
+matmat(mop, mat);
 
-#if DEBUG_MAKE_SYMMAT
-ARR3SET(tmp, vec);
-tmp[3] = 0.0;
-vec4mat(mat, tmp);
-P3VEC("[2]    orientation vector:", tmp);
-#endif
-
-/* use a spatial point to determine translation required for origin coincidence */
-ARR3SET(tmp, a);
-tmp[3] = 1.0;
-vec4mat(mat, tmp);
-
-#if DEBUG_MAKE_SYMMAT
-P3VEC("pt 1:", tmp);
-#endif
-
-/* translate to origin */
-mat[3] -= tmp[0];
-mat[7] -= tmp[1];
-mat[11] -= tmp[2];
-
-/* main operation here */
-switch(type)
-  {
-  case REFLECTION:
-    VEC4SET(&mop[0],  1.0, 0.0, 0.0, 0.0);
-    VEC4SET(&mop[4],  0.0, 1.0, 0.0, 0.0);
-    VEC4SET(&mop[8],  0.0, 0.0, -1.0, 0.0);
-    VEC4SET(&mop[12], 0.0, 0.0, 0.0, 1.0);
-    break;
-
-  case PAXIS:
-    make_rotmat(rot, D2R*angle, ROLL);
-    VEC4SET(&mop[0],  rot[0], rot[1], rot[2], 0.0);
-    VEC4SET(&mop[4],  rot[3], rot[4], rot[5], 0.0);
-    VEC4SET(&mop[8],  rot[6], rot[7], rot[8], 0.0);
-    VEC4SET(&mop[12], 0.0, 0.0, 0.0, 1.0);
-    break;
-
-  case ALIGNMENT:
-/* TODO - x,y as well? */
-    return;
-
-  default:
-    g_assert_not_reached();
-  }
-mat4mat(mop, mat);
-
-/* translation inverse */
-mat[3] += tmp[0];
-mat[7] += tmp[1];
-mat[11] += tmp[2];
-
-/* rot x inverse */
-VEC4SET(&mop[0], 1.0, 0.0, 0.0, 0.0);
-VEC4SET(&mop[4], 0.0, vec[2], proj, 0.0);
-VEC4SET(&mop[8], 0.0, -proj, vec[2], 0.0);
-VEC4SET(&mop[12], 0.0, 0.0, 0.0, 1.0);
-mat4mat(mop, mat);
-
-/* rot z inverse */
-if (proj > FRACTION_TOLERANCE)
-  {
-  VEC4SET(&mop[0], vec[1]/proj, vec[0]/proj, 0.0, 0.0);
-  VEC4SET(&mop[4], -vec[0]/proj,  vec[1]/proj, 0.0, 0.0);
-  VEC4SET(&mop[8], 0.0, 0.0, 1.0, 0.0);
-  VEC4SET(&mop[12], 0.0, 0.0, 0.0, 1.0);
-  mat4mat(mop, mat);
-  }
-#if DEBUG_MAKE_SYMMAT
-else
-  printf("plane is already yz\n");
-#endif
-
-#if DEBUG_MAKE_SYMMAT
-P4MAT("transf: ", mat);
+#if DEBUG_Z_ALIGNMENT
+ARR3SET(n, vec);
+vecmat(mat, n);
+P3MAT("mat: ", mat);
+P3VEC("v0: ", vec);
+P3VEC("v1: ", n);
 #endif
 }
 
-/********************************************/
-/* construct matrix for rotation about axis */
-/********************************************/
-/* NB: vec is assumed to point from the origin */
-/* and is the normal for a mirror, and the axis for a rotation */
-#define DEBUG_GET_MATRIX 0
-void get_rot_matrix(gdouble *mat, gint type, gdouble *vec, gint order) 
+/********************************************************/
+/* arbitrary alignment, maps v1 to be co-linear with v2 */
+/********************************************************/
+#define DEBUG_V_ALIGNMENT 0
+void matrix_v_alignment(gdouble *m1, gdouble *v1, gdouble *v2)
 {
-gdouble proj, len, angle;
-gdouble mop[9];
+gdouble m2[9];
 
-switch(type)
-  {
-  case IDENTITY:
-    VEC3SET(&mat[0], 1.0, 0.0, 0.0);
-    VEC3SET(&mat[3], 0.0, 1.0, 0.0);
-    VEC3SET(&mat[6], 0.0, 0.0, 1.0);
-    break;
+/* map v1 to z, then to v2 via inverse of v2's z alignment */
+matrix_z_alignment(m1, v1);
+matrix_z_alignment(m2, v2);
+matrix_invert(m2);
+matmat(m2, m1);
+}
 
-/* proper rotation axis */
-  case PAXIS:
-#if DEBUG_GET_MATRIX
-P3VEC("direction vector: ",vec);
-printf("constructing for order %d axis...\n", order);
-#endif
-/* constants */
-    len = VEC3MAG(vec);
-    proj = sqrt(vec[1]*vec[1] + vec[2]*vec[2]);
-    angle = 2.0*PI/order;
+/****************************************************/
+/* construct rotation matrix about arbitrary vector */
+/****************************************************/
+void matrix_v_rotation(gdouble *mat, gdouble *v, gdouble angle)
+{
+gdouble m1[9], m2[9];
 
-/* init the return matrix */
-    VEC3SET(&mat[0], 1.0, 0.0, 0.0);
-    VEC3SET(&mat[3], 0.0, 1.0, 0.0);
-    VEC3SET(&mat[6], 0.0, 0.0, 1.0);
-
-/* yz plane coincidence */
-    VEC3SET(&mop[0], 1.0, 0.0, 0.0);
-    VEC3SET(&mop[3], 0.0, vec[2]/proj, vec[1]/proj);
-    VEC3SET(&mop[6], 0.0, -1.0*vec[1]/proj, vec[2]/proj);
-    matmat(mop,mat);
-
-/* z axis coincidence */
-    VEC3SET(&mop[0], proj/len, 0.0, vec[0]/len);
-    VEC3SET(&mop[3], 0.0, 1.0, 0.0);
-    VEC3SET(&mop[6], -1.0*vec[0]/len, 0.0, proj/len);
-    matmat(mop,mat);
+/* align with z */
+matrix_z_alignment(m1, v);
+memcpy(mat, m1, 9*sizeof(gdouble));
 
 /* rotation operation (about z) */
-    VEC3SET(&mop[0], cos(angle), sin(angle), 0.0);
-    VEC3SET(&mop[3], -1.0*sin(angle), cos(angle), 0.0);
-    VEC3SET(&mop[6], 0.0, 0.0, 1.0);
-    matmat(mop,mat);
+matrix_rotation(m2, angle, YAW);
+matmat(m2, mat);
 
-/* z axis inverse */
-    VEC3SET(&mop[0], proj/len, 0.0, -1.0*vec[0]/len);
-    VEC3SET(&mop[3], 0.0, 1.0, 0.0);
-    VEC3SET(&mop[6], vec[0]/len, 0.0, proj/len);
-    matmat(mop,mat);
+/* inverse z alignment */
+matrix_invert(m1);
+matmat(m1, mat);
+}
 
-/* yz plane inverse */
-    VEC3SET(&mop[0], 1.0, 0.0, 0.0);
-    VEC3SET(&mop[3], 0.0, vec[2]/proj, -1.0*vec[1]/proj);
-    VEC3SET(&mop[6], 0.0, vec[1]/proj, vec[2]/proj);
-    matmat(mop,mat);
-    break;
+/****************************************************/
+/* construct rotation matrix about arbitrary vector */
+/****************************************************/
+void matrix_v_reflection(gdouble *mat, gdouble *v)
+{
+gdouble m1[9], m2[9];
 
-/* mirror plane */
-  case PLANE:
-#if DEBUG_GET_MATRIX
-printf("NIY - constructing for mirror plane\n");
-#endif
-    break;
+/* align with z */
+matrix_z_alignment(m1, v);
+memcpy(mat, m1, 9*sizeof(gdouble));
 
-  default:;
-  }
+/* reflection about z */
+matrix_identity(m2);
+m2[8] = -1.0;
+matmat(m2, mat);
+
+/* inverse z alignment */
+matrix_invert(m1);
+matmat(m1, mat);
 }
 
 /*******************************/
 /* construct a rotation matrix */
 /*******************************/
-void make_rotmat(gdouble *rot, gdouble da, gint type)
+void matrix_rotation(gdouble *rot, gdouble da, gint type)
 {
 gdouble cosa, sina;
 
@@ -795,21 +674,24 @@ sina = sin(da);
 /* dependant matrix elements */
 switch(type)
   {
-  case YAW:
-    VEC3SET(&rot[0], cosa, 0.0,-sina);
-    VEC3SET(&rot[3],  0.0, 1.0,  0.0);
-    VEC3SET(&rot[6], sina, 0.0, cosa);
-    break;
   case PITCH:
     VEC3SET(&rot[0], 1.0,  0.0, 0.0);
     VEC3SET(&rot[3], 0.0, cosa,-sina);
     VEC3SET(&rot[6], 0.0, sina, cosa);
     break;
-  case ROLL:
-    VEC3SET(&rot[0], cosa,-sina, 0.0);
-    VEC3SET(&rot[3], sina, cosa, 0.0);
+
+  case YAW:
+    VEC3SET(&rot[0], cosa, sina, 0.0);
+    VEC3SET(&rot[3], -sina, cosa, 0.0);
     VEC3SET(&rot[6],  0.0,  0.0, 1.0);
     break;
+
+  case ROLL:
+    VEC3SET(&rot[0], cosa, 0.0,-sina);
+    VEC3SET(&rot[3],  0.0, 1.0,  0.0);
+    VEC3SET(&rot[6], sina, 0.0, cosa);
+    break;
+
   case INITIAL:
   default:
     VEC3SET(&rot[0], 1.0, 0.0, 0.0);
@@ -819,11 +701,66 @@ switch(type)
   }
 }
 
+/**************************************************************/
+/* get reciprocal lattice vectors from direct lattice vectors */
+/**************************************************************/
+#define DEBUG_RLAT 0
+void matrix_reciprocal_init(struct model_pak *data)
+{
+gdouble norm;
+gdouble a[3], b[3], c[3];
+gdouble axb[3], bxc[3], cxa[3];
+
+#if DEBUG_RLAT
+P3MAT("direct lattice matrix:",data->latmat);
+#endif
+
+/* extract direct lattice vectors */
+a[0] = data->latmat[0];
+a[1] = data->latmat[3];
+a[2] = data->latmat[6];
+b[0] = data->latmat[1];
+b[1] = data->latmat[4];
+b[2] = data->latmat[7];
+c[0] = data->latmat[2];
+c[1] = data->latmat[5];
+c[2] = data->latmat[8];
+
+/* calc intermediate products */
+crossprod(bxc, b, c);
+crossprod(cxa, c, a);
+crossprod(axb, a, b);
+norm = 1.0/(a[0]*bxc[0] + a[1]*bxc[1] + a[2]*bxc[2]);
+
+/* create reciprocal lattice vectors */
+ARR3SET(a, bxc);
+ARR3SET(b, cxa);
+ARR3SET(c, axb);
+VEC3MUL(a, norm);
+VEC3MUL(b, norm);
+VEC3MUL(c, norm);
+
+/* store */
+data->rlatmat[0] = a[0];
+data->rlatmat[3] = a[1];
+data->rlatmat[6] = a[2];
+data->rlatmat[1] = b[0];
+data->rlatmat[4] = b[1];
+data->rlatmat[7] = b[2];
+data->rlatmat[2] = c[0];
+data->rlatmat[5] = c[1];
+data->rlatmat[8] = c[2];
+
+#if DEBUG_RLAT
+P3MAT("reciprocal lattice matrix:",data->rlatmat);
+#endif
+}
+
 /******************************************/
 /* construct the unit translation vectors */
 /******************************************/
 #define DEBUG_XLAT 0
-void make_latmat(struct model_pak *data)
+void matrix_lattice_init(struct model_pak *data)
 {
 gdouble n[3], v1[3], v2[3], v3[3];
 gdouble b1, b2, b3, c1, c2, c3;
@@ -894,15 +831,15 @@ printf("constructing latmat...\n");
   VEC3MUL(&data->latmat[6], data->pbc[2]);
 
 /* get vectors in cols */
-  transpose(data->latmat);
+  matrix_transpose(data->latmat);
   }
 
 /* update dependents */
 make_axes(data);
 make_cell(data);
 memcpy(data->ilatmat, data->latmat, 9*sizeof(gdouble));
-invmat(data->ilatmat);
-make_rlat(data);
+matrix_invert(data->ilatmat);
+matrix_reciprocal_init(data);
 switch(data->periodic)
   {
   case 3:
@@ -926,107 +863,13 @@ P3MAT("lattice matrix:",data->latmat);
 return;
 }
 
-/**************************************************************/
-/* get reciprocal lattice vectors from direct lattive vectors */
-/**************************************************************/
-#define DEBUG_RLAT 0
-void make_rlat(struct model_pak *data)
-{
-gdouble norm;
-gdouble a[3], b[3], c[3];
-gdouble axb[3], bxc[3], cxa[3];
-
-#if DEBUG_RLAT
-P3MAT("direct lattice matrix:",data->latmat);
-#endif
-
-/* extract direct lattice vectors */
-a[0] = data->latmat[0];
-a[1] = data->latmat[3];
-a[2] = data->latmat[6];
-b[0] = data->latmat[1];
-b[1] = data->latmat[4];
-b[2] = data->latmat[7];
-c[0] = data->latmat[2];
-c[1] = data->latmat[5];
-c[2] = data->latmat[8];
-
-/* calc intermediate products */
-crossprod(bxc, b, c);
-crossprod(cxa, c, a);
-crossprod(axb, a, b);
-norm = 1.0/(a[0]*bxc[0] + a[1]*bxc[1] + a[2]*bxc[2]);
-
-/* create reciprocal lattice vectors */
-ARR3SET(a, bxc);
-ARR3SET(b, cxa);
-ARR3SET(c, axb);
-VEC3MUL(a, norm);
-VEC3MUL(b, norm);
-VEC3MUL(c, norm);
-
-/* store */
-data->rlatmat[0] = a[0];
-data->rlatmat[3] = a[1];
-data->rlatmat[6] = a[2];
-data->rlatmat[1] = b[0];
-data->rlatmat[4] = b[1];
-data->rlatmat[7] = b[2];
-data->rlatmat[2] = c[0];
-data->rlatmat[5] = c[1];
-data->rlatmat[8] = c[2];
-
-#if DEBUG_RLAT
-P3MAT("reciprocal lattice matrix:",data->rlatmat);
-#endif
-}
-
-/******************************************/
-/* convert cartesian coords to fractional */
-/******************************************/
-/* TODO - change the function name */
-#define DEBUG_LATMAT_FUDGE 0
-void latmat_fudge(struct model_pak *data)
-{
-GSList *list;
-struct core_pak *core;
-struct shel_pak *shel;
-struct vertex_pak *vec;
-
-#if DEBUG_LATMAT_FUDGE
-P3MAT("latmat: ", data->latmat);
-P3MAT("invmat: ", data->ilatmat);
-#endif
-
-/* transform cores */
-for (list=data->cores ; list ; list=g_slist_next(list))
-  {
-  core = (struct core_pak *) list->data;
-  vecmat(data->ilatmat, core->x);
-  }
-
-/* transform shells */
-for (list=data->shels ; list ; list=g_slist_next(list))
-  {
-  shel = (struct shel_pak *) list->data;
-  vecmat(data->ilatmat, shel->x);
-  }
-
-/* morphology vertices */
-for (list=data->vertices ; list ; list=g_slist_next(list))
-  {
-  vec = (struct vertex_pak *) list->data;
-  vecmat(data->ilatmat, vec->x);
-  }
-}
-
 /****************************/
-/* NEW - new lattice vector */
+/* build new lattice matrix */
 /****************************/
 /* v - new orientation (may need projection if model is 2D) */
 /* ix - which lattice vector to replace (0-2) */
 #define DEBUG_NEW_LATTICE 0
-void new_lattice(gdouble *latmat, struct model_pak *model)
+void matrix_lattice_new(gdouble *latmat, struct model_pak *model)
 {
 gint i;
 gint ia, ib, ic, ma, mb, mc;
@@ -1051,20 +894,20 @@ memcpy(tmat, latmat, 9*sizeof(gdouble));
 
 /* transpose the source lattice matrix */
 memcpy(mat1, model->latmat, 9*sizeof(gdouble));
-transpose(mat1);
+matrix_transpose(mat1);
 
 /* get the new lattice matrix */
 matmat(tmat, mat1);
-transpose(mat1);
+matrix_transpose(mat1);
 
 #if DEBUG_NEW_LATTICE
 P3MAT("new latmat:", mat1);
 #endif
 
 memcpy(mat2, mat1, 9*sizeof(gdouble));
-if (invmat(mat2))
+if (matrix_invert(mat2))
   {
-  show_text(ERROR, "Two or more lattice vectors are not independant.");
+  gui_text_show(ERROR, "Two or more lattice vectors are not independant.");
   return;
   }
 
@@ -1072,8 +915,8 @@ if (invmat(mat2))
 dest = model_new();
 dest->periodic = model->periodic;
 memcpy(dest->latmat, mat1, 9*sizeof(gdouble));
-copy_gulp_data(model, dest);
-copy_gulp_extra(model, dest);
+gulp_data_copy(model, dest);
+gulp_extra_copy(model, dest);
 
 /* setup repeats required to fill the new cell */
 ma=mb=mc=1;
@@ -1151,7 +994,7 @@ for (ic=0 ; ic<mc ; ic++)
 /* init new model */
 dest->fractional = FALSE;
 dest->construct_pbc = TRUE;
-prep_model(dest);
+model_prep(dest);
 tree_model_add(dest);
 redraw_canvas(SINGLE);
 }
@@ -1159,7 +1002,7 @@ redraw_canvas(SINGLE);
 /************************/
 /* distinguish identity */
 /************************/
-gint is_identity(gdouble *mat)
+gint matrix_is_identity(gdouble *mat)
 {
 if (*(mat+0) != 1.0)
   return(FALSE);
@@ -1188,7 +1031,7 @@ return(TRUE);
 /*************************/
 /* distinguish inversion */
 /*************************/
-gint is_inversion(gdouble *mat)
+gint matrix_is_inversion(gdouble *mat)
 {
 if (*(mat+0) != -1.0)
   return(FALSE);
@@ -1217,7 +1060,7 @@ return(TRUE);
 /**************************************/
 /* get the order of a symmetry matrix */
 /**************************************/
-gint order(gdouble *mat)
+gint matrix_order(gdouble *mat)
 {
 gint i;
 gdouble m1[9], m2[9];
@@ -1233,7 +1076,7 @@ ARR3SET(&m2[6], (mat+6));
 i=1;
 while(i<17)
   {
-  if (is_identity(m2))
+  if (matrix_is_identity(m2))
     return(i);
   matmat(m1, m2);
   i++;
@@ -1243,3 +1086,32 @@ while(i<17)
 return(0);
 }
 
+/************************************************/
+/* return the order if a matrix is a z rotation */
+/************************************************/
+/* FIXME - only does 2 fold (for defect setup) */
+gint matrix_is_z_rotation(gdouble *mat)
+{
+if (*(mat+0) != -1.0)
+  return(0);
+if (*(mat+1) != 0.0)
+  return(0);
+if (*(mat+2) != 0.0)
+  return(0);
+
+if (*(mat+3) != 0.0)
+  return(0);
+if (*(mat+4) != -1.0)
+  return(0);
+if (*(mat+5) != 0.0)
+  return(0);
+
+if (*(mat+6) != 0.0)
+  return(0);
+if (*(mat+7) != 0.0)
+  return(0);
+if (*(mat+8) != 1.0)
+  return(0);
+
+return(2);
+}
