@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -28,6 +28,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "coords.h"
 #include "matrix.h"
 #include "measure.h"
+#include "select.h"
 #include "interface.h"
 #include "opengl.h"
 
@@ -43,7 +44,6 @@ struct measure_pak
 gint type;                                /* distance, angle, etc. */
 gchar *value;
 gpointer core[MEASURE_MAX_CORES];         /* participants */
-gdouble xlat[MEASURE_MAX_CORES][3];       /* vector offset for each core */
 gint image[MEASURE_MAX_CORES][3];         /* perioidic image offset for each core */
 gdouble colour[3];
 };
@@ -66,12 +66,13 @@ for (list=model->measure_list ; list ; list=g_slist_next(list))
 
   switch (mp->type)
     {
+    case MEASURE_BOND:
     case MEASURE_INTER:
     case MEASURE_INTRA:
     case MEASURE_DISTANCE:
       c1 = mp->core[0];
       c2 = mp->core[1];
-      printf("[%s-%s] : ", c1->label, c2->label);
+      printf("[%s-%s] : ", c1->atom_label, c2->atom_label);
 #if DEBUG_DUMP_ALL
       printf("[%d %d %d] ", mp->image[0][0], mp->image[0][1], mp->image[0][2]);
       printf("[%d %d %d] : ", mp->image[1][0], mp->image[1][1], mp->image[1][2]);
@@ -83,7 +84,10 @@ for (list=model->measure_list ; list ; list=g_slist_next(list))
       c1 = mp->core[0];
       c2 = mp->core[1];
       c3 = mp->core[2];
-      printf("[%s-%s-%s] : %s\n", c1->label, c2->label, c3->label, mp->value);
+      printf("[%s-%s-%s] : %s\n", c1->atom_label,
+                                  c2->atom_label,
+                                  c3->atom_label,
+                                  mp->value);
       break;
 
     case MEASURE_TORSION:
@@ -91,8 +95,11 @@ for (list=model->measure_list ; list ; list=g_slist_next(list))
       c2 = mp->core[1];
       c3 = mp->core[2];
       c4 = mp->core[3];
-      printf("[%s-%s-%s-%s] : %s\n", c1->label, c2->label, c3->label, c4->label,
-                                                                     mp->value);
+      printf("[%s-%s-%s-%s] : %s\n", c1->atom_label,
+                                     c2->atom_label,
+                                     c3->atom_label,
+                                     c4->atom_label,
+                                     mp->value);
       break;
     }
   }
@@ -119,14 +126,13 @@ for (list=model->measure_list ; list ; list=g_slist_next(list))
     case MEASURE_INTER:
     case MEASURE_INTRA:
     case MEASURE_DISTANCE:
-      model->selection = g_slist_prepend(model->selection, m->core[0]);
-      model->selection = g_slist_prepend(model->selection, m->core[1]);
+      select_core(m->core[0], FALSE, model);
+      select_core(m->core[1], FALSE, model);
       break;
-
     case MEASURE_ANGLE:
-      model->selection = g_slist_prepend(model->selection, m->core[0]);
-      model->selection = g_slist_prepend(model->selection, m->core[1]);
-      model->selection = g_slist_prepend(model->selection, m->core[2]);
+      select_core(m->core[0], FALSE, model);
+      select_core(m->core[1], FALSE, model);
+      select_core(m->core[2], FALSE, model);
       break;
     }
   }
@@ -157,6 +163,29 @@ g_assert(mp != NULL);
 return(mp->value);
 }
 
+/********************************/
+/* measurement cores extraction */
+/********************************/
+GSList *measure_cores_get(gpointer data)
+{
+GSList *list=NULL;
+struct measure_pak *mp = data;
+
+g_assert(mp != NULL);
+
+switch (mp->type)
+  {
+  case MEASURE_TORSION:
+    list = g_slist_prepend(list, mp->core[3]);
+  case MEASURE_ANGLE:
+    list = g_slist_prepend(list, mp->core[2]);
+  default:
+    list = g_slist_prepend(list, mp->core[1]);
+    list = g_slist_prepend(list, mp->core[0]);
+  }
+return(list);
+}
+
 /************************************/
 /* measurement type label creation */
 /***********************************/
@@ -220,7 +249,8 @@ switch(mp->type)
     core[1] = mp->core[1];
 g_assert(core[0] != NULL);
 g_assert(core[1] != NULL);
-    text = g_strdup_printf("%4s : %-4s", core[0]->label, core[1]->label);
+    text = g_strdup_printf("%4s : %-4s", core[0]->atom_label,
+                                         core[1]->atom_label);
     break;
 
   case MEASURE_ANGLE:
@@ -232,8 +262,9 @@ g_assert(core[0] != NULL);
 g_assert(core[1] != NULL);
 g_assert(core[2] != NULL);
 
-    text = g_strdup_printf("%4s : %-4s : %-4s",
-                           core[0]->label, core[1]->label, core[2]->label);
+    text = g_strdup_printf("%4s : %-4s : %-4s", core[0]->atom_label,
+                                                core[1]->atom_label,
+                                                core[2]->atom_label);
     break;
 
   case MEASURE_TORSION:
@@ -245,9 +276,10 @@ g_assert(core[0] != NULL);
 g_assert(core[1] != NULL);
 g_assert(core[2] != NULL);
 g_assert(core[3] != NULL);
-    text = g_strdup_printf("%4s : %-4s : %-4s : %-4s", 
-                           core[0]->label, core[1]->label,
-                           core[2]->label, core[3]->label);
+    text = g_strdup_printf("%4s : %-4s : %-4s : %-4s", core[0]->atom_label,
+                                                       core[1]->atom_label,
+                                                       core[2]->atom_label,
+                                                       core[3]->atom_label);
     break;
 
   default:
@@ -269,20 +301,28 @@ ARR3SET(colour, mp->colour);
 /***************************************************************/
 /* extract nth measurement component location (periodic aware) */
 /***************************************************************/
-void measure_coord_get(gdouble *x, gint n, gpointer data)
+/* NB: always returns closest (image aware) positions relative to 0th core */
+void measure_coord_get(gdouble *x, gint n, gpointer data, struct model_pak *model)
 {
-struct core_pak *core;
+gdouble dx[3];
+struct core_pak *core1, *core2;
 struct measure_pak *mp = data;
 
 /* checks */
 g_assert(mp != NULL);
+g_assert(n >= 0);
+g_assert(n < MEASURE_MAX_CORES);
 
-/* retrieve core position and add periodic translation */
-if (n >= 0 && n < MEASURE_MAX_CORES)
+core1 = mp->core[0];
+ARR3SET(x, core1->rx);
+if (n)
   {
-  core = mp->core[n];
-  ARR3SET(x, core->rx);
-  ARR3ADD(x, &mp->xlat[n][0])
+  core2 = mp->core[n];
+  ARR3SET(dx, core1->x);
+  ARR3SUB(dx, core2->x);
+  fractional_minsq(dx, model->periodic);
+  vecmat(model->latmat, dx);
+  ARR3SUB(x, dx);
   }
 }
 
@@ -296,90 +336,6 @@ struct measure_pak *mp = data;
 VEC3SET(mp->colour, r, g, b);
 }
 
-/**************************************************/
-/* compute a measurement's translation components */
-/**************************************************/
-/* only useful for measurements defined by bonded atoms */
-void measure_xlat_get(gdouble *x, gint type, struct core_pak **core, struct model_pak *model)
-{
-gint i, length=MEASURE_MAX_CORES;
-gdouble xlat[MEASURE_MAX_CORES*4];
-GSList *list;
-struct bond_pak *bond;
-
-/* initialize translation vectors */
-for (i=MEASURE_MAX_CORES ; i-- ; )
-  {
-  VEC4SET(&xlat[i*4], 0.0, 0.0, 0.0, 0.0);
-  }
-
-/* update the unit translation vectors */
-switch (type)
-  {
-  case MEASURE_BOND:
-    length = 2;
-    for (list=core[0]->bonds ; list ; list=g_slist_next(list))
-      {
-      bond = list->data;
-
-/* NB: move the second core */
-      if (bond->atom1 == core[1])
-        {
-        ARR3SET(&xlat[4], bond->pic);
-        }
-      if (bond->atom2 == core[1])
-        {
-        ARR3SET(&xlat[4], bond->pic);
-        VEC3MUL(&xlat[4], -1.0);
-        }
-      }
-    break;
-
-  case MEASURE_ANGLE:
-    length = 3;
-    for (list=core[1]->bonds ; list ; list=g_slist_next(list))
-      {
-      bond = list->data;
-
-/* NB: move the non-central cores */
-      if (bond->atom1 == core[0])
-        {
-        ARR3SET(&xlat[0], bond->pic);
-        VEC3MUL(&xlat[0], -1.0);
-        }
-      if (bond->atom2 == core[0])
-        {
-        ARR3SET(&xlat[0], bond->pic);
-        }
-      if (bond->atom1 == core[2])
-        {
-        ARR3SET(&xlat[8], bond->pic);
-        VEC3MUL(&xlat[8], -1.0);
-        }
-      if (bond->atom2 == core[2])
-        {
-        ARR3SET(&xlat[8], bond->pic);
-        }
-      }
-    break;
-
-/* TODO */
-  case MEASURE_TORSION:
-    break;
-
-  }
-
-/* get the cartesian translation */
-for (i=length ; i-- ; )
-  vec4mat(model->display_lattice, &xlat[i*4]);
-
-/* store the updated translations */
-for (i=length ; i-- ; )
-  {
-  ARR3SET(&x[i*4], &xlat[i*4]);
-  }
-}
-
 /******************************************/
 /* measurement duplicate search primitive */
 /******************************************/
@@ -470,10 +426,8 @@ m = g_malloc(sizeof(struct measure_pak));
 for (i=MEASURE_MAX_CORES ; i-- ; )
   {
   m->core[i] = NULL;
-  VEC3SET(m->xlat[i], 0.0, 0.0, 0.0);
   VEC3SET(m->image[i], 0, 0, 0);
   }
-
 return(m);
 }
 
@@ -524,6 +478,19 @@ for (i=MEASURE_MAX_CORES ; i-- ; )
 return(FALSE);
 }
 
+/*********************************/
+/* absolute distance computation */
+/*********************************/
+gdouble measure_distance(gdouble *x1, gdouble *x2)
+{
+gdouble a[3];
+
+ARR3SET(a, x2);
+ARR3SUB(a, x1);
+
+return(VEC3MAG(a));
+}
+
 /******************************/
 /* absolute angle computation */
 /******************************/
@@ -543,13 +510,11 @@ return(R2D*via(a,b,3));
 /*********************************************/
 /* recompute a measurement value for a model */
 /*********************************************/
-void measure_update_single(gpointer data, struct model_pak *model)
+gdouble measure_update_single(gpointer data, struct model_pak *model)
 {
-gint i;
-gdouble value;
-gdouble x1[3], x2[3], x3[3], xlat[MEASURE_MAX_CORES*4];
+gdouble value=0.0;
+gdouble x1[3], x2[3], x3[3];
 struct measure_pak *mp = data;
-struct core_pak *core[4];
 
 /* checks */
 g_assert(model != NULL);
@@ -558,37 +523,12 @@ g_assert(mp != NULL);
 switch (mp->type)
   {
   case MEASURE_BOND:
-    for (i=2 ; i-- ; )
-      core[i] = mp->core[i];
-    measure_xlat_get(xlat, MEASURE_BOND, core, model);
-    for (i=2 ; i-- ; )
-      {
-      ARR3SET(&mp->xlat[i][0], &xlat[i*4]);
-      }
-/* get measurement component coords */
-    measure_coord_get(x1, 0, mp);
-    measure_coord_get(x2, 1, mp);
-/* update angle value */
-    ARR3SUB(x1, x2);
-    value = VEC3MAG(x1);
-    g_free(mp->value);
-    mp->value = g_strdup_printf("%8.3f", value); 
-    break;
-
   case MEASURE_DISTANCE:
   case MEASURE_INTER:
   case MEASURE_INTRA:
-    ARR3SET(&xlat[0], &mp->image[0][0]);
-    xlat[3] = 0.0;
-    ARR3SET(&xlat[4], &mp->image[1][0]);
-    xlat[7] = 0.0;
-    vec4mat(model->display_lattice, &xlat[0]);
-    vec4mat(model->display_lattice, &xlat[4]);
-    ARR3SET(&mp->xlat[0][0], &xlat[0]);
-    ARR3SET(&mp->xlat[1][0], &xlat[4]);
 /* get measurement component coords */
-    measure_coord_get(x1, 0, mp);
-    measure_coord_get(x2, 1, mp);
+    measure_coord_get(x1, 0, mp, model);
+    measure_coord_get(x2, 1, mp, model);
 /* update distance value */
     ARR3SUB(x1, x2);
     value = VEC3MAG(x1);
@@ -597,30 +537,17 @@ switch (mp->type)
     break;
 
   case MEASURE_ANGLE:
-/* update xlat with bond pic/display lattice transformation */
-/* compiler complains about this - why??? */
-/*
-    measure_xlat_get(xlat, MEASURE_ANGLE, mp->core, model);
-*/
-/* substitute for above */
-    for (i=3 ; i-- ; )
-      core[i] = mp->core[i];
-    measure_xlat_get(xlat, MEASURE_ANGLE, core, model);
-
-    for (i=3 ; i-- ; )
-      {
-      ARR3SET(&mp->xlat[i][0], &xlat[i*4]);
-      }
 /* get measurement component coords */
-    measure_coord_get(x1, 0, mp);
-    measure_coord_get(x2, 1, mp);
-    measure_coord_get(x3, 2, mp);
+    measure_coord_get(x1, 0, mp, model);
+    measure_coord_get(x2, 1, mp, model);
+    measure_coord_get(x3, 2, mp, model);
 /* update angle value */
     value = measure_angle(x1, x2, x3);
     g_free(mp->value);
     mp->value = g_strdup_printf("%8.3f", value); 
     break;
   }
+return(value);
 }
 
 /************************************************/
@@ -653,6 +580,7 @@ g_assert(model != NULL);
 /* determine if filter label is an element or an exact (strict) label, */
 /* or neither - and set up initial list accordingly */
 code = elem_symbol_test(label);
+
 if (code)
   {
   if (strlen(label) != strlen(elements[code].symbol))
@@ -690,7 +618,7 @@ while (list)
     {
     if (strict)
       {
-      if (g_ascii_strcasecmp(label, core->label) != 0)
+      if (g_ascii_strcasecmp(label, core->atom_label) != 0)
         output = g_slist_remove(output, core);
       }
     else
@@ -705,7 +633,7 @@ while (list)
 for (list=output ; list ; list=g_slist_next(list))
   {
   core = list->data;
-  printf(" [%s]", core->label);
+  printf(" [%s]", core->atom_label);
   }
 printf("\n");
 #endif
@@ -733,9 +661,6 @@ g_assert(model != NULL);
 VEC4SET(&xlat[0], 0.0, 0.0, 0.0, 0.0);
 ARR3SET(&xlat[4], image);
 xlat[7] = 0.0;
-/*
-vec4mat(model->display_lattice, &xlat[4]);
-*/
 mp = measure_search(type, c, xlat, model);
 
 if (!mp)
@@ -771,7 +696,6 @@ gpointer measure_bond_test(struct core_pak **core,
 gint t[3], cutoff_test;
 gdouble r1sq, r2sq, rsq;
 gdouble x1[3], x2[3];
-gdouble xlat[12];
 struct measure_pak *mp=NULL;
 
 /* checks */
@@ -785,16 +709,13 @@ if (r2 < FRACTION_TOLERANCE)
 else
   cutoff_test = TRUE;
 
-/* get cartesian coordinates of component cores */
-ARR3SET(x1, core[0]->rx);
-ARR3SET(x2, core[1]->rx);
-
 /* setup cutoffs */
 r1sq = r1*r1;
 r2sq = r2*r2;
 
 /* get measurement component coords */
-measure_xlat_get(xlat, MEASURE_BOND, core, model);
+ARR3SET(x1, core[0]->x);
+ARR3SET(x2, core[1]->x);
 
 /*
 printf("[%s - %s]\n", core[0]->label, core[1]->label);
@@ -804,18 +725,16 @@ P3VEC("xlat1:", &xlat[0]);
 P3VEC("xlat2:", &xlat[4]);
 */
 
-/* perform the translations */
-ARR3ADD(x1, &xlat[0]);
-ARR3ADD(x2, &xlat[4]);
-
-/* test if angle satisfies the cutoffs */
+/* test if distance satisfies the cutoffs */
 ARR3SUB(x1, x2);
+fractional_minsq(x1, model->periodic);
+vecmat(model->latmat, x1);
 rsq = VEC3MAGSQ(x1);
+
 if (cutoff_test)
   if (rsq < r1sq || rsq > r2sq)
      return(NULL);
 
-/* as with angles, bonds (currently) don't have periodic images */
 VEC3SET(t, 0, 0, 0);
 mp = measure_distance_create(MEASURE_BOND, core, t, model);
 
@@ -849,8 +768,9 @@ for (item1=list1 ; item1 ; item1=g_slist_next(item1))
     {
     bond = item2->data;
 
-    if (bond->type == BOND_HBOND)
-      continue;
+    if (!model->build_hydrogen)
+      if (bond->type == BOND_HBOND)
+        continue;
 
     if (core[0] == bond->atom1)
       core[1] = bond->atom2;
@@ -860,13 +780,13 @@ for (item1=list1 ; item1 ; item1=g_slist_next(item1))
     if (g_slist_find(list2, core[1]))
       {
 #if DEBUG_BOND_SEARCH
-printf("valid: %s - %s\n", core[0]->label, core[1]->label);
+printf("valid: %s - %s\n", core[0]->atom_label, core[1]->atom_label);
 #endif
       measure_bond_test(core, r1, r2, model);
       }
     }
   }
-calc_coords(REFRESH, model);
+coords_compute(model);
 
 measure_dump_all(model);
 }
@@ -959,8 +879,8 @@ return(mp);
 /**************************************/
 /* measurement calculation primitives */
 /**************************************/
-void measure_distance_search(gint type,
-                             const gchar **label,
+void measure_distance_search(const gchar **label,
+                             gint type,
                              gdouble r1, gdouble r2,
                              struct model_pak *model)
 {
@@ -990,19 +910,7 @@ for (item1=list1 ; item1 ; item1=g_slist_next(item1))
     measure_distance_test(type, core, r1, r2, model);
     }
   }
-calc_coords(REFRESH, model);
-}
-
-/************************************/
-/* measurement distance calculation */
-/************************************/
-void measure_dist_search(const gchar **label, gdouble r1, gdouble r2, struct model_pak *model)
-{
-measure_distance_search(MEASURE_DISTANCE, label, r1, r2, model);
-}
-void measure_inter_search(const gchar **label, gdouble r1, gdouble r2, struct model_pak *model)
-{
-measure_distance_search(MEASURE_INTER, label, r1, r2, model);
+coords_compute(model);
 }
 
 /**********************************/
@@ -1032,7 +940,6 @@ if (!mp)
   for (i=3 ; i-- ; )
     {
     mp->core[i] = c[i];
-    ARR3SET(&mp->xlat[i][0], &xlat[3*i]);
     VEC3SET(&mp->image[i][0], 0, 0, 0);
     }
   if (label)
@@ -1068,13 +975,6 @@ ARR3SET(x1, core[0]->rx);
 ARR3SET(x2, core[1]->rx);
 ARR3SET(x3, core[2]->rx);
 
-/* compute the approriate translation for the non-central atoms */
-measure_xlat_get(xlat, MEASURE_ANGLE, core, model);
-
-/* perform the translations */
-ARR3ADD(x1, &xlat[0]);
-ARR3ADD(x3, &xlat[8]);
-
 /* create a new angle new measurement if it satisfies the cutoffs */
 angle = measure_angle(x1, x2, x3);
 if (angle >= a1 && angle <= a2)
@@ -1091,7 +991,73 @@ return(mp);
 /**************************************/
 /* measurement calculation primitives */
 /**************************************/
-void measure_angle_search(const gchar **label, gdouble a1, gdouble a2, struct model_pak *model)
+void measure_angle_search(const gchar **label, gdouble *range, struct model_pak *model)
+{
+gdouble r, r12_min, r12_max, r23_min, r23_max;
+gdouble x[3];
+GSList *item1, *item2, *item3, *list1, *list2, *list3;
+struct core_pak *core[3];
+
+/* checks */
+g_assert(model != NULL);
+
+/* get pattern matching entries */
+list1 = measure_filter(label[0], model);
+list2 = measure_filter(label[1], model);
+list3 = measure_filter(label[2], model);
+
+/* setup range tests */
+r12_min = range[0]*range[0];
+r12_max = range[1]*range[1];
+r23_min = range[2]*range[2];
+r23_max = range[3]*range[3];
+
+/* loop over center atoms */
+for (item2=list2 ; item2 ; item2=g_slist_next(item2))
+  {
+  core[1] = item2->data;
+
+/* search for valid arm 1 */
+  for (item1=list1 ; item1 ; item1=g_slist_next(item1))
+    {
+    core[0] = item1->data;
+    if (core[1] == core[0])
+      continue;
+
+    ARR3SET(x, core[1]->rx);
+    ARR3SUB(x, core[0]->rx);
+    r = VEC3MAGSQ(x);
+
+    if (r < r12_min || r > r12_max)
+      continue;
+
+/* search for valid arm 2 */
+    for (item3=list3 ; item3 ; item3=g_slist_next(item3))
+      {
+      core[2] = item3->data;
+      if (core[2] == core[0])
+        continue;
+      if (core[2] == core[1])
+        continue;
+
+      ARR3SET(x, core[1]->rx);
+      ARR3SUB(x, core[2]->rx);
+      r = VEC3MAGSQ(x);
+
+      if (r < r23_min || r > r23_max)
+        continue;
+
+      measure_angle_test(core, range[4], range[5], model);
+      }
+    }
+  }
+coords_compute(model);
+}
+
+/**************************************/
+/* measurement calculation primitives */
+/**************************************/
+void measure_bangle_search(const gchar **label, gdouble a1, gdouble a2, struct model_pak *model)
 {
 gint m1, m2;
 GSList *item1, *item2, *item3, *list1, *list2, *list3;
@@ -1118,10 +1084,9 @@ for (item2=list2 ; item2 ; item2=g_slist_next(item2))
     bond1 = item1->data;
     item1 = g_slist_next(item1);
 
-/*
-    if (bond1->type == BOND_HBOND)
-      continue;
-*/
+    if (!model->build_hydrogen)
+      if (bond1->type == BOND_HBOND)
+        continue;
 
     if (core[1] == bond1->atom1)
       core[0] = bond1->atom2;
@@ -1143,10 +1108,9 @@ for (item2=list2 ; item2 ; item2=g_slist_next(item2))
         bond2 = item3->data;
         item3 = g_slist_next(item3);
 
-/*
-        if (bond2->type == BOND_HBOND)
-          continue;
-*/
+        if (!model->build_hydrogen)
+          if (bond2->type == BOND_HBOND)
+            continue;
 
         if (core[1] == bond2->atom1)
           core[2] = bond2->atom2;
@@ -1192,7 +1156,7 @@ printf("testing valid candidate: %s - %s - %s\n", core1->label, core2->label, co
       }
     }
   }
-calc_coords(REFRESH, model);
+coords_compute(model);
 }
 
 /*******************************/
@@ -1200,8 +1164,8 @@ calc_coords(REFRESH, model);
 /*******************************/
 gdouble measure_torsion(struct core_pak **core)
 {
-gdouble len;
-gdouble a[3], b[3], n[3];
+gdouble len, sign;
+gdouble a[3], b[3], c[3], n[3];
 
 /* compute end atom vectors (from 1-2 axis) */
 ARR3SET(a, core[0]->rx);
@@ -1214,6 +1178,13 @@ ARR3SET(n, core[1]->rx);
 ARR3SUB(n, core[2]->rx);
 normalize(n, 3);
 
+/* CURRENT - compute the direction sense (sign) */
+crossprod(c, a, b);
+if (via(c,n,3) < 0.5*PI)
+  sign = -1.0;
+else
+  sign = 1.0;
+
 /* n[0..2] is the normal of the plane */
 /* project 1-0 onto the normal (ie dist to plane) */
 len = a[0]*n[0] + a[1]*n[1] + a[2]*n[2];
@@ -1228,7 +1199,7 @@ b[0] -= len*n[0];
 b[1] -= len*n[1];
 b[2] -= len*n[2];
 /* compute angle between projected vectors */
-return(180.0*via(a,b,3)/PI); 
+return(sign * R2D*via(a,b,3)); 
 }
 
 /********************************/
