@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -21,8 +21,13 @@ The GNU GPL can also be found at http://www.gnu.org
 */
 
 struct model_pak *model_new(void);
+void model_init(struct model_pak *);
+gint model_prep(struct model_pak *);
+void model_free(struct model_pak *);
 void model_delete(struct model_pak *);
+gpointer model_dup(struct model_pak *);
 
+void model_content_refresh(struct model_pak *);
 
 void property_free(gpointer);
 void property_add_ranked(guint, 
