@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -20,10 +20,34 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 The GNU GPL can also be found at http://www.gnu.org
 */
 
+/* NB: property table is still assoc with model */
+/* configs should link to relevant dialog & settings */
+struct config_pak
+{
+gint id;
+gchar *label;            /* tree label */
+gpointer data;           /* computational code structure */
+};
+
+struct project_pak
+{
+gchar *label;          /* id / tree label */
+gchar *path;           /* job directory */
+
+GSList *models;        /* associated structures */
+GSList *configs;       /* associated code configs */
+
+GHashTable *data;      /* project specific data items */
+};
+
 /* prototypes */
-gpointer project_create(gpointer, struct model_pak *);
-void project_remove(gpointer, struct model_pak *);
-void project_remove_all(struct model_pak *);
-gpointer project_data(gpointer);
+gpointer project_new(const gchar *);
+gpointer project_model_new(gpointer);
+gpointer project_model_get(gint, gpointer);
+gpointer project_data_get(gchar *, gpointer);
+
+void project_model_add(struct model_pak *, gpointer);
+void project_data_set(gchar *, gpointer, gpointer);
+
 gchar *project_path(gpointer);
 
