@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -27,66 +27,43 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "gdis.h"
 #include "coords.h"
 #include "matrix.h"
-#include "spatial.h"
+#include "quaternion.h"
 #include "measure.h"
+#include "spatial.h"
 #include "opengl.h"
+#include "render.h"
 #include "select.h"
 #include "zone.h"
 #include "interface.h"
-#include "gtkshorts.h"
+#include "shortcuts.h"
 
 extern struct sysenv_pak sysenv;
 extern struct elem_pak elements[];
 
 gint backbone = 0;
 
-/**********************/
-/* set selection mode */
-/**********************/
-void set_selection_mode(GtkWidget *w, gpointer *data)
+/*********************************/
+/* update selection connectivity */
+/*********************************/
+void select_connect_update(struct model_pak *model)
 {
-gint len;
-const gchar *tmp;
+GSList *list;
 
-tmp = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(data)->entry));
-len = strlen(tmp);
-switch(len)
-  {
-  case 7:
-    sysenv.select_mode = REGION;
-    break;
-  case 8:
-    sysenv.select_mode = ELEM;
-    break;
-  case 9:
-    sysenv.select_mode = MOL;
-    break;
-  case 11:
-    sysenv.select_mode = ATOM_LABEL;
-    break;
+/* remove all old bonds first */
+for (list=model->selection ; list ; list=g_slist_next(list))
+  connect_atom_clear(list->data, model);
 
-/* NEW */
-  case 19:
-    sysenv.select_mode = FRAGMENT;
-    switch_mode(SELECT_FRAGMENT);
-    return;
+/* now create the new connectivity */
+for (list=model->selection ; list ; list=g_slist_next(list))
+  connect_atom_compute(list->data, model);
 
-  case 20:
-    sysenv.select_mode = ELEM_MOL;
-    break;
-  default:
-    sysenv.select_mode = CORE;
-    break;
-  }
-/* NEW - switch to normal selection mode */
-/* is this sensible behaviour??? */
-switch_mode(FREE);
+connect_molecules(model);
 }
 
 /****************************/
 /* mark selection as ghosts */
 /****************************/
-void set_selection_ghosts(void)
+void select_flag_ghost(void)
 {
 GSList *list;
 struct core_pak *core;
@@ -99,7 +76,7 @@ if (!data)
 /* clean up selection highlighting */
 for (list=data->selection ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
   core->ghost = TRUE;
   init_atom_colour(core, data);
   }
@@ -109,7 +86,7 @@ redraw_canvas(SINGLE);
 /****************************/
 /* mark selection as normal */
 /****************************/
-void set_selection_normal(void)
+void select_flag_normal(void)
 {
 GSList *list;
 struct core_pak *core;
@@ -122,7 +99,7 @@ if (!data)
 /* clean up selection highlighting */
 for (list=data->selection ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
   core->ghost = FALSE;
   init_atom_colour(core, data);
   }
@@ -157,7 +134,7 @@ struct core_pak *core;
 
 for (list=cores ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
   core->render_mode = mode;
   }
 }
@@ -169,7 +146,7 @@ struct core_pak *core;
 
 for (list=cores ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
   core->render_wire = wire;
   }
 }
@@ -177,27 +154,21 @@ for (list=cores ; list ; list=g_slist_next(list))
 /*******************/
 /* clear selection */
 /*******************/
-void select_clear(void)
+void select_clear(struct model_pak *model)
 {
 GSList *list;
-struct model_pak *data;
 struct core_pak *core;
 
-data = sysenv.active_model;
-if (!data)
-  return;
+g_assert(model != NULL);
 
 /* clean up selection highlighting */
-for (list=data->selection ; list ; list=g_slist_next(list))
+for (list=model->selection ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
   core->status &= ~SELECT;
   }
-g_slist_free(data->selection);
-data->selection=NULL;
-
-/* redraw */
-redraw_canvas(SINGLE);
+g_slist_free(model->selection);
+model->selection=NULL;
 }
 
 /******************/
@@ -224,7 +195,7 @@ void select_paste(void)
 {
 struct model_pak *data, *src;
 struct core_pak *core, *orig;
-GSList *clist;
+GSList *clist, *slist;
 
 /* checks */
 data = sysenv.active_model;
@@ -234,18 +205,15 @@ if (data == NULL || src == NULL)
 if (data->id == NODATA || !src->selection)
   return;
 
-for (clist=data->selection ; clist ; clist=g_slist_next(clist))
-  {
-  core = (struct core_pak *) clist->data;
-  core->status &= ~SELECT;
-  }
-g_slist_free(data->selection);
-data->selection = NULL;
+/* copy the selection - just in case src == data */
+slist = g_slist_copy(src->selection);
+/* ensure pasted atoms will be the only ones that are selected */
+select_clear(data);
 
 /* duplicate cores */
-for (clist=src->selection ; clist ; clist=g_slist_next(clist))
+for (clist=slist ; clist ; clist=g_slist_next(clist))
   {
-  orig = (struct core_pak *) clist->data;
+  orig = clist->data;
 
 /* duplicate the core & transform into the new model's lattice space */
   core = copy_core(orig, src, data);
@@ -255,17 +223,16 @@ for (clist=src->selection ; clist ; clist=g_slist_next(clist))
     core->region = REGION1A;
 
 /* add core to the selection */
-  data->selection = g_slist_prepend(data->selection, core);
-  core->status |= SELECT;
+  select_add_core(core, data);
   }
 
-/* updates */
-init_objs(INIT_COORDS, data);
-zone_init(data);
-calc_bonds(data);
-calc_mols(data);
+g_slist_free(slist);
 
-data->mode = FREE;
+/* updates */
+coords_compute(data);
+zone_init(data);
+connect_bonds(data);
+connect_molecules(data);
 redraw_canvas(SINGLE);
 }
 
@@ -291,7 +258,7 @@ P3VEC("colour: ", colour);
 
 for (list=model->selection ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
 
   ARR3SET(core->colour, colour);
   VEC3MUL(core->colour, 65535.0);
@@ -329,7 +296,7 @@ if (!data)
 list = data->selection;
 while (list)
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
   list = g_slist_next(list);
   delete_core(core);
   }
@@ -361,7 +328,7 @@ if (!data->selection)
 /* hide */
 for (list=data->selection ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
   core->status |= HIDDEN;
   core->status &= ~SELECT;
   }
@@ -397,7 +364,7 @@ data->selection = g_slist_copy(data->cores);
 /* update the highlighting */
 for (list=data->selection ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
   core->status |= SELECT;
   }
 redraw_canvas(SINGLE);
@@ -423,7 +390,7 @@ new = g_slist_copy(data->cores);
 /* remove the old selection */
 for (list=data->selection ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
 
   core->status &= ~SELECT;
   new = g_slist_remove(new, core);
@@ -436,7 +403,7 @@ data->selection = new;
 /* update the highlighting */
 for (list=data->selection ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
 
   core->status |= SELECT;
   }
@@ -461,13 +428,28 @@ if (core->status & HIDDEN)
 data->selection = g_slist_append(data->selection, core);
 core->status |= SELECT;
 
-/* update the atom info box */
-if (g_slist_length(data->selection) == 1)
-  atom_properties_update(core, data);
-
+gui_refresh(GUI_MODEL_PROPERTIES);
 return(0);
 }
 
+/***********************************************/
+/* select all cores of a given atom label type */
+/***********************************************/
+void select_all_types(struct core_pak *core, struct model_pak *model)
+{
+GSList *list;
+struct core_pak *comp;
+
+for (list=model->cores ; list ; list=g_slist_next(list))
+  {
+  comp = list->data;
+
+  if (core->atom_type)
+    if (g_ascii_strcasecmp(comp->atom_type, core->atom_type) == 0)
+      select_add_core(comp, model);
+  }
+}
+
 /***********************************************/
 /* select all cores of a given atom label type */
 /***********************************************/
@@ -478,9 +460,9 @@ struct core_pak *comp;
 
 for (list=model->cores ; list ; list=g_slist_next(list))
   {
-  comp = (struct core_pak *) list->data;
+  comp = list->data;
  
-  if (g_ascii_strcasecmp(comp->label, core->label) == 0)
+  if (g_ascii_strcasecmp(comp->atom_label, core->atom_label) == 0)
     select_add_core(comp, model);
   }
 }
@@ -497,7 +479,7 @@ g_assert(model != NULL);
 
 for (list=model->cores ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
  
   if (core->atom_code == code)
     select_add_core(core, model);
@@ -517,7 +499,7 @@ g_assert(core != NULL);
 
 for (list=model->cores ; list ; list=g_slist_next(list))
   {
-  comp = (struct core_pak *) list->data;
+  comp = list->data;
   if (core->atom_code == comp->atom_code && core->molecule == comp->molecule)
     select_add_core(comp, model);
   }
@@ -537,7 +519,7 @@ if (!mol)
 
 for (list=mol->cores ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
   select_add_core(core, model);
   }
 }
@@ -548,103 +530,24 @@ for (list=mol->cores ; list ; list=g_slist_next(list))
 #define DEBUG_ADD_FRAGMENT 0
 void select_add_fragment(struct core_pak *core, struct model_pak *model)
 {
-static struct core_pak *first=NULL, *core1, *core2;
-GSList *list, *item1, *item2, *branch_all, *branch_new;
+GSList *list, *cores;
+static struct core_pak *first=NULL;
 
 /* test for first or second call */
 if (model->state)
   {
   g_assert(first != NULL);
 
-/* clear the pruned flag */
-  for (item1=model->cores ; item1 ; item1=g_slist_next(item1))
-    {
-    core1 = (struct core_pak *) item1->data;
-    core1->status &= ~PRUNED;
-    }
-
-/* get initial branches */
-  branch_new = exit_branches(first, core);
-  item1 = branch_all = g_slist_prepend(branch_new, core);
-
-#if DEBUG_ADD_FRAGMENT
-printf("seeking branches for: %p -> %p : ", first, core);
-printf("initial branches: %d\n", g_slist_length(branch_new));
-for (item3=branch_new ; item3 ; item3 = g_slist_next(item3))
-  {
-  core3 = (struct core_pak *) item3->data;
-  printf(" - %p (%s)\n", core3, core3->label);
-  }
-#endif
-
-/* iterate through upper branch level */
-  while (item1)
-    {
-    core1 = (struct core_pak *) item1->data;
-    item1 = g_slist_next(item1);
-    g_assert(core1 != NULL);
-
-/* iterate through lower branch level and aquire next level of branches */
-    item2 = branch_new;
-    branch_new = NULL;
-    while (item2)
-      {
-      core2 = (struct core_pak *) item2->data;
-      g_assert(core2 != NULL);
-      item2 = g_slist_next(item2);
-      if (core2->status & PRUNED)
-        continue;
-
-/* prune this core, so its branches are not reexamined */
-      core2->status |= PRUNED;
-
-      list = exit_branches(core1, core2);
-
-#if DEBUG_ADD_FRAGMENT
-{
-struct core_pak *core3;
-GSList *item3;
-
-printf("branches: %p (%s) -> %p (%s)", core1, core1->label, core2, core2->label);
-printf(" : found %d.\n", g_slist_length(list));
-for (item3=list ; item3 ; item3 = g_slist_next(item3))
-  {
-  core3 = (struct core_pak *) item3->data;
-  printf(" - %p (%s)\n", core3, core3->label);
-  }
-}
-#endif
-
-/* aquire all branches at current level */
-      branch_new = g_slist_concat(branch_new, list);
-      }
-
-/* test if we arrive back at first core (from a different direction) */
-    if (g_slist_find(branch_new, first))
-      {
-      show_text(ERROR, "Bad fragment.\n");
-      g_slist_free(branch_all);
-      model->state = 0;
-      first = NULL;
-      return;
-      }
-
-/* accumlate new branches into main list */
-    branch_all = g_slist_concat(branch_all, branch_new);
-    }
+/* aqcuire the fragment */
+  connect_fragment_init(model);
+  cores = connect_fragment_get(first, core, model);
 
 /* add all cores to the selection */
-/* FIXME - why is the first core's selection getting wiped??? */
   select_add_core(first, model);
-  select_add_core(core, model);
-  for (item1=branch_all ; item1 ; item1=g_slist_next(item1))
-    {
-    core1 = (struct core_pak *) item1->data;
-    select_add_core(core1, model);
-    }
+  for (list=cores ; list ; list=g_slist_next(list))
+    select_add_core(list->data, model);
 
-/* done */
-  g_slist_free(branch_all);
+  g_slist_free(list);
   model->state = 0;
   first = NULL;
   }
@@ -667,13 +570,9 @@ struct core_pak *comp;
 g_assert(model != NULL);
 g_assert(core != NULL);
 
-/*
-printf("select region: %d in: %p\n", core->region, model);
-*/
-
 for (list=model->cores ; list ; list=g_slist_next(list))
   {
-  comp = (struct core_pak *) list->data;
+  comp = list->data;
   if (core->region == comp->region)
     select_add_core(comp, model);
   }
@@ -691,6 +590,48 @@ core->status &= ~SELECT;
 model->selection = g_slist_remove(model->selection, core);
 }
 
+/******************************************************/
+/* select atom(s) according to current selection mode */
+/******************************************************/
+void select_core(struct core_pak *core, gint toggle, struct model_pak *model)
+{
+switch (sysenv.select_mode)
+  {
+  case ATOM_LABEL:
+    select_all_labels(core, model);
+    break;
+  case ATOM_TYPE:
+    select_all_types(core, model);
+    break;
+  case CORE:
+/* toggle = true -> if selected already; deselect */
+/* toggle = false -> if selected already; remain selected */
+    if (toggle)
+      {
+      if (select_add_core(core, model))
+        select_del_core(core, model);
+      }
+    else
+      select_add_core(core, model);
+    break;
+  case ELEM:
+    select_all_elem(core->atom_code, model);
+    break;
+  case ELEM_MOL:
+    select_all_elem_in_mol(core, model);
+    break;
+  case MOL:
+    select_add_mol(core->mol, model);
+    break;
+  case FRAGMENT:
+    select_add_fragment(core, model);
+    break;
+  case REGION:
+    select_add_region(core, model);
+    break;
+  }
+}
+
 /********************************/
 /* rotate the current selection */
 /********************************/
@@ -709,8 +650,8 @@ if (!g_slist_length(data->selection))
 VEC3SET(scent, 0.0, 0.0, 0.0);
 for (list=data->selection ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
-  ARR3ADD(scent, core->rx);
+  core = list->data;
+  ARR3ADD(scent, core->x);
   }
 VEC3MUL(scent, 1.0/g_slist_length(data->selection));
 
@@ -721,47 +662,39 @@ P3VEC("scent: ", scent);
 
 for (list=data->selection ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
 
-  ARR3SET(vec, core->rx);
+  ARR3SET(vec, core->x);
   ARR3SUB(vec, scent);
-
-/* rotate to get positions */
   vecmat(mat, vec);
   ARR3ADD(vec, scent);
-/* lattice/rotation back transform */
-  vecmat(data->irotmat, vec);
-  vecmat(data->ilatmat, vec);
-  ARR3ADD(vec, data->centroid);
-/* store */
   ARR3SET(core->x, vec);
+
 /* shells */
   if (core->shell)
     {
-    shell = (struct shel_pak *) core->shell;
+    shell = core->shell;
 
-    ARR3SET(vec, shell->rx);
+    ARR3SET(vec, shell->x);
     ARR3SUB(vec, scent);
 
 /* rotate to get on screen positions, and store the values */
     vecmat(mat, vec);
     ARR3ADD(vec, scent);
-/* lattice/rotation back transform */
-    vecmat(data->irotmat, vec);
-    vecmat(data->ilatmat, vec);
-    ARR3ADD(vec, data->centroid);
-/* store */
     ARR3SET(shell->x, vec);
     }
   }
 
-/* redo connectivity */
-if (data->cbu)
-  {
-/* slow, but correct */
-  calc_bonds(data);
-  calc_mols(data);
-  }
+/* updates */
+coords_compute(data);
+zone_init(data);
+
+/* FIXME - fine grained approach sometimes misses bonds */
+/*
+select_connect_update(data);
+*/
+connect_bonds(data);
+connect_molecules(data);
 
 meas_graft_model(data);
 }
@@ -769,32 +702,66 @@ meas_graft_model(data);
 /*************************/
 /* translate a selection */
 /*************************/
-void xlat_select(gdouble *x, struct model_pak *data)
+void select_translate(gint px, gint py, struct model_pak *model)
 {
+gdouble x[3], r[3], qi[9], tm[9], ti[9];
 GSList *list;
 struct core_pak *core;
 struct shel_pak *shel;
 
-for (list=data->selection ; list ; list=g_slist_next(list))
+g_assert(model != NULL);
+
+/* translation */
+VEC3SET(x, px, 0.0, -py);
+VEC3MUL(x, 0.01);
+
+/* lattice transformation */
+memcpy(ti, model->latmat, 9*sizeof(gdouble));
+
+/* camera transformation */
+quat_matrix(tm, camera_q(model->camera));
+quat_matrix(qi, camera_q(model->camera));
+matrix_invert(qi);
+
+/* build transform matrices */
+matmat(qi, ti);
+matmat(model->ilatmat, tm);
+
+for (list=model->selection ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
-  ARR3ADD(core->x, x);
+  core = list->data;
+
+/* transform */
+  ARR3SET(r, core->x);
+  vecmat(ti, r);
+  ARR3ADD(r, x);
+  vecmat(tm, r);
+  ARR3SET(core->x, r);
+
   if (core->shell)
     {
-    shel = (struct shel_pak *) core->shell;
-    ARR3ADD(shel->x, x);
+    shel = core->shell;
+
+/* transform */
+    ARR3SET(r, shel->x);
+    vecmat(ti, r);
+    ARR3ADD(r, x);
+    vecmat(tm, r);
+    ARR3SET(shel->x, r);
     }
   }
 
-/* redo connectivity */
-if (data->cbu)
-  {
-/* slow, but correct */
-  calc_bonds(data);
-  calc_mols(data);
-  }
+/* updates */
+coords_compute(model);
+zone_init(model);
+/* FIXME - fine grained approach sometimes misses bonds */
+/*
+select_connect_update(model);
+*/
+connect_bonds(model);
+connect_molecules(model);
 
-meas_graft_model(data);
+meas_graft_model(model);
 }
 
 /************************************/
@@ -809,7 +776,7 @@ struct core_pak *exit;
 
 for (list=core->bonds ; list ; list=g_slist_next(list))
   {
-  bond = (struct bond_pak *) list->data;
+  bond = list->data;
 
   if (core == bond->atom1)
     exit = bond->atom2;
@@ -824,7 +791,6 @@ for (list=core->bonds ; list ; list=g_slist_next(list))
     if (exit->atom_code == backbone)
       branches++;
   }
-
 return(branches);
 }
 
@@ -844,9 +810,7 @@ if (backbone)
 
 for (list1=core2->bonds ; list1 ; list1=g_slist_next(list1))
   {
-  bond = (struct bond_pak *) list1->data;
-  if (bond->periodic == BOND_SPLIT)
-    continue;
+  bond = list1->data;
   if (bond->type == BOND_HBOND)
     continue;
 
@@ -915,7 +879,7 @@ struct node_pak *node_data;
 
 for (list=node_list ; list ; list=g_list_next(list))
   {
-  node_data = (struct node_pak *) list->data;
+  node_data = list->data;
 
   if (node_data->core == core)
     return(node_data);
@@ -947,9 +911,9 @@ link->size = 0;
 link->id = link_id++;
 
 list = g_list_last(chain);
-link->last = (struct core_pak *) list->data;
+link->last = list->data;
 list = chain;
-link->first = (struct core_pak *) list->data;
+link->first = list->data;
 
 while (list)
   {
@@ -982,13 +946,13 @@ struct core_pak *curr=NULL, *prev=NULL, *next=NULL;
 
 /* current atom */
 list = g_list_last(chain);
-curr = (struct core_pak *) list->data;
+curr = list->data;
 
 /* attempt to get the atom we last left */
 list = g_list_previous(list);
 
 if (list)
-  prev = (struct core_pak *) list->data;
+  prev = list->data;
 else
   {
 /* last submitted link of current node */
@@ -996,12 +960,12 @@ else
 /* last atom in this list should be the current one - get previous */
   if (list)
     {
-    last_link = (struct link_pak *) list->data;
+    last_link = list->data;
     list = g_list_last(last_link->chain);
     list = g_list_previous(list);
 
     g_assert(list != NULL);
-    prev = (struct core_pak *) list->data;
+    prev = list->data;
     }
   }
 
@@ -1014,7 +978,7 @@ if (node->core == curr)
 /* go through candidate pathways */
   while (slist)
     {
-    next = (struct core_pak *) slist->data;
+    next = slist->data;
 /* if explored previously, then skip */
     slist2 = node->exit_list;
     while (slist2)
@@ -1049,7 +1013,7 @@ printf("traverse() explored: %d\n", node->num_explored);
   }
 else
   if (slist)
-    next = (struct core_pak *) slist->data;
+    next = slist->data;
 
 #if DEBUG_TRAVERSE
 printf("traverse() next atom: %p\n", next);
@@ -1083,7 +1047,7 @@ if (rnode)
   list = g_list_last(list);
   list = g_list_previous(list);
   g_assert(list != NULL);
-  core = (struct core_pak *) list->data;
+  core = list->data;
 
 /* CURRENT - add to node's explored list */
   rnode->exit_list = g_slist_append(rnode->exit_list, core);
@@ -1101,7 +1065,7 @@ printf("terminating cyclic group.\n");
   list = cnode->link_list;
   while(list)
     {
-    temp_link = (struct link_pak *) list->data;
+    temp_link = list->data;
  
 /* TODO - other cases where only one of these match - eg type = JOINED */
 /* loop: A-C-B-D */
@@ -1150,7 +1114,7 @@ if (dest_link)
 /* loop through the input chain & store at destination */
   while (list)
     {
-    core = (struct core_pak *) list->data;
+    core = list->data;
     dest_list = g_list_append(dest_list, core);
     dest_link->size++;
     list = g_list_next(list);
@@ -1197,31 +1161,44 @@ for (list=core->bonds ; list ; list=g_slist_next(list))
 #define DEBUG_PREPROCESS 0
 void preprocess(struct core_pak *start, struct model_pak *data)
 {
-gint type, flag;
+gint flag;
 GSList *list;
 struct core_pak *core;
 
-type = start->atom_code;
+g_assert (data != NULL);
+
+/* set global - yuck */
+if (start)
+  backbone = start->atom_code;
+else
+  backbone = 0;
 
 /* clear the pruned flag for backbone & prune all others */
 for (list=data->cores ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
 
-  if (core->atom_code == type)
+  if (backbone)
+    {
+  if (core->atom_code == backbone)
     core->status &= ~PRUNED;
   else
     core->status |= PRUNED;
+    }
+  else
+    core->status &= ~PRUNED;
   }
 
 /* prune "dangling" backbone type links */
+if (backbone)
+  {
 flag=1;
 while (flag)
   {
   flag=0;
   for (list=data->cores ; list ; list=g_slist_next(list))
     {
-    core = (struct core_pak *) list->data;
+    core = list->data;
 
 /* only prune once */
     if (core->status & PRUNED)
@@ -1248,11 +1225,12 @@ while (flag)
       }
     }
   }
+  }
 
 #if DEBUG_PREPROCESS
 for (list=data->cores ; list ; list=g_slist_next(list))
   {
-  core = (struct core_pak *) list->data;
+  core = list->data;
   if (core->status & PRUNED)
     core->status |= HIDDEN;
   }
@@ -1280,7 +1258,6 @@ g_assert(data != NULL);
 
 /* clicked atom */
 core = start;
-backbone = start->atom_code;
 
 /* prune the model first (ie remove dangling chains) */
 /* to make tracing the connectivity easier */
@@ -1312,7 +1289,7 @@ printf("atom: %p is not a node.\n", core);
 if (!tmp_list)
   return;
 
-    path = (struct core_pak *) tmp_list->data; 
+    path = tmp_list->data; 
     g_slist_free(tmp_list);
 
     while (total_branches(path) < 3)
@@ -1396,7 +1373,7 @@ while (current_node)
       list = node_list;
       while (list)
         {
-        temp_node = (struct node_pak *) list->data;
+        temp_node = list->data;
         if (core == temp_node->core)
           {
 /* repeated node - join incomplete chains of the current node */
@@ -1432,7 +1409,7 @@ repeat_node = get_node_by_atom(core, node_list);
         list = g_list_previous(list);
         g_assert(list != NULL);
 
-        path = (struct core_pak *) list->data;
+        path = list->data;
 
         temp_node->exit_list = g_slist_append(temp_node->exit_list, path);
         temp_node->num_explored++;
@@ -1446,7 +1423,7 @@ repeat_node = get_node_by_atom(core, node_list);
       list = node_list;
       while (list)
         {
-        current_node = (struct node_pak *) list->data;
+        current_node = list->data;
         if (current_node->num_explored < current_node->num_branches)
           break;
         list = g_list_next(list);
@@ -1484,13 +1461,13 @@ current_link=NULL;
 list = node_list;
 while (list)
   {
-  current_node = (struct node_pak *) list->data;
+  current_node = list->data;
   printf("node: %p, branches: %d, explored: %d\n", 
   current_node->core, current_node->num_branches, current_node->num_explored);
   link_list = current_node->link_list; 
   while (link_list)
     {
-    current_link = (struct link_pak *) link_list->data;
+    current_link = link_list->data;
     printf("link type: %d, size: %d\n", current_link->type, current_link->size);
     printf("\t");
     list2 = current_link->chain;
@@ -1511,7 +1488,7 @@ while (list)
 g_list_free(current_chain);
 for (tmp_list=data->cores ; tmp_list ; tmp_list=g_slist_next(tmp_list))
   {
-  core = (struct core_pak *) tmp_list->data;
+  core = tmp_list->data;
   core->status &= ~PRUNED;
   }
 
@@ -1520,14 +1497,14 @@ construct_ribbon(node_list, data);
 backbone = 0;
 
 /* recalc coords (ie the ribbon coords) & draw */
-init_objs(REDO_COORDS, data);
+coords_init(REDO_COORDS, data);
 redraw_canvas(SINGLE);
 }
 
 /******************************************************/
 /* general function for initializing a special object */
 /******************************************************/
-/* TODO - put this somewhere else */
+/* deprec by the new spatial code */
 struct object_pak *create_object(gint type, struct model_pak *data)
 {
 struct object_pak *obj_data;
@@ -1541,6 +1518,7 @@ switch (type)
   {
   case RIBBON:
     break;
+
   default:
     g_assert_not_reached();
   }
@@ -1549,9 +1527,6 @@ switch (type)
 data->ribbons = g_slist_reverse(data->ribbons);
 data->ribbons = g_slist_prepend(data->ribbons, (gpointer *) obj_data);
 data->ribbons = g_slist_reverse(data->ribbons);
-/*
-data->ribbons = g_slist_append(data->ribbons, (gpointer *) obj_data);
-*/
 
 return(obj_data);
 }
@@ -1585,12 +1560,12 @@ ribbon = NULL;
 list = node_list;
 while (list)
   {
-  current_node = (struct node_pak *) list->data;
+  current_node = list->data;
 
   link_list = current_node->link_list; 
   while (link_list)
     {
-    current_link = (struct link_pak *) link_list->data;
+    current_link = link_list->data;
 
 /* find open chains */
     if (current_link->type == OPEN)
@@ -1600,11 +1575,11 @@ while (list)
         join1=join2=NULL;
         while (list2)
           {
-          node2 = (struct node_pak *) list2->data;
+          node2 = list2->data;
           list3 = node2->link_list; 
           while (list3)
             {
-            link2 = (struct link_pak *) list3->data;
+            link2 = list3->data;
 
 /* match only cyclic groups */
             if (link2->type == CLOSED)
@@ -1613,7 +1588,7 @@ while (list)
               
               while (list4)
                 {
-                core = (struct core_pak *) list4->data;
+                core = list4->data;
                 if (current_link->first == core)
                   {
                   group1 = link2;
@@ -1644,7 +1619,7 @@ printf("Two cyclic groups connected via: [%p] [%p], sizes: (%d) (%d)\n",
        join1, join2, group1->size, group2->size);
 #endif
 
-ribbon1 = (struct ribbon_pak *) g_malloc(sizeof(struct ribbon_pak));
+ribbon1 = g_malloc(sizeof(struct ribbon_pak));
 
 ARR3SET(ribbon1->colour, sysenv.render.ribbon_colour);
 ribbon1->colour[3] = sysenv.render.transmit;
@@ -1663,7 +1638,7 @@ ribbon1->id2 = group2->id;
         list2 = g_list_next(list2);
         while (list2)
           {
-          core = (struct core_pak *) list2->data;
+          core = list2->data;
           ARR3ADD(vec, core->x);
           list2 = g_list_next(list2);
           }
@@ -1677,7 +1652,7 @@ ribbon1->id2 = group2->id;
         list2 = g_list_next(list2);
         while (list2)
           {
-          core = (struct core_pak *) list2->data;
+          core = list2->data;
           ARR3ADD(vec, core->x);
           list2 = g_list_next(list2);
           }
@@ -1718,7 +1693,7 @@ ribbon1->id2 = group2->id;
 rlist1 = ribbon;
 while (rlist1)
   {
-  ribbon1 = (struct ribbon_pak *) rlist1->data;
+  ribbon1 = rlist1->data;
 
 #if DEBUG_CONSTRUCT_RIBBON
 printf("ribbon segment: [%d] - [%d]\n", ribbon1->id1, ribbon1->id2);
@@ -1740,7 +1715,7 @@ printf("ribbon segment: [%d] - [%d]\n", ribbon1->id1, ribbon1->id2);
   rlist2 = g_slist_next(rlist1);
   while (rlist2)
     {
-    ribbon2 = (struct ribbon_pak *) rlist2->data;
+    ribbon2 = rlist2->data;
 
 /* if match - get the two orientation vectors at the ribbon meeting point */
     v1 = v2 = NULL;
