@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -29,9 +29,11 @@ The GNU GPL can also be found at http://www.gnu.org
 
 #include "gdis.h"
 #include "coords.h"
+#include "model.h"
 #include "matrix.h"
 #include "sginfo.h"
 #include "space.h"
+#include "zone.h"
 #include "dialog.h"
 #include "interface.h"
 
@@ -162,7 +164,7 @@ struct shel_pak *sr_shell;
 
 /* include an inversion (ie mult by -1) operation? */
 smin = 0;
-if (model->sginfo.centric == -1)
+if (model->sginfo.inversion == -1)
   smin = -2;
 
 /* s = +1 or -1 for inversion of matrix elements */
@@ -175,7 +177,7 @@ for (s=1 ; s>smin ; s-=2)
 /* loop over asymetric cores */
     for (list=model->cores ; list ; list=g_slist_next(list))
       {
-      core = (struct core_pak *) list->data;
+      core = list->data;
       if (!core->primary)
         continue;
 
@@ -195,29 +197,48 @@ for (s=1 ; s>smin ; s-=2)
         {
         sr_shell = sr_core->shell;
         model->shels = g_slist_prepend(model->shels, sr_shell);
-
         sr_shell->primary = FALSE;
         sr_shell->primary_shell = sr_shell;
-
         VEC3MUL(sr_shell->x, s);
         vecmat(*(model->sginfo.matrix+j), sr_shell->x);
         sr_shell->x[0] += *(*(model->sginfo.offset+j)+0);
         sr_shell->x[1] += *(*(model->sginfo.offset+j)+1);
         sr_shell->x[2] += *(*(model->sginfo.offset+j)+2);
         }
-      }         /* loop - operations in the group */
+      }
     }
-  }             /* loop - primary atoms */
+  }
 
 /* update lists */
 model->cores = g_slist_concat(model->cores, g_slist_reverse(cores));
 
 /* remove any duplicates */
 /* NB: do this before pbc constrain, as it updates core-shell linkages */
-remove_duplicates(model);
+zone_init(model);
+delete_duplicate_cores(model);
+shell_make_links(model);
+}
 
-/* NB: core-shell linkages must be correct before we call this */
-pbc_constrain_atoms(model);
+/****************************************/
+/* initialize the space group structure */
+/****************************************/
+void space_init(gpointer data)
+{
+struct space_pak *space=data;
+
+g_assert(space != NULL);
+
+space->lookup=TRUE;
+space->spacenum=0;
+space->lattice=0;
+space->pointgroup=0;
+space->cellchoice=0;
+space->inversion=FALSE;
+space->order=0;
+space->spacename=NULL;
+space->latticename=NULL;
+space->matrix=NULL;
+space->offset=NULL;
 }
 
 /********************************/
@@ -228,10 +249,12 @@ void space_free(gpointer data)
 gint i;
 struct space_pak *space=data;
 
+g_assert(space != NULL);
+
 /* free data */
 g_free(space->spacename);
 g_free(space->latticename);
-for (i=0 ; i<space->order ; i++)
+for (i=space->order ; i-- ; )
   {
   g_free(*(space->matrix+i));
   g_free(*(space->offset+i));
@@ -240,11 +263,35 @@ g_free(space->matrix);
 g_free(space->offset);
 
 /* enforce blankness */
-space->spacename=NULL;
-space->latticename=NULL;
-space->matrix=NULL;
-space->offset=NULL;
-space->order=0;
+space_init(space);
+}
+
+/********************************/
+/* hexagonal/rhombohedral check */
+/********************************/
+gint space_primitive_cell(struct model_pak *model)
+{
+gdouble dx, x2;
+
+dx = (model->pbc[0] - model->pbc[1]);
+x2 = dx*dx;
+dx = (model->pbc[0] - model->pbc[2]);
+x2 += dx*dx;
+dx = (model->pbc[1] - model->pbc[2]);
+x2 += dx*dx;
+
+/* better tolerances? */
+if (x2 < 0.001)
+  {
+/*
+printf("Treating as rhombohedral.\n");
+*/
+  return(TRUE);
+  }
+/*
+printf("Treating as hexagonal.\n");
+*/
+return(FALSE);
 }
 
 /**************************/
@@ -257,7 +304,6 @@ gint f, i, j, nt;
 gint m;
 gint iList, nTrV, iTrV, nLoopInv, iLoopInv, iMatrix;
 gchar *label;
-gdouble dx, x2;
 GString *name;
 const T_RTMx  *lsmx;
 const gint *r, *t, *TrV;
@@ -270,7 +316,6 @@ g_return_val_if_fail(data != NULL, 1);
 /* where symmetry is broken in subsequent frames */
 if (!data->sginfo.lookup)
   {
-pbc_constrain_atoms(data);
   return(0);
   }
 
@@ -292,30 +337,16 @@ if (data->sginfo.cellchoice)
   g_string_sprintfa(name, ":%d", data->sginfo.cellchoice);
 
 /* if trigonal lattice, determine if cell is in hex or rhombo format */
-if (g_strrstr(name->str, "R") || data->sginfo.spacenum == 146 || data->sginfo.spacenum == 148
-                              || data->sginfo.spacenum == 155 || data->sginfo.spacenum == 160
-                              || data->sginfo.spacenum == 161 || data->sginfo.spacenum == 166
+if (g_strrstr(name->str, "R") || data->sginfo.spacenum == 146
+                              || data->sginfo.spacenum == 148
+                              || data->sginfo.spacenum == 155
+                              || data->sginfo.spacenum == 160
+                              || data->sginfo.spacenum == 161
+                              || data->sginfo.spacenum == 166
                               || data->sginfo.spacenum == 167)
   {
-  dx = (data->pbc[0] - data->pbc[1]);
-  x2 = dx*dx;
-  dx = (data->pbc[0] - data->pbc[2]);
-  x2 += dx*dx;
-  dx = (data->pbc[1] - data->pbc[2]);
-  x2 += dx*dx;
-
-/* better tolerances? */
-  if (x2 < 0.001)
-    {
-#if DEBUG_GEN_POS
-printf("Treating as rhombohedral.\n");
-#endif
-    g_string_sprintfa(name, ":R");   /* primitive (rhombohedral) */
-    }
-#if DEBUG_GEN_POS
-  else
-    printf("Treating as hexagonal.\n");
-#endif
+  if (space_primitive_cell(data))
+    g_string_sprintfa(name, ":R");
   }
 
 #if DEBUG_GEN_POS
@@ -328,14 +359,8 @@ if (BuildSgInfo(&SgInfo, name->str) != 0)
 /* CURRENT - if order > 0 -> fill the cell */
   if (data->sginfo.order)
     {
-    show_text(WARNING, "No spacegroup: using explicit matrices.\n");
+    gui_text_show(WARNING, "No spacegroup: using explicit matrices.\n");
     space_fill_cell(data);
-/* NB: we must constrain here if the periodic bond calculation is */
-/* to be done correctly. Since we can have auto unfragment on */
-/* this shouldn't be a problem for models with parts sticking out */
-/*
-    pbc_constrain_atoms(data);
-*/
     return(0);
     }
   return(2);
@@ -400,16 +425,17 @@ switch(SgInfo.XtalSystem)
 
 /* point group */
 data->sginfo.pointgroup = SgInfo.PointGroup;
-/* centred */
-data->sginfo.centric = SgInfo.Centric;
+data->sginfo.inversion = SgInfo.Centric;
+data->sginfo.centering = SgInfo.LatticeInfo->Code;
 
 #if DEBUG_GEN_POS
-printf("   Space group name: %s\n",data->sginfo.spacename);
-printf(" Space group number: %d\n",data->sginfo.spacenum);
-printf("       Lattice type: %c\n",SgInfo.LatticeInfo->Code);
-printf("              Order: %d\n",SgInfo.OrderL);
+printf("   Space group name: %s\n", data->sginfo.spacename);
+printf("        Point group: %d\n", data->sginfo.pointgroup);
+printf(" Space group number: %d\n", data->sginfo.spacenum);
+printf("       Lattice type: %c\n", data->sginfo.centering);
+printf("              Order: %d\n", SgInfo.OrderL);
 printf("   Inversion center: ");
-if (SgInfo.Centric == -1)
+if (data->sginfo.inversion == -1)
   printf("yes\n");
 else
   printf("no\n");
@@ -491,10 +517,15 @@ printf("Skipping symmetry matrix generation.\n");
 printf("Symmetry matrices: %d\n", m);
 for (i=0 ; i<m ; i++)
   {
-  printf("matrix %d :",i);
+  printf("matrix %d",i);
+
+P3MAT(" : ", *(data->sginfo.matrix+i));
+
+/*
   for (j=0 ; j<9 ; j++)
     printf(" %4.1f",*(*(data->sginfo.matrix+i)+j));
   printf("\n");
+*/
 
   printf("offset %d :",i);
   for (j=0 ; j<3 ; j++)
@@ -536,18 +567,6 @@ struct shel_pak *shell;
 /* checks */
 g_assert(model != NULL);
 
-/* set spacegroup */
-/*
-model->sginfo.spacenum = 1;
-model->sginfo.spacename = g_strdup("P 1");
-model->sginfo.cellchoice = 0;
-model->sginfo.order = 0;
-*/
-
-space_free(&model->sginfo);
-model->sginfo.spacenum = 1;
-model->sginfo.cellchoice = 0;
-
 /* initialize atoms */
 for (list=model->cores ; list ; list=g_slist_next(list))
   {
@@ -561,6 +580,12 @@ for (list=model->shels ; list ; list=g_slist_next(list))
   shell->orig = TRUE;
   shell->primary = TRUE;
   }
+
+/* set spacegroup */
+space_free(&model->sginfo);
+model->sginfo.spacenum = 1;
+model->sginfo.cellchoice = 0;
+space_lookup(model);
 }
 
 /***********************************************/
@@ -586,7 +611,7 @@ clist = slist = NULL;
 /* loop over images 1st, so we preserve core ordering in cell images */
 for (ilist=model->images ; ilist ; ilist=g_slist_next(ilist))
   {
-  image = (struct image_pak *) ilist->data;
+  image = ilist->data;
   for (list=model->cores ; list ; list=g_slist_next(list))
     {
     core = list->data;
@@ -664,7 +689,6 @@ model->image_limit[4] = 0.0;
 model->image_limit[5] = 1.0;
 
 space_make_p1(model);
-prep_model(model);
 }
 
 /***********************************/
@@ -746,7 +770,7 @@ switch(mode)
   }
 
 /*
-gtksh_relation_update(model);
+gui_relation_update(model);
 */
 }
 
