@@ -1,7 +1,7 @@
 /*
 Copyright (C) 2003 by Sean David Fleming
 
-sean@power.curtin.edu.au
+sean@ivec.org
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -20,20 +20,16 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 The GNU GPL can also be found at http://www.gnu.org
 */
 
+void zone_free(gpointer);
+gpointer zone_make(gdouble, struct model_pak *);
 void zone_init(struct model_pak *);
-void zone_free(struct model_pak *);
-void zone_info(struct model_pak *);
-
-void zone_append_core(gpointer, struct core_pak *);
-void zone_remove_core(gpointer, struct core_pak *);
-void zone_append_shell(gpointer, struct shel_pak *);
-void zone_remove_shell(gpointer, struct shel_pak *);
-
-gpointer zone_ptr(struct core_pak *, struct model_pak *);
-gint *zone_grid(gpointer);
-gint zone_index(gdouble *, struct model_pak *);
+gint zone_index(gdouble *, gpointer);
+gpointer zone_get(gdouble *, gpointer);
 
 GSList *zone_cores(gpointer);
 GSList *zone_shells(gpointer);
-GSList *zone_area_cores(gint, gpointer, struct model_pak *);
+GSList *zone_area_cores(gint, gpointer, gpointer);
+
+void zone_display_init(gpointer, struct model_pak *);
+void zone_visible_init(struct model_pak *);
 
